@@ -32,13 +32,31 @@
 #ifndef lint
 static char copyright[] =
 "@(#) Copyright 1994 Purdue Research Foundation.\nAll rights reserved.\n";
-static char *rcsid = "$Id: arg.c,v 1.31 10/.0/.1 .0:.5:.4 abe Exp $";
+static char *rcsid = "$Id: arg.c,v 1.36 2001/02/13 02:06:23 abe Exp $";
 #endif
 
 
 #include "lsof.h"
 
 
+/*
+ * Local definitions
+ */
+
+#define	CMDRXINCR	32		/* CmdRx[] allocation increment */
+
+
+/*
+ * Local static variables
+ */
+
+static int NCmdRxA = 0;			/* space allocated to CmdRx[] */
+
+
+/*
+ * Local function prototypes
+ */
+
 _PROTOTYPE(static int ckfd_range,(char *first, char *dash, char *last, int *lo, int *hi));
 _PROTOTYPE(static void enter_fd_lst,(char *nm, int lo, int hi));
 _PROTOTYPE(static int enter_nwad,(struct nwad *n, int sp, int ep, char *s, struct hostent *he));
@@ -173,6 +191,7 @@ ck_file_arg(i, ac, av, fv, rs, sbp)
 		for (mx = 0; mx < nm; mx++) {
 		    if (strcmp(mp->dir, mmp[mx]->dir) == 0
 		    &&  mp->dev == mmp[mx]->dev
+		    &&  mp->rdev == mmp[mx]->rdev
 		    &&  mp->inode == mmp[mx]->inode)
 			break;
 		}
@@ -246,6 +265,11 @@ ck_file_arg(i, ac, av, fv, rs, sbp)
 			    ErrStat = 1;
 			    continue;
 			}
+
+#if	defined(HASSPECDEVD)
+			(void) HASSPECDEVD(fnm, &sb);
+#endif	/* defined(HASSPECDEVD) */
+
 		    }
 		    sfp->i = sb.st_ino;
 		    sfp->mode = sb.st_mode & S_IFMT;
@@ -256,26 +280,11 @@ ck_file_arg(i, ac, av, fv, rs, sbp)
 		 * already-expanded local mount info table device numbers.
 		 * (This is an EP/IX 2.1.1 and above artifact.)
 		 */
-		    dev = expdev(sb.st_dev);
-		    rdev = expdev(sb.st_rdev);
-#endif	/* defined(CKFA_EXPDEV) */
-
-		    if (sfp->mode == S_IFBLK || sfp->mode == S_IFCHR
-
-#if	defined(CKFA_XDEVTST)
-		    ||  CKFA_XDEVTST
-#endif	/* defined(CKFA_XDEVTST) */
-
-		    )
-
-#if	defined(CKFA_EXPDEV)
-			sfp->dev = rdev;
-		    else
-			sfp->dev = dev;
+		    sfp->dev = expdev(sb.st_dev);
+		    sfp->rdev = expdev(sb.st_rdev);
 #else	/* !defined(CKFA_EXPDEV) */
-			sfp->dev = sb.st_rdev;
-		    else
-			sfp->dev = sb.st_dev;
+		    sfp->dev = sb.st_dev;
+		    sfp->rdev = sb.st_rdev;
 #endif	/* defined(CKFA_EXPDEV) */
 
 #if	defined(CKFA_MPXCHAN)
@@ -306,11 +315,12 @@ ck_file_arg(i, ac, av, fv, rs, sbp)
 		/*
 		 * Derive file name and file system name for a mount point.
 		 *
-		 * Save the device number, inode number, and modes.
+		 * Save the device numbers, inode number, and modes.
 		 */
 		    fnm = mp->dir;
 		    fsnm = mp->fsname;
 		    sfp->dev = mp->dev;
+		    sfp->rdev = mp->rdev;
 		    sfp->i = mp->inode;
 		    sfp->mode = mp->mode & S_IFMT;
 		}
@@ -357,10 +367,10 @@ ck_file_arg(i, ac, av, fv, rs, sbp)
 		if (!Mtprocfs || Procsrch)
 		    continue;
 
-# if	defined(HASFSTYPE)
+# if	defined(HASFSTYPE) && HASFSTYPE==1
 		if (strcmp(sb.st_fstype, HASPROCFS) != 0)
 		    continue;
-# endif	/* defined(HASFSTYPE) */
+# endif	/* defined(HASFSTYPE) && HASFSTYPE==1 */
 
 		if (pfsnl == -1)
 		    pfsnl = strlen(Mtprocfs->dir);
@@ -524,6 +534,162 @@ ctrl_dcache(c)
 #endif	/* defined(HASDCACHE) */
 
 
+/*
+ * enter_cmd_rx() - enter command regular expression
+ */
+
+int
+enter_cmd_rx(x)
+	char *x;			/* regular expression */
+{
+	int bmod = 0;
+	int bxmod = 0;
+	int i, re;
+	int imod = 0;
+	int xmod = 0;
+	int co = REG_NOSUB|REG_EXTENDED;
+	char reb[256], *xb, *xe, *xm;
+	MALLOC_S xl;
+	char *xp = (char *)NULL;
+/*
+ * Make sure the supplied string starts a regular expression.
+ */
+	if (!*x || (*x != '/')) {
+	    (void) fprintf(stderr, "%s: regexp doesn't begin with '/': ", Pn);
+	    if (x)
+		safestrprt(x, stderr, 1);
+	    return(1);
+	}
+/*
+ * Skip to the end ('/') of the regular expression.
+ */
+	xb = x + 1;
+	for (xe = xb; *xe; xe++) {
+	    if (*xe == '/')
+		break;
+	}
+	if (*xe != '/') {
+	    (void) fprintf(stderr, "%s: regexp doesn't end with '/': ", Pn);
+	    safestrprt(x, stderr, 1);
+	    return(1);
+	}
+/*
+ * Decode any regular expression modifiers.
+ */
+	for (i = 0, xm = xe + 1; *xm; xm++) {
+	    switch(*xm) {
+	    case 'b':			/* This is a basic expression. */
+		if (++bmod > 1) {
+		    if (bmod == 2) {
+			(void) fprintf(stderr,
+			    "%s: b regexp modifier already used: ", Pn);
+			safestrprt(x, stderr, 1);
+		    }
+		    i = 1;
+		} else if (xmod) {
+		    if (++bxmod == 1) {
+			(void) fprintf(stderr,
+			    "%s: b and x regexp modifiers conflict: ", Pn);
+			safestrprt(x, stderr, 1);
+		    }
+		    i = 1;
+		} else
+		    co &= ~REG_EXTENDED;
+		break;
+	    case 'i':			/* Ignore case. */
+		if (++imod > 1) {
+		    if (imod == 2) {
+			(void) fprintf(stderr,
+			    "%s: i regexp modifier already used: ", Pn);
+			safestrprt(x, stderr, 1);
+		    }
+		    i = 1;
+		} else
+		    co |= REG_ICASE;
+		break;
+	    case 'x':			/* This is an extended expression. */
+		if (++xmod > 1) {
+		    if (xmod == 2) {
+			(void) fprintf(stderr,
+			    "%s: x regexp modifier already used: ", Pn);
+			safestrprt(x, stderr, 1);
+		    }
+		    i = 1;
+		} else if (bmod) {
+		    if (++bxmod == 1) {
+			(void) fprintf(stderr,
+			    "%s: b and x regexp modifiers conflict: ", Pn);
+			safestrprt(x, stderr, 1);
+		    }
+		    i = 1;
+		} else
+		    co |= REG_EXTENDED;
+		break;
+	    default:
+		(void) fprintf(stderr, "%s: invalid regexp modifier: %c\n",
+		Pn, (int)*xm);
+		i = 1;
+	    }
+	}
+	if (i)
+	    return(1);
+/*
+ * Allocate space to hold expression and copy it there.
+ */
+	xl = (MALLOC_S)(xe - xb);
+	if (!(xp = (char *)malloc(xl + 1))) {
+	    (void) fprintf(stderr, "%s: no regexp space for: ", Pn);
+	    safestrprt(x, stderr, 1);
+	    Exit(1);
+	}
+	(void) strncpy(xp, xb, xl);
+	xp[(int)xl] = '\0';
+/*
+ * Assign a new CmdRx[] slot for this expression.
+ */
+	if (NCmdRxA >= NCmdRxU) {
+
+	/*
+	 * More CmdRx[] space must be assigned.
+	 */
+	    NCmdRxA += CMDRXINCR;
+	    xl = (MALLOC_S)(NCmdRxA * sizeof(lsof_rx_t));
+	    if (CmdRx)
+		CmdRx = (lsof_rx_t *)realloc((MALLOC_P *)CmdRx, xl);
+	    else
+		CmdRx = (lsof_rx_t *)malloc(xl);
+	    if (!CmdRx) {
+		(void) fprintf(stderr, "%s: no space for regexp: ", Pn);
+		safestrprt(x, stderr, 1);
+		Exit(1);
+	    }
+	}
+	i = NCmdRxU;
+	CmdRx[i].exp = xp;
+/*
+ * Compile the expression.
+ */
+	if ((re = regcomp(&CmdRx[i].cx, xp, co))) {
+	    (void) fprintf(stderr, "%s: regexp error: ", Pn);
+	    safestrprt(x, stderr, 0);
+	    (void) regerror(re, &CmdRx[i].cx, &reb[0], sizeof(reb));
+	    (void) fprintf(stderr, ": %s\n", reb);
+	    if (xp) {
+		(void) free((FREE_P *)xp);
+		xp = (char *)NULL;
+	    }
+	    return(1);
+	}
+/*
+ * Complete the CmdRx[] table entry.
+ */
+	CmdRx[i].mc = 0;
+	CmdRx[i].exp = xp;
+	NCmdRxU++;
+	return(0);
+}
+
+
 /*
  * enter_fd() - enter file descriptor list for searching
  */
@@ -697,6 +863,11 @@ enter_dir(d, descend)
 	    }
 	    return(1);
 	}
+
+#if	defined(HASSPECDEVD)
+	(void) HASSPECDEVD(dn, &sb);
+#endif	/* defined(HASSPECDEVD) */
+
 	ddev = sb.st_dev;
 /*
  * Stack the directory and record it in Sfile for searching.
@@ -754,7 +925,7 @@ enter_dir(d, descend)
 		    Exit(1);
 		}
 	    }
-	    (void) sprintf(fp, "%s%s", dn, sl ? "/" : "");
+	    (void) snpf(fp, (size_t)fpl, "%s%s", dn, sl ? "/" : "");
 	    (void) free((FREE_P *)dn);
 	    dn = (char *)NULL;
 	/*
@@ -819,6 +990,11 @@ enter_dir(d, descend)
 		    }
 		    continue;
 		}
+
+#if	defined(HASSPECDEVD)
+		(void) HASSPECDEVD(fp, &sb);
+#endif	/* defined(HASSPECDEVD) */
+
 		if ((sb.st_mode & S_IFMT) == S_IFLNK || sb.st_dev != ddev)
 		    continue;
 		if (av[0]) {
@@ -883,12 +1059,12 @@ enter_dir(d, descend)
 
 
 /*
- * enter_id() - enter PGRP or PID for searching
+ * enter_id() - enter PGID or PID for searching
  */
 
 int
 enter_id(ty, p)
-	enum IDType ty;			/* type: PGRP or PID */
+	enum IDType ty;			/* type: PGID or PID */
 	char *p;			/* process group ID string pointer */
 {
 	char *cp;
@@ -897,17 +1073,17 @@ enter_id(ty, p)
 
 	if (!p) {
 	    (void) fprintf(stderr, "%s: no process%s ID specified\n",
-		Pn, (ty == PGRP) ? " group" : "");
+		Pn, (ty == PGID) ? " group" : "");
 	    return(1);
 	}
 /*
  * Set up variables for the type of ID.
  */
 	switch (ty) {
-	case PGRP:
-	    mx = Mxpgrp;
-	    n = Npgrp;
-	    s = Spgrp;
+	case PGID:
+	    mx = Mxpgid;
+	    n = Npgid;
+	    s = Spgid;
 	    break;
 	case PID:
 	    mx = Mxpid;
@@ -938,7 +1114,7 @@ enter_id(ty, p)
 
 		{
 		    (void) fprintf(stderr, "%s: illegal process%s ID: ",
-			Pn, (ty == PGRP) ? " group" : "");
+			Pn, (ty == PGID) ? " group" : "");
 		    safestrprt(p, stderr, 1);
 		    return(1);
 		}
@@ -968,7 +1144,7 @@ enter_id(ty, p)
 			(MALLOC_S)(sizeof(struct int_lst) * mx));
 		if (!s) {
 		    (void) fprintf(stderr, "%s: no space for %d process%s IDs",
-			Pn, mx, (ty == PGRP) ? " group" : "");
+			Pn, mx, (ty == PGID) ? " group" : "");
 		    Exit(1);
 		}
 	    }
@@ -978,10 +1154,10 @@ enter_id(ty, p)
 /*
  * Save variables for the type of ID.
  */
-	if (ty == PGRP) {
-	    Mxpgrp = mx;
-	    Npgrp = n;
-	    Spgrp = s;
+	if (ty == PGID) {
+	    Mxpgid = mx;
+	    Npgid = n;
+	    Spgid = s;
 	} else {
 	    Mxpid = mx;
 	    Npid = Npuns = n;
@@ -1515,8 +1691,8 @@ enter_str_lst(opt, s, lp)
  */
 
 int
-enter_uid(u)
-	char *u;			/* User IDentifier string pointer */
+enter_uid(us)
+	char *us;			/* User IDentifier string pointer */
 {
 	int err, i, j, lnml, nn;
 	unsigned char excl;
@@ -1526,11 +1702,11 @@ enter_uid(u)
 	char *s, *st;
 	uid_t uid;
 
-	if (!u) {
+	if (!us) {
 	    (void) fprintf(stderr, "%s: no UIDs specified\n", Pn);
 	    return(1);
 	}
-	for (err = 0, s = u; *s;) {
+	for (err = 0, s = us; *s;) {
 
 	/*
 	 * Assemble next User IDentifier.
