@@ -32,7 +32,7 @@
 #ifndef lint
 static char copyright[] =
 "@(#) Copyright 1994 Purdue Research Foundation.\nAll rights reserved.\n";
-static char *rcsid = "$Id: dfile.c,v 1.4 99/03/29 07:09:57 abe Exp $";
+static char *rcsid = "$Id: dfile.c,v 1.8 2001/07/18 14:01:21 abe Exp $";
 #endif
 
 
@@ -56,6 +56,9 @@ struct hsfile {
 static struct hsfile *HbyFdi =		/* hash by file buckets */
 	(struct hsfile *)NULL;
 static int HbyFdiCt = 0;		/* HbyFdi entry count */
+static struct hsfile *HbyFrd =		/* hash by file raw device buckets */
+	(struct hsfile *)NULL;
+static int HbyFrdCt = 0;		/* HbyFrd entry count */
 static struct hsfile *HbyFsd =		/* hash by file system buckets */
 	(struct hsfile *)NULL;
 static int HbyFsdCt = 0;		/* HbyFsd entry count */
@@ -82,6 +85,13 @@ static int HbyNmCt = 0;			/* HbyNm entry count */
 #define	SFMPCHASH	1024		/* Sfile hash by MPC device number */
 #define	SFNMHASH	4096		/* Sfile hash by name bucket count
 					   (power of 2!) */
+#define	SFRDHASH	1024		/* Sfile hash by raw device number
+					 * bucket count (power of 2!) */
+#define SFHASHRDEVI(maj, min, rmaj, rmin, ino, mod) ((int)(((int)((((int)(maj+1))*((int)((min+1))))+((int)(rmaj+1)*(int)(rmin+1))+ino)*31415)&(mod-1)))
+					/* hash for Sfile by major device,
+					 * minor device, major raw device,
+					 * minor raw device, and inode, modulo
+					 * mod (mod must be a power of 2) */
 
 
 /*
@@ -113,6 +123,14 @@ hashSfile()
 		Pn, SFDIHASH);
 	    Exit(1);
 	}
+	if (!(HbyFrd = (struct hsfile *)calloc((MALLOC_S)SFRDHASH,
+					       sizeof(struct hsfile))))
+	{
+	    (void) fprintf(stderr,
+		"%s: can't allocate space for %d rdev hash buckets\n",
+		Pn, SFRDHASH);
+	    Exit(1);
+	}
 	if (!(HbyFsd = (struct hsfile *)calloc((MALLOC_S)SFFSHASH,
 					       sizeof(struct hsfile))))
 	{
@@ -143,7 +161,7 @@ hashSfile()
  * name hash bucket chains.
  */
 	for (s = Sfile; s; s = s->next) {
-	    for (i = 0; i < 3; i++) {
+	    for (i = 0; i < 4; i++) {
 		if (i == 0) {
 		    if (!s->aname)
 			continue;
@@ -163,14 +181,30 @@ hashSfile()
 						 SFFSHASH)];
 			HbyFsdCt++;
 		    }
-		} else {
-		    if (!s->type || s->mode != S_IFCHR)
+		} else if (i == 2) {
+		    if (s->type && (s->ch < 0) && (s->mode == S_IFCHR))
+		    {
+			sh = &HbyMPC[SFHASHDEVINO(major(s->dev),
+						  minor(s->dev),
+						  0,
+						  SFMPCHASH)];
+			HbyMPCCt++;
+		    } else
+			continue;
+		} else if (i == 3) {
+		    if (s->type
+		    &&  (((s->mode == S_IFCHR) && (s->ch < 0))
+		    ||   ((s->mode == S_IFBLK))))
+		    {
+			sh = &HbyFrd[SFHASHRDEVI(major(s->dev),
+						 minor(s->dev),
+						 major(s->rdev),
+						 minor(s->rdev),
+						 s->i,
+						 SFRDHASH)];
+			HbyFrdCt++;
+		    } else
 			continue;
-		    sh = &HbyMPC[SFHASHDEVINO(major(s->dev),
-					      minor(s->dev),
-					      0,
-					      SFMPCHASH)];
-		    HbyMPCCt++;
 		}
 		if (!sh->s) {
 		    sh->s = s;
@@ -206,9 +240,11 @@ is_file_named(p, ty, ch, ic)
 	chan_t ch;			/* gnode channel */
 	int ic;				/* is clone file (4.1.4 and above) */
 {
+	char *ep;
 	int f = 0;
 	struct sfile *s;
 	struct hsfile *sh;
+	size_t sz;
 
 #if	AIXV>=4140
 /*
@@ -218,17 +254,18 @@ is_file_named(p, ty, ch, ic)
 	if (ic
 	&&  HbyFdiCt
 	&&  CloneMaj >= 0
-	&&  Lf->dev_def
+	&&  (Lf->dev_def && (Lf->dev == DevDev))
+	&&  Lf->rdev_def
 	&&  (Lf->inp_ty == 1 || Lf->inp_ty == 3))
 	{
-	    for (sh = &HbyFdi[SFHASHDEVINO(CloneMaj, major(Lf->dev), Lf->inode,
+	    for (sh = &HbyFdi[SFHASHDEVINO(CloneMaj, major(Lf->rdev), Lf->inode,
 					   SFDIHASH)];
 		 sh;
 		 sh = sh->next)
 	    {
 		if ((s = sh->s)
-		&&  (major(s->dev) == CloneMaj)
-		&&  (minor(s->dev) == major(Lf->dev))
+		&&  (major(s->rdev) == CloneMaj)
+		&&  (minor(s->rdev) == major(Lf->rdev))
 		&&  (s->i == (ino_t)Lf->inode))
 		{
 		    f = 3;
@@ -253,16 +290,19 @@ is_file_named(p, ty, ch, ic)
  * Check for a regular AIX multiplexed file, matching the channel if
  * it was supplied by the caller.
  */
-	if (!f && HbyMPCCt && ty == VMPC && Lf->dev_def) {
-	    for (sh = &HbyMPC[SFHASHDEVINO(major(Lf->dev),
-					   minor(Lf->dev),
+	if (!f && HbyMPCCt && ty == VMPC
+	&&  (Lf->dev_def && (Lf->dev == DevDev))
+	&&  Lf->rdev_def)
+	{
+	    for (sh = &HbyMPC[SFHASHDEVINO(major(Lf->rdev),
+					   minor(Lf->rdev),
 					   0,
 					   SFMPCHASH)];
 		 sh;
 		 sh = sh->next)
 	    {
 		if ((s = sh->s)
-		&&  s->dev == Lf->dev
+		&&  s->dev == Lf->rdev
 		&&  (s->ch < 0 || ch == s->ch)) {
 		    f = 1;
 		    break;
@@ -299,10 +339,33 @@ is_file_named(p, ty, ch, ic)
 		 sh = sh->next)
 	    {
 		if ((s = sh->s) && Lf->dev == s->dev) {
-		    if (!(ty == VCHR && s->mode != S_IFCHR)) {
-			f = 1;
-			break;
-		    }
+		    f = 1;
+		    break;
+		}
+	    }
+	}
+/*
+ * Check for a character or block device file.
+ */
+	if (!f && HbyFrdCt
+	&&  ((ty == VCHR) || (ty == VBLK))
+	&&  (Lf->dev_def && (Lf->dev == DevDev))
+	&&  Lf->rdev_def
+	&& (Lf->inp_ty == 1 || Lf->inp_ty == 3))
+	{
+	    for (sh = &HbyFrd[SFHASHRDEVI(major(Lf->dev), minor(Lf->dev),
+					  major(Lf->rdev), minor(Lf->rdev),
+					  Lf->inode, SFRDHASH)];
+		 sh;
+		 sh = sh->next)
+	    {
+		if ((s = sh->s)
+		&&  (s->rdev == Lf->rdev)
+		&&  (((ty == VCHR) && (s->mode == S_IFCHR) && (s->ch < 0))
+		||   ((ty == VBLK) && (s->mode == S_IFBLK))))
+		{
+		    f = 1;
+		    break;
 		}
 	    }
 	}
@@ -311,14 +374,13 @@ is_file_named(p, ty, ch, ic)
  */
 	if (f) {
 	    if (f == 2) {
-		(void) strcpy(Namech, p);
+		(void) snpf(Namech, Namechl, "%s", p);
 
 #if	AIXV>=4140
 	    } else if (f == 3 && ClonePtc >= 0
 		   &&  major(Lf->dev) == ClonePtc)
 	    {
-		(void) strcpy(Namech, s->name);
-		(void) sprintf(endnm(), "/%d", minor(Lf->dev));
+		(void) snpf(Namech, Namechl, "%s/%d", s->name, minor(Lf->dev));
 
 #endif	/* AIXV>=4140 */
 
@@ -328,11 +390,15 @@ is_file_named(p, ty, ch, ic)
 	     * If the search argument isn't a file system, propagate it
 	     * to Namech[]; otherwise, let printname() compose the name.
 	     */
-		(void) strcpy(Namech, s->name);
-		if (ty == VMPC && s->ch < 0)
-		    (void) sprintf(endnm(), "/%d", ch);
-		if (s->devnm)
-		    (void) sprintf(endnm(), " (%s)", s->devnm);
+		(void) snpf(Namech, Namechl, "%s", s->name);
+		if (ty == VMPC && s->ch < 0) {
+		    ep = endnm(&sz);
+		    (void) snpf(ep, sz, "/%d", ch);
+		}
+		if (s->devnm) {
+		    ep = endnm(&sz);
+		    (void) snpf(ep, sz, " (%s)", s->devnm);
+		}
 	    }
 	    s->f = 1;
 	    return(1);
@@ -346,18 +412,173 @@ is_file_named(p, ty, ch, ic)
  */
 
 char *
-print_dev(lf)
+print_dev(lf, dev)
 	struct lfile *lf;		/* file whose device to be printed */
+	dev_t *dev;			/* pointer to device to be printed */
+
 {
 	static char buf[128];
 
+#if	AIXA<1
+	int maj = major(*dev);
+	int min = minor(*dev);
+#else	/* AIXA>=1 */
+	int maj = major64(*dev);
+	int min = minor64(*dev);
+#endif	/* AIXA<1 */
+
 #if	AIXV>=3200
-	if (lf->dev & SDEV_REMOTE) {
-	    (void) sprintf(buf, "NFS,%d", minor(lf->dev));
+# if	AIXA<1
+	if (*dev & SDEV_REMOTE) {
+	    (void) snpf(buf, sizeof(buf), "NFS,%d", min);
 	    return(buf);
 	}
+# else	/* AIXA>=1 */
+	if (min & SDEV_REMOTE) {
+	    (void) snpf(buf, sizeof(buf), "NFS,%d", (min & ~SDEV_REMOTE));
+	    return(buf);
+	}
+# endif	/* AIXA<1 */
 #endif	/* AIXV>=3200 */
 
-	(void) sprintf(buf, "%d,%d", major(lf->dev), minor(lf->dev));
+	(void) snpf(buf, sizeof(buf), "%d,%d", maj, min);
 	return(buf);
 }
+
+
+/*
+ * readvfs() - read vfs structure
+ */
+
+struct l_vfs *
+readvfs(vn)
+	struct vnode *vn;		/* vnode */
+{
+	struct gfs g;
+	void *mp;
+	char *s1, *s2;
+	uint ul;
+	struct vfs v;
+	struct vmount *vm;
+	struct l_vfs *vp;
+
+
+	if (!vn->v_vfsp)
+	    return((struct l_vfs *)NULL);
+	for (vp = Lvfs; vp; vp = vp->next) {
+	    if ((KA_T)vn->v_vfsp == vp->addr)
+		return(vp);
+	}
+	if (!(vp = (struct l_vfs *)malloc((MALLOC_S)sizeof(struct l_vfs)))) {
+	    (void) fprintf(stderr, "%s: PID %d, no space for vfs\n",
+		Pn, Lp->pid);
+	    Exit(1);
+	}
+	vp->dir = (char *)NULL;
+	vp->fsname = (char *)NULL;
+/*
+ * Read the vfs structure.
+ */
+	if (kread((KA_T)vn->v_vfsp, (char *)&v, sizeof(v))) {
+
+vfs_exit:
+	    (void) free((FREE_P *)vp);
+	    return((struct l_vfs *)NULL);
+	}
+/*
+ * Locate AIX mount information.
+ */
+	if (!v.vfs_gfs || kread((KA_T)v.vfs_gfs, (char *)&g, sizeof(g)))
+	    goto vfs_exit;
+	if (!v.vfs_mdata
+	||  kread((KA_T)((char *)v.vfs_mdata
+		  + offsetof(struct vmount, vmt_length)),
+		  (char *)&ul, sizeof(ul)))
+	    goto vfs_exit;
+	if (!(mp = (void *)malloc((MALLOC_S)ul))) {
+	    (void) fprintf(stderr, "%s: PID %d, no space for mount data\n",
+		Pn, Lp->pid);
+	    Exit(1);
+	}
+	if (kread((KA_T)v.vfs_mdata, (char *)mp, (int)ul)) {
+	    (void) free((FREE_P *)mp);
+	    goto vfs_exit;
+	}
+	vm = (struct vmount *)mp;
+	vp->vmt_flags = vm->vmt_flags;
+	vp->vmt_gfstype = vm->vmt_gfstype;
+
+#if	AIXV>=3200
+	if (vp->vmt_flags & MNT_REMOTE) {
+	    vp->dev = 0x80000000 | vm->vmt_vfsnumber;
+# if	AIXA>=1
+	    vp->dev |= 0x8000000000000000;
+# endif	/* AIXA>=1 */
+	} else
+#endif	/* AIXV>=3200 */
+
+#if	defined(HAS_AFS)
+	    if (vm->vmt_gfstype == MNT_AFS)
+		vp->dev = AFSDEV;
+	    else
+#endif	/* defined(HAS_AFS) */
+
+#if	AIXA>=1
+	if (vm->vmt_gfstype == MNT_PROCFS) {
+
+	/*
+	 * !!!DEBUG!!!   !!!DEBUG!!!   !!!DEBUG!!!   !!!DEBUG!!!   !!!DEBUG!!!
+	 *
+	 * The following *hack* is required to make the vmount structure's
+	 * device number match what stat(2) errnoneously returns in ia64
+	 * AIX 5L.
+	 *
+	 * REMOVE THIS CODE WHEN STAT(2) IS FIXED!!!
+	 */
+		vp->dev = (dev_t)(vm->vmt_fsid.fsid_dev & 0x7fffffffffffffff);
+	/*
+	 * !!!DEBUG!!!   !!!DEBUG!!!   !!!DEBUG!!!   !!!DEBUG!!!   !!!DEBUG!!!
+	 */
+
+	}
+	else
+#endif	/* AIXA>=1 */
+
+		vp->dev = (dev_t)vm->vmt_fsid.fsid_dev;
+	if ((s1 = vmt2dataptr(vm, VMT_STUB))) {
+	    if (!(vp->dir = mkstrcpy(s1, (MALLOC_S *)NULL))) {
+
+readvfs_aix1:
+		(void) fprintf(stderr, "%s: PID %d, readvfs, no space\n",
+		    Pn, Lp->pid);
+		Exit(1);
+	    }
+	} else
+	    vp->dir = (char *)NULL;
+	s1 = vmt2dataptr(vm, VMT_HOST);
+	if (!(s2 = vmt2dataptr(vm, VMT_OBJECT)) || *s1 == '\0')
+	    s2 = g.gfs_name;
+	if (!s1 && !s2)
+	    vp->fsname = (char *)NULL;
+	else {
+	    if (vm->vmt_flags & MNT_REMOTE) {
+		if (!(vp->fsname = mkstrcat(s1 ? s1 : "", -1, ":", 1, s2, -1,
+				   (MALLOC_S *)NULL)))
+		    goto readvfs_aix1;
+	    } else {
+		if (!(vp->fsname = mkstrcpy(s2, (MALLOC_S *)NULL)))
+		    goto readvfs_aix1;
+	    }
+	}
+	(void) free((FREE_P *)mp);
+	vp->next = Lvfs;
+	vp->addr = (KA_T)vn->v_vfsp;
+
+#if	defined(HAS_AFS)
+	if (!AFSVfsp && vm->vmt_gfstype == MNT_AFS)
+	    AFSVfsp = (KA_T)vn->v_vfsp;
+#endif	/* defined(HAS_AFS) */
+
+	Lvfs = vp;
+	return(vp);
+}
