@@ -32,7 +32,7 @@
 #ifndef lint
 static char copyright[] =
 "@(#) Copyright 1994 Purdue Research Foundation.\nAll rights reserved.\n";
-static char *rcsid = "$Id: dmnt.c,v 1.6 98/07/17 08:48:47 abe Exp $";
+static char *rcsid = "$Id: dmnt.c,v 1.9 2001/07/18 14:01:37 abe Exp $";
 #endif
 
 
@@ -59,7 +59,7 @@ readmnt()
 	struct mounts *mtp;
 	int nm;
 	struct stat sb;
-	unsigned sz;
+	MALLOC_S sz;
 	struct vmount *v;
 	struct vmount *vt = (struct vmount *)NULL;
 
@@ -113,9 +113,29 @@ readmnt()
 			break;
 #endif	/* defined(HAS_AFS) */
 
+#if	defined(HASPROCFS)
+		    case MNT_PROCFS:
+			ty = HASPROCFS;
+			break;
+#endif	/* defined(HASPROCFS) */
+
+#if	defined(MNT_AIX) && defined(MNT_J2) && MNT_AIX==MNT_J2
+		    case MNT_AIX:
+			ty = "jfs2";
+			break;
+#else	/* !defined(MNT_AIX) || !defined(MNT_J2) || MNT_AIX!=MNT_J2 */
+# if	defined(MNT_AIX)
 		    case MNT_AIX:
 			ty = "oaix";
 			break;
+# endif	/* defined(MNT_AIX) */
+# if	defined(MNT_J2)
+		    case MNT_J2:
+			ty = "jfs2";
+			break;
+# endif	/* defined(MNT_J2) */
+#endif	/* defined(MNT_AIX) && defined(MNT_H2) && MNT_AIX==MNT_J2 */
+
 		    case MNT_NFS:
 			ty = "nfs";
 			break;
@@ -138,9 +158,16 @@ readmnt()
 	     * Assemble alternate device number and mode flags.
 	     */
 		(void) bzero((char *)&sb, sizeof(sb));
-		if (v->vmt_flags & MNT_REMOTE)
+		if (v->vmt_flags & MNT_REMOTE) {
+
+#if	AIXA<1
 		    sb.st_dev = (dev_t)(SDEV_REMOTE | v->vmt_vfsnumber);
-		else {
+#else	/* AIXA>=1 */
+		    sb.st_dev = (dev_t)(SDEV_REMOTE | (SDEV_REMOTE << 32)
+			      |         v->vmt_vfsnumber);
+#endif	/*AIXA<1 */
+
+		} else {
 
 #if	defined(HAS_AFS)
 		    if (v->vmt_gfstype == MNT_AFS)
@@ -148,6 +175,28 @@ readmnt()
 		    else
 #endif	/* defined(HAS_AFS) */
 
+#if	AIXA>=1 && defined(HASPROCFS)
+		    if (v->vmt_gfstype == MNT_PROCFS) {
+
+		    /*
+		     * !!!DEBUG!!!   !!!DEBUG!!!   !!!DEBUG!!!   !!!DEBUG!!!
+		     *
+		     * The following *hack* is required to make the vmount
+		     * structure's device number match what stat(2)
+		     * errnoneously returns on ia64 AIX 5L.
+ 		     *
+		     * REMOVE THIS CODE WHEN STAT(2) IS FIXED!!!
+		     */
+			sb.st_dev = (dev_t)(v->vmt_fsid.val[0]
+				  &         0x7fffffffffffffff);
+		    /*
+		     * !!!DEBUG!!!   !!!DEBUG!!!   !!!DEBUG!!!   !!!DEBUG!!!
+ 		     */
+
+		    }
+		    else
+#endif	/* AIXA>=1 && defined(HASPROCFS) */
+
 			sb.st_dev = (dev_t)v->vmt_fsid.val[0];
 		}
 		if (!Fwarn)
@@ -184,7 +233,8 @@ no_space_for_mount:
 	/*
 	 * Allocate a local mounts structure and fill the directory information.
 	 */
-	    if (!(mtp = (struct mounts *)malloc(sizeof(struct mounts))))
+	    if (!(mtp = (struct mounts *)malloc(
+			(MALLOC_S)sizeof(struct mounts))))
 		goto no_space_for_mount;
 	    mtp->dir = dn;
 	    dn = (char *)NULL;
@@ -192,6 +242,11 @@ no_space_for_mount:
 	    mtp->inode = sb.st_ino;
 	    mtp->mode = sb.st_mode;
 	    mtp->rdev = sb.st_rdev;
+
+#if	defined(HASFSTYPE)
+	    mtp->fstype = sb.st_vfstype;
+#endif	/* defined(HASFSTYPE) */
+
 	    mtp->next = Lmi;
 	/*
 	 * Form the file system (mounted-on) device name.  Resolve any
@@ -229,116 +284,3 @@ no_space_for_mount:
 	Lmist = 1;
 	return(Lmi);
 }
-
-
-/*
- * readvfs() - read vfs structure
- */
-
-struct l_vfs *
-readvfs(vn)
-	struct vnode *vn;		/* vnode */
-{
-	struct gfs g;
-	void *mp;
-	char *s1, *s2;
-	u_long ul;
-	struct vfs v;
-	struct vmount *vm;
-	struct l_vfs *vp;
-
-	if (!vn->v_vfsp)
-	    return((struct l_vfs *)NULL);
-	for (vp = Lvfs; vp; vp = vp->next) {
-	    if ((KA_T)vn->v_vfsp == vp->addr)
-		return(vp);
-	}
-	if (!(vp = (struct l_vfs *)malloc(sizeof(struct l_vfs)))) {
-	    (void) fprintf(stderr, "%s: PID %d, no space for vfs\n",
-		Pn, Lp->pid);
-	    Exit(1);
-	}
-	vp->dir = (char *)NULL;
-	vp->fsname = (char *)NULL;
-/*
- * Read the vfs structure.
- */
-	if (kread((KA_T)vn->v_vfsp, (char *)&v, sizeof(v))) {
-
-vfs_exit:
-	    (void) free((FREE_P *)vp);
-	    return((struct l_vfs *)NULL);
-	}
-/*
- * Locate AIX mount information.
- */
-	if (!v.vfs_gfs || kread((KA_T)v.vfs_gfs, (char *)&g, sizeof(g)))
-	    goto vfs_exit;
-	if (!v.vfs_mdata
-	||  kread((KA_T)v.vfs_mdata + (KA_T)sizeof(u_long), (char *)&ul,
-		  sizeof(ul))
-	)
-	    goto vfs_exit;
-	if (!(mp = (void *)malloc((size_t)ul))) {
-	    (void) fprintf(stderr, "%s: PID %d, no space for mount data\n",
-		Pn, Lp->pid);
-	    Exit(1);
-	}
-	if (kread((KA_T)v.vfs_mdata, (char *)mp, (int)ul)) {
-	    (void) free((FREE_P *)mp);
-	    goto vfs_exit;
-	}
-	vm = (struct vmount *)mp;
-	vp->vmt_flags = vm->vmt_flags;
-	vp->vmt_gfstype = vm->vmt_gfstype;
-
-#if	AIXV>=3200
-	if (vp->vmt_flags & MNT_REMOTE)
-	    vp->dev = 0x80000000 | vm->vmt_vfsnumber;
-	else
-#endif	/* AIXV>=3200 */
-
-#if	defined(HAS_AFS)
-	    if (vm->vmt_gfstype == MNT_AFS)
-		vp->dev = AFSDEV;
-	    else
-#endif	/* defined(HAS_AFS) */
-
-		vp->dev = (dev_t)vm->vmt_fsid.fsid_dev;
-	if ((s1 = vmt2dataptr(vm, VMT_STUB))) {
-	    if (!(vp->dir = mkstrcpy(s1, (MALLOC_S *)NULL))) {
-
-readvfs_aix1:
-		(void) fprintf(stderr, "%s: PID %d, readvfs, no space\n",
-		    Pn, Lp->pid);
-		Exit(1);
-	    }
-	} else
-	    vp->dir = (char *)NULL;
-	s1 = vmt2dataptr(vm, VMT_HOST);
-	if (!(s2 = vmt2dataptr(vm, VMT_OBJECT)) || *s1 == '\0')
-	    s2 = g.gfs_name;
-	if (!s1 && !s2)
-	    vp->fsname = (char *)NULL;
-	else {
-	    if (vm->vmt_flags & MNT_REMOTE) {
-		if (!(vp->fsname = mkstrcat(s1 ? s1 : "", -1, ":", 1, s2, -1,
-				   (MALLOC_S *)NULL)))
-		    goto readvfs_aix1;
-	    } else {
-		if (!(vp->fsname = mkstrcpy(s2, (MALLOC_S *)NULL)))
-		    goto readvfs_aix1;
-	    }
-	}
-	(void) free((FREE_P *)mp);
-	vp->next = Lvfs;
-	vp->addr = (KA_T)vn->v_vfsp;
-
-#if	defined(HAS_AFS)
-	if (!AFSVfsp && vm->vmt_gfstype == MNT_AFS)
-	    AFSVfsp = (KA_T)vn->v_vfsp;
-#endif	/* defined(HAS_AFS) */
-
-	Lvfs = vp;
-	return(vp);
-}
