@@ -32,7 +32,7 @@
 #ifndef lint
 static char copyright[] =
 "@(#) Copyright 1994 Purdue Research Foundation.\nAll rights reserved.\n";
-static char *rcsid = "$Id: dsock.c,v 1.11 99/11/28 06:47:32 abe Exp $";
+static char *rcsid = "$Id: dsock.c,v 1.16 2001/02/08 18:14:04 abe Exp $";
 #endif
 
 
@@ -41,11 +41,12 @@ static char *rcsid = "$Id: dsock.c,v 1.11 99/11/28 06:47:32 abe Exp $";
 
 /*
  * We include <sys/domain.h> here instead of "dlsof.h" for gcc's benefit.
- * It's loader can't handle the multiple CONST u_char arrays declared in
+ * Its loader can't handle the multiple CONST u_char arrays declared in
  * <net/net_globals.h> -- e.g., etherbroadcastaddr[].  (<sys/domain.h>
  * #include's <net/net_globals.h>.)
  */
 
+#include <net/netopt.h>
 #include <sys/domain.h>
 
 
@@ -66,7 +67,6 @@ process_socket(sa)
 	struct inpcb inp;
 	int is = 0;
 	unsigned char *la = (unsigned char *)NULL;
-	struct mbuf mb;
 	struct protosw p;
 	struct socket s;
 	struct tcpcb t;
@@ -74,10 +74,11 @@ process_socket(sa)
 	struct sockaddr_un *ua = (struct sockaddr_un *)NULL;
 	struct sockaddr_un un;
 	struct vnode v;
+	struct mbuf mb;
 /*
  * Set socket file variables.
  */
-	(void) strcpy(Lf->type, "sock");
+	(void) snpf(Lf->type, sizeof(Lf->type), "sock");
 	Lf->inp_ty = 2;
 /*
  * Read socket and protocol switch structures.
@@ -87,8 +88,8 @@ process_socket(sa)
 	    return;
 	}
 	if (kread(sa, (char *) &s, sizeof(s))) {
-	    (void) sprintf(Namech, "can't read socket struct from %s",
-		print_kptr(sa, (char *)NULL));
+	    (void) snpf(Namech, Namechl, "can't read socket struct from %s",
+		print_kptr(sa, (char *)NULL, 0));
 	    enter_nm(Namech);
 	    return;
 	}
@@ -98,8 +99,8 @@ process_socket(sa)
 	}
 	if (!s.so_proto
 	||  kread((KA_T)s.so_proto, (char *)&p, sizeof(p))) {
-	    (void) sprintf(Namech, "can't read protocol switch from %s",
-		print_kptr((KA_T)s.so_proto, (char *)NULL));
+	    (void) snpf(Namech, Namechl, "can't read protocol switch from %s",
+		print_kptr((KA_T)s.so_proto, (char *)NULL, 0));
 	    enter_nm(Namech);
 	    return;
 	}
@@ -128,8 +129,8 @@ process_socket(sa)
  */
 	if (!p.pr_domain
 	||  kread((KA_T)p.pr_domain, (char *)&d, sizeof(d))) {
-	    (void) sprintf(Namech, "can't read domain struct from %s",
-		print_kptr((KA_T)p.pr_domain, (char *)NULL));
+	    (void) snpf(Namech, Namechl, "can't read domain struct from %s",
+		print_kptr((KA_T)p.pr_domain, (char *)NULL, 0));
 	    enter_nm(Namech);
 	    return;
 	}
@@ -148,9 +149,10 @@ process_socket(sa)
 	    printiproto(p.pr_protocol);
 
 #if	defined(HASIPv6)
-	    (void) strcpy(Lf->type, fam == AF_INET ? "IPv4" : "IPv6");
+	    (void) snpf(Lf->type, sizeof(Lf->type),
+			fam == AF_INET ? "IPv4" : "IPv6");
 #else	/* !defined(HASIPv6) */
-	    (void) strcpy(Lf->type, "inet");
+	    (void) snpf(Lf->type, sizeof(Lf->type), "inet");
 #endif	/* defined(HASIPv6) */
 
 	/*
@@ -159,14 +161,12 @@ process_socket(sa)
 	    if (!s.so_pcb
 	    ||  kread((KA_T) s.so_pcb, (char *) &inp, sizeof(inp))) {
 		if (!s.so_pcb) {
-		    (void) strcpy(Namech, "no PCB");
-		    if (s.so_state & SS_CANTSENDMORE)
-			(void) strcpy(endnm(), ", CANTSENDMORE");
-		    if (s.so_state & SS_CANTRCVMORE)
-			(void) strcpy(endnm(), ", CANTRCVMORE");
+		    (void) snpf(Namech, Namechl, "no PCB%s%s",
+			(s.so_state & SS_CANTSENDMORE) ? ", CANTSENDMORE" : "",
+			(s.so_state & SS_CANTRCVMORE)  ? ", CANTRCVMORE"  : "");
 		} else {
-		    (void) sprintf(Namech, "can't read inpcb at %s",
-			print_kptr((KA_T)s.so_pcb, (char *)NULL));
+		    (void) snpf(Namech, Namechl, "can't read inpcb at %s",
+			print_kptr((KA_T)s.so_pcb, (char *)NULL, 0));
 		}
 		enter_nm(Namech);
 		return;
@@ -176,19 +176,17 @@ process_socket(sa)
 	 */
 	    enter_dev_ch(print_kptr((KA_T)(inp.inp_ppcb ? inp.inp_ppcb
 							: s.so_pcb),
-				    (char *)NULL));
+				    (char *)NULL, 0));
 
 #if	defined(HASIPv6)
 	/*
 	 * If this is an IPv6 (AF_INET6) socket and IPv4 compatibility
 	 * mode is enabled, use the IPv4 address, change the family
-	 * indicator from AF_INET6 to AF_INET, and change the file type
-	 * from "IPv6" to "IPv4".  Othwerise, use the IPv6 address.
-	 * Don't ignore empty addresses.
+	 * indicator from AF_INET6 to AF_INET.  Otherwise, use the IPv6
+	 * address.  Don't ignore empty addresses.
 	 */
 	    if (fam == AF_INET6) {
 		if (inp.inp_flags & INP_COMPATV4) {
-		    Lf->type[3] = '4';
 		    fam = AF_INET;
 		    la = (unsigned char *)&inp.inp_laddr;
 		} else
@@ -210,12 +208,10 @@ process_socket(sa)
 	    /*
 	     * If this is an IPv6 (AF_INET6) socket and IPv4 compatibility
 	     * mode is enabled, use the IPv4 address, change the family
-	     * indicator from AF_INET6 to AF_INET, and change the file type
-	     * from "IPv6" to "IPv4".  Othwerise, use the IPv6 address.
-	     * Ignore empty addresses.
+	     * indicator from AF_INET6 to AF_INET.  Otherwise, use the IPv6
+	     * address.  Ignore empty addresses.
 	     */
 		if (inp.inp_flags & INP_COMPATV4) {
-		    Lf->type[3] = '4';
 		    fam = AF_INET;
 		    if (inp.inp_faddr.s_addr != INADDR_ANY
 		    || inp.inp_fport != 0)
@@ -244,11 +240,11 @@ process_socket(sa)
  * Process a ROUTE domain socket.
  */
 	case AF_ROUTE:
-	    (void) strcpy(Lf->type, "rte");
+	    (void) snpf(Lf->type, sizeof(Lf->type), "rte");
 	    if (s.so_pcb)
-		enter_dev_ch(print_kptr((KA_T)(s.so_pcb), (char *)NULL));
+		enter_dev_ch(print_kptr((KA_T)(s.so_pcb), (char *)NULL, 0));
 	    else
-		(void) strcpy(Namech, "no protocol control block");
+		(void) snpf(Namech, Namechl, "no protocol control block");
 	    if (!Fsize)
 		Lf->off_def = 1;
 	    break;
@@ -258,31 +254,31 @@ process_socket(sa)
 	case AF_UNIX:
 	    if (Funix)
 		Lf->sf |= SELUNX;
-	    (void) strcpy(Lf->type, "unix");
+	    (void) snpf(Lf->type, sizeof(Lf->type), "unix");
 	/*
 	 * Read Unix protocol control block and the Unix address structure.
 	 */
-	    enter_dev_ch(print_kptr(sa, (char *)NULL));
-	    if (kread((KA_T) s.so_pcb, (char *) &unp, sizeof(unp))) {
-		(void) sprintf(Namech, "can't read unpcb at %s",
-		    print_kptr((KA_T)s.so_pcb, (char *)NULL));
+	    enter_dev_ch(print_kptr(sa, (char *)NULL, 0));
+	    if (kread((KA_T) s.so_pcb, (char *)&unp, sizeof(unp))) {
+		(void) snpf(Namech, Namechl, "can't read unpcb at %s",
+		    print_kptr((KA_T)s.so_pcb, (char *)NULL, 0));
 		break;
 	    }
 	    if ((struct socket *)sa != unp.unp_socket) {
-		(void) sprintf(Namech, "unp_socket (%s) mismatch",
-		    print_kptr((KA_T)unp.unp_socket, (char *)NULL));
+		(void) snpf(Namech, Namechl, "unp_socket (%s) mismatch",
+		    print_kptr((KA_T)unp.unp_socket, (char *)NULL, 0));
 		break;
 	    }
 	    if (unp.unp_addr) {
-		if (kread((KA_T) unp.unp_addr, (char *) &mb, sizeof(mb))) {
-		    (void) sprintf(Namech, "can't read unp_addr at %s",
-			print_kptr((KA_T)unp.unp_addr, (char *)NULL));
+		if (kread((KA_T) unp.unp_addr, (char *)&mb, sizeof(mb))) {
+		    (void) snpf(Namech, Namechl, "can't read unp_addr at %s",
+			print_kptr((KA_T)unp.unp_addr, (char *)NULL, 0));
 		    break;
 		}
 
 #if	AIXV>=3200
-		ua = (struct sockaddr_un *) ((char *) &mb
-		   + (mb.m_hdr.mh_data - (caddr_t) unp.unp_addr));
+		ua = (struct sockaddr_un *)((char *)&mb
+		   + (mb.m_hdr.mh_data - (caddr_t)unp.unp_addr));
 #else	/* AIXV<3200 */
 		ua = (struct sockaddr_un *)(((char *)&mb) + mb.m_off);
 #endif	/* AIXV>=3200 */
@@ -302,17 +298,16 @@ process_socket(sa)
 		if (ua->sun_family == AF_UNSPEC) {
 		    if (unp.unp_conn) {
 			if (kread((KA_T)unp.unp_conn, (char *)&uc, sizeof(uc)))
-			    (void) sprintf(Namech, "can't read unp_conn at %s",
-				print_kptr((KA_T)unp.unp_conn,
-				(char *)NULL));
+			    (void) snpf(Namech, Namechl,
+				"can't read unp_conn at %s",
+				print_kptr((KA_T)unp.unp_conn,(char *)NULL,0));
 			else
-			    (void) sprintf(Namech, "->%s",
-				print_kptr((KA_T)uc.unp_socket,
-				(char *)NULL));
+			    (void) snpf(Namech, Namechl, "->%s",
+				print_kptr((KA_T)uc.unp_socket,(char *)NULL,0));
 		    } else
-			(void) strcpy(Namech, "->(none)");
+			(void) snpf(Namech, Namechl, "->(none)");
 		} else
-		    (void) sprintf(Namech, "unknown sun_family (%d)",
+		    (void) snpf(Namech, Namechl, "unknown sun_family (%d)",
 			ua->sun_family);
 		break;
 	    }
@@ -350,9 +345,9 @@ process_socket(sa)
 		if (Sfile && is_file_named(ua->sun_path, VSOCK, 0, 0))
 		    Lf->sf |= SELNM;
 		if (!Namech[0])
-		    (void) strcpy(Namech, ua->sun_path);
+		    (void) snpf(Namech, Namechl, "%s", ua->sun_path);
 	    } else
-		(void) strcpy(Namech, "no address");
+		(void) snpf(Namech, Namechl, "no address");
 	    break;
 
 	default:
