@@ -31,7 +31,7 @@
 
 
 /*
- * $Id: machine.h,v 1.22 99/07/08 14:49:45 abe Exp $
+ * $Id: machine.h,v 1.31 2001/07/18 14:01:49 abe Exp $
  */
 
 
@@ -69,6 +69,10 @@
 
 #include <sys/types.h>
 
+#   if	AIXA>=1
+#include <sys/resource.h>
+#   endif	/* AIXA>=1 */
+
 # if	defined(__GNUC__)
 typedef	long long	aligned_off64_t __attribute__ ((aligned (8)));
 typedef	long long	aligned_offset_t  __attribute__ ((aligned (8)));
@@ -257,14 +261,21 @@ typedef	long long	aligned_offset_t  __attribute__ ((aligned (8)));
 
 /*
  * HASNCACHE is defined for those dialects that have a kernel name cache
- * that lsof can search.
+ * that lsof can search.  A value of 1 directs printname() to prefix the
+ * cache value with the file system directory name; 2, avoid the prefix.
  *
  * HASNCAPID is defined for those dialects with a searchable kernel name
  * cache whose cache and vnodes are linked by a capability ID.
+ *
+ * NCACHELDPFX is a set of C commands to execute before calling ncache_load().
+ *
+ * NCACHELDSFX is a set of C commands to execute after calling ncache_load().
  */
 
 /* #define	HASNCACHE	1	*/
 /* #define	HASNCAPID	1	*/
+/* #define	NCACHELDPFX	??? */
+/* #define	NCACHELDSFX	??? */
 
 
 /*
@@ -351,12 +362,27 @@ typedef	long long	aligned_offset_t  __attribute__ ((aligned (8)));
 /* #define	HASPRIVNMCACHE	<function name>	*/
 
 
+/*
+ * HASPRIVPRIPP is defined for dialects that have a private function for
+ * printing IP protocol names.  When HASPRIVPRIPP isn't defined, the
+ * IP protocol name printing function defaults to printiprto().
+ */
+
+/* #define	HASPRIVPRIPP	1	*/
+
+
 /*
  * HASPROCFS is defined for those dialects that have a proc file system --
  * usually /proc and usually in SYSV4 derivatives.
  *
- * HASFSTYPE is defined for those systems that have a file system type
- * string, st_fstype, in the stat() buffer.
+ * HASFSTYPE is defined as 1 for those systems that have a file system type
+ * string, st_fstype, in the stat() buffer; 2, for those systems that have a
+ * file system type integer in the stat() buffer, named MOUNTS_STAT_FSTYPE;
+ * 0, for systems whose stat(2) structure has no file system type member.  The
+ * additional symbols MOUNTS_FSTYPE, RMNT_FSTYPE, and RMNT_STAT_FSTYPE may be
+ * defined in dlsof.h to direct how the readmnt() function in lib/rmnt.c
+ * preserves these stat(2) and getmntent(3) buffer values in the local mounts
+ * structure.
  *
  * The defined value is the string that names the file system type.
  *
@@ -369,18 +395,25 @@ typedef	long long	aligned_offset_t  __attribute__ ((aligned (8)));
  * by inode number.
  */
 
+#if	AIXA<1
 /* #define	HASPROCFS	"proc?" */
 /* #define	HASFSTYPE	1 */
 /* #define	HASPINODEN	1 */
+#else	/* AIXA>=1 */
+#define	HASPROCFS	"proc"
+#define	HASFSTYPE	2
+#define	HASPINODEN	1
+#endif	/* AIXA<1 */
 
 
 /*
  * HASRNODE is defined for those dialects that have rnodes.
+ *
+ * Note: while AIX has rnodes, they are processed privately, so this
+ *	 definition should be disabled.
  */
 
-#if	defined(HAS_NFS)
-#define	HASRNODE	1
-#endif	/* defined(HAS_NFS) */
+/* #define	HASRNODE	1 */
 
 
 /*
@@ -393,6 +426,14 @@ typedef	long long	aligned_offset_t  __attribute__ ((aligned (8)));
 /* #define	HASSECURITY	1	*/
 
 
+/*
+ * HASSETLOCALE is defined for those dialects that have <locale.h> and
+ * setlocale().
+ */
+
+#define	HASSETLOCALE	1
+
+
 /*
  * HASSNODE is defined for those dialects that have snodes.
  */
@@ -400,6 +441,24 @@ typedef	long long	aligned_offset_t  __attribute__ ((aligned (8)));
 /* #define	HASSNODE	1 */
 
 
+/*
+ * Define HASSPECDEVD to be the name of a function that handles the results
+ * of a successful stat(2) of a file name argument.
+ *
+ * For example, HASSPECDEVD() for Darwin makes sure that st_dev is set to
+ * what stat("/dev") returns -- i.e., what's in DevDev.
+ *
+ * The function takes two arguments:
+ *
+ *	1: pointer to the full path name of file
+ *	2: pointer to the stat(2) result
+ *
+ * The function returns void.
+ */
+
+/* #define	HASSPECDEVD	process_dev_stat */
+
+
 /*
  * HASSTREAMS is defined for those dialects that support streams.
  */
@@ -487,9 +546,12 @@ typedef	long long	aligned_offset_t  __attribute__ ((aligned (8)));
 #define	USE_LIB_PRINT_TCPTPI			1	/* ptti.c */
 /* #define	USE_LIB_READDEV			1	   rdev.c */
 /* #define	USE_LIB_READMNT			1	   rmnt.c */
+/* #define	USE_LIB_REGEX			1	   regex.c */
 /* #define	USE_LIB_RNAM			1	   rnam.c */
 /* #define	USE_LIB_RNCH			1	   rnch.c */
 /* #define	USE_LIB_RNMH			1	   rnmh.c */
+/* #define	USE_LIB_SNPF			1	   snpf.c */
+#define	snpf	snprintf	   /* use the system's snprintf() */
 
 
 /*
