@@ -31,7 +31,7 @@
 
 
 /*
- * $Id: dlsof.h,v 1.11 99/11/28 06:38:51 abe Exp $
+ * $Id: dlsof.h,v 1.15 2001/02/13 13:49:17 abe Exp $
  */
 
 
@@ -45,6 +45,11 @@
 #include <signal.h>
 #include <string.h>
 #include <unistd.h>
+
+# if	BSDIV>=40100
+#include <arpa/inet.h>
+# endif	/* BSDIV>=40100 */
+
 #include <sys/filedesc.h>
 #include <sys/mbuf.h>
 #define	NFS
@@ -125,7 +130,7 @@
 #define	P_COMM		kp_proc.p_comm
 #define	P_FD		kp_proc.p_fd
 #define	P_PID		kp_proc.p_pid
-#define	P_PGRP		kp_eproc.e_pgid
+#define	P_PGID		kp_eproc.e_pgid
 #define	P_PPID		kp_eproc.e_ppid
 #define	P_STAT		kp_proc.p_stat
 #define	P_TEXTVP	kp_proc.p_textvp
@@ -166,6 +171,10 @@ typedef	u_long		KA_T;
 #define	READLEN_T	int
 #define STRNCPY_L	size_t
 #define SWAP		"/dev/drum"
+#define	SZOFFTYPE	long long	/* size and offset internal storage
+					 * type */
+#define	SZOFFPSPEC	"ll"		/* SZOFFTYPE print specification
+					 * modifier */
 
 
 /*
@@ -219,6 +228,7 @@ struct sfile {
 	char *name;			/* file name (after readlink()) */
 	char *devnm;			/* device name (optional) */
 	dev_t dev;			/* device */
+	dev_t rdev;			/* raw device */
 	u_short mode;			/* S_IFMT mode bits from stat() */
 	int type;			/* file type: 0 = file system
 				 	 *	      1 = regular file */
