@@ -32,16 +32,16 @@
 #ifndef lint
 static char copyright[] =
 "@(#) Copyright 1994 Purdue Research Foundation.\nAll rights reserved.\n";
-static char *rcsid = "$Id: dnode.c,v 1.11 99/06/22 08:12:38 abe Exp $";
+static char *rcsid = "$Id: dnode.c,v 1.16 2001/02/13 13:49:44 abe Exp $";
 #endif
 
 
 #include "lsof.h"
 
 
-#if	defined(HASFDESCFS)
+#if	defined(HASFDESCFS) && HASFDESCFS==1
 _PROTOTYPE(static int lkup_dev_tty,(dev_t *dr, unsigned long *ir));
-#endif	/* defined(HASFDESCFS) */
+#endif	/* defined(HASFDESCFS) && HASFDESCFS==1 */
 
 _PROTOTYPE(static void process_lock,(KA_T lp));
 
@@ -75,7 +75,7 @@ getmemsz(pid)
 #endif	/* defined(HASPROCFS) */
 
 
-#if	defined(HASFDESCFS)
+#if	defined(HASFDESCFS) && HASFDESCFS==1
 /*
  * lkup_dev_tty() - look up /dev/tty
  */
@@ -118,7 +118,7 @@ lkup_dev_tty_again:
 
 	return(-1);
 }
-#endif	/* defined(HASFDESCFS) */
+#endif	/* defined(HASFDESCFS) && HASFDESCFS==1 */
 
 
 /*
@@ -180,24 +180,30 @@ process_node(va)
 {
 	struct iso_node cd;
 	u_int cn;
-	dev_t dev;
+	dev_t dev, rdev;
+	unsigned char devs = 0;
+	unsigned char rdevs = 0;
 	struct denode dn;
-	int lt;
+	char *ep, *ty;
 	struct inode i;
 	struct mfsnode m;
 	struct nfsnode n;
 	enum nodetype {NONODE, INODE, MFSNODE, NFSNODE, CDFSNODE, PCFSNODE,
 		FDESCNODE, PFSNODE} nty = NONODE;
-	char *ty;
+	size_t sz;
 	enum vtype type;
 	struct vnode *v, vb;
 	struct l_vfs *vfs;
 
 #if	defined(HASFDESCFS)
 	struct fdescnode f;
+
+# if	HASFDESCFS==1
 	static dev_t f_tty_dev;
 	static unsigned long f_tty_ino;
 	static int f_tty_s = 0;
+# endif	/* HASFDESCFS==1 */
+
 #endif	/* defined(HASFDESCFS) */
 
 #if	defined(HASPROCFS)
@@ -261,7 +267,7 @@ process_node(va)
 # endif	/* defined(HASPROCFS) */
 
 		else {
-		    (void) sprintf(Namech,
+		    (void) snpf(Namech, Namechl,
 			"unsupported file system type: %d", vfs->type);
 		    enter_nm(Namech);
 		    return;
@@ -285,7 +291,7 @@ process_node(va)
 		else {
 		    if (vfs->typnm[0]
 		    ||  (v->v_type != VNON && v->v_type != VBAD)) {
-			(void) sprintf(Namech,
+			(void) snpf(Namech, Namechl,
 			    "unsupported file system type: %s", vfs->typnm);
 			enter_nm(Namech);
 			return;
@@ -312,8 +318,8 @@ process_node(va)
 	switch (v->v_tag) {
 	case VT_ISOFS:
 	    if (kread((KA_T)v->v_data, (char *)&cd, sizeof(cd))) {
-		(void) sprintf(Namech, "can't read cd9660_node at: %s",
-		    print_kptr((KA_T)v->v_data, (char *)NULL));
+		(void) snpf(Namech, Namechl, "can't read cd9660_node at: %s",
+		    print_kptr((KA_T)v->v_data, (char *)NULL, 0));
 		enter_nm(Namech);
 		return;
 	    }
@@ -322,8 +328,8 @@ process_node(va)
 	    break;
 	case VT_MFS:
 	    if (kread((KA_T)v->v_data, (char *)&m, sizeof(m))) {
-		(void) sprintf(Namech, "can't read mfsnode at: %s",
-		    print_kptr((KA_T)v->v_data, (char *)NULL));
+		(void) snpf(Namech, Namechl, "can't read mfsnode at: %s",
+		    print_kptr((KA_T)v->v_data, (char *)NULL, 0));
 		enter_nm(Namech);
 		return;
 	    }
@@ -331,8 +337,8 @@ process_node(va)
 	    break;
 	case VT_MSDOSFS:
 	    if (kread((KA_T)v->v_data, (char *)&dn, sizeof(dn))) {
-		(void) sprintf(Namech, "can't read denode at: %s",
-		    print_kptr((KA_T)v->v_data, (char *)NULL));
+		(void) snpf(Namech, Namechl, "can't read denode at: %s",
+		    print_kptr((KA_T)v->v_data, (char *)NULL, 0));
 		enter_nm(Namech);
 		return;
 	    }
@@ -341,8 +347,8 @@ process_node(va)
 	    break;
 	case VT_NFS:
 	    if (kread((KA_T)v->v_data, (char *)&n, sizeof(n))) {
-		(void) sprintf(Namech, "can't read nfsnode at: %s",
-		    print_kptr((KA_T)v->v_data, (char *)NULL));
+		(void) snpf(Namech, Namechl, "can't read nfsnode at: %s",
+		    print_kptr((KA_T)v->v_data, (char *)NULL, 0));
 		enter_nm(Namech);
 		return;
 	    }
@@ -352,8 +358,8 @@ process_node(va)
 #if	defined(HASFDESCFS)
 	case VT_FDESC:
 	    if (kread((KA_T)v->v_data, (char *)&f, sizeof(f))) {
-		(void) sprintf(Namech, "can't read fdescnode at: %s",
-		    print_kptr((KA_T)v->v_data, (char *)NULL));
+		(void) snpf(Namech, Namechl, "can't read fdescnode at: %s",
+		    print_kptr((KA_T)v->v_data, (char *)NULL, 0));
 		enter_nm(Namech);
 		return;
 	    }
@@ -364,8 +370,8 @@ process_node(va)
 #if	defined(HASPROCFS)
 	case VT_PROCFS:
 	    if (kread((KA_T)v->v_data, (char *)&p, sizeof(p))) {
-		(void) sprintf(Namech, "can't read pfsnode at: %s",
-		    print_kptr((KA_T)v->v_data, (char *)NULL));
+		(void) snpf(Namech, Namechl, "can't read pfsnode at: %s",
+		    print_kptr((KA_T)v->v_data, (char *)NULL, 0));
 		enter_nm(Namech);
 		return;
 	    }
@@ -375,8 +381,8 @@ process_node(va)
 
 	case VT_UFS:
 	    if (kread((KA_T)v->v_data, (char *)&i, sizeof(i))) {
-		(void) sprintf(Namech, "can't read inode at: %s",
-		    print_kptr((KA_T)v->v_data, (char *)NULL));
+		(void) snpf(Namech, Namechl, "can't read inode at: %s",
+		    print_kptr((KA_T)v->v_data, (char *)NULL, 0));
 		enter_nm(Namech);
 		return;
 	    }
@@ -390,19 +396,24 @@ process_node(va)
 	type = v->v_type;
 	switch (nty) {
 	case INODE:
-	    if (type == VCHR || type == VBLK)
-		dev = i.i_din.di_rdev;
-	    else
-		dev = i.i_dev;
+	    dev = i.i_dev;
+	    devs = 1;
+	    if (type == VCHR || type == VBLK) {
+		rdev = i.i_din.di_rdev;
+		rdevs = 1;
+	    }
 	    break;
 	case NFSNODE:
 	    dev = n.n_vattr.va_fsid;
+	    devs = 1;
 	    break;
 	case CDFSNODE:
 	    dev = cd.i_dev;
+	    devs = 1;
 	    break;
 	case PCFSNODE:
 	    dev = dn.de_dev;
+	    devs = 1;
 	}
 /*
  * Obtain the inode number.
@@ -411,21 +422,29 @@ process_node(va)
 
 #if	defined(HASFDESCFS)
 	case FDESCNODE:
+
+# if	defined(HASFDLINK)
 	    if (f.fd_link
-	    &&  !kread((KA_T)f.fd_link, Namech, sizeof(Namech) - 1)) {
-		Namech[sizeof(Namech) - 1] = '\0';
+	    &&  !kread((KA_T)f.fd_link, Namech, (int)(Namechl - 1))) {
+		Namech[(int)(Namechl - 1)] = '\0';
 		break;
 	    }
+# endif	/* defined(HASFDLINK)
+
+# if	HASFDESCFS==1
 	    if (f.fd_type == Fctty) {
 		if (f_tty_s == 0)
 		    f_tty_s = lkup_dev_tty(&f_tty_dev, &f_tty_ino);
 		if (f_tty_s == 1) {
 		    dev = f_tty_dev;
+		    devs = 1;
 		    Lf->inode = f_tty_ino;
 		    Lf->inp_ty = 1;
 		}
 	    }
 	    break;
+# endif	/* HASFDESCFS==1 */
+
 #endif	/* defined(HASFDESCFS) */
 
 	case INODE:
@@ -579,8 +598,14 @@ process_node(va)
 	    Lf->fsdev = vfs->fsname;
 	}
 /*
+ * Save the device numbers and their states.
+ *
  * Format the vnode type, and possibly the device name.
  */
+	Lf->dev = dev;
+	Lf->dev_def = devs;
+	Lf->rdev = rdev;
+	Lf->rdev_def = rdevs;
 	switch (type) {
 	case VNON:
 	    ty ="VNON";
@@ -588,19 +613,13 @@ process_node(va)
 	case VREG:
 	case VDIR:
 	    ty = (type == VREG) ? "VREG" : "VDIR";
-	    Lf->dev_def = 1;
-	    Lf->dev = dev;
 	    break;
 	case VBLK:
 	    ty = "VBLK";
-	    Lf->dev_def = 1;
-	    Lf->dev = dev;
 	    Ntype = N_BLK;
 	    break;
 	case VCHR:
 	    ty = "VCHR";
-	    Lf->dev = dev;
-	    Lf->dev_def = 1;
 	    Ntype = N_CHR;
 	    break;
 	case VLNK:
@@ -617,20 +636,18 @@ process_node(va)
 	    ty = "VBAD";
 	    break;
 	case VFIFO:
-	    Lf->dev = dev;
-	    Lf->dev_def = 1;
 	    ty = "FIFO";
 	    break;
 	default:
 	    if (type > 9999)
-		(void) sprintf(Lf->type, "*%03d", type % 1000);
+		(void) snpf(Lf->type, sizeof(Lf->type), "*%03d", type % 1000);
 	    else
-		(void) sprintf(Lf->type, "%4d", type);
-	    (void) strcpy(Namech, "unknown type");
+		(void) snpf(Lf->type, sizeof(Lf->type), "%4d", type);
+	    (void) snpf(Namech, Namechl, "unknown type");
 	    ty = (char *)NULL;
 	}
 	if (ty)
-	    (void) strcpy(Lf->type, ty);
+	    (void) snpf(Lf->type, sizeof(Lf->type), ty);
 	Lf->ntype = Ntype;
 /*
  * Handle some special cases:
@@ -641,61 +658,70 @@ process_node(va)
  */
 
 	if (type == VBAD)
-	    (void) strcpy(Namech, "(revoked)");
+	    (void) snpf(Namech, Namechl, "(revoked)");
 	else if (nty == MFSNODE) {
-	    Lf->dev_def = 0;
-	    (void) sprintf(Namech, "%#x", m.mfs_baseoff);
+	    devs = Lf->dev_def = Lf->rdev_def = rdevs = 0;
+	    (void) snpf(Namech, Namechl, "%#x", m.mfs_baseoff);
 	    enter_dev_ch("memory");
 	}
 
 #if	defined(HASPROCFS)
 	else if (nty == PFSNODE) {
-	    Lf->dev_def = 0;
+	    devs = Lf->dev_def = Lf->rdev_def = rdevs = 0;
 	    ty = (char *)NULL;
-	    (void) sprintf(Namech, "/%s", HASPROCFS);
+	    (void) snpf(Namech, Namechl, "/%s", HASPROCFS);
 	    switch (p.pfs_type) {
 	    case Proot:
 		ty = "PDIR";
 		break;
 	    case Pproc:
-		(void) sprintf(endnm(), "/%d", p.pfs_pid);
+		ep = endnm(&sz);
+		(void) snpf(ep, sz, "/%d", p.pfs_pid);
 		ty = "PDIR";
 		break;
 	    case Pfile:
-		(void) sprintf(endnm(), "/%d/file", p.pfs_pid);
+		ep = endnm(&sz);
+		(void) snpf(ep, sz, "/%d/file", p.pfs_pid);
 		ty = "PFIL";
 		break;
 	    case Pmem:
-		(void) sprintf(endnm(), "/%d/mem", p.pfs_pid);
+		ep = endnm(&sz);
+		(void) snpf(ep, sz, "/%d/mem", p.pfs_pid);
 		ty = "PMEM";
 		break;
 	    case Pregs:
-		(void) sprintf(endnm(), "/%d/regs", p.pfs_pid);
+		ep = endnm(&sz);
+		(void) snpf(ep, sz, "/%d/regs", p.pfs_pid);
 		ty = "PREG";
 		break;
 	    case Pfpregs:
-		(void) sprintf(endnm(), "/%d/fpregs", p.pfs_pid);
+		ep = endnm(&sz);
+		(void) snpf(ep, sz, "/%d/fpregs", p.pfs_pid);
 		ty = "PFPR";
 		break;
 	    case Pctl:
-		(void) sprintf(endnm(), "/%d/ctl", p.pfs_pid);
+		ep = endnm(&sz);
+		(void) snpf(ep, sz, "/%d/ctl", p.pfs_pid);
 		ty = "PCTL";
 		break;
 	    case Pstatus:
-		(void) sprintf(endnm(), "/%d/status", p.pfs_pid);
+		ep = endnm(&sz);
+		(void) snpf(ep, sz, "/%d/status", p.pfs_pid);
 		ty = "PSTA";
 		break;
 	    case Pnote:
-		(void) sprintf(endnm(), "/%d/note", p.pfs_pid);
+		ep = endnm(&sz);
+		(void) snpf(ep, sz, "/%d/note", p.pfs_pid);
 		ty = "PNTF";
 		break;
 	    case Pnotepg:
-		(void) sprintf(endnm(), "/%d/notepg", p.pfs_pid);
-		ty = "PGRP";
+		ep = endnm(&sz);
+		(void) snpf(ep, sz, "/%d/notepg", p.pfs_pid);
+		ty = "PGID";
 		break;
 	    }
 	    if (ty)
-		(void) strcpy(Lf->type, ty);
+		(void) snpf(Lf->type, sizeof(Lf->type), "%s", ty);
 	    enter_nm(Namech);
 	}
 #endif	/* defined(HASPROCFS) */
@@ -705,7 +731,7 @@ process_node(va)
  * If this is a VBLK file and it's missing an inode number, try to
  * supply one.
  */
-	if (Lf->inp_ty == 0 && type == VBLK && Lf->dev_def)
+	if ((Lf->inp_ty == 0) && (type == VBLK))
 	    find_bl_ino();
 #endif	/* defined(HASBLKDEV) */
 
@@ -713,7 +739,7 @@ process_node(va)
  * If this is a VCHR file and it's missing an inode number, try to
  * supply one.
  */
-	if (Lf->inp_ty == 0 && type == VCHR && Lf->dev_def)
+	if ((Lf->inp_ty == 0) && (type == VCHR))
 	    find_ch_ino();
 /*
  * Test for specified file.
@@ -734,10 +760,8 @@ process_node(va)
 
 		    ) {
 			pfi->f = 1;
-			if (Namech[0] && pfi->nm) {
-			    (void) strncpy(Namech, pfi->nm, MAXPATHLEN - 1);
-			    Namech[MAXPATHLEN - 1] = '\0';
-			}
+			if (Namech[0] && pfi->nm)
+			    (void) snpf(Namech, Namechl, "%s", pfi->nm);
 			Lf->sf |= SELNM;
 			break;
 		    }
@@ -747,7 +771,9 @@ process_node(va)
 #endif	/* defined(HASPROCFS) */
 
 	{
-	    if (Sfile && is_file_named((char *)NULL, (type == VCHR) ? 1: 0))
+	    if (Sfile && is_file_named((char *)NULL,
+				       ((type == VCHR) || (type == VBLK)) ? 1
+									  : 0))
 		Lf->sf |= SELNM;
 	}
 /*
