@@ -32,7 +32,7 @@
 #ifndef lint
 static char copyright[] =
 "@(#) Copyright 1994 Purdue Research Foundation.\nAll rights reserved.\n";
-static char *rcsid = "$Id: dproc.c,v 1.5 99/05/09 14:55:18 abe Exp $";
+static char *rcsid = "$Id: dproc.c,v 1.7 2001/02/13 13:50:06 abe Exp $";
 #endif
 
 #include "lsof.h"
@@ -172,14 +172,6 @@ gather_proc_info()
 		Pn, kvm_geterr(Kd));
 	    Exit(1);
 	}
-
-#if     defined(HASNCACHE)
-/*
- * Read kernel name cache.
- */
-	ncache_load();
-#endif  /* defined(HASNCACHE) */
-
 /*
  * Examine proc structures and their associated information.
  */
@@ -196,7 +188,7 @@ gather_proc_info()
 	 * Read file structure pointers.
 	 */
 	    uid = p->kp_eproc.e_ucred.cr_uid;
-	    if (is_proc_excl((int)p->P_PID, (int)p->P_PGRP, (UID_ARG)uid,
+	    if (is_proc_excl((int)p->P_PID, (int)p->P_PGID, (UID_ARG)uid,
 		 &pss, &sf))
 	    {
 		continue;
@@ -211,7 +203,7 @@ gather_proc_info()
 	 */
 	    if (is_cmd_excl(p->P_COMM, &pss, &sf))
 		continue;
-	    alloc_lproc((int)p->P_PID, (int)p->P_PGRP, (int)p->P_PPID,
+	    alloc_lproc((int)p->P_PID, (int)p->P_PGID, (int)p->P_PPID,
 		(UID_ARG)uid, p->P_COMM, (int)pss, (int)sf);
 	    Plf = (struct lfile *)NULL;
 	    Kpa = (KA_T)p->P_ADDR;
