@@ -33,7 +33,7 @@
 #ifndef lint
 static char copyright[] =
 "@(#) Copyright 1994 Purdue Research Foundation.\nAll rights reserved.\n";
-static char *rcsid = "$Id: ddev.c,v 1.11 99/06/22 08:13:44 abe Exp $";
+static char *rcsid = "$Id: ddev.c,v 1.13 2000/12/04 14:24:15 abe Exp $";
 #endif
 
 #include "lsof.h"
@@ -48,6 +48,17 @@ static struct stat Dirsb;
 #endif	/* defined(USELOCALREADDIR) */
 
 
+/*
+ * Local definitions
+ */
+
+#define	LIKE_NODE_TTL	"like device special "
+
+
+/*
+ * Local function prototypes
+ */
+
 _PROTOTYPE(static int rmdupdev,(struct l_dev ***dp, int n, char *nm));
 
 
@@ -77,8 +88,9 @@ clr_sect()
  */
 
 int
-printchdevname(rdev, f)
-	dev_t *rdev;			/* device */
+printchdevname(dev, rdev, f)
+	dev_t *dev;			/* device */
+	dev_t *rdev;			/* raw device */
 	int f;				/* 1 = follow with '\n' */
 {
 	struct clone *c;
@@ -95,7 +107,9 @@ printchdevname_again:
 
 #endif  /* defined(HASDCACHE) */
 
-	if (Clone && HAVECLONEMAJ && major(*rdev) == CLONEMAJ) {
+	if (Clone && HAVECLONEMAJ && (*dev == DevDev)
+	&&  (major(*rdev) == CLONEMAJ))
+	{
 	    for (c = Clone; c; c = c->next) {
 		if (Devtp[c->dx].rdev == *rdev) {
 
@@ -110,12 +124,34 @@ printchdevname_again:
 	    }
 	}
 /*
- * Search device table for match.
+ * Search device table for a full match.
  */
-	if ((dp = lkupdev(rdev, 1, 0))) {
+	if ((dp = lkupdev(dev, rdev, 1, 0))) {
 	    safestrprt(dp->name, stdout, f);
 	    return(1);
 	}
+/*
+ * Search device table for a match without inode number and dev.
+ */
+	if ((dp = lkupdev(&DevDev, rdev, 0, 0))) {
+
+	/*
+	 * A raw device match was found.  Record it as a name column addition.
+	 */
+	    char *cp;
+	    int len;
+
+	    len = (int)(1 + strlen(LIKE_NODE_TTL) + strlen(dp->name) + 1);
+	    if (!(cp = (char *)malloc((MALLOC_S)(len + 1)))) {
+		(void) fprintf(stderr, "%s: no nma space for: (%s%s)\n",
+		    Pn, LIKE_NODE_TTL, dp->name);
+		Exit(1);
+	    }
+	    (void) snpf(cp, len + 1, "(%s%s)", LIKE_NODE_TTL, dp->name);
+	    (void) add_nma(cp, len);
+	    (void) free((MALLOC_P *)cp);
+	    return(0);
+	}
 
 #if     defined(HASDCACHE)
 /*
@@ -376,11 +412,15 @@ readdev(skip)
 	    (void) qsort((QSORT_P *)BSdev, (size_t)BNdev,
 		(size_t)sizeof(struct l_dev *), compdev);
 	    BNdev = rmdupdev(&BSdev, BNdev, "block");
-	} else {
+	}
+
+# if	!defined(NOWARNBLKDEV)
+	else {
 	    if (!Fwarn)
 		(void) fprintf(stderr,
 		    "%s: WARNING: no block devices found\n", Pn);
 	}
+# endif	/* !defined(NOWARNBLKDEV) */
 #endif	/* defined(HASBLKDEV) */
 
 	if (Ndev) {
