@@ -33,7 +33,7 @@
 #ifndef lint
 static char copyright[] =
 "@(#) Copyright 1994 Purdue Research Foundation.\nAll rights reserved.\n";
-static char *rcsid = "$Id: dfile.c,v 1.9 99/10/13 09:17:18 abe Exp $";
+static char *rcsid = "$Id: dfile.c,v 1.11 2000/12/04 14:24:15 abe Exp $";
 #endif
 
 
@@ -102,14 +102,15 @@ print_advfs_path(lf)
  */
 
 char *
-print_dev(lf)
+print_dev(lf, dev)
 	struct lfile *lf;		/* file whose device is to be printed */
+	dev_t *dev;			/* device to be printed */
 {
 	static char buf[128];
 
-	if (minor(lf->dev) > 9999999)
-	    (void) sprintf(buf, "%d,%#x", major(lf->dev), minor(lf->dev));
+	if (minor(*dev) > 9999999)
+	    (void) snpf(buf, sizeof(buf), "%d,%#x", major(*dev), minor(*dev));
 	else
-	    (void) sprintf(buf, "%d,%d", major(lf->dev), minor(lf->dev));
+	    (void) snpf(buf, sizeof(buf), "%d,%d", major(*dev), minor(*dev));
 	return(buf);
 }
