@@ -32,7 +32,7 @@
 #ifndef lint
 static char copyright[] =
 "@(#) Copyright 1994 Purdue Research Foundation.\nAll rights reserved.\n";
-static char *rcsid = "$Id: dnode.c,v 1.13 99/10/22 08:22:08 abe Exp $";
+static char *rcsid = "$Id: dnode.c,v 1.17 2001/01/09 15:00:07 abe Exp $";
 #endif
 
 
@@ -43,6 +43,74 @@ static char *rcsid = "$Id: dnode.c,v 1.13 99/10/22 08:22:08 abe Exp $";
  * Local definitions
  */
 
+#if	ADVFSV>=500
+/*
+ * AdvFS (MSFS) definitions for AdvFS version 5.0 and above.
+ */
+
+struct fs_stat {			/* file system stat(2) info structure
+					 * for AdvFS 5.0 and above */
+	unsigned int num;		/* node number */
+	unsigned int d1;
+	mode_t d2;
+	uid_t d3;
+	gid_t d4;
+	dev_t rdev;			/* character or block device number */
+	off_t size;			/* file size */
+
+# if	defined(__arch32__)
+	unsigned int d5;
+# endif	/* defined(__arch32__) */
+
+	time_t d6;
+	int d7;
+	time_t d8;
+	int d9;
+	time_t d10;
+	int d11;
+	unsigned int d12[5];
+	unsigned short nlink;		/* link count */
+};
+
+struct fsContext {			/* file system context for AdvFS 5.0
+					 * and above */
+	short d1[2];
+	unsigned int d2[2];
+	long d3;
+	int d4[2];
+	lock_data_t d5;
+	long d6;
+	simple_lock_data_t d7;
+	unsigned int d8[2];
+	long d9;
+	struct fs_stat st;		/* file stats */
+};
+
+struct advfsnode {			/* AdvFS (MSFS) node definition for
+					 * AdvFS 5.0 and above */
+	unsigned long d1;
+	struct fsContext *a_con;	/* context pointer */
+};
+#endif	/* ADVFSV>=500 */
+
+#if	DUV>=50000
+typedef struct cnode {			/* CFS node structure definition for
+					 * Tru64 UNIX 5.0 and above */
+        udecl_simple_lock_data(, d1)
+	unsigned int d2;
+	time_t d3;
+	unsigned long d4[3];
+
+# if	DUV<50100
+	int d5[2];
+	off_t d6;
+# endif	/* DUV<50100 */
+
+	unsigned long d7;
+        vattr_t          c_attr;        /* 96:Cached vnode attributes */
+} cnode_t;
+#endif	/* DUV>=50000 */
+
 struct l_lock {				/* local lock info */
 	struct eflock set;		/* lock data */
 	struct l_lock *next;
@@ -307,8 +375,9 @@ process_node(va)
 {
 	struct advfsnode *a = (struct advfsnode *)NULL;
 	struct cdnode *c = (struct cdnode *)NULL;
-	dev_t dev;
-	int devs;
+	dev_t dev, rdev;
+	unsigned char devs = 0;
+	unsigned char rdevs = 0;
 	struct inode *i = (struct inode *)NULL;
 	struct mfsnode *m = (struct mfsnode *)NULL;
 	struct procnode *p = (struct procnode *)NULL;
@@ -326,8 +395,15 @@ process_node(va)
 #if	DUV>=30000
 	struct fifonode *f = (struct fifonode *)NULL;
 	struct fifonode fn;
+	static struct vnode *fv = (struct vnode *)NULL;
 #endif	/* DUV>=30000 */
 
+#if	DUV>=50000
+	cnode_t *cn = (cnode_t *)NULL;
+	struct fsContext fsc;
+	int fscs = 0;
+#endif	/* DUV>=50000 */
+
 /*
  * Read the vnode.
  */
@@ -344,6 +420,14 @@ process_node(va)
 		(void) fprintf(stderr, "%s: no space for vnode buffer\n", Pn);
 		Exit(1);
 	    }
+
+#if	DUV>=30000
+	    if (!(fv = (struct vnode *)malloc(sizeof(struct vnode)-1+Vnmxp))) {
+		(void) fprintf(stderr, "%s: no space for fvnode buffer\n", Pn);
+		Exit(1);
+	    }
+#endif	/* DUV>=30000 */
+
 	}
 	if (readvnode(va, v)) {
 	    enter_nm(Namech);
@@ -413,6 +497,13 @@ process_node(va)
 	case VT_CDFS:
 	    c = (struct cdnode *)v->v_data;
 	    break;
+
+#if	DUV>=50000
+	case VT_CFS:
+	    cn = (cnode_t *)v->v_data;
+	    break;
+#endif	/* DUV>=50000 */
+
 	case VT_MFS:
 	    m = (struct mfsnode *)v->v_data;
 	    break;
@@ -426,6 +517,10 @@ process_node(va)
 	    &&  !kread((KA_T)v->v_specinfo, (char *)&sn, sizeof(sn)))
 		s = &sn;
 	    else
+#else	/* DUV>=20000 */
+# if	DUV>=30000
+	    if (!f)
+# endif	/* DUV>=30000 */
 #endif  /* DUV<20000 */
 
 		s = (struct spec_node *)v->v_data;
@@ -438,62 +533,138 @@ process_node(va)
 	    break;
 	case VT_MSFS:
 	    a = (struct advfsnode *)v->v_data;
+
+#if	ADVFSV>=500
+	    if (a->a_con
+	    &&  !kread((KA_T)a->a_con, (char *)&fsc, sizeof(fsc)))
+		fscs = 1;
+#endif	/* ADVFSV>=500 */
+
 	    break;
 	case VT_UFS:
 	    i = (struct inode *)v->v_data;
 	    break;
+	default:
+	    (void) snpf(Namech, Namechl, "unknown node type, v_tag=%d",
+		v->v_tag);
+	    enter_nm(Namech);
+	    return;
 	}
 /*
  * Get device and type for printing.
  */
 	type = v->v_type;
-	dev = devs = 0;
 	if (a) {
-	    if (type == VCHR) {
-		dev = a->a_rdev;
-		devs = 1;
-	    } else if (vfs) {
+	    if (vfs && vfs->dev) {
 		dev = vfs->dev;
 		devs = 1;
 	    }
+	    if ((type == VCHR) || (type == VBLK)) {
+
+#if	ADVFSV>=500
+		if (fscs) {
+		    rdev = fsc.st.rdev;
+		    rdevs = 1;
+		}
+#else	/* ADVFSV<500 */
+		rdev = a->a_rdev;
+		rdevs = 1;
+#endif	/* ADVFSV>=500 */
+
+	    }
 	} else if (c) {
 	    dev = c->cd_dev;
 	    devs = 1;
-	} else if (i) {
-	    dev = (type == VCHR) ? i->i_din.di_db[0] : i->i_dev;
-	    devs = 1;
+	}
+
+#if	DUV>=50000
+	else if (cn) {
+	    if (vfs && vfs->dev) {
+		dev = vfs->dev;
+		devs = 1;
+	    }
+	    if ((type == VCHR) || (type == VBLK)) {
+		if (cn->c_attr.va_mask & AT_RDEV) {
+		    rdev = cn->c_attr.va_rdev;
+		    rdevs = 1;
+		}
+	    }
+	}
+#endif	/* DUV>=50000 */
+
+	else if (i) {
+	    if (i->i_dev) {
+		dev = i->i_dev;
+		devs = 1;
+	    } else if (vfs && vfs->dev) {
+		dev = vfs->dev;
+		devs = 1;
+	    }
+	    if ((type == VCHR) || (type == VBLK)) {
+		rdev = i->i_din.di_db[0];
+		rdevs = 1;
+	    }
 	} else if (r) {
 	    dev = r->r_attr.va_fsid;
 	    devs = 1;
 	} else if (s) {
+	    if (vfs && vfs->dev)
+		dev = vfs->dev;
+	    else
+		dev = DevDev;
 	    devs = 1;
-	    dev = s->sn_vattr.va_rdev;
-	    if (!lkupdev(&dev, 0, 0) && HaveCloneMaj)
-		dev = makedev(CloneMaj, major(dev));
+	    rdev = s->sn_vattr.va_rdev;
+	    rdevs = 1;
+	    if (!lkupdev(&dev, &rdev, 0, 0) && HaveCloneMaj)
+		rdev = makedev(CloneMaj, major(rdev));
 	} else if (s5) {
 	    dev = s5->i_dev;
 	    devs = 1;
+	} else if (f) {
+	    if (vfs && vfs->dev) {
+		dev = vfs->dev;
+		devs = 1;
+	    }
 	}
 /*
  * Obtain the inode number.
  */
 	if (a) {
+
+#if	ADVFSV>=500
+	    if (fscs) {
+		Lf->inode = (unsigned long)fsc.st.num;
+		Lf->inp_ty = 1;
+	    }
+#else	/* ADVFSV<500 */
 	    Lf->inode = (unsigned long)a->a_number;
 	    Lf->inp_ty = 1;
+#endif	/* ADVFSV>=500 */
 
-#if	DUV>=40000 && ADVFSV>=400
+
+#if	defined(HASTAGTOPATH)
 	/*
 	 * Record the Digital UNIX 4.0 or greater, ADVFS 4.0 or greater
 	 * ADVFS sequence number for later use with tag_to_path().
 	 */
 	    Lf->advfs_seq = a->a_seq;
 	    Lf->advfs_seq_stat = 1;
-#endif	/* DUV>=40000 && ADVFSV>=400 */
+#endif	/* defined(HASTAGTOPATH) */
 
 	} else if (c) {
 	    Lf->inode = (unsigned long)c->cd_number;
 	    Lf->inp_ty = 1;
 	}
+
+#if	DUV>=50000
+	else if (cn) {
+	    if (cn->c_attr.va_mask & AT_NODEID) {
+		Lf->inode = (unsigned long)cn->c_attr.va_fileid;
+		Lf->inp_ty = 1;
+	    }
+	}
+#endif	/* DUV>=50000 */
+
 	else if (i) {
 	    Lf->inode = (unsigned long)i->i_number;
 	    Lf->inp_ty = 1;
@@ -553,12 +724,32 @@ process_node(va)
 	    case N_REGLR:
 		if (type == VREG || type == VDIR) {
 		    if (a) {
+
+#if	ADVFSV>=500
+			if (fscs) {
+			    Lf->sz = (SZOFFTYPE)fsc.st.size;
+			    Lf->sz_def = 1;
+			}
+#else	/* ADVFSV<500 */
 			Lf->sz = (SZOFFTYPE)a->a_size;
 			Lf->sz_def = 1;
+#endif	/* ADVFSV>=500 */
+
 		    } else if (c) {
 			Lf->sz = (SZOFFTYPE)c->cd_size;
 			Lf->sz_def = 1;
-		    } else if (i) {
+		    }
+
+#if	DUV>=50000
+		    else if (cn) {
+			if (cn->c_attr.va_mask & AT_SIZE) {
+			    Lf->sz = (SZOFFTYPE)cn->c_attr.va_qsize;
+			    Lf->sz_def = 1;
+			}
+		    }
+#endif	/* DUV>=50000 */
+
+		    else if (i) {
 			Lf->sz = (SZOFFTYPE)i->i_din.di_qsize;
 			Lf->sz_def = 1;
 		    } else if (m) {
@@ -587,8 +778,17 @@ process_node(va)
 
 #  if	ADVFSV>=400
 		if (a) {
+
+#if	ADVFSV>=500
+		    if (fscs) {
+			Lf->nlink = (long)fsc.st.nlink;
+			Lf->nlink_def = 1;
+		    }
+#else	/* ADVFSV<500 */
 		    Lf->nlink = (long)a->a_nlink;
 		    Lf->nlink_def = 1;
+#endif	/* ADVFSV>=500 */
+
 		    break;
 		}
 #  endif	/* ADVFSV>=400 */
@@ -596,7 +796,18 @@ process_node(va)
 		if (c) {
 		    Lf->nlink = (long)c->cd_nlink;
 		    Lf->nlink_def = 1;
-		} else if (i) {
+		}
+
+#if	DUV>=50000
+		else if (cn) {
+		    if (cn->c_attr.va_mask & AT_NLINK) {
+			Lf->nlink = (long)cn->c_attr.va_nlink;
+			Lf->nlink_def = 1;
+		    }
+		}
+#endif	/* DUV>=50000 */
+
+		else if (i) {
 		    Lf->nlink = (long)i->i_din.di_nlink;
 		    Lf->nlink_def = 1;
 		} else if (s5) {
@@ -613,8 +824,14 @@ process_node(va)
 	if (Ntype == N_NFS && Fnfs)
 	    Lf->sf |= SELNFS;
 /*
+ * Save the device numbers and their states.
+ *
  * Format the vnode type, and possibly the device name.
  */
+	Lf->dev = dev;
+	Lf->dev_def = devs;
+	Lf->rdev = rdev;
+	Lf->rdev_def = rdevs;
 	switch (type) {
 	case VNON:
 	    ty ="VNON";
@@ -622,16 +839,12 @@ process_node(va)
 	case VREG:
 	case VDIR:
 	    ty = (type == VREG) ? "VREG" : "VDIR";
-	    Lf->dev_def = 1;
-	    Lf->dev = dev;
 	    break;
 	case VBLK:
 	    ty = "VBLK";
 	    break;
 	case VCHR:
 	    ty = "VCHR";
-	    Lf->dev = dev;
-	    Lf->dev_def = 1;
 	    Ntype = N_CHR;
 	    break;
 	case VLNK:
@@ -648,31 +861,28 @@ process_node(va)
 	    ty = "VBAD";
 	    break;
 	case VFIFO:
-	    if (devs && dev) {
-		Lf->dev = dev;
-		Lf->dev_def = 1;
-	    }
+	    ty = "FIFO";
 
 #if	DUV>=30000
-	    else if (f) {
+	    if ((!devs || !dev) && f) {
 		vfs = (struct l_vfs *)NULL;
+		devs = Lf->dev_def = 0;
 		ul = (unsigned long)v->v_fifonode;
-		enter_dev_ch(print_kptr((KA_T)(ul & 0xffffffff), (char *)NULL));
+		enter_dev_ch(print_kptr((KA_T)(ul&0xffffffff),(char *)NULL,0));
 	    }
 #endif	/* DUV>=30000 */
 
-	    ty = "FIFO";
 	    break;
 	default:
 	    if (type > 9999)
-		(void) sprintf(Lf->type, "*%03d", type % 1000);
+		(void) snpf(Lf->type, sizeof(Lf->type), "*%03d", type % 1000);
 	    else
-		(void) sprintf(Lf->type, "%4d", type);
-	    (void) strcpy(Namech, "unknown type");
+		(void) snpf(Lf->type, sizeof(Lf->type), "%4d", type);
+	    (void) snpf(Namech, Namechl, "unknown type");
 	    ty = (char *)NULL;
 	}
 	if (ty)
-	    (void) strcpy(Lf->type, ty);
+	    (void) snpf(Lf->type, sizeof(Lf->type), "%s", ty);
 	Lf->ntype = Ntype;
 /*
  * Save the file system names.
@@ -699,23 +909,33 @@ process_node(va)
  */
 
 	if (type == VBAD)
-	    (void) strcpy(Namech, "(revoked)");
+	    (void) snpf(Namech, Namechl, "(revoked)");
 	if (m) {
-	    Lf->dev_def = 0;
-	    (void) sprintf(Namech, "%#x", m->mfs_baseoff);
+	    devs = Lf->dev_def = Lf->rdev_def = rdevs = 0;
+	    (void) snpf(Namech, Namechl, "%#x", m->mfs_baseoff);
 	    (void) enter_dev_ch("memory");
 	} else if (p) {
-	    Lf->dev_def = 0;
+	    devs = Lf->dev_def = Lf->rdev_def = rdevs = 0;
 	    if (type != VDIR)
-		(void) sprintf(Namech, "/proc/%d", p->prc_pid);
+		(void) snpf(Namech, Namechl, "/proc/%d", p->prc_pid);
 	    else
-		(void) strcpy(Namech, "/proc");
+		(void) snpf(Namech, Namechl, "/proc");
 	}
+
+#if	defined(HASBLKDEV)
+/*
+ * If this is a VBLK file and it's missing an inode number, try to
+ * supply one.
+ */
+	if ((Lf->inp_ty == 0) && (type == VBLK))
+	    find_bl_ino();
+#endif	/* defined(HASBLKDEV) */
+
 /*
  * If this is a VCHR file and it's missing an inode number, try to
  * supply one.
  */
-	if (Lf->inp_ty == 0 && type == VCHR && Lf->dev_def)
+	if ((Lf->inp_ty == 0) && (type == VCHR))
 	    find_ch_ino();
 /*
  * Test for specified file.
@@ -762,8 +982,8 @@ readvnode(va, v)
 {
 
 	if (kread((KA_T)va, (char *)v, sizeof(struct vnode) - 1 + Vnmxp)) {
-	    (void) sprintf(Namech, "can't read vnode at %s",
-		print_kptr(va, (char *)NULL));
+	    (void) snpf(Namech, Namechl, "can't read vnode at %s",
+		print_kptr(va, (char *)NULL, 0));
 	    return(1);
 	}
 	return(0);
