@@ -32,16 +32,16 @@
 #ifndef lint
 static char copyright[] =
 "@(#) Copyright 1994 Purdue Research Foundation.\nAll rights reserved.\n";
-static char *rcsid = "$Id: dnode.c,v 1.13 99/11/28 06:40:24 abe Exp $";
+static char *rcsid = "$Id: dnode.c,v 1.21 2001/06/08 15:47:42 abe Exp $";
 #endif
 
 
 #include "lsof.h"
 
 
-#if	defined(HASFDESCFS)
+#if	defined(HASFDESCFS) && HASFDESCFS==1
 _PROTOTYPE(static int lkup_dev_tty,(dev_t *dr, unsigned long *ir));
-#endif	/* defined(HASFDESCFS) */
+#endif	/* defined(HASFDESCFS) && HASFDESCFS==1 */
 
 
 #if	FREEBSDV>=200
@@ -77,7 +77,7 @@ getmemsz(pid)
 #endif	/* FREEBSDV>=200 */
 
 
-#if	defined(HASFDESCFS)
+#if	defined(HASFDESCFS) && HASFDESCFS==1
 /*
  * lkup_dev_tty() - look up /dev/tty
  */
@@ -120,7 +120,7 @@ lkup_dev_tty_again:
 
 	return(-1);
 }
-#endif	/* defined(HASFDESCFS) */
+#endif	/* defined(HASFDESCFS) && HASFDESCFS==1 */
 
 
 /*
@@ -131,14 +131,13 @@ void
 process_node(va)
 	KA_T va;			/* vnode kernel space address */
 {
-
-	dev_t dev;
-	int devs = 0;
-	char dev_ch[32];
+	dev_t dev, rdev;
+	unsigned char devs = 0;
+	unsigned char lt;
+	unsigned char rdevs = 0;
+	char dev_ch[32], *ep;
 	struct inode *i = (struct inode *)NULL;
 	struct lockf lf, *lff, *lfp;
-	int lt;
-	struct mfsnode *m = (struct mfsnode *)NULL;
 	struct nfsnode *n = (struct nfsnode *)NULL;
 	char *ty;
 	enum vtype type;
@@ -147,13 +146,20 @@ process_node(va)
 
 #if	FREEBSDV>=200
 	struct inode ib;
-	struct mfsnode mb;
 	struct nfsnode nb;
+	size_t sz;
 # if	FREEBSDV>=400
-	struct specinfo udev;
+	struct specinfo si;
 # endif	/* FREEBSDV>=400 */
 #endif	/* FREEBSDV>=200 */
 
+#if	FREEBSDV<500
+	struct mfsnode *m = (struct mfsnode *)NULL;
+# if	FREEBSDV>=200
+	struct mfsnode mb;
+# endif	/* FREEBSDV>=200 */
+#endif	/* FREEBSDV<500 */
+
 #if	defined(HAS9660FS)
 	dev_t iso_dev;
 	int iso_dev_def = 0;
@@ -164,14 +170,24 @@ process_node(va)
 
 #if	defined(HASFDESCFS)
 	struct fdescnode *f = (struct fdescnode *)NULL;
+
+# if	HASFDESCFS==1
 	static dev_t f_tty_dev;
 	static unsigned long f_tty_ino;
 	static int f_tty_s = 0;
+# endif	/* HASFDESCFS==1 */
+
 # if	FREEBSDV>=200
 	struct fdescnode fb;
 # endif	/* FREEBSDV>=200 */
+
 #endif	/* defined(HASFDESCFS) */
 
+#if	FREEBSDV>=500
+	struct devfs_dirent de;
+	struct devfs_dirent *d = (struct devfs_dirent *)NULL;
+#endif	/* FREEBSDV>=500 */
+
 #if	defined(HASPROCFS)
 	struct pfsnode *p = (struct pfsnode *)NULL;
 	struct procfsid *pfi;
@@ -187,13 +203,13 @@ process_node(va)
  * Read the vnode.
  */
 	if ( ! va) {
-		enter_nm("no vnode address");
-		return;
+	    enter_nm("no vnode address");
+	    return;
 	}
 	v = &vb;
 	if (readvnode(va, v)) {
-		enter_nm(Namech);
-		return;
+	    enter_nm(Namech);
+	    return;
 	}
 
 #if	defined(HASNCACHE)
@@ -253,12 +269,36 @@ process_node(va)
  */
 	switch (v->v_tag) {
 
+#if	FREEBSDV>=500
+	case VT_DEVFS:
+	    if (!v->v_data
+	    ||  kread((KA_T)v->v_data, (char *)&de, sizeof(de)))
+	    {
+		(void) snpf(Namech, Namechl, "no devfs node: %s",
+		    print_kptr((KA_T)v->v_data, (char *)NULL, 0));
+		enter_nm(Namech);
+		return;
+	    }
+	    d = &de;
+	    if (v->v_type == VDIR) {
+		if (!d->de_dir
+		||  kread((KA_T)d->de_dir, (char *)&de, sizeof(de))) {
+		    (void) snpf(Namech, Namechl, "no devfs dir node: %s",
+			print_kptr((KA_T)d->de_dir, (char *)NULL, 0));
+		    enter_nm(Namech);
+		    return;
+		}
+	    }
+	    break;
+#endif	/* FREEBSDV>=500 */
+
 #if	defined(HAS9660FS)
 	case VT_ISOFS:
 	    if (read_iso_node(v, &iso_dev, &iso_dev_def, &iso_ino, &iso_links,
 			      &iso_sz))
 	    {
-		(void) sprintf(Namech, "no iso node: %#x", v->v_data);
+		(void) snpf(Namech, Namechl, "no iso node: %s",
+		    print_kptr((KA_T)v->v_data, (char *)NULL, 0));
 		enter_nm(Namech);
 		return;
 	    }
@@ -266,19 +306,22 @@ process_node(va)
 	    break;
 #endif	/* defined(HAS9660FS) */
 
+#if	FREEBSDV<500
 	case VT_MFS:
 
-#if	FREEBSDV<200
+# if	FREEBSDV<200
 	    m = (struct mfsnode *)v->v_data;
-#else	/* FREEBSDV>=200 */
+# else	/* FREEBSDV>=200 */
 	    if (!v->v_data
 	    ||  kread((KA_T)v->v_data, (char *)&mb, sizeof(mb))) {
-		(void) sprintf(Namech, "no mfs node: %#x", v->v_data);
+		(void) snpf(Namech, Namechl, "no mfs node: %s",
+		    print_kptr((KA_T)v->v_data, (char *)NULL, 0));
 		enter_nm(Namech);
 		return;
 	    }
 	    m = &mb;
-#endif	/* FREEBSDV<200 */
+# endif	/* FREEBSDV<200 */
+#endif	/* FREEBSDV<500 */
 
 	    break;
 	case VT_NFS:
@@ -288,7 +331,8 @@ process_node(va)
 #else	/* FREEBSDV>=200 */
 	    if (!v->v_data
 	    ||  kread((KA_T)v->v_data, (char *)&nb, sizeof(nb))) {
-		(void) sprintf(Namech, "no nfs node: %#x", v->v_data);
+		(void) snpf(Namech, Namechl, "no nfs node: %s",
+		    print_kptr((KA_T)v->v_data, (char *)NULL, 0));
 		enter_nm(Namech);
 		return;
 	    }
@@ -304,8 +348,8 @@ process_node(va)
 	    f = (struct fdescnode *)v->v_data;
 # else	/* FREEBSDV>=200 */
 	    if (kread((KA_T)v->v_data, (char *)&fb, sizeof(fb)) != 0) {
-		(void) sprintf(Namech, "can't read fdescnode at: %x",
-		    v->v_data);
+		(void) snpf(Namech, Namechl, "can't read fdescnode at: %s",
+		    print_kptr((KA_T)v->v_data, (char *)NULL, 0));
 		enter_nm(Namech);
 		return;
 	    }
@@ -322,7 +366,8 @@ process_node(va)
 # else	/* FREEBSDV>=200 */
 	    if (!v->v_data
 	    ||  kread((KA_T)v->v_data, (char *)&pb, sizeof(pb))) {
-		(void) sprintf(Namech, "no pfs node: %#x", v->v_data);
+		(void) snpf(Namech, Namechl, "no pfs node: %s",
+		    print_kptr((KA_T)v->v_data, (char *)NULL, 0));
 		enter_nm(Namech);
 		return;
 	    }
@@ -339,7 +384,8 @@ process_node(va)
 #else	/* FREEBSDV>=200 */
 	    if (!v->v_data
 	    ||  kread((KA_T)v->v_data, (char *)&ib, sizeof(ib))) {
-		(void) sprintf(Namech, "no ifs node: %#x", v->v_data);
+		(void) snpf(Namech, Namechl, "no ufs node: %s",
+		    print_kptr((KA_T)v->v_data, (char *)NULL, 0));
 		enter_nm(Namech);
 		return;
 	    }
@@ -390,7 +436,7 @@ process_node(va)
 	default:
 	    if (v->v_type == VBAD || v->v_type == VNON)
 		break;
-	    (void) sprintf(Namech, "unknown file system type: %d", v->v_tag);
+	    (void) snpf(Namech,Namechl,"unknown file system type: %d",v->v_tag);
 	    enter_nm(Namech);
 	    return;
 	}
@@ -401,42 +447,56 @@ process_node(va)
 	if (n) {
 	    dev = n->n_vattr.va_fsid;
 	    devs = 1;
+	    if ((type == VCHR) || (type == VBLK)) {
+		rdev = n->n_vattr.va_rdev;
+		rdevs = 1;
+	    }
 	} else if (i) {
-	    if (type == VCHR) {
-		dev = i->i_rdev ;
-		devs = 1;
-	    } else {
 
 #if	FREEBSDV>=400
-		if (i->i_dev
-		&&  !kread((KA_T)i->i_dev, (char *)&udev, sizeof(udev))) {
-		    dev = udev.si_udev;
-		    devs = 1;
-		}
-#else	/* FREEBSDV<400 */
-		dev = i->i_dev;
+	    if (i->i_dev
+	    &&  !kread((KA_T)i->i_dev, (char *)&si, sizeof(si))) {
+		dev = si.si_udev;
 		devs = 1;
+	    }
+#else	/* FREEBSDV<400 */
+	    dev = i->i_dev;
+	    devs = 1;
 #endif	/* FREEBSDV>=400 */
 
+	    if ((type == VCHR) || (type == VBLK)) {
+		rdev = i->i_rdev ;
+		rdevs = 1;
 	    }
 	}
 
-#if	defined(HASFDESCFS)
+#if	defined(HASFDESCFS) && (defined(HASFDLINK) || HASFDESCFS==1)
 	else if (f) {
+
+# if	defined(HASFDLINK)
 	    if (f->fd_link
-	    &&  kread((KA_T)f->fd_link, Namech, sizeof(Namech) - 1) == 0)
-		    Namech[sizeof(Namech) - 1] = '\0';
-	    else if (f->fd_type == Fctty) {
-		if (f_tty_s == 0)
-		    f_tty_s = lkup_dev_tty(&f_tty_dev, &f_tty_ino);
-		if (f_tty_s == 1) {
-		    dev = f_tty_dev;
-		    Lf->inode = f_tty_ino;
-		    devs = Lf->inp_ty = 1;
+	    &&  kread((KA_T)f->fd_link, Namech, Namechl - 1) == 0)
+		Namech[Namechl - 1] = '\0';
+
+#  if	HASFDESCFS==1
+	    else
+#  endif	/* HASFDESFS==1 */
+# endif	/* defined(HASFDLINK) */
+
+# if	HASFDESCFS==1
+		if (f->fd_type == Fctty) {
+		    if (f_tty_s == 0)
+			f_tty_s = lkup_dev_tty(&f_tty_dev, &f_tty_ino);
+		    if (f_tty_s == 1) {
+			dev = f_tty_dev;
+			Lf->inode = f_tty_ino;
+			devs = Lf->inp_ty = 1;
+		    }
 		}
-	    }
+# endif	/* HASFDESFS==1 */
+
 	}
-#endif	/* defined(HASFDESCFS) */
+#endif	defined(HASFDESCFS) && (defined(HASFDLINK) || HASFDESCFS==1)
 
 #if	defined(HAS9660FS)
 	else if (iso_stat && iso_dev_def) {
@@ -445,6 +505,23 @@ process_node(va)
 	}
 #endif	/* defined(HAS9660FS) */
 
+#if	FREEBSDV>=500
+	else if (d) {
+	    if (vfs) {
+		dev = vfs->fsid.val[0];
+		devs = 1;
+	    } else {
+		dev = DevDev;
+		devs = 1;
+	    }
+	    if ((type == VCHR) && v->v_rdev) {
+		if (!kread((KA_T)v->v_rdev, (char *)&si, sizeof(si))) {
+		    rdev = si.si_udev;
+		    rdevs = 1;
+		}
+	    }
+	}
+#endif	/* FREEBSDV>=500 */
 
 /*
  * Obtain the inode number.
@@ -475,6 +552,14 @@ process_node(va)
 # endif	/* FREEBSDV>=200 */
 #endif	/* defined(HASPROCFS) */
 
+#if	FREEBSDV>=500
+	else if (d) {
+	    Lf->inode = (unsigned long)d->de_inode;
+	    Lf->inp_ty = 1;
+	}
+#endif	/* FREEBSDV>=500 */
+
+
 /*
  * Obtain the file size.
  */
@@ -536,10 +621,14 @@ process_node(va)
 		    if (i) {
 			Lf->sz = (SZOFFTYPE)i->i_size;
 			Lf->sz_def = 1;
-		    } else if (m) {
+		    }
+
+#if	FREEBSDV<500
+		    else if (m) {
 			Lf->sz = (SZOFFTYPE)m->mfs_size;
 			Lf->sz_def = 1;
 		    }
+#endif	/* FREEBSDV<500 */
 
 #if	defined(HAS9660FS)
 		    else if (iso_stat) {
@@ -578,6 +667,14 @@ process_node(va)
 		}
 #endif	/* defined(HAS9660FS) */
 
+#if	FREEBSDV>=500
+		else if (d) {
+		    Lf->nlink = d->de_links;
+		    Lf->nlink_def = 1;
+		}
+#endif	/* FREEBSDV>=500 */
+
+
 		break;
 	    }
 	    if (Lf->nlink_def && Nlink && (Lf->nlink < Nlink))
@@ -596,35 +693,28 @@ process_node(va)
 	    Lf->fsdev = vfs->fsname;
 	}
 /*
+ * Save the device numbers and their states.
+ *
  * Format the vnode type, and possibly the device name.
  */
+	Lf->dev = dev;
+	Lf->dev_def = devs;
+	Lf->rdev = rdev;
+	Lf->rdev_def = rdevs;
 	switch (type) {
-
 	case VNON:
 	    ty ="VNON";
 	    break;
 	case VREG:
 	case VDIR:
 	    ty = (type == VREG) ? "VREG" : "VDIR";
-	    if (devs) {
-		Lf->dev_def = 1;
-		Lf->dev = dev;
-	    }
 	    break;
 	case VBLK:
 	    ty = "VBLK";
-	    if (devs) {
-		Lf->dev_def = 1;
-		Lf->dev = dev;
-	    }
 	    Ntype = N_BLK;
 	    break;
 	case VCHR:
 	    ty = "VCHR";
-	    if (devs) {
-		Lf->dev = dev;
-		Lf->dev_def = 1;
-	    }
 	    Ntype = N_CHR;
 	    break;
 	case VLNK:
@@ -641,22 +731,18 @@ process_node(va)
 	    ty = "VBAD";
 	    break;
 	case VFIFO:
-	    if (devs) {
-		Lf->dev = dev;
-		Lf->dev_def = 1;
-	    }
 	    ty = "FIFO";
 	    break;
 	default:
 	    if (type > 9999)
-		(void) sprintf(Lf->type, "*%03d", type % 1000);
+		(void) snpf(Lf->type, sizeof(Lf->type), "*%03d", type % 1000);
 	    else
-		(void) sprintf(Lf->type, "%4d", type);
-	    (void) strcpy(Namech, "unknown type");
+		(void) snpf(Lf->type, sizeof(Lf->type), "%4d", type);
+	    (void) snpf(Namech, Namechl, "unknown type");
 	    ty = (char *)NULL;
 	}
 	if (ty)
-	    (void) strcpy(Lf->type, ty);
+	    (void) snpf(Lf->type, sizeof(Lf->type), "%s", ty);
 	Lf->ntype = Ntype;
 /*
  * Handle some special cases:
@@ -667,83 +753,98 @@ process_node(va)
  */
 
 	if (type == VBAD)
-	    (void) strcpy(Namech, "(revoked)");
+	    (void) snpf(Namech, Namechl, "(revoked)");
+
+#if	FREEBSDV<500
 	else if (m) {
-	    Lf->dev_def = 0;
-	    (void) sprintf(Namech, "%#x", m->mfs_baseoff);
-	    (void) strcpy(dev_ch, "    memory");
+	    Lf->dev_def = Lf->rdev_def = 0;
+	    (void) snpf(Namech, Namechl, "%#x", m->mfs_baseoff);
+	    (void) snpf(dev_ch, sizeof(dev_ch), "    memory");
 	    enter_dev_ch(dev_ch);
 	}
+#endif	/* FREEBSDV<500 */
+
 
 #if	defined(HASPROCFS)
 	else if (p) {
-	    Lf->dev_def = 0;
+	    Lf->dev_def = Lf->rdev_def = 0;
 
 # if	FREEBSDV<200
 	    if (type == VDIR)
-		(void) sprintf(Namech, "/%s", HASPROCFS);
+		(void) snpf(Namech, Namechl, "/%s", HASPROCFS);
 	    else
-		(void) sprintf(Namech, "/%s/%0*d", HASPROCFS, PNSIZ,
+		(void) snpf(Namech, Namechl, "/%s/%0*d", HASPROCFS, PNSIZ,
 		    p->pfs_pid);
 	    enter_nm(Namech);
 # else	/* FREEBSDV>=200 */
 	    ty = (char *)NULL;
-	    (void) sprintf(Namech, "/%s", HASPROCFS);
+	    (void) snpf(Namech, Namechl, "/%s", HASPROCFS);
 	    switch (p->pfs_type) {
 	    case Proot:
 		ty = "PDIR";
 		break;
 	    case Pproc:
-		(void) sprintf(endnm(), "/%d", p->pfs_pid);
+		ep = endnm(&sz);
+		(void) snpf(ep, sz, "/%d", p->pfs_pid);
 		ty = "PDIR";
 		break;
 	    case Pfile:
-		(void) sprintf(endnm(), "/%d/file", p->pfs_pid);
+		ep = endnm(&sz);
+		(void) snpf(ep, sz, "/%d/file", p->pfs_pid);
 		ty = "PFIL";
 		break;
 	    case Pmem:
-		(void) sprintf(endnm(), "/%d/mem", p->pfs_pid);
+		ep = endnm(&sz);
+		(void) snpf(ep, sz, "/%d/mem", p->pfs_pid);
 		ty = "PMEM";
 		break;
 	    case Pregs:
-		(void) sprintf(endnm(), "/%d/regs", p->pfs_pid);
+		ep = endnm(&sz);
+		(void) snpf(ep, sz, "/%d/regs", p->pfs_pid);
 		ty = "PREG";
 		break;
 	    case Pfpregs:
-		(void) sprintf(endnm(), "/%d/fpregs", p->pfs_pid);
+		ep = endnm(&sz);
+		(void) snpf(ep, sz, "/%d/fpregs", p->pfs_pid);
 		ty = "PFPR";
 		break;
 	    case Pctl:
-		(void) sprintf(endnm(), "/%d/ctl", p->pfs_pid);
+		ep = endnm(&sz);
+		(void) snpf(ep, sz, "/%d/ctl", p->pfs_pid);
 		ty = "PCTL";
 		break;
 	    case Pstatus:
-		(void) sprintf(endnm(), "/%d/status", p->pfs_pid);
+		ep = endnm(&sz);
+		(void) snpf(ep, sz, "/%d/status", p->pfs_pid);
 		ty = "PSTA";
 		break;
 	    case Pnote:
-		(void) sprintf(endnm(), "/%d/note", p->pfs_pid);
+		ep = endnm(&sz);
+		(void) snpf(ep, sz, "/%d/note", p->pfs_pid);
 		ty = "PNTF";
 		break;
 	    case Pnotepg:
-		(void) sprintf(endnm(), "/%d/notepg", p->pfs_pid);
-		ty = "PGRP";
+		ep = endnm(&sz);
+		(void) snpf(ep, sz, "/%d/notepg", p->pfs_pid);
+		ty = "PGID";
 		break;
 
 #  if	FREEBSDV>=300
 	    case Pmap:
-		(void) sprintf(endnm(), "/%d/map", p->pfs_pid);
+		ep = endnm(&sz);
+		(void) snpf(ep, sz, "/%d/map", p->pfs_pid);
 		ty = "PMAP";
 		break;
 	    case Ptype:
-		(void) sprintf(endnm(), "/%d/etype", p->pfs_pid);
+		ep = endnm(&sz);
+		(void) snpf(ep, sz, "/%d/etype", p->pfs_pid);
 		ty = "PETY";
 		break;
 #  endif	/* FREEBSDV>=300 */
 
 	    }
 	    if (ty)
-		(void) strcpy(Lf->type, ty);
+		(void) snpf(Lf->type, sizeof(Lf->type), "%s", ty);
 	    enter_nm(Namech);
 
 # endif	/* FREEBSDV<200 */
@@ -755,7 +856,7 @@ process_node(va)
  * If this is a VBLK file and it's missing an inode number, try to
  * supply one.
  */
-	if (Lf->inp_ty == 0 && type == VBLK && Lf->dev_def)
+	if ((Lf->inp_ty == 0) && (type == VBLK))
 	    find_bl_ino();
 #endif	/* defined(HASBLKDEV) */
 
@@ -763,7 +864,7 @@ process_node(va)
  * If this is a VCHR file and it's missing an inode number, try to
  * supply one.
  */
-	if (Lf->inp_ty == 0 && type == VCHR && Lf->dev_def)
+	if ((Lf->inp_ty == 0) && (type == VCHR))
 	    find_ch_ino();
 /*
  * Test for specified file.
@@ -786,10 +887,8 @@ process_node(va)
 
 		    ) {
 			pfi->f = 1;
-			if (!Namech[0]) {
-			    (void) strncpy(Namech, pfi->nm, MAXPATHLEN - 1);
-			    Namech[MAXPATHLEN - 1] = '\0';
-			}
+			if (!Namech[0])
+			    (void) snpf(Namech, Namechl, "%s", pfi->nm);
 			Lf->sf |= SELNM;
 			break;
 		    }
@@ -799,7 +898,9 @@ process_node(va)
 #endif	/* defined(HASPROCFS) */
 
 	{
-	    if (Sfile && is_file_named((char *)NULL, (type == VCHR) ? 1 : 0))
+	    if (Sfile && is_file_named((char *)NULL,
+				       ((type == VCHR) || (type == VBLK)) ? 1
+									  : 0))
 		Lf->sf |= SELNM;
 	}
 /*
@@ -819,17 +920,19 @@ void
 process_pipe(pa)
 	KA_T pa;			/* pipe structure address */
 {
-	char dev_ch[32];
+	char dev_ch[32], *ep;
 	struct pipe p;
+	size_t sz;
 
 	if (!pa || kread(pa, (char *)&p, sizeof(p))) {
-	    (void) sprintf(Namech,
-		"can't read DTYPE_PIPE pipe struct: %#x", pa);
+	    (void) snpf(Namech, Namechl,
+		"can't read DTYPE_PIPE pipe struct: %s",
+		print_kptr((KA_T)pa, (char *)NULL, 0));
 	    enter_nm(Namech);
 	    return;
 	}
-	(void) strcpy(Lf->type, "PIPE");
-	(void) sprintf(dev_ch, "%#x", pa);
+	(void) snpf(Lf->type, sizeof(Lf->type), "PIPE");
+	(void) snpf(dev_ch, sizeof(dev_ch), "%#x", pa);
 	enter_dev_ch(dev_ch);
 	if (Foffset)
 	    Lf->off_def = 1;
@@ -838,15 +941,22 @@ process_pipe(pa)
 	    Lf->sz_def = 1;
 	}
 	if (p.pipe_peer)
-	    (void) sprintf(Namech, "->%#x", p.pipe_peer);
+	    (void) snpf(Namech, Namechl, "->%s",
+		print_kptr((KA_T)p.pipe_peer, (char *)NULL, 0));
 	else
 	    Namech[0] = '\0';
-	if (p.pipe_buffer.cnt)
-	    (void) sprintf(endnm(), "%, cnt=%d", p.pipe_buffer.cnt);
-	if (p.pipe_buffer.in)
-	    (void) sprintf(endnm(), "%, in=%d", p.pipe_buffer.in);
-	if (p.pipe_buffer.out)
-	    (void) sprintf(endnm(), "%, out=%d", p.pipe_buffer.out);
+	if (p.pipe_buffer.cnt) {
+	    ep = endnm(&sz);
+	    (void) snpf(ep, sz, ", cnt=%d", p.pipe_buffer.cnt);
+	}
+	if (p.pipe_buffer.in) {
+	    ep = endnm(&sz);
+	    (void) snpf(ep, sz, ", in=%d", p.pipe_buffer.in);
+	}
+	if (p.pipe_buffer.out) {
+	    ep = endnm(&sz);
+	    (void) snpf(ep, sz, ", out=%d", p.pipe_buffer.out);
+	}
 /*
  * Enter name characters.
  */
