@@ -32,13 +32,25 @@
 #ifndef lint
 static char copyright[] =
 "@(#) Copyright 1994 Purdue Research Foundation.\nAll rights reserved.\n";
-static char *rcsid = "$Id: dsock.c,v 1.10 99/11/28 06:40:31 abe Exp $";
+static char *rcsid = "$Id: dsock.c,v 1.15 2001/01/02 12:34:19 abe Exp $";
 #endif
 
 
 #include "lsof.h"
 
 
+#if	defined(HASIPv6)
+
+/*
+ * IPv6_2_IPv4()  -- macro to define the address of an IPv4 address contained
+ *		     in an IPv6 address
+ */
+
+#define IPv6_2_IPv4(v6)	(((uint8_t *)((struct in6_addr *)v6)->s6_addr)+12)
+#endif	/* defined(HASIPv6) */
+
+
+
 /*
  * process_socket() - process socket
  */
@@ -57,11 +69,16 @@ process_socket(sa)
 	struct protosw p;
 	struct socket s;
 	struct tcpcb t;
+	KA_T ta = (KA_T)NULL;
 	struct unpcb uc, unp;
 	struct sockaddr_un *ua = NULL;
 	struct sockaddr_un un;
 
-	(void) strcpy(Lf->type, "sock");
+#if	defined(HASIPv6) && !defined(HASINRIAIPv6)
+	struct in6pcb in6p;
+#endif	/* defined(HASIPv6) && !defined(HASINRIAIPv6) */
+
+	(void) snpf(Lf->type, sizeof(Lf->type), "sock");
 	Lf->inp_ty = 2;
 /*
  * Read the socket, protocol, and domain structures.
@@ -71,8 +88,8 @@ process_socket(sa)
 	    return;
 	}
 	if (kread(sa, (char *) &s, sizeof(s))) {
-	    (void) sprintf(Namech, "can't read socket struct from %s",
-		print_kptr(sa, (char *)NULL));
+	    (void) snpf(Namech, Namechl, "can't read socket struct from %s",
+		print_kptr(sa, (char *)NULL, 0));
 	    enter_nm(Namech);
 	    return;
 	}
@@ -82,15 +99,15 @@ process_socket(sa)
 	}
 	if (!s.so_proto
 	||  kread((KA_T)s.so_proto, (char *)&p, sizeof(p))) {
-	    (void) sprintf(Namech, "can't read protocol switch from %s",
-		print_kptr((KA_T)s.so_proto, (char *)NULL));
+	    (void) snpf(Namech, Namechl, "can't read protocol switch from %s",
+		print_kptr((KA_T)s.so_proto, (char *)NULL, 0));
 	    enter_nm(Namech);
 	    return;
 	}
 	if (!p.pr_domain
 	||  kread((KA_T)p.pr_domain, (char *)&d, sizeof(d))) {
-	    (void) sprintf(Namech, "can't read domain struct from %s",
-		print_kptr((KA_T)p.pr_domain, (char *)NULL));
+	    (void) snpf(Namech, Namechl, "can't read domain struct from %s",
+		print_kptr((KA_T)p.pr_domain, (char *)NULL, 0));
 	    enter_nm(Namech);
 	    return;
 	}
@@ -132,70 +149,116 @@ process_socket(sa)
 	    printiproto(p.pr_protocol);
 
 #if	defined(HASIPv6)
-	    (void) strcpy(Lf->type, (fam == AF_INET) ? "IPv4" : "IPv6");
+	    (void) snpf(Lf->type, sizeof(Lf->type),
+		(fam == AF_INET) ? "IPv4" : "IPv6");
 #else	/* !defined(HASIPv6) */
-	    (void) strcpy(Lf->type, "inet");
+	    (void) snpf(Lf->type, sizeof(Lf->type), "inet");
+#endif	/* defined(HASIPv6) */
+
+#if	defined(HASIPv6) && !defined(HASINRIAIPv6)
+	    if (fam == AF_INET6) {
+
+	    /*
+	     * Read IPv6 protocol control block.
+	     */
+		if (!s.so_pcb
+		||  kread((KA_T)s.so_pcb, (char *)&in6p, sizeof(in6p))) {
+		    (void) snpf(Namech, Namechl, "can't read in6pcb at %s",
+			print_kptr((KA_T)s.so_pcb, (char *)NULL, 0));
+		    enter_nm(Namech);
+		    return;
+		}
+	    /*
+	     * Save IPv6 address information.
+	     */
+		enter_dev_ch(print_kptr((KA_T)(in6p.in6p_ppcb ? in6p.in6p_ppcb
+							      : s.so_pcb),
+					       (char *)NULL, 0));
+		if (p.pr_protocol == IPPROTO_TCP)
+		    ta = (KA_T)in6p.in6p_ppcb;
+	        la = (unsigned char *)&in6p.in6p_laddr;
+	        lp = (int)ntohs(in6p.in6p_lport);
+		if (!IN6_IS_ADDR_UNSPECIFIED(&in6p.in6p_faddr)
+		||  in6p.in6p_fport)
+		{
+		    fa = (unsigned char *)&in6p.in6p_faddr;
+		    fp = (int)ntohs(in6p.in6p_fport);
+		}
+	    } else
+#endif	/* defined(HASIPv6) && !defined(HASINRIAIPv6) */
+
+	    {
+
+	    /*
+	     * Read Ipv4 or IPv6 (INRIA) protocol control block.
+	     */
+		if (!s.so_pcb
+		||  kread((KA_T) s.so_pcb, (char *) &inp, sizeof(inp))) {
+		    if (!s.so_pcb) {
+			(void) snpf(Namech, Namechl, "no PCB%s%s",
+			    (s.so_state & SS_CANTSENDMORE) ? ", CANTSENDMORE"
+							   : "",
+			    (s.so_state & SS_CANTRCVMORE) ? ", CANTRCVMORE"
+							  : "");
+		    } else {
+			(void) snpf(Namech, Namechl, "can't read inpcb at %s",
+			    print_kptr((KA_T)s.so_pcb, (char *)NULL, 0));
+		    }
+		    enter_nm(Namech);
+		    return;
+		}
+		enter_dev_ch(print_kptr((KA_T)(inp.inp_ppcb ? inp.inp_ppcb
+							    : s.so_pcb),
+					       (char *)NULL, 0));
+		if (p.pr_protocol == IPPROTO_TCP)
+		    ta = (KA_T)inp.inp_ppcb;
+		lp = (int)ntohs(inp.inp_lport);
+		if (fam == AF_INET) {
+
+		/*
+		 * Save IPv4 address information.
+		 */
+		    la = (unsigned char *)&inp.inp_laddr;
+		    if (inp.inp_faddr.s_addr != INADDR_ANY || inp.inp_fport) {
+			fa = (unsigned char *)&inp.inp_faddr;
+			fp = (int)ntohs(inp.inp_fport);
+		    }
+		}
+		
+#if	defined(HASIPv6) && defined(HASINRIAIPv6)
+		else {
+		    la = (unsigned char *)&inp.inp_laddr6;
+		    if (!IN6_IS_ADDR_UNSPECIFIED(&inp.inp_faddr6)
+		    ||  inp.inp_fport)
+		    {
+			fa = (unsigned char *)&inp.inp_faddr6;
+			fp = (int)ntohs(inp.inp_fport);
+		    }
+		}
+#endif	/* defined(HASIPv6) && defined(HASINRIAIPv6) */
+
+	    }
+
+
+#if	defined(HASIPv6)
+	    if ((fam == AF_INET6)
+	    &&  ((la && IN6_IS_ADDR_V4MAPPED((struct in6_addr *)la))
+	    ||  ((fa && IN6_IS_ADDR_V4MAPPED((struct in6_addr *)fa))))) {
+
+	    /*
+	     * Adjust for IPv4 addresses mapped in IPv6 addresses.
+	     */
+		if (la)
+		    la = (unsigned char *)IPv6_2_IPv4(la);
+		if (fa)
+		    fa = (unsigned char *)IPv6_2_IPv4(fa);
+		fam = AF_INET;
+	    }
 #endif	/* defined(HASIPv6) */
 
 	/*
-	 * Read protocol control block.
-	 */
-	    if (!s.so_pcb
-	    ||  kread((KA_T) s.so_pcb, (char *) &inp, sizeof(inp))) {
-		(void) sprintf(Namech, "can't read inpcb at %s",
-		    print_kptr((KA_T)s.so_pcb, (char *)NULL));
-		enter_nm(Namech);
-		return;
-	    }
-	/*
-	 * Print Internet socket information.
-	 */
-	    enter_dev_ch(print_kptr((KA_T)(inp.inp_ppcb ? inp.inp_ppcb
-							: s.so_pcb),
-				    (char *)NULL));
-
-#if	defined(HASIPv6) && defined(HASINRIAIPv6)
-	    if (inp.inp_latype == IPATYPE_UNBD)
-		la = (unsigned char *)&in6addr_any;
-	    else if (inp.inp_latype == IPATYPE_IPV6) {
-		fam = AF_INET6;
-		Lf->type[3] = '6';
-		la = (unsigned char *)&inp.inp_laddr6;
-	    } else {
-		if (inp.inp_latype == IPATYPE_IPV4) {
-		    fam = AF_INET;
-		    Lf->type[3] = '4';
-		}
-		la = (unsigned char *)&inp.inp_laddr;
-	    }
-#else	/* !defined(HASIPv6) || !defined(HASINRIAIPv6) */
-	    la = (unsigned char *)&inp.inp_laddr;
-#endif	/* defined(HASIPv6) && defined(HASINRIAIPv6) */
-
-	    lp = (int)ntohs(inp.inp_lport);
-
-#if	defined(HASIPv6) && defined(HASINRIAIPv6)
-	    if (inp.inp_fatype != IPATYPE_UNBD || inp.inp_fport) {
-		if (inp.inp_fatype == IPATYPE_IPV6) {
-		    fa = (unsigned char *)&inp.inp_faddr6;
-		    fam = AF_INET6;
-		    Lf->type[3] = '6';
-		} else {
-		    fa = (unsigned char *)&inp.inp_faddr;
-		    if (inp.inp_fatype == IPATYPE_IPV4) {
-			fam = AF_INET;
-			Lf->type[3] = '4';
-		    }
-		}
-		fp = (int)ntohs(inp.inp_fport);
-	    }
-#else	/* !defined(HASIPv6) || !defined(HASINRIAIPv6) */
-	    if (inp.inp_faddr.s_addr != INADDR_ANY || inp.inp_fport) {
-		fa = (unsigned char *)&inp.inp_faddr;
-		fp = (int)ntohs(inp.inp_fport);
-	    }
-#endif	/* defined(HASIPv6) && defined(HASINRIAIPv6) */
-
+ 	 * Enter local and remote addresses by address family.
+ 	 */
 	    if (fa || la)
 		(void) ent_inaddr(la, lp, fa, fp, fam);
 	    if (p.pr_protocol == IPPROTO_TCP && inp.inp_ppcb
@@ -208,11 +271,11 @@ process_socket(sa)
  * Process a ROUTE domain socket.
  */
 	case AF_ROUTE:
-	    (void) strcpy(Lf->type, "rte");
+	    (void) snpf(Lf->type, sizeof(Lf->type), "rte");
 	    if (s.so_pcb)
-		enter_dev_ch(print_kptr((KA_T)(s.so_pcb), (char *)NULL));
+		enter_dev_ch(print_kptr((KA_T)(s.so_pcb), (char *)NULL, 0));
 	    else
-		(void) strcpy(Namech, "no protocol control block");
+		(void) snpf(Namech, Namechl, "no protocol control block");
 	    if (!Fsize)
 		Lf->off_def = 1;
 	    break;
@@ -222,26 +285,26 @@ process_socket(sa)
 	case AF_UNIX:
 	    if (Funix)
 		Lf->sf |= SELUNX;
-	    (void) strcpy(Lf->type, "unix");
+	    (void) snpf(Lf->type, sizeof(Lf->type), "unix");
 	/*
 	 * Read Unix protocol control block and the Unix address structure.
 	 */
 
-	    enter_dev_ch(print_kptr(sa, (char *)NULL));
+	    enter_dev_ch(print_kptr(sa, (char *)NULL, 0));
 	    if (kread((KA_T) s.so_pcb, (char *) &unp, sizeof(unp))) {
-		(void) sprintf(Namech, "can't read unpcb at %s",
-		    print_kptr((KA_T)s.so_pcb, (char *)NULL));
+		(void) snpf(Namech, Namechl, "can't read unpcb at %s",
+		    print_kptr((KA_T)s.so_pcb, (char *)NULL, 0));
 		break;
 	    }
 	    if ((struct socket *)sa != unp.unp_socket) {
-		(void) sprintf(Namech, "unp_socket (%s) mismatch",
-		    print_kptr((KA_T)unp.unp_socket, (char *)NULL));
+		(void) snpf(Namech, Namechl, "unp_socket (%s) mismatch",
+		    print_kptr((KA_T)unp.unp_socket, (char *)NULL, 0));
 		break;
 	    }
 	    if (unp.unp_addr) {
 		if (kread((KA_T) unp.unp_addr, (char *) &mb, sizeof(mb))) {
-		    (void) sprintf(Namech, "can't read unp_addr at %s",
-			print_kptr((KA_T)unp.unp_addr, (char *)NULL));
+		    (void) snpf(Namech, Namechl, "can't read unp_addr at %s",
+			print_kptr((KA_T)unp.unp_addr, (char *)NULL, 0));
 		    break;
 		}
 		if (mb.m_hdr.mh_len == sizeof(struct sockaddr_un))
@@ -262,15 +325,16 @@ process_socket(sa)
 		if (ua->sun_family == AF_UNSPEC) {
 		    if (unp.unp_conn) {
 			if (kread((KA_T)unp.unp_conn, (char *)&uc, sizeof(uc)))
-			    (void) sprintf(Namech, "can't read unp_conn at %s",
-				print_kptr((KA_T)unp.unp_conn, (char *)NULL));
+			    (void) snpf(Namech, Namechl,
+				"can't read unp_conn at %s",
+				print_kptr((KA_T)unp.unp_conn,(char *)NULL,0));
 			else
-			    (void) sprintf(Namech, "->%s",
-				print_kptr((KA_T)uc.unp_socket, (char *)NULL));
+			    (void) snpf(Namech, Namechl, "->%s",
+				print_kptr((KA_T)uc.unp_socket,(char *)NULL,0));
 		    } else
-			(void) strcpy(Namech, "->(none)");
+			(void) snpf(Namech, Namechl, "->(none)");
 		} else
-		    (void) sprintf(Namech, "unknown sun_family (%d)",
+		    (void) snpf(Namech, Namechl, "unknown sun_family (%d)",
 			ua->sun_family);
 		break;
 	    }
@@ -281,9 +345,9 @@ process_socket(sa)
 		if (Sfile && is_file_named(ua->sun_path, 0))
 		    Lf->sf |= SELNM;
 		if (!Namech[0])
-		    (void) strcpy(Namech, ua->sun_path);
+		    (void) snpf(Namech, Namechl, "%s", ua->sun_path);
 	    } else
-		(void) strcpy(Namech, "no address");
+		(void) snpf(Namech, Namechl, "no address");
 	    break;
 	default:
 	    printunkaf(fam, 1);
