@@ -31,7 +31,7 @@
 
 
 /*
- * $Id: machine.h,v 1.20 10/.0/.1 .0:.0:.0 abe Exp $
+ * $Id: machine.h,v 1.27 2001/06/06 17:59:14 abe Exp $
  */
 
 
@@ -213,14 +213,21 @@
 
 /*
  * HASNCACHE is defined for those dialects that have a kernel name cache
- * that lsof can search.
+ * that lsof can search.  A value of 1 directs printname() to prefix the
+ * cache value with the file system directory name; 2, avoid the prefix.
  *
  * HASNCAPID is defined for those dialects with a searchable kernel name
  * cache whose cache and vnodes are linked by a capability ID.
+ *
+ * NCACHELDPFX is a set of C commands to execute before calling ncache_load().
+ *
+ * NCACHELDSFX is a set of C commands to execute after calling ncache_load().
  */
 
 #define	HASNCACHE	1
 #define	HASNCAPID	1
+/* #define	NCACHELDPFX	??? */
+/* #define	NCACHELDSFX	??? */
 
 
 /*
@@ -309,6 +316,15 @@
 /* #define	HASPRIVNMCACHE	<function name>	*/
 
 
+/*
+ * HASPRIVPRIPP is defined for dialects that have a private function for
+ * printing IP protocol names.  When HASPRIVPRIPP isn't defined, the
+ * IP protocol name printing function defaults to printiprto().
+ */
+
+/* #define	HASPRIVPRIPP	1	*/
+
+
 /*
  * HASPROCFS is defined for those dialects that have a proc file system --
  * usually /proc and usually in SYSV4 derivatives.  For FreeBSD 2.0 and
@@ -316,8 +332,14 @@
  * /usr/src/sys/miscfs/procfs/procfs.h; below 2.0, Configure always defines
  * HASPROCFS.
  *
- * HASFSTYPE is defined for those systems that have a file system type
- * string, st_fstype, in the stat() buffer.
+ * HASFSTYPE is defined as 1 for those systems that have a file system type
+ * string, st_fstype, in the stat() buffer; 2, for those systems that have a
+ * file system type integer in the stat() buffer, named MOUNTS_STAT_FSTYPE;
+ * 0, for systems whose stat(2) structure has no file system type member.  The
+ * additional symbols MOUNTS_FSTYPE, RMNT_FSTYPE, and RMNT_STAT_FSTYPE may be
+ * defined in dlsof.h to direct how the readmnt() function in lib/rmnt.c
+ * preserves these stat(2) and getmntent(3) buffer values in the local mounts
+ * structure.
  *
  * The defined value is the string that names the file system type.
  *
@@ -360,6 +382,14 @@
 /* #define	HASSECURITY	1 */
 
 
+/*
+ * HASSETLOCALE is defined for those dialects that have <locale.h> and
+ * setlocale().
+ */
+
+#define	HASSETLOCALE	1
+
+
 /*
  * HASSNODE is defined for those dialects that have snodes.
  */
@@ -367,6 +397,24 @@
 /* #define	HASSNODE	1 */
 
 
+/*
+ * Define HASSPECDEVD to be the name of a function that handles the results
+ * of a successful stat(2) of a file name argument.
+ *
+ * For example, HASSPECDEVD() for Darwin makes sure that st_dev is set to
+ * what stat("/dev") returns -- i.e., what's in DevDev.
+ *
+ * The function takes two arguments:
+ *
+ *	1: pointer to the full path name of file
+ *	2: pointer to the stat(2) result
+ *
+ * The function returns void.
+ */
+
+/* #define	HASSPECDEVD	process_dev_stat */
+
+
 /*
  * HASSTREAMS is defined for those dialects that support streams.
  */
@@ -444,6 +492,7 @@
 #define	USE_LIB_PRINT_TCPTPI			1	/* ptti.c */
 #define	USE_LIB_READDEV				1	/* rdev.c */
 /* #define	USE_LIB_READMNT			1	   rmnt.c */
+/* #define	USE_LIB_REGEX			1	   regex.c */
 
 # if	FREEBSDV<210
 #define	USE_LIB_RNAM				1	/* rnam.c */
@@ -452,6 +501,8 @@
 # endif	/* FREEBSDV<210 */
 
 /* #define	USE_LIB_RNCH			1	   rnch.c */
+/* #define	USE_LIB_SNPF			1	   snpf.c */
+#define	snpf	snprintf	   /* use the system's snprintf() */
 
 
 /*
