@@ -32,7 +32,7 @@
 #ifndef lint
 static char copyright[] =
 "@(#) Copyright 1994 Purdue Research Foundation.\nAll rights reserved.\n";
-static char *rcsid = "$Id: dnode.c,v 1.12 99/11/28 06:46:07 abe Exp $";
+static char *rcsid = "$Id: dnode.c,v 1.16 2001/06/19 13:29:51 abe Exp $";
 #endif
 
 #if	defined(HPUXKERNBITS) && HPUXKERNBITS>=64
@@ -52,6 +52,7 @@ _PROTOTYPE(static int islocked,(KA_T lp));
 
 _PROTOTYPE(static int getnodety,(struct vnode *v));
 _PROTOTYPE(static int readinode,(KA_T ia, struct inode *i));
+_PROTOTYPE(static int read_nmn,(KA_T na, KA_T ia, struct mvfsnode *m));
 
 
 #if	HPUXV>=900
@@ -158,6 +159,7 @@ getnodety(v)
 	static KA_T avops;
 	static KA_T cvops;
 	static KA_T fvops;
+	static KA_T mvops;
 	static KA_T nvops;
 	static KA_T nvops3;
 	static KA_T nv3ops;
@@ -175,6 +177,8 @@ getnodety(v)
 		cvops = (unsigned long)0;
 	    if (get_Nl_value("fvops", Drive_Nl, &fvops) < 0)
 		fvops = (unsigned long)0;
+	    if (get_Nl_value("mvops", Drive_Nl, &mvops) < 0)
+		mvops = (unsigned long)0;
 	    if (get_Nl_value("nvops", Drive_Nl, &nvops) < 0)
 		nvops = (unsigned long)0;
 	    if (get_Nl_value("nvops3", Drive_Nl, &nvops3) < 0)
@@ -202,6 +206,8 @@ getnodety(v)
 	    return(N_NFS);
 	else if (nv3ops && nv3ops == (unsigned long)v->v_op)
 	    return(N_NFS);
+	else if (mvops && mvops == (unsigned long)v->v_op)
+	    return(N_MVFS);
 
 #if	defined(HASVXFS)
 	else if (vvops && vvops == (unsigned long)v->v_op)
@@ -233,9 +239,9 @@ getnodety(v)
 	    else {
 
 unknown_v_op:
-		(void) sprintf(Namech,
+		(void) snpf(Namech, Namechl,
 		    "unknown file system type; v_op: %s",
-		    print_kptr((KA_T)v->v_op, (char *)NULL));
+		    print_kptr((KA_T)v->v_op, (char *)NULL, 0));
 		enter_nm(Namech);
 		return(-1);
 	    }
@@ -262,9 +268,9 @@ unknown_v_op:
 	}
 #else	/* !defined(HAS_AFS) */
 	else {
-	    (void) sprintf(Namech,
+	    (void) snpf(Namech, Namechl,
 		"unknown file system type; v_op: %s",
-		print_kptr((KA_T)v->v_op, (char *)NULL));
+		print_kptr((KA_T)v->v_op, (char *)NULL, 0));
 	    enter_nm(Namech);
 	    return(-1);
 	}
@@ -287,11 +293,13 @@ process_node(va)
 	struct afsnode an;
 #endif	/* defined(HAS_AFS) */
 
-	dev_t dev;
-	int devs = 1;
+	dev_t dev, rdev;
+	int devs = 0;
 	struct inode i;
 	int ins = 0;
+	struct mvfsnode m;
 	struct rnode r;
+	int rdevs = 0;
 	int rns = 0;
 	char tbuf[32], *ty;
 	enum vtype type;
@@ -392,9 +400,9 @@ process_node(va)
 	if (!v->v_vfsp)
 	    vfs = (struct l_vfs *)NULL;
 	else if (!(vfs = readvfs(v))) {
-	    (void) sprintf(Namech, "can't read vfs for %s at %s",
-		print_kptr(va, tbuf),
-		print_kptr((KA_T)v->v_vfsp, (char *)NULL));
+	    (void) snpf(Namech, Namechl, "can't read vfs for %s at %s",
+		print_kptr(va, tbuf, sizeof(tbuf)),
+		print_kptr((KA_T)v->v_vfsp, (char *)NULL, 0));
 	    enter_nm(Namech);
 	    return;
 	}
@@ -412,11 +420,11 @@ process_node(va)
 
 #if	defined(HASVXFS)
 	case N_VXFS:
-	    if (!v->v_data || read_vxnode(v, vfs, &dev)) {
-		(void) sprintf(Namech,
+	    if (!v->v_data || read_vxnode(v, vfs, &dev, &devs, &rdev, &rdevs)) {
+		(void) snpf(Namech, Namechl,
 		    "vnode at %s: can't read vx_inode (%s)",
-		    print_kptr(va, tbuf),
-		    print_kptr((KA_T)v->v_data, (char *)NULL));
+		    print_kptr(va, tbuf, sizeof(tbuf)),
+		    print_kptr((KA_T)v->v_data, (char *)NULL, 0));
 		enter_nm(Namech);
 		return;
 	    }
@@ -427,10 +435,10 @@ process_node(va)
 	case N_CDFS:
 	    if (!v->v_data
 	    ||  kread((KA_T)v->v_data, (char *)&c, sizeof(c))) {
-		(void) sprintf(Namech,
+		(void) snpf(Namech, Namechl,
 		    "vnode at %s: can't read cdnode (%s)",
-		    print_kptr(va, tbuf),
-		    print_kptr((KA_T)v->v_data, (char *)NULL));
+		    print_kptr(va, tbuf, sizeof(tbuf)),
+		    print_kptr((KA_T)v->v_data, (char *)NULL, 0));
 		enter_nm(Namech);
 		return;
 	    }
@@ -439,10 +447,10 @@ process_node(va)
 	case N_PIPE:
 	    if (!v->v_data
 	    ||  kread((KA_T)v->v_data, (char *)&f, sizeof(f))) {
-		(void) sprintf(Namech,
+		(void) snpf(Namech, Namechl,
 		    "vnode at %s: can't read fifonode (%s)",
-		    print_kptr(va, tbuf),
-		    print_kptr((KA_T)v->v_data, (char *)NULL));
+		    print_kptr(va, tbuf, sizeof(tbuf)),
+		    print_kptr((KA_T)v->v_data, (char *)NULL, 0));
 		enter_nm(Namech);
 		return;
 	    }
@@ -453,12 +461,16 @@ process_node(va)
 	    break;
 #endif	/* HPUXV>=1000 */
 
+	case N_MVFS:
+	    if (read_nmn(va, (KA_T)v->v_data, &m))
+		return;
+	    break;
 	case N_NFS:
 	    if (!v->v_data || readrnode((KA_T)v->v_data, &r)) {
-		(void) sprintf(Namech,
+		(void) snpf(Namech, Namechl,
 		    "vnode at %s: can't read rnode (%s)",
-		    print_kptr(va, tbuf), 
-		    print_kptr((KA_T)v->v_data, (char *)NULL));
+		    print_kptr(va, tbuf, sizeof(tbuf)), 
+		    print_kptr((KA_T)v->v_data, (char *)NULL, 0));
 		enter_nm(Namech);
 		return;
 	    }
@@ -469,30 +481,31 @@ process_node(va)
 	case N_SPEC:
 	    if (v->v_type == VCHR) {
 		if (!v->v_data || readsnode((KA_T)v->v_data, &s)) {
-		    (void) sprintf(Namech,
+		    (void) snpf(Namech, Namechl,
 			"vnode at %s: can't read snode(%s)",
-			print_kptr(va, tbuf),
-			print_kptr((KA_T)v->v_data, (char *)NULL));
+			print_kptr(va, tbuf, sizeof(tbuf)),
+			print_kptr((KA_T)v->v_data, (char *)NULL, 0));
 		    enter_nm(Namech);
 		    return;
 		}
 		if (!s.s_realvp
 		||  readvnode((KA_T)s.s_realvp, &rv)) {
-		    (void) sprintf(Namech,
+		    (void) snpf(Namech, Namechl,
 			"snode at %s: can't read real vnode (%s)",
-			print_kptr((KA_T)v->v_data, tbuf),
-			print_kptr((KA_T)s.s_realvp, (char *)NULL));
+			print_kptr((KA_T)v->v_data, tbuf, sizeof(tbuf)),
+			print_kptr((KA_T)s.s_realvp, (char *)NULL, 0));
 		    enter_nm(Namech);
 		    return;
 		}
 
 #if	defined(HASVXFS)
 		if (getnodety(&rv) == N_VXFS) {
-		    if (!rv.v_data || read_vxnode(&rv, vfs, &dev)) {
-			(void) sprintf(Namech,
+		    if (!rv.v_data
+		    ||  read_vxnode(&rv, vfs, &dev, &devs, &rdev, &rdevs)) {
+			(void) snpf(Namech, Namechl,
 			    "vnode at %s: can't read vx_inode (%s)",
-			    print_kptr(va, tbuf),
-			    print_kptr((KA_T)rv.v_data, (char *)NULL));
+			    print_kptr(va, tbuf, sizeof(tbuf)),
+			    print_kptr((KA_T)rv.v_data, (char *)NULL, 0));
 			enter_nm(Namech);
 			return;
 		    }
@@ -502,10 +515,10 @@ process_node(va)
 #endif	/* defined(HASVXFS) */
 
 		if (!rv.v_data || readinode((KA_T)rv.v_data, &i)) {
-		    (void) sprintf(Namech,
+		    (void) snpf(Namech, Namechl,
 			"snode at %s: can't read inode (%s)",
-			print_kptr((KA_T)v->v_data, tbuf),
-			print_kptr((KA_T)rv.v_data, (char *)NULL));
+			print_kptr((KA_T)v->v_data, tbuf, sizeof(tbuf)),
+			print_kptr((KA_T)rv.v_data, (char *)NULL, 0));
 		    enter_nm(Namech);
 		    return;
 		}
@@ -513,10 +526,10 @@ process_node(va)
 		break;
 	    }
 	    if (!v->v_data || readinode((KA_T)v->v_data, &i)) {
-		(void) sprintf(Namech,
+		(void) snpf(Namech, Namechl,
 		    "vnode at %s: can't read inode (%s)",
-		    print_kptr(va, tbuf),
-		    print_kptr((KA_T)v->v_data, (char *)NULL));
+		    print_kptr(va, tbuf, sizeof(tbuf)),
+		    print_kptr((KA_T)v->v_data, (char *)NULL, 0));
 		enter_nm(Namech);
 		return;
 	    }
@@ -528,26 +541,26 @@ process_node(va)
 	case N_FIFO:
 	    if (v->v_fstype == VNFS_FIFO) {
 		if (!v->v_data || readsnode((KA_T)v->v_data, &s)) {
-		    (void) sprintf(Namech,
+		    (void) snpf(Namech, Namechl,
 			"vnode at %s: can't read snode (%s)",
-			print_kptr(va, tbuf),
-			print_kptr((KA_T)v->v_data, (char *)NULL));
+			print_kptr(va, tbuf, sizeof(tbuf)),
+			print_kptr((KA_T)v->v_data, (char *)NULL, 0));
 		    enter_nm(Namech);
 		    return;
 		}
 		if (!s.s_realvp || readvnode((KA_T)s.s_realvp, &rv)) {
-		    (void) sprintf(Namech,
+		    (void) snpf(Namech, Namechl,
 			"snode at %s: can't read real vnode (%s)",
-			print_kptr((KA_T)v->v_data, tbuf),
-			print_kptr((KA_T)s.s_realvp, (char *)NULL));
+			print_kptr((KA_T)v->v_data, tbuf, sizeof(tbuf)),
+			print_kptr((KA_T)s.s_realvp, (char *)NULL, 0));
 		    enter_nm(Namech);
 		    return;
 		}
 		if (!rv.v_data || readrnode((KA_T)rv.v_data, &r)) {
-		    (void) sprintf(Namech,
+		    (void) snpf(Namech, Namechl,
 			"snode at %s: can't read real rnode (%s)",
-			print_kptr((KA_T)v->v_data, tbuf),
-			print_kptr((KA_T)s.s_realvp, (char *)NULL));
+			print_kptr((KA_T)v->v_data, tbuf, sizeof(tbuf)),
+			print_kptr((KA_T)s.s_realvp, (char *)NULL, 0));
 		    enter_nm(Namech);
 		    return;
 		}
@@ -561,10 +574,10 @@ process_node(va)
 	default:
 	    if (v->v_type != VBLK) {
 		if (!v->v_data || readinode((KA_T)v->v_data, &i)) {
-		    (void) sprintf(Namech,
+		    (void) snpf(Namech, Namechl,
 			"vnode at %s: can't read inode (%s)",
-			print_kptr(va, tbuf),
-			print_kptr((KA_T)v->v_data, (char *)NULL));
+			print_kptr(va, tbuf, sizeof(tbuf)),
+			print_kptr((KA_T)v->v_data, (char *)NULL, 0));
 		    enter_nm(Namech);
 		    return;
 		}
@@ -594,6 +607,12 @@ process_node(va)
 	    break;
 #endif	/* defined(HAS_AFS) */
 
+	case N_MVFS:
+	    if (vfs) {
+		dev = vfs->dev;
+		devs = 1;
+	    }
+	    break;
 	case N_NFS:
 	    dev = vfs ? vfs->dev : 0;
 	    devs = 1;
@@ -613,7 +632,7 @@ process_node(va)
 		dev = (dev_t)vat.va_fsid;
 		devs = 1;
 	    } else
-		enter_dev_ch(print_kptr(va, (char *)NULL));
+		enter_dev_ch(print_kptr(va, (char *)NULL, 0));
 	    break;
 #endif	/* _HPUX>=1000 */
 
@@ -627,24 +646,26 @@ process_node(va)
 	default:
 
 #if	HPUXV>=800
-	    if (v->v_type == VCHR || v->v_type == N_BLK) {
-		dev = v->v_rdev;
-		devs = 1;
-	    } else if (vfs && vfs->fsname) {
+	    if (vfs && vfs->fsname) {
 		dev = vfs->dev;
 		devs = 1;
 	    } else if (ins) {
 		dev = i.i_dev;
 		devs = 1;
 	    }
+	    if ((v->v_type == VCHR) || (v->v_type == N_BLK)) {
+		rdev = v->v_rdev;
+		rdevs = 1;
+	    }
 #else	/* HPUXV<800 */
-	    if (v->v_type == VCHR || v->v_type == VBLK) {
-		dev = v->v_rdev;
-		devs = 1;
-	    } else if (ins) {
+	    if (ins) {
 		dev = i.i_dev;
 		devs = 1;
 	    }
+	    if ((v->v_type == VCHR) || (v->v_type == VBLK)) {
+		rdev = v->v_rdev;
+		rdevs = 1;
+	    }
 #endif	/* HPUXV>=800 */
 
 	}
@@ -663,6 +684,10 @@ process_node(va)
 	    break;
 #endif	/* defined(HAS_AFS) */
 
+	case N_MVFS:
+	    Lf->inode = (unsigned long)m.m_ino;
+	    Lf->inp_ty = 1;
+	    break;
 	case N_NFS:
 
 #if	HPUXV<1030
@@ -725,10 +750,12 @@ process_node(va)
  */
 	if (v->v_stream) {
 	    KA_T ip, pcb;
-	    char *pn;
+	    char *pn = (char *)NULL;
 
 	    Lf->dev = dev;
 	    Lf->dev_def = devs;
+	    Lf->rdev = rdev;
+	    Lf->rdev_def = rdevs;
 	    if (read_mi((KA_T)v->v_stream, &ip, &pcb, &pn))
 		return;
 	    if (ip && pcb) {
@@ -787,7 +814,7 @@ process_node(va)
 
 		if (Fsize || (Lf->access != 'r' && Lf->access != 'w')) {
 		    if (fns || ins) {
-			(void) sprintf(fb, "rd=%#x; wr=%#x", rp, wp);
+			(void) snpf(fb, sizeof(fb), "rd=%#x; wr=%#x", rp, wp);
 			(void) enter_nma(fb);
 		    }
 		    if (fns || ins || rns) {
@@ -799,7 +826,7 @@ process_node(va)
 		if (fns || ins) {
 		    Lf->off = (unsigned long)((Lf->access == 'r') ? rp
 								  : wp);
-		    (void) sprintf(fb, "%s=%#x",
+		    (void) snpf(fb, sizeof(fb), "%s=%#x",
 			(Lf->access == 'r') ? "rd" : "wr",
 			(Lf->access == 'r') ?  rp  :  wp);
 		    (void) enter_nma(fb);
@@ -808,6 +835,9 @@ process_node(va)
 		break;
 #endif	/* HPUXV>=900 */
 
+	    case N_MVFS:
+		/* The location of the file size isn't known. */
+		break;
 	    case N_NFS:
 
 #if	HPUXV<1030
@@ -849,6 +879,9 @@ process_node(va)
 		break;
 # endif	/* defined(HAS_AFS) */
 
+	    case N_MVFS:
+		/* The location of the link count isn't known. */
+		break;
 	    case N_NFS:
 		break;
 
@@ -904,8 +937,14 @@ process_node(va)
 
 	}
 /*
+ * Save the device numbers and their states.
+ *
  * Format the vnode type, and possibly the device name.
  */
+	Lf->dev = dev;
+	Lf->dev_def = devs;
+	Lf->rdev = rdev;
+	Lf->rdev_def = rdevs;
 	switch (type) {
 	case VNON:
 	    ty ="VNON";
@@ -913,19 +952,13 @@ process_node(va)
 	case VREG:
 	case VDIR:
 	    ty = (type == VREG) ? "VREG" : "VDIR";
-	    Lf->dev_def = devs;
-	    Lf->dev = dev;
 	    break;
 	case VBLK:
 	    ty = "VBLK";
-	    Lf->dev_def = devs;
-	    Lf->dev = dev;
 	    Ntype = N_BLK;
 	    break;
 	case VCHR:
 	    ty = "VCHR";
-	    Lf->dev = dev;
-	    Lf->dev_def = devs;
 	    Ntype = N_CHR;
 	    break;
 	case VLNK:
@@ -946,37 +979,27 @@ process_node(va)
 
 #if	HPUXV>=1000
 	    case N_FIFO:
-		if (vfs && vfs->fsname) {
-		    Lf->dev = dev;
-		    Lf->dev_def = devs;
-		}
 		ty = "FIFO";
 		break;
 	    case N_PIPE:
-		if (vfs && vfs->fsname) {
-		    Lf->dev = dev;
-		    Lf->dev_def = devs;
-		}
 		ty = "PIPE";
 		break;
 #endif	/* HPUXV>=1000 */
 
 	    default:
-		Lf->dev = dev;
-		Lf->dev_def = devs;
 		ty = "FIFO";
 	    }
 	    break;
 	default:
 	    if (type > 9999)
-		(void) sprintf(Lf->type, "*%03d", type % 1000);
+		(void) snpf(Lf->type, sizeof(Lf->type), "*%03d", type % 1000);
 	    else
-		(void) sprintf(Lf->type, "%4d", type);
-	    (void) strcpy(Namech, "unknown type");
+		(void) snpf(Lf->type, sizeof(Lf->type), "%4d", type);
+	    (void) snpf(Namech, Namechl, "unknown type");
 	    ty = (char *)NULL;
 	}
 	if (ty)
-	    (void) strcpy(Lf->type, ty);
+	    (void) snpf(Lf->type, sizeof(Lf->type), "%s", ty);
 	Lf->ntype = Ntype;
 
 #if	defined(HASBLKDEV)
@@ -984,7 +1007,7 @@ process_node(va)
  * If this is a VBLK file and it's missing an inode number, try to
  * supply one.
  */
-	if (Lf->inp_ty == 0 && type == VBLK && Lf->dev_def)
+	if ((Lf->inp_ty == 0) && (type == VBLK))
 	    find_bl_ino();
 #endif	/* defined(HASBLKDEV) */
 
@@ -992,12 +1015,13 @@ process_node(va)
  * If this is a VCHR file and it's missing an inode number, try to
  * supply one.
  */
-	if (Lf->inp_ty == 0 && type == VCHR && Lf->dev_def)
+	if ((Lf->inp_ty == 0) && (type == VCHR))
 	    find_ch_ino();
 /*
  * Test for specified file.
  */
-	if (Sfile && is_file_named((char *)NULL, (type == VCHR) ? 1 : 0))
+	if (Sfile && is_file_named((char *)NULL,
+				   ((type == VCHR) || (type == VBLK) ? 1 : 0)))
 	    Lf->sf |= SELNM;
 /*
  * Enter name characters.
@@ -1018,8 +1042,31 @@ readinode(ia, i)
 	struct inode *i;		/* inode buffer */
 {
 	if (kread((KA_T)ia, (char *)i, sizeof(struct inode))) {
-	    (void) sprintf(Namech, "can't read inode at %s",
-		print_kptr(ia, (char *)NULL));
+	    (void) snpf(Namech, Namechl, "can't read inode at %s",
+		print_kptr(ia, (char *)NULL, 0));
+	    return(1);
+	}
+	return(0);
+}
+
+
+/*
+ * read_nmn() - read node's mvfsnode
+ */
+
+static int
+read_nmn(na, ma, m)
+	KA_T na;                        /* containing node's address */
+	KA_T ma;                        /* kernel mvfsnode address */
+	struct mvfsnode *m;             /* mvfsnode receiver */
+{
+	char tbuf[32];
+
+	if (!ma || kread((KA_T)ma, (char *)m, sizeof(struct mvfsnode))) {
+	    (void) snpf(Namech, Namechl, "node at %s: can't read mvfsnode: %s",
+		print_kptr(na, tbuf, sizeof(tbuf)),
+		print_kptr(ma, (char *)NULL, 0));
+	    enter_nm(Namech);
 	    return(1);
 	}
 	return(0);
