@@ -32,7 +32,7 @@
 #ifndef lint
 static char copyright[] =
 "@(#) Copyright 1994 Purdue Research Foundation.\nAll rights reserved.\n";
-static char *rcsid = "$Id: dproc.c,v 1.11 99/06/22 08:17:49 abe Exp $";
+static char *rcsid = "$Id: dproc.c,v 1.15 2001/02/13 13:54:36 abe Exp $";
 #endif
 
 #if	defined(HPUXKERNBITS)
@@ -79,7 +79,7 @@ struct proc {
 	caddr_t d3[9];			/* dummy to occupy space */
 	uid_t p_uid;			/* UID */
 	caddr_t d4[2];			/* dummy to occupy space */
-	gid_t p_pgrp;			/* process group ID */
+	gid_t p_pgid;			/* process group ID */
 	pid_t p_pid;			/* PID */
 	pid_t p_ppid;			/* parent PID */
 	caddr_t d5[9];			/* dummy to occupy space */
@@ -103,7 +103,7 @@ struct proc {
 	uid_t p_uid;			/* real UID */
 	uid_t p_suid;			/* effective UID */
 	caddr_t d4;			/* dummy to occupy space */
-	gid_t p_pgrp;			/* process group ID */
+	gid_t p_pgid;			/* process group ID */
 	pid_t p_pid;			/* PID */
 	pid_t p_ppid;			/* parent PID */
 	caddr_t d5[9];			/* dummy to occupy space */
@@ -197,10 +197,6 @@ struct ofile_t {
 # endif	/* HPUXV>=1020 || (HPUXV>=900 && HPUXV<1000) */
 #endif	/* defined(HASFSTRUCT) */
 
-#if     defined(HASNCACHE)
-        ncache_load();
-#endif  /* defined(HASNCACHE) */
-
 /*
  * Examine proc structures and their associated information.
  */
@@ -222,7 +218,7 @@ struct ofile_t {
 	/*
 	 * See if process is excluded.
 	 */
-	    if (is_proc_excl(p->p_pid, (int)p->p_pgrp, (UID_ARG)p->p_uid,
+	    if (is_proc_excl(p->p_pid, (int)p->p_pgid, (UID_ARG)p->p_uid,
 			     &pss, &sf))
 		continue;
 
@@ -257,18 +253,16 @@ struct ofile_t {
 	 * Skip to the last component of the first path name.  Also skip any
 	 * leading `-', signifying a login shell.  Copy the result to u_comm[].
 	 */
-	    if (*c == '/' || *c == '.') {
-		for (s = c; *s && *s != ' '; s++) {
-		    if (*s == '/')
-			c = s + 1;
-		}
-	    }
 	    if (*c == '-')
 		c++;
+	    for (s = c; *c && (*c != ' '); c++) {
+		if (*c == '/')
+		    s = c + 1;
+	    }
 	    for (i = 0; i < MAXCOMLEN; i++) {
-		if (*c == '\0' || *c == ' ' || *c == '/')
+		if (*s == '\0' || *s == ' ' || *s == '/')
 		    break;
-		u->u_comm[i] = *c++;
+		u->u_comm[i] = *s++;
 	    }
 	    u->u_comm[i] = '\0';
 #else	/* HPUXV<800 */
@@ -324,7 +318,7 @@ struct ofile_t {
 	 */
 	    if (is_cmd_excl(u->u_comm, &pss, &sf))
 		continue;
-	    alloc_lproc(p->p_pid, (int)p->p_pgrp, (int)p->p_ppid,
+	    alloc_lproc(p->p_pid, (int)p->p_pgid, (int)p->p_ppid,
 			(UID_ARG)p->p_uid, u->u_comm, (int)pss, (int)sf);
 	    Plf = (struct lfile *)NULL;
 	/*
@@ -771,7 +765,7 @@ process_text(vasp)
 		alloc_lfile(" mem", -1);
 		break;
 	    default:
-		(void) sprintf(fd, "R%02d", p.p_type);
+		(void) snpf(fd, sizeof(fd), "R%02d", p.p_type);
 		alloc_lfile(fd, -1);
 	    }
 	/*
