@@ -32,7 +32,7 @@
 #ifndef lint
 static char copyright[] =
 "@(#) Copyright 1994 Purdue Research Foundation.\nAll rights reserved.\n";
-static char *rcsid = "$Id: dsock.c,v 1.13 99/11/28 06:46:09 abe Exp $";
+static char *rcsid = "$Id: dsock.c,v 1.15 2001/06/19 13:29:19 abe Exp $";
 #endif
 
 #if	defined(HPUXKERNBITS) && HPUXKERNBITS>=64
@@ -112,12 +112,12 @@ print_tcptpi(nl)
 		    cp = "TIME_WAIT";
 		    break;
 		default:
-		    (void) sprintf(sbuf, "UknownState_%d", i);
+		    (void) snpf(sbuf, sizeof(sbuf), "UknownState_%d", i);
 		    cp = sbuf;
 		}
 		break;
 	    case 1:				/* TPI */
-		switch ((u = Lf->lts.state.u)) {
+		switch ((u = Lf->lts.state.ui)) {
 		case TS_UNINIT:
 		    cp = "Uninitialized";
 		    break;
@@ -168,7 +168,7 @@ print_tcptpi(nl)
 		    cp = "Internal";
 		    break;
 		default:
-		    (void) sprintf(sbuf, "UNKNOWN_TPI_STATE_%u", u);
+		    (void) snpf(sbuf, sizeof(sbuf), "UNKNOWN_TPI_STATE_%u", u);
 		    cp = sbuf;
 		}
 	    }
@@ -264,17 +264,19 @@ void
 process_lla(la)
 	KA_T la;			/* link level CB address in kernel */
 {
+	char *ep;
 	struct lla_cb lcb;
+	size_t sz;
 
-	(void) strcpy(Lf->type, "lla");
+	(void) snpf(Lf->type, sizeof(Lf->type), "lla");
 	Lf->inp_ty = 2;
-	enter_dev_ch(print_kptr(la, (char *)NULL));
+	enter_dev_ch(print_kptr(la, (char *)NULL, 0));
 /*
  * Read link level access control block.
  */
 	if (!la || kread((KA_T)la, (char *)&lcb, sizeof(lcb))) {
-	    (void) sprintf(Namech, "can't read LLA CB (%s)",
-		print_kptr(la, (char *)NULL));
+	    (void) snpf(Namech, Namechl, "can't read LLA CB (%s)",
+		print_kptr(la, (char *)NULL, 0));
 	    enter_nm(Namech);
 	    return;
 	}
@@ -291,20 +293,23 @@ process_lla(la)
  * Determine the open mode, if possible.
  */
 	if (lcb.lla_flags & LLA_IS_ETHER)
-	    (void) strcpy(Lf->iproto, "Ether");
+	    (void) snpf(Lf->iproto, sizeof(Lf->iproto), "Ether");
 	else if (lcb.lla_flags & (LLA_IS_8025|LLA_IS_SNAP8025|LLA_IS_FA8025)) {
-	    (void) strcpy(Lf->iproto, "802.5");
+	    (void) snpf(Lf->iproto, sizeof(Lf->iproto), "802.5");
 	    if (lcb.lla_flags & LLA_IS_SNAP8025)
-		(void) strcpy(Namech, "SNAP");
+		(void) snpf(Namech, Namechl, "SNAP");
 	    else if (lcb.lla_flags & LLA_IS_FA8025)
-		(void) strcpy(Namech, "function address");
+		(void) snpf(Namech, Namechl, "function address");
 	}
 /*
  * Add any significant flags.
  */
-	if (lcb.lla_flags & ~(LLA_FWRITE | LLA_FREAD))
-	    (void) sprintf(endnm(), "%s(flags = %#x)",
-		(Namech[0] == '\0') ? "" : " ", lcb.lla_flags);
+	if (lcb.lla_flags & ~(LLA_FWRITE | LLA_FREAD)) {
+	    ep = endnm(&sz);
+	    (void) snpf(ep, sz, "%s(flags = %#x)",
+		(ep == Namech) ? "" : " ",
+		lcb.lla_flags);
+	}
 	if (Namech[0])
 	    enter_nm(Namech);
 }
@@ -320,12 +325,13 @@ process_socket(sa)
 	KA_T sa;			/* socket address in kernel */
 {
 	unsigned char *fa = (unsigned char *)NULL;
+	char *ep, tbuf[32];
 	int fam;
 	int fp, mbl, lp;
 	unsigned char *la = (unsigned char *)NULL;
 	struct protosw p;
 	struct socket s;
-	char tbuf[32];
+	size_t sz;
 	struct unpcb uc, unp;
 	struct sockaddr_un *ua = (struct sockaddr_un *)NULL;
 	struct sockaddr_un un;
@@ -334,7 +340,6 @@ process_socket(sa)
 	struct domain d;
 
 # if	defined(HPUX_CCITT)
-	char *cp;
 	int i;
 	struct x25pcb xp;
 	struct x25pcb_extension xpe;
@@ -354,7 +359,7 @@ process_socket(sa)
 # endif	/* HPUXV<1030 */
 #endif	/* HPUXV>=800 */
 
-	(void) strcpy(Lf->type, "sock");
+	(void) snpf(Lf->type, sizeof(Lf->type), "sock");
 	Lf->inp_ty = 2;
 /*
  * Read socket structure.
@@ -364,8 +369,8 @@ process_socket(sa)
 	    return;
 	}
 	if (kread((KA_T) sa, (char *) &s, sizeof(s))) {
-	    (void) sprintf(Namech, "can't read socket struct from %s",
-		print_kptr(sa, (char *)NULL));
+	    (void) snpf(Namech, Namechl, "can't read socket struct from %s",
+		print_kptr(sa, (char *)NULL, 0));
 	    enter_nm(Namech);
 	    return;
 	}
@@ -373,21 +378,21 @@ process_socket(sa)
  * Read protocol switch and domain structure (HP-UX 8 and above).
  */
 	if (!s.so_type) {
-	    (void) strcpy(Namech, "no socket type");
+	    (void) snpf(Namech, Namechl, "no socket type");
 	    enter_nm(Namech);
 	    return;
 	}
 	if (!s.so_proto
 	||  kread((KA_T) s.so_proto, (char *) &p, sizeof(p))) {
-	    (void) strcpy(Namech, "no protocol switch");
+	    (void) snpf(Namech, Namechl, "no protocol switch");
 	    enter_nm(Namech);
 	    return;
 	}
 
 #if	HPUXV>=800
 	if (kread((KA_T) p.pr_domain, (char *) &d, sizeof(d))) {
-	    (void) sprintf(Namech, "can't read domain struct from %s",
-		print_kptr((KA_T)p.pr_domain, (char *)NULL));
+	    (void) snpf(Namech, Namechl, "can't read domain struct from %s",
+		print_kptr((KA_T)p.pr_domain, (char *)NULL, 0));
 	    enter_nm(Namech);
 	    return;
 	}
@@ -434,61 +439,62 @@ process_socket(sa)
 	case AF_CCITT:
 	    if (Fnet)
 		Lf->sf |= SELNET;
-	    (void) strcpy(Lf->type, "x.25");
-	    (void) sprintf(Lf->iproto, "%.*s", IPROTOL, "CCITT");
+	    (void) snpf(Lf->type, sizeof(Lf->type), "x.25");
+	    (void) snpf(Lf->iproto, sizeof(Lf->iproto), "%.*s", IPROTOL,
+		"CCITT");
 	/*
 	 * Get the X25 PCB and its extension.
 	 */
 	    if (!s.so_pcb
 	    ||  kread((KA_T)s.so_pcb, (char *)&xp, sizeof(xp))) {
-		(void) sprintf(Namech, "can't read x.25 pcb at %s",
-		    print_kptr((KA_T)s.so_pcb, (char *)NULL));
+		(void) snpf(Namech, Namechl, "can't read x.25 pcb at %s",
+		    print_kptr((KA_T)s.so_pcb, (char *)NULL, 0));
 		enter_nm(Namech);
 		return;
 	    }
-	    enter_dev_ch(print_kptr((KA_T)s.so_pcb, (char *)NULL));
+	    enter_dev_ch(print_kptr((KA_T)s.so_pcb, (char *)NULL, 0));
 	    if (!xp.x25pcb_extend
 	    ||  kread((KA_T)xp.x25pcb_extend, (char *)&xpe, sizeof(xpe))) {
-		(void) sprintf(Namech,
+		(void) snpf(Namech, Namechl,
 		    "can't read x.25 pcb (%s) extension at %s",
-		    print_kptr((KA_T)s.so_pcb, tbuf),
-		    print_kptr((KA_T)xp.x25pcb_extend, (char *)NULL));
+		    print_kptr((KA_T)s.so_pcb, tbuf, sizeof(tbuf)),
+		    print_kptr((KA_T)xp.x25pcb_extend, (char *)NULL, 0));
 		enter_nm(Namech);
 		return;
 	    }
 	/*
 	 * Format local address.
 	 */
-	    for (i = 0, cp = Namech;
-		 i < xpe.x25pcbx_local_addr.x25hostlen/2;
-		 i++, cp += 2)
-	    {
-		(void) sprintf(cp, "%02x", xpe.x25pcbx_local_addr.x25_host[i]);
+	    for (i = 0; i < xpe.x25pcbx_local_addr.x25hostlen/2; i++) {
+		ep = endnm(&sz);
+		(void) snpf(ep, sz, "%02x", xpe.x25pcbx_local_addr.x25_host[i]);
 	    }
 	    if (i*2 != xpe.x25pcbx_local_addr.x25hostlen) {
-		(void) sprintf(cp, "%01x",
+		ep = endnm(&sz);
+		(void) snpf(ep, sz, "%01x",
 		    xpe.x25pcbx_local_addr.x25_host[i] >> 4);
-		cp++;
 	    }
 	/*
 	 * Display the virtual connection number, if it's defined.
 	 */
-	    if (xp.x25pcb_vcn >= 0)
-		(void) sprintf(cp, ":%d", xp.x25pcb_vcn + 1);
+	    if (xp.x25pcb_vcn >= 0) {
+		ep = endnm(&sz);
+		(void) snpf(ep, sz, ":%d", xp.x25pcb_vcn + 1);
+	    }
 	/*
 	 * Format peer address, if there is one.
 	 */
 	    if (xpe.x25pcbx_peer_addr.x25hostlen > 0) {
-		(void) strcpy(endnm(), "->");
-		for (i = 0, cp = endnm();
-		     i < xpe.x25pcbx_peer_addr.x25hostlen/2;
-		     i++, cp += 2)
-		{
-		    (void) sprintf(cp, "%02x",
+		ep = endnm(&sz);
+		(void) snpf(ep, sz, "->");
+		for (i = 0; i < xpe.x25pcbx_peer_addr.x25hostlen/2; i++) {
+		    ep = endnm(&sz);
+		    (void) snpf(ep, sz, "%02x",
 			xpe.x25pcbx_peer_addr.x25_host[i]);
 		}
 		if (i*2 != xpe.x25pcbx_peer_addr.x25hostlen) {
-		    (void) sprintf(cp, "%01x",
+		    ep = endnm(&sz);
+		    (void) snpf(ep, sz, "%01x",
 			xpe.x25pcbx_peer_addr.x25_host[i] >> 4);
 		}
 	    }
@@ -502,7 +508,7 @@ process_socket(sa)
 	case AF_INET:
 	    if (Fnet)
 		Lf->sf |= SELNET;
-	    (void) strcpy(Lf->type, "inet");
+	    (void) snpf(Lf->type, sizeof(Lf->type), "inet");
 	    printiproto(p.pr_protocol);
 
 #if	HPUXV>=1030
@@ -512,7 +518,7 @@ process_socket(sa)
 	    if (s.so_sth) {
 
 		KA_T ip, pcb;
-		char *pn;
+		char *pn = (char *)NULL;
 	    /*
 	     * Read module information.
 	     */
@@ -533,7 +539,7 @@ process_socket(sa)
 	     */
 
 		if (ip && !Lf->dev_def)
-		    enter_dev_ch(print_kptr(ip, (char *)NULL));
+		    enter_dev_ch(print_kptr(ip, (char *)NULL, 0));
 		if (!strlen(Namech)) {
 
 		/*
@@ -541,10 +547,10 @@ process_socket(sa)
 		 * message.
 		 */
 		    if (!ip) {
-			(void) strcpy(Namech,
+			(void) snpf(Namech, Namechl,
 			    "no IP module for stream socket");
 		    } else {
-			(void) strcpy(Namech,
+			(void) snpf(Namech, Namechl,
 			    "no TCP/UDP module for stream socket");
 		    }
 		}
@@ -567,14 +573,14 @@ process_socket(sa)
 	     */
 		if (kread((KA_T)s.so_pcb, (char *)&raw, sizeof(raw))
 		||  (struct socket *)sa != (struct socket *)raw.rcb_socket) {
-		    (void) sprintf(Namech, "can't read rawcb at %s",
-			print_kptr((KA_T)s.so_pcb, (char *)NULL));
+		    (void) snpf(Namech, Namechl, "can't read rawcb at %s",
+			print_kptr((KA_T)s.so_pcb, (char *)NULL, 0));
 		    enter_nm(Namech);
 		    return;
 		}
 		enter_dev_ch(print_kptr((KA_T)(raw.rcb_pcb ? raw.rcb_pcb
 							   : s.so_pcb),
-					(char *)NULL));
+					(char *)NULL, 0));
 		if (raw.rcb_laddr.sa_family == AF_INET)
 		    la = (unsigned char *)&raw.rcb_laddr.sa_data[2];
 		else if (raw.rcb_laddr.sa_family)
@@ -582,7 +588,8 @@ process_socket(sa)
 		if (raw.rcb_faddr.sa_family == AF_INET)
 		    fa = (unsigned char *)&raw.rcb_faddr.sa_data[2];
 		else if (raw.rcb_faddr.sa_family) {
-		    (void) strcat(endnm(), "->");
+		    ep = endnm(&sz);
+		    (void) snpf(ep, sz, "->");
 		    printrawaddr(&raw.rcb_faddr);
 		}
 		if (fa || la)
@@ -593,14 +600,14 @@ process_socket(sa)
 	     * Print Internet socket information.
 	     */
 		if (kread((KA_T)s.so_pcb, (char *)&inp, sizeof(inp))) {
-		    (void) sprintf(Namech, "can't read inpcb at %s",
-			print_kptr((KA_T)s.so_pcb, (char *)NULL));
+		    (void) snpf(Namech, Namechl, "can't read inpcb at %s",
+			print_kptr((KA_T)s.so_pcb, (char *)NULL, 0));
 		    enter_nm(Namech);
 		    return;
 		}
 		enter_dev_ch(print_kptr((KA_T)(inp.inp_ppcb ? inp.inp_ppcb
 							    : s.so_pcb),
-					(char *)NULL));
+					(char *)NULL, 0));
 		la = (unsigned char *)&inp.inp_laddr;
 		lp = (int)ntohs(inp.inp_lport);
 		if (inp.inp_faddr.s_addr != INADDR_ANY || inp.inp_fport != 0) {
@@ -624,7 +631,7 @@ process_socket(sa)
 	case AF_UNIX:
 	    if (Funix)
 		Lf->sf |= SELUNX;
-	    (void) strcpy(Lf->type, "unix");
+	    (void) snpf(Lf->type, sizeof(Lf->type), "unix");
 
 #if	HPUXV>=1030
 /*
@@ -651,15 +658,15 @@ process_socket(sa)
 	/*
 	 * Read Unix protocol control block and the Unix address structure.
 	 */
-	    enter_dev_ch(print_kptr(sa, (char *)NULL));
+	    enter_dev_ch(print_kptr(sa, (char *)NULL, 0));
 	    if (kread((KA_T) s.so_pcb, (char *) &unp, sizeof(unp))) {
-		(void) sprintf(Namech, "can't read unpcb at %s",
-		    print_kptr((KA_T)s.so_pcb, (char *)NULL));
+		(void) snpf(Namech, Namechl, "can't read unpcb at %s",
+		    print_kptr((KA_T)s.so_pcb, (char *)NULL, 0));
 		break;
 	    }
 	    if ((struct socket *)sa != unp.unp_socket) {
-		(void) sprintf(Namech, "unp_socket (%s) mismatch",
-		    print_kptr((KA_T)unp.unp_socket, (char *)NULL));
+		(void) snpf(Namech, Namechl, "unp_socket (%s) mismatch",
+		    print_kptr((KA_T)unp.unp_socket, (char *)NULL, 0));
 		break;
 	    }
 
@@ -669,8 +676,8 @@ process_socket(sa)
 	 */
 	    if (unp.unp_addr) {
 		if (kread((KA_T) unp.unp_addr, (char *) &mb, sizeof(mb))) {
-		    (void) sprintf(Namech, "can't read unp_addr at %s",
-			print_kptr((KA_T)unp.unp_addr, (char *)NULL));
+		    (void) snpf(Namech, Namechl, "can't read unp_addr at %s",
+			print_kptr((KA_T)unp.unp_addr, (char *)NULL, 0));
 		    break;
 		}
 		ua = (struct sockaddr_un *)(((char *)&mb) + mb.m_off);
@@ -729,15 +736,16 @@ process_socket(sa)
 		if (ua->sun_family == AF_UNSPEC) {
 		    if (unp.unp_conn) {
 			if (kread((KA_T)unp.unp_conn, (char *) &uc, sizeof(uc)))
-			    (void) sprintf(Namech, "can't read unp_conn at %s",
-				print_kptr((KA_T)unp.unp_conn, (char *)NULL));
+			    (void) snpf(Namech, Namechl,
+				"can't read unp_conn at %s",
+				print_kptr((KA_T)unp.unp_conn,(char *)NULL,0));
 			else
-			    (void) sprintf(Namech, "->%s",
-				print_kptr((KA_T)uc.unp_socket, (char *)NULL));
+			    (void) snpf(Namech, Namechl, "->%s",
+				print_kptr((KA_T)uc.unp_socket,(char *)NULL,0));
 		    } else
-			(void) strcpy(Namech, "->(none)");
+			(void) snpf(Namech, Namechl, "->(none)");
 		} else
-		    (void) sprintf(Namech, "unknown sun_family (%d)",
+		    (void) snpf(Namech, Namechl, "unknown sun_family (%d)",
 			ua->sun_family);
 		break;
 	    }
@@ -748,9 +756,9 @@ process_socket(sa)
 		if (Sfile && is_file_named(ua->sun_path, 0))
 		    Lf->sf |= SELNM;
 		if (!Namech[0])
-		    (void) strcpy(Namech, ua->sun_path);
+		    (void) snpf(Namech, Namechl, "%s", ua->sun_path);
 	    } else
-		(void) strcpy(Namech, "no address");
+		(void) snpf(Namech, Namechl, "no address");
 		break;
 	default:
 	    printunkaf(fam, 1);
@@ -773,50 +781,55 @@ process_stream_sock(ip, pcb, pn, vt)
 	enum vtype vt;			/* vnode type */
 {
 	unsigned char *fa = (unsigned char *)NULL;
+	char *ep;
 	int fp, lp, rq, sq;
 	struct ipc_s ic;
 	unsigned char *la = (unsigned char *)NULL;
-	size_t nl;
-	char *np;
+	size_t sz;
 	u_short pt;
 	struct tcp_s tc;
 	tcph_t th;
 	struct udp_s ud;
 /*
- * Set current Namech address, file type, and protocol.  If AF_INET selection
- * is in effect, set its flag.
+ * Set file type and protocol.  If AF_INET selection is in effect, set its flag.
  */
-	np = endnm();
-	nl = (size_t)(np - Namech);
 	if (Fnet)
 	    Lf->sf |= SELNET;
-	(void) strcpy(Lf->type, "inet");
-	if (Sfile && vt != VNON && Lf->dev_def && Lf->inp_ty == 1) {
+	(void) snpf(Lf->type, sizeof(Lf->type), "inet");
+	if (pn) {
+	    (void) snpf(Lf->iproto, sizeof(Lf->iproto), pn);
+	    Lf->inp_ty = 2;
+	} else if (Sfile && (vt != VNON) && Lf->dev_def && (Lf->inp_ty == 1)) {
+
+	/*
+	 * If the protocol name isn't known and this stream socket's vnode type
+	 * isn't VNON, the stream socket will be handled mostly as a stream.
+	 * Thus, a named file check is appropriate.
+	 */
 	    if (is_file_named((char *)NULL, (vt == VCHR) ? 1 : 0))
 		Lf->sf |= SELNM;
 	}
-	if (pn) {
-	    (void) strcpy(Lf->iproto, pn);
-	    Lf->inp_ty = 2;
-	}
 /*
  * Get IP structure.
  */
+	*Namech = '\0';
 	if (!ip || kread(ip, (char *)&ic, sizeof(ic))) {
-	    (void) sprintf(np, "%scan't read IP control structure from %s",
-		nl ? " " : "", print_kptr(ip, (char *)NULL));
+	    ep = endnm(&sz);
+	    (void) snpf(ep, sz, "%scan't read IP control structure from %s",
+		sz ? " " : "", print_kptr(ip, (char *)NULL, 0));
 	    enter_nm(Namech);
 	    return;
 	}
 	if (!Lf->dev_def)
-	    enter_dev_ch(print_kptr(ip, (char *)NULL));
+	    enter_dev_ch(print_kptr(ip, (char *)NULL, 0));
 /*
  * Check for protocol control block address.  Enter if non-NULL and clear
  * device definition flag.
  */
 	if (!pcb) {
-	    (void) sprintf(np, "%ssocket stream has no TCP or UDP module",
-		nl ? " " : "");
+	    ep = endnm(&sz);
+	    (void) snpf(ep, sz, "%ssocket stream has no TCP or UDP module",
+		sz ? " " : "");
 	    enter_nm(Namech);
 	    return;
 	}
@@ -829,8 +842,9 @@ process_stream_sock(ip, pcb, pn, vt)
 	 * Process TCP socket.
 	 */
 	    if (kread(pcb, (char *)&tc, sizeof(tc))) {
-		(void) sprintf(np, "%scan't read TCP PCB from %s",
-		    nl ? " " : "", print_kptr(pcb, (char *)NULL));
+		ep = endnm(&sz);
+		(void) snpf(ep, sz, "%scan't read TCP PCB from %s",
+		    sz ? " " : "", print_kptr(pcb, (char *)NULL, 0));
 		enter_nm(Namech);
 		return;
 	    }
@@ -909,8 +923,9 @@ process_stream_sock(ip, pcb, pn, vt)
 	 * Process UDP socket.
 	 */
 	    if (kread(pcb, (char *)&ud, sizeof(ud))) {
-		(void) sprintf(np, "%scan't read UDP PCB from %s",
-		    nl ? " " : "", print_kptr(pcb, (char *)NULL));
+		ep = endnm(&sz);
+		(void) snpf(ep, sz, "%scan't read UDP PCB from %s",
+		    sz ? " " : "", print_kptr(pcb, (char *)NULL, 0));
 		enter_nm(Namech);
 		return;
 	    }
@@ -932,7 +947,7 @@ process_stream_sock(ip, pcb, pn, vt)
 	    if (!Fsize)
 		Lf->off_def = 1;
 	    Lf->lts.type = 1;
-	    Lf->lts.state.u = (unsigned int)ud.udp_state;
+	    Lf->lts.state.ui = (unsigned int)ud.udp_state;
 	    Namech[0] = '\0';
 	    return;
 	} else {
@@ -940,8 +955,9 @@ process_stream_sock(ip, pcb, pn, vt)
 	/*
 	 * Record an unknown protocol.
 	 */
-	    (void) sprintf(np, "%sunknown stream protocol: %s",
-		nl ? " " : "", pn ? pn : "NUll");
+	    ep = endnm(&sz);
+	    (void) snpf(ep, sz, "%sunknown stream protocol: %s",
+		sz ? " " : "", pn ? pn : "NUll");
 	}
 	if (Namech[0])
 	    enter_nm(Namech);
