@@ -32,16 +32,16 @@
 #ifndef lint
 static char copyright[] =
 "@(#) Copyright 1994 Purdue Research Foundation.\nAll rights reserved.\n";
-static char *rcsid = "$Id: dnode.c,v 1.15 99/06/22 08:19:52 abe Exp Locker: abe $";
+static char *rcsid = "$Id: dnode.c,v 1.21 2001/02/13 14:16:14 abe Exp $";
 #endif
 
 
 #include "lsof.h"
 
 
-#if	defined(HASFDESCFS)
+#if	defined(HASFDESCFS) && HASFDESCFS==1
 _PROTOTYPE(static int lkup_dev_tty,(dev_t *dr, unsigned long *ir));
-#endif	/* defined(HASFDESCFS) */
+#endif	/* defined(HASFDESCFS) && HASFDESCFS==1 */
 
 #if	defined(HASPROCFS)
 _PROTOTYPE(static void getmemsz,(pid_t pid));
@@ -79,7 +79,7 @@ getmemsz(pid)
 #endif	/* defined(HASPROCFS) */
 
 
-#if	defined(HASFDESCFS)
+#if	defined(HASFDESCFS) && HASFDESCFS==1
 /*
  * lkup_dev_tty() - look up /dev/tty
  */
@@ -122,7 +122,7 @@ lkup_dev_tty_again:
 
 	return(-1);
 }
-#endif	/* defined(HASFDESCFS) */
+#endif	/* defined(HASFDESCFS) && HASFDESCFS==1 */
 
 
 /*
@@ -133,18 +133,22 @@ void
 process_node(va)
 	KA_T va;			/* vnode kernel space address */
 {
-
-	dev_t dev;
-	int devs = 0;
+	dev_t dev, rdev;
+	struct denode d;
+	u_long dpb;
+	unsigned char devs = 0;
+	unsigned char lt;
+	unsigned char ns;
+	unsigned char rdevs = 0;
+	char *ep, *ty;
 	struct lockf lf, *lff, *lfp;
-	int lt;
 	struct inode i;
 	struct mfsnode m;
 	struct nfsnode n;
-	int ns;
-	enum nodetype {NONODE, CDFSNODE, EXT2NODE, FDESCNODE, INODE,
+	u_long long nn;
+	enum nodetype {NONODE, CDFSNODE, DOSNODE, EXT2NODE, FDESCNODE, INODE,
 		KERNFSNODE, MFSNODE, NFSNODE, PFSNODE} nty = NONODE;
-	char *ty;
+	struct msdosfsmount pm;
 	enum vtype type;
 	struct vnode *v, vb;
 	struct l_vfs *vfs;
@@ -158,9 +162,13 @@ process_node(va)
 
 #if	defined(HASFDESCFS)
 	struct fdescnode f;
+
+# if	HASFDESCFS==1
 	static dev_t f_tty_dev;
 	static unsigned long f_tty_ino;
 	static int f_tty_s = 0;
+# endif	/* HASFDESCFS==1 */
+
 #endif	/* defined(HASFDESCFS) */
 
 #if	defined(HASKERNFS)
@@ -182,6 +190,7 @@ process_node(va)
 #if	defined(HASPROCFS)
 	struct pfsnode p;
 	struct procfsid *pfi;
+	size_t sz;
 #endif	/* defined(HASPROCFS) */
 
 /*
@@ -254,8 +263,8 @@ process_node(va)
 	    if (read_iso_node(v, &iso_dev, (unsigned long *)&iso_ino,
 		  &iso_nlink, (SZOFFTYPE *)&iso_sz))
 	    {
-		(void) sprintf(Namech, "can't read iso_node at: %s",
-		    print_kptr((KA_T)v->v_data, (char *)NULL));
+		(void) snpf(Namech, Namechl, "can't read iso_node at: %s",
+		    print_kptr((KA_T)v->v_data, (char *)NULL, 0));
 		enter_nm(Namech);
 		return;
 	    }
@@ -273,8 +282,9 @@ process_node(va)
 	    if (!v->v_data
 	    ||  kread((KA_T)v->v_data, (char *)&kn, sizeof(kn))) {
 		if (v->v_type != VDIR || !(v->v_flag && VROOT)) {
-		    (void) sprintf(Namech, "can't read kernfs_node at: %s",
-			print_kptr((KA_T)v->v_data, (char *)NULL));
+		    (void) snpf(Namech, Namechl,
+			"can't read kernfs_node at: %s",
+			print_kptr((KA_T)v->v_data, (char *)NULL, 0));
 		    enter_nm(Namech);
 		    return;
 		} else
@@ -298,7 +308,7 @@ process_node(va)
 			ktnl = MAXPATHLEN - strlen(_PATH_KERNFS) - 2;
 			ktnm[ktnl] = '\0';
 		    }
-		    (void) sprintf(Namech, "%s/%s", _PATH_KERNFS, ktnm);
+		    (void) snpf(Namech, Namechl, "%s/%s", _PATH_KERNFS, ktnm);
 		}
 	    }
 	/*
@@ -307,7 +317,7 @@ process_node(va)
 	 * inode number and size.
 	 */
 	    if (v->v_type == VDIR && (v->v_flag & VROOT)) {
-		(void) strcpy(Namech, _PATH_KERNFS);
+		(void) snpf(Namech, Namechl, "%s", _PATH_KERNFS);
 		ksb.st_ino = 2;
 		ksb.st_size = DEV_BSIZE;
 		ksbs = 1;
@@ -320,18 +330,28 @@ process_node(va)
 	case VT_MFS:
 	    if (!v->v_data
 	    ||  kread((KA_T)v->v_data, (char *)&m, sizeof(m))) {
-		(void) sprintf(Namech, "can't read mfsnode at: %s",
-		    print_kptr((KA_T)v->v_data, (char *)NULL));
+		(void) snpf(Namech, Namechl, "can't read mfsnode at: %s",
+		    print_kptr((KA_T)v->v_data, (char *)NULL, 0));
 		enter_nm(Namech);
 		return;
 	    }
 	    nty = MFSNODE;
 	    break;
+	case VT_MSDOSFS:
+	    if (!v->v_data
+	    ||  kread((KA_T)v->v_data, (char *)&d, sizeof(d))) {
+		(void) snpf(Namech, Namechl, "can't read denode at: %s",
+		    print_kptr((KA_T)v->v_data, (char *)NULL, 0));
+		enter_nm(Namech);
+		return;
+	    }
+	    nty = DOSNODE;
+	    break;
 	case VT_NFS:
 	    if (!v->v_data
 	    ||  kread((KA_T)v->v_data, (char *)&n, sizeof(n))) {
-		(void) sprintf(Namech, "can't read nfsnode at: %s",
-		    print_kptr((KA_T)v->v_data, (char *)NULL));
+		(void) snpf(Namech, Namechl, "can't read nfsnode at: %s",
+		    print_kptr((KA_T)v->v_data, (char *)NULL, 0));
 		enter_nm(Namech);
 		return;
 	    }
@@ -339,8 +359,8 @@ process_node(va)
 #if	defined(HASNFSVATTRP)
 	    if (!n.n_vattr
 	    ||  kread((KA_T)n.n_vattr, (char *)&nv, sizeof(nv))) {
-		(void) sprintf(Namech, "can't read n_vattr at: %x",
-		    print_kptr((KA_T)n.n_vattr, (char *)NULL));
+		(void) snpf(Namech, Namechl, "can't read n_vattr at: %x",
+		    print_kptr((KA_T)n.n_vattr, (char *)NULL, 0));
 		enter_nm(Namech);
 		return;
 	    }
@@ -353,8 +373,8 @@ process_node(va)
 	case VT_FDESC:
 	    if (!v->v_data
 	    ||  kread((KA_T)v->v_data, (char *)&f, sizeof(f))) {
-		(void) sprintf(Namech, "can't read fdescnode at: %x",
-		    print_kptr((KA_T)v->v_data, (char *)NULL));
+		(void) snpf(Namech, Namechl, "can't read fdescnode at: %x",
+		    print_kptr((KA_T)v->v_data, (char *)NULL, 0));
 		enter_nm(Namech);
 		return;
 	    }
@@ -366,8 +386,8 @@ process_node(va)
 	case VT_PROCFS:
 	    if (!v->v_data
 	    ||  kread((KA_T)v->v_data, (char *)&p, sizeof(p))) {
-		(void) sprintf(Namech, "can't read pfsnode at: %s",
-		    print_kptr((KA_T)v->v_data, (char *)NULL));
+		(void) snpf(Namech, Namechl, "can't read pfsnode at: %s",
+		    print_kptr((KA_T)v->v_data, (char *)NULL, 0));
 		enter_nm(Namech);
 		return;
 	    }
@@ -382,8 +402,8 @@ process_node(va)
 	case VT_UFS:
 	    if (!v->v_data
 	    ||  kread((KA_T)v->v_data, (char *)&i, sizeof(i))) {
-		(void) sprintf(Namech, "can't read inode at: %s",
-		    print_kptr((KA_T)v->v_data, (char *)NULL));
+		(void) snpf(Namech, Namechl, "can't read inode at: %s",
+		    print_kptr((KA_T)v->v_data, (char *)NULL, 0));
 		enter_nm(Namech);
 		return;
 	    }
@@ -435,7 +455,7 @@ process_node(va)
 	default:
 	    if (v->v_type == VBAD || v->v_type == VNON)
 		break;
-	    (void) sprintf(Namech, "unknown file system type: %d",
+	    (void) snpf(Namech, Namechl, "unknown file system type: %d",
 		v->v_tag);
 	    enter_nm(Namech);
 	    return;
@@ -446,23 +466,35 @@ process_node(va)
 	type = v->v_type;
 	switch (nty) {
 
+	case DOSNODE:
+	    dev = d.de_dev;
+	    devs = 1;
+	    break;
+
 #if	defined(HASFDESCFS)
 	case FDESCNODE:
+
+# if	defined(HASFDLINK)
 	    if (f.fd_link
-	    &&  !kread((KA_T)f.fd_link, Namech, sizeof(Namech) - 1)) {
-		Namech[sizeof(Namech) - 1] = '\0';
+	    &&  !kread((KA_T)f.fd_link, Namech, Namechl - 1)) {
+		Namech[Namechl - 1] = '\0';
 		break;
 	    }
+# endif	/* defined(HASFDLINK) */
+
+# if	HASFDESCFS==1
 	    if (f.fd_type == Fctty) {
 		if (f_tty_s == 0)
 		    f_tty_s = lkup_dev_tty(&f_tty_dev, &f_tty_ino);
 		if (f_tty_s == 1) {
-		    dev = f_tty_dev;
+		    dev = DevDev;
+		    rdev = f_tty_dev;
 		    Lf->inode = f_tty_ino;
-		    devs = Lf->inp_ty = 1;
+		    devs = Lf->inp_ty = rdevs = 1;
 		}
 	    }
 	    break;
+# endif	/* HASFDESCFS==1 */
 #endif	/* defined(HASFDESCFS) */
 
 #if	defined(HASEXT2FS)
@@ -470,24 +502,32 @@ process_node(va)
 
 # if	defined(HASI_FFS)
 	    dev = i.i_dev;
+	    devs = 1;
 # else	/* !defined(HASI_FFS) */
-	    dev = (type == VCHR || type == VBLK) ? i.i_rdev : i.i_dev;
+	    dev = i.i_dev;
+	    devs = 1;
+	    if ((type == VCHR) || (type == VBLK)) {
+		rdev = i.i_rdev;
+		rdevs = 1;
+	    }
 # endif	/* defined(HASI_FFS) */
 
 	    break;
 #endif	/* defined(HASEXT2FS) */
 
 	case INODE:
-	    dev = (type == VCHR || type == VBLK) ?
+	    dev = i.i_dev;
+	    devs = 1;
+	    if ((type == VCHR) || (type == VBLK)) {
 
 #if	defined(HASI_FFS)
-						   i.i_ffs_rdev
+		rdev = i.i_ffs_rdev;
 #else	/* !defined(HASI_FFS) */
-						   i.i_rdev
+		rdev = i.i_rdev;
 #endif	/* defined(HASI_FFS) */
 
-						 : i.i_dev;
-	    devs = 1;
+		rdevs = 1;
+	    }
 	    break;
 
 #if	defined(HASKERNFS)
@@ -517,6 +557,25 @@ process_node(va)
  * Obtain the inode number.
  */
 	switch (nty) {
+	case DOSNODE:
+	    if (d.de_pmp && !kread((KA_T)d.de_pmp, (char *)&pm, sizeof(pm))) {
+		dpb = (u_long)(pm.pm_BytesPerSec / sizeof(struct direntry));
+		if (d.de_Attributes & ATTR_DIRECTORY) {
+		    if (d.de_StartCluster == MSDOSFSROOT)
+			nn = (u_long)1;
+		    else
+			nn = (u_long)(cntobn(&pm, d.de_StartCluster) * dpb);
+		} else {
+		    if (d.de_dirclust == MSDOSFSROOT)
+			nn = (u_long)(roottobn(&pm, 0) * dpb);
+		    else
+			nn = (u_long)(cntobn(&pm, d.de_dirclust) * dpb);
+		    nn += (u_long)(d.de_diroffset / sizeof(struct direntry));
+		}
+		Lf->inode = (unsigned long)nn;
+		Lf->inp_ty = 1;
+	    }
+	    break;
 
 #if	defined(HASEXT2FS)
 	case EXT2NODE:
@@ -642,6 +701,9 @@ process_node(va)
 			Lf->sz = (SZOFFTYPE)i.i_size;
 #endif	/* defined(HASI_FFS) */
 
+			Lf->sz_def = 1;
+		    } else if (nty == DOSNODE) {
+			Lf->sz = (SZOFFTYPE)d.de_FileSize;
 			Lf->sz_def = 1;
 		    } else if (nty == MFSNODE) {
 			Lf->sz = (SZOFFTYPE)m.mfs_size;
@@ -706,6 +768,9 @@ process_node(va)
 		    Lf->nlink = (long)i.i_nlink;
 # endif	/* defined(HASI_FFS) */
 
+		    Lf->nlink_def = 1;
+		} else if (nty == DOSNODE) {
+		    Lf->nlink = (long)d.de_refcnt;
 		    Lf->nlink_def = 1;
 		}
 
@@ -734,34 +799,30 @@ process_node(va)
 	    Lf->fsdev = vfs->fsname;
 	}
 /*
+ * Save the deice numbers and their states.
+ *
  * Format the vnode type, and possibly the device name.
  */
+	Lf->dev = dev;
+	Lf->dev_def = devs;
+	Lf->rdev = rdev;
+	Lf->rdev_def = rdevs;
 	switch (type) {
 	case VNON:
 	    ty ="VNON";
 	    break;
 	case VREG:
+	    ty = "VREG";
+	    break;
 	case VDIR:
-	    ty = (type == VREG) ? "VREG" : "VDIR";
-	    if (devs) {
-		Lf->dev_def = 1;
-		Lf->dev = dev;
-	    }
+	    ty = "VDIR";
 	    break;
 	case VBLK:
 	    ty = "VBLK";
-	    if (devs) {
-		Lf->dev_def = 1;
-		Lf->dev = dev;
-	    }
 	    Ntype = N_BLK;
 	    break;
 	case VCHR:
 	    ty = "VCHR";
-	    if (devs) {
-		Lf->dev = dev;
-		Lf->dev_def = 1;
-	    }
 	    Ntype = N_CHR;
 	    break;
 	case VLNK:
@@ -778,22 +839,18 @@ process_node(va)
 	    ty = "VBAD";
 	    break;
 	case VFIFO:
-	    if (devs) {
-		Lf->dev = dev;
-		Lf->dev_def = 1;
-	    }
 	    ty = "FIFO";
 	    break;
 	default:
 	    if (type > 9999)
-		(void) sprintf(Lf->type, "*%03d", type % 1000);
+		(void) snpf(Lf->type, sizeof(Lf->type), "*%03d", type % 1000);
 	    else
-		(void) sprintf(Lf->type, "%4d", type);
-	    (void) strcpy(Namech, "unknown type");
+		(void) snpf(Lf->type, sizeof(Lf->type), "%4d", type);
+	    (void) snpf(Namech, Namechl, "unknown type");
 	    ty = NULL;
 	}
 	if (ty)
-	    (void) strcpy(Lf->type, ty);
+	    (void) snpf(Lf->type, sizeof(Lf->type), "%s", ty);
 	Lf->ntype = Ntype;
 /*
  * Handle some special cases:
@@ -805,65 +862,75 @@ process_node(va)
  */
 
 	if (type == VBAD)
-	    (void) strcpy(Namech, "(revoked)");
+	    (void) snpf(Namech, Namechl, "(revoked)");
 	else if (nty == MFSNODE) {
-	    Lf->dev_def = 0;
-	    (void) sprintf(Namech, "%#x", m.mfs_baseoff);
+	    Lf->dev_def = Lf->rdev_def = 0;
+	    (void) snpf(Namech, Namechl, "%#x", m.mfs_baseoff);
 	    enter_dev_ch("memory");
 	}
 
 #if	defined(HASPROCFS)
 	else if (nty == PFSNODE) {
-	    Lf->dev_def = 0;
+	    Lf->dev_def= Lf->rdev_def = 0;
 	    ty = NULL;
-	    (void) sprintf(Namech, "/%s", HASPROCFS);
+	    (void) snpf(Namech, Namechl, "/%s", HASPROCFS);
 	    switch (p.pfs_type) {
 	    case Proot:
 		ty = "PDIR";
 		break;
 	    case Pcurproc:
-		(void) strcpy(endnm(), "/curproc");
+		ep = endnm(&sz);
+		(void) snpf(ep, sz, "/curproc");
 		ty = "PCUR";
 		break;
 	    case Pproc:
-		(void) sprintf(endnm(), "/%d", p.pfs_pid);
+		ep = endnm(&sz);
+		(void) snpf(ep, sz, "/%d", p.pfs_pid);
 		ty = "PDIR";
 		break;
 	    case Pfile:
-		(void) sprintf(endnm(), "/%d/file", p.pfs_pid);
+		ep = endnm(&sz);
+		(void) snpf(ep, sz, "/%d/file", p.pfs_pid);
 		ty = "PFIL";
 		break;
 	    case Pmem:
-		(void) sprintf(endnm(), "/%d/mem", p.pfs_pid);
+		ep = endnm(&sz);
+		(void) snpf(ep, sz, "/%d/mem", p.pfs_pid);
 		ty = "PMEM";
 		break;
 	    case Pregs:
-		(void) sprintf(endnm(), "/%d/regs", p.pfs_pid);
+		ep = endnm(&sz);
+		(void) snpf(ep, sz, "/%d/regs", p.pfs_pid);
 		ty = "PREG";
 		break;
 	    case Pfpregs:
-		(void) sprintf(endnm(), "/%d/fpregs", p.pfs_pid);
+		ep = endnm(&sz);
+		(void) snpf(ep, sz, "/%d/fpregs", p.pfs_pid);
 		ty = "PFPR";
 		break;
 	    case Pctl:
-		(void) sprintf(endnm(), "/%d/ctl", p.pfs_pid);
+		ep = endnm(&sz);
+		(void) snpf(ep, sz, "/%d/ctl", p.pfs_pid);
 		ty = "PCTL";
 		break;
 	    case Pstatus:
-		(void) sprintf(endnm(), "/%d/status", p.pfs_pid);
+		ep = endnm(&sz);
+		(void) snpf(ep, sz, "/%d/status", p.pfs_pid);
 		ty = "PSTA";
 		break;
 	    case Pnote:
-		(void) sprintf(endnm(), "/%d/note", p.pfs_pid);
+		ep = endnm(&sz);
+		(void) snpf(ep, sz, "/%d/note", p.pfs_pid);
 		ty = "PNTF";
 		break;
 	    case Pnotepg:
-		(void) sprintf(endnm(), "/%d/notepg", p.pfs_pid);
-		ty = "PGRP";
+		ep = endnm(&sz);
+		(void) snpf(ep, sz, "/%d/notepg", p.pfs_pid);
+		ty = "PGID";
 		break;
 	    }
 	    if (ty)
-		(void) strcpy(Lf->type, ty);
+		(void) snpf(Lf->type, sizeof(Lf->type), ty);
 	    if (Namech[0])
 		enter_nm(Namech);
 	}
@@ -874,7 +941,7 @@ process_node(va)
  * If this is a VBLK file and it's missing an inode number, try to
  * supply one.
  */
-	if (Lf->inp_ty == 0 && type == VBLK && Lf->dev_def)
+	if ((Lf->inp_ty == 0) && (type == VBLK))
 	    find_bl_ino();
 #endif	/* defined(HASBLKDEV) */
 
@@ -882,7 +949,7 @@ process_node(va)
  * If this is a VCHR file and it's missing an inode number, try to
  * supply one.
  */
-	if (Lf->inp_ty == 0 && type == VCHR && Lf->dev_def)
+	if ((Lf->inp_ty == 0) && (type == VCHR))
 	    find_ch_ino();
 /*
  * Test for specified file.
@@ -903,10 +970,8 @@ process_node(va)
 
 		    ) {
 			pfi->f = 1;
-			if (Namech[0] && pfi->nm) {
-			    (void) strncpy(Namech, pfi->nm, MAXPATHLEN - 1);
-			    Namech[MAXPATHLEN - 1] = '\0';
-			}
+			if (Namech[0] && pfi->nm)
+			    (void) snpf(Namech, Namechl, "%s", pfi->nm);
 			Lf->sf |= SELNM;
 			break;
 		    }
@@ -921,7 +986,9 @@ process_node(va)
 		ns = 1;
 	    } else
 		ns = 0;
-	    if (Sfile && is_file_named(NULL, (type == VCHR) ? 1 : 0))
+	    if (Sfile && is_file_named((char *)NULL,
+				       ((type == VCHR) || (type == VBLK)) ? 1
+									  : 0))
 		Lf->sf |= SELNM;
 	    if (ns)
 		Namech[0] = '\0';
@@ -943,16 +1010,19 @@ void
 process_pipe(pa)
 	KA_T pa;			/* pipe structure kernel address */
 {
+	char *ep;
 	struct pipe p;
+	size_t sz;
 
 	if (!pa || kread((KA_T)pa, (char *)&p, sizeof(p))) {
-	    (void) sprintf(Namech, "can't read DTYPE_PIPE pipe struct: %#s",
-		print_kptr(pa, (char *)NULL));
+	    (void) snpf(Namech, Namechl,
+		"can't read DTYPE_PIPE pipe struct: %#s",
+		print_kptr(pa, (char *)NULL, 0));
 	    enter_nm(Namech);
 	    return;
 	}
-	(void) strcpy(Lf->type, "PIPE");
-	enter_dev_ch(print_kptr(pa, (char *)NULL));
+	(void) snpf(Lf->type, sizeof(Lf->type), "PIPE");
+	enter_dev_ch(print_kptr(pa, (char *)NULL, 0));
 	if (Foffset)
 	    Lf->off_def = 1;
 	else {
@@ -960,16 +1030,22 @@ process_pipe(pa)
 	    Lf->sz_def = 1;
 	}
 	if (p.pipe_peer)
-	    (void) sprintf(Namech, "->%s",
-		print_kptr((KA_T)p.pipe_peer, (char *)NULL));
+	    (void) snpf(Namech, Namechl, "->%s",
+		print_kptr((KA_T)p.pipe_peer, (char *)NULL, 0));
 	else
 	    Namech[0] = '\0';
-	if (p.pipe_buffer.cnt)
-	    (void) sprintf(endnm(), "%, cnt=%d", p.pipe_buffer.cnt);
-	if (p.pipe_buffer.in)
-	    (void) sprintf(endnm(), "%, in=%d", p.pipe_buffer.in);
-	if (p.pipe_buffer.out)
-	    (void) sprintf(endnm(), "%, out=%d", p.pipe_buffer.out);
+	if (p.pipe_buffer.cnt) {
+	    ep = endnm(&sz);
+	    (void) snpf(ep, sz, ", cnt=%d", p.pipe_buffer.cnt);
+	}
+	if (p.pipe_buffer.in) {
+	    ep = endnm(&sz);
+	    (void) snpf(ep, sz, ", in=%d", p.pipe_buffer.in);
+	}
+	if (p.pipe_buffer.out) {
+	    ep = endnm(&sz);
+	    (void) snpf(ep, sz, ", out=%d", p.pipe_buffer.out);
+	}
 /*
  * Enter name characters.
  */
