@@ -32,7 +32,7 @@
 #ifndef lint
 static char copyright[] =
 "@(#) Copyright 1995 Purdue Research Foundation.\nAll rights reserved.\n";
-static char *rcsid = "$Id: dfile.c,v 1.9 99/06/22 08:21:33 abe Exp $";
+static char *rcsid = "$Id: dfile.c,v 1.11 2000/12/04 14:32:49 abe Exp $";
 #endif
 
 #include "lsof.h"
@@ -69,14 +69,15 @@ get_max_fd()
  */
 
 char *
-print_dev(lf)
+print_dev(lf, dev)
 	struct lfile *lf;		/* file whose device is to be printed */
+	dev_t *dev;			/* device to be printed */
 {
 	static char buf[128];
 
-	(void) sprintf(buf, "%d,%d",
-	    lf->is_nfs ? ((~(lf->dev >> 8)) & 0xff) : emajor(lf->dev),
-	    eminor(lf->dev));
+	(void) snpf(buf, sizeof(buf), "%d,%d",
+	    lf->is_nfs ? ((~(*dev >> 8)) & 0xff) : emajor(*dev),
+	    eminor(*dev));
 	return(buf);
 }
 
@@ -91,7 +92,7 @@ print_ino(lf)
 {
 	static char buf[128];
 
-	(void) sprintf(buf, (lf->inode & 0x80000000) ? "%#x" : "%lu",
+	(void) snpf(buf, sizeof(buf), (lf->inode & 0x80000000) ? "%#x" : "%lu",
 	    lf->inode);
 	return(buf);
 }
@@ -109,8 +110,8 @@ process_file(fp)
 	int flag;
 
 	if (kread(fp, (char *)&f, sizeof(f))) {
-	    (void) sprintf(Namech, "can't read file struct from %s",
-		print_kptr(fp, (char *)NULL));
+	    (void) snpf(Namech, Namechl, "can't read file struct from %s",
+		print_kptr(fp, (char *)NULL, 0));
 	    enter_nm(Namech);
 	    return;
 	}
