@@ -32,7 +32,7 @@
 #ifndef lint
 static char copyright[] =
 "@(#) Copyright 1995 Purdue Research Foundation.\nAll rights reserved.\n";
-static char *rcsid = "$Id: dproc.c,v 1.13 99/11/28 06:42:23 abe Exp $";
+static char *rcsid = "$Id: dproc.c,v 1.15 2000/11/03 18:10:48 abe Exp abe $";
 #endif
 
 #include "lsof.h"
@@ -152,14 +152,6 @@ gather_proc_info()
 		Exit(1);
 	    }
 	}
-
-#if     defined(HASNCACHE)
-/*
- * Read kernel name caches.
- */
-        ncache_load();
-#endif  /* defined(HASNCACHE) */
-
 /*
  * Examine proc structures and their associated information.
  */
@@ -308,7 +300,7 @@ get_cdevsw()
 {
 	char buf[16];
 	struct cdevsw *c, *tmp;
-	int i, j;
+	int i, j, len;
 	struct stat sb;
 	KA_T v[2];
 /*
@@ -363,12 +355,13 @@ get_cdevsw()
 	    }
 	    if (!buf[0])
 		continue;
-	    if (!(Cdevsw[i] = (char *)malloc(strlen(buf) + 1))) {
+	    len = strlen(buf) + 1;
+	    if (!(Cdevsw[i] = (char *)malloc(len))) {
 		(void) fprintf(stderr, "%s: no space for cdevsw[%d] name: %s\n",
 		   Pn, i, buf);
 		Exit(1);
 	    }
-	    (void) strcpy(Cdevsw[i], buf);
+	    (void) snpf(Cdevsw[i], len, "%s", buf);
 	    if (!HaveCloneMajor && strcmp(buf, "clone") == 0) {
 		CloneMajor = i;
 		HaveCloneMajor = 1;
@@ -450,7 +443,7 @@ get_kernel_access()
 	||  Npp < 1) {
 	    (void) fprintf(stderr,
 		"%s: can't read pregion count (%d) from %s\n", Pn, Npp,
-		    print_kptr(v, (char *)NULL));
+		    print_kptr(v, (char *)NULL, 0));
 	    Exit(1);
 	}
 	Prsz = (MALLOC_S)(Npp * sizeof(struct pregion));
@@ -481,7 +474,7 @@ get_kernel_access()
 	{
 	    if (!Fwarn)
 		(void) fprintf(stderr, "%s: WARNING: can't read Hz from %s\n",
-		    Pn, print_kptr(v, (char *)NULL));
+		    Pn, print_kptr(v, (char *)NULL, 0));
 	    Hz = -1;
 	}
 	if (get_Nl_value("lbolt", Drive_Nl, &Lbolt) < 0 || !v
@@ -490,7 +483,7 @@ get_kernel_access()
 	    if (!Fwarn)
 		(void) fprintf(stderr,
 		     "%s: WARNING: can't read lightning bolt timer from %s\n",
-		     Pn, print_kptr(v, (char *)NULL));
+		     Pn, print_kptr(v, (char *)NULL, 0));
 	    Lbolt = (KA_T)0;
 	}
 /*
@@ -527,7 +520,7 @@ get_kernel_access()
 	{
 	    (void) fprintf(stderr,
 		"%s: bad extended device table size (%d) at %s.\n",
-		Pn, nxdevmaps, print_kptr(v, (char *)NULL));
+		Pn, nxdevmaps, print_kptr(v, (char *)NULL, 0));
 	    Exit(1);
 	}
 	len = (MALLOC_S)((nxdevmaps + 1) * sizeof(struct XDEVMAP));
@@ -625,7 +618,7 @@ get_nlist_path(ap)
 		len = strlen(dir[i]) + strlen(de->d_name) + 1;
 		if (len >= sizeof(b2))
 		    continue;
-		(void) sprintf(b2, "%s%s", dir[i], de->d_name);
+		(void) snpf(b2, sizeof(b2), "%s%s", dir[i], de->d_name);
 		if (bfp && strcmp(b2, bfp) == 0)
 		    continue;
 	    /*
@@ -650,7 +643,7 @@ get_nlist_return_path:
 			    Pn, len , pp);
 			Exit(1);
 		    }
-		    (void) strcpy(tp, pp);
+		    (void) snpf(tp, len, "%s", pp);
 		    return(tp);
 		}
 	    }
@@ -887,7 +880,7 @@ process_text(prp)
  		ty = " m86";
  		break;
   	    default:		/* all others as a hex number */
-		(void) sprintf(tyb, " M%02x", p->p_type & 0xff);
+		(void) snpf(tyb, sizeof(tyb), " M%02x", p->p_type & 0xff);
  		ty = tyb;
 	    }
 	    if (ty) {
@@ -908,7 +901,7 @@ static void
 readfsinfo()
 {
 	char buf[FSTYPSZ+1];
-	int i;
+	int i, len;
 
 	if ((Fsinfomax = sysfs(GETNFSTYP)) == -1) {
 	    (void) fprintf(stderr, "%s: sysfs(GETNFSTYP) error: %s\n",
@@ -929,12 +922,13 @@ readfsinfo()
 		Exit(1);
 	    }
 	    buf[FSTYPSZ] = '\0';
-	    if (!(Fsinfo[i-1] = (char *)malloc((MALLOC_S)(strlen(buf)+1)))) {
+	    len = strlen(buf) + 1;
+	    if (!(Fsinfo[i-1] = (char *)malloc((MALLOC_S)len))) {
 		(void) fprintf(stderr,
 		    "%s: no space for file system entry %s\n", Pn, buf);
 		Exit(1);
 	    }
-	    (void) strcpy(Fsinfo[i-1], buf);
+	    (void) snpf(Fsinfo[i-1], len, "%s", buf);
 	}
 }
 
@@ -1623,7 +1617,7 @@ ncache_lookup(buf, blen, fp)
 		    return(*cp ? cp : (char *)NULL);
 		cp = buf + blen - nl - 1;
 		rlen = blen - nl - 1;
-		(void) strcpy(cp, lc->nm);
+		(void) snpf(cp, nl + 1, "%s", lc->nm);
 	    /*
 	     * Look up the NFS name cache entries that are parents of the
 	     * rnode address.  Quit when:
@@ -1665,7 +1659,7 @@ ncache_lookup(buf, blen, fp)
 		return(*cp ? cp : (char *)NULL);
 	    cp = buf + blen - nl - 1;
 	    rlen = blen - nl - 1;
-	    (void) strcpy(cp, lc->nm);
+	    (void) snpf(cp, nl + 1, "%s", lc->nm);
 	/*
 	 * Look up the LNC name cache entries that are parents of the
 	 * device and inode number.  Quit when:
