@@ -31,7 +31,7 @@
 
 
 /*
- * $Id: dlsof.h,v 1.12 99/06/22 08:23:59 abe Exp $
+ * $Id: dlsof.h,v 1.15 2001/06/06 16:16:41 abe Exp $
  */
 
 
@@ -124,6 +124,10 @@
 # endif	/* PTXV<400 */
 
 # if	defined(HAS_NFS)
+#  if	PTXV>=451
+#undef	UDPMSGSIZE
+#include <sys/pathname.h>
+#  endif	/* PTXV>=451 */
 #include <nfs/nfs.h>
 #include <nfs/nfs_clnt.h>
 
@@ -196,6 +200,25 @@
 #include <netinet/tcp_var.h>
 
 
+# if	PTXV>=450 && PTXV<460
+/*
+ * This definition for dnlc_t is missing from the <sys/dnlc.h> of PTX 4.5[.1].
+ *
+ * Laurent Montaron <lpm@sequent.com> filed PR #250929, requesting that the
+ * dnlc_t definition be moved from vfs_dnlc.c back to <sys/dnlc.h>, and that
+ * was done at PTX 4.6.
+ */
+
+typedef struct {
+	lock_t		dnlc_mutex;
+	struct nc_hash	*dnlc_hash;
+	struct ncache	*dnlc_cache;
+	struct ncache	*dnlc_lru;
+	long		dnlc_pad[12];
+} dnlc_t;
+# endif	/* PTXV>=450 && PTXV<460 */
+
+
 #define	COMP_P		const void
 #define QSORT_P		void
 
@@ -267,6 +290,8 @@ struct mounts {
         struct mounts *next;    	/* forward link */
 };
 
+#define X_DNLCA		"dnlc_array"
+#define	X_DNLCM		"dnlc_mask"
 #define	X_NCACHE	"ncache"
 #define	X_NCSIZE	"ncsize"
 #define	NL_NAME		n_name
@@ -278,6 +303,7 @@ struct sfile {
 	char *name;			/* file name (after readlink()) */
 	char *devnm;			/* device name (optional) */
 	dev_t dev;			/* device */
+	dev_t rdev;			/* raw device */
 	mode_t mode;			/* S_IFMT mode bits from stat() */
 	int type;			/* file type: 0 = file system
 				 	 *	      1 = regular file */
