@@ -32,7 +32,7 @@
 #ifndef lint
 static char copyright[] =
 "@(#) Copyright 1995 Purdue Research Foundation.\nAll rights reserved.\n";
-static char *rcsid = "$Id: dnode.c,v 1.12 99/06/22 08:24:12 abe Exp $";
+static char *rcsid = "$Id: dnode.c,v 1.15 2000/12/04 14:33:45 abe Exp $";
 #endif
 
 
@@ -49,18 +49,9 @@ static char *rcsid = "$Id: dnode.c,v 1.12 99/06/22 08:24:12 abe Exp $";
 #endif	/* PTXV<400 */
 
 
-#if	PTXV>=400
-_PROTOTYPE(static void ent_fa,(KA_T *a1, KA_T *a2, char *d));
-#endif	/* PTXV>=400 */
-
-_PROTOTYPE(static struct l_dev * finddev,(dev_t *dev, int stream));
-_PROTOTYPE(static int isip,(char *mn));
-_PROTOTYPE(static char isvlocked,(struct vnode *va));
-
-#if	TCPV>=450
-_PROTOTYPE(static int read_vl_un,(KA_T addr, int len));
-#endif	/* TCPV>=450 */
-
+/*
+ * Local structure definitions
+ */
 
 struct iptab_proto {
 	char *mn;			/* module name */
@@ -74,6 +65,23 @@ struct iptab_proto {
 };
 
 
+/*
+ * Local function prototypes
+ */
+
+#if	PTXV>=400
+_PROTOTYPE(static void ent_fa,(KA_T *a1, KA_T *a2, char *d));
+#endif	/* PTXV>=400 */
+
+_PROTOTYPE(static struct l_dev * finddev,(dev_t *dev, dev_t *rdev, int stream));
+_PROTOTYPE(static int isip,(char *mn));
+_PROTOTYPE(static char isvlocked,(struct vnode *va));
+
+#if	TCPV>=450
+_PROTOTYPE(static int read_vl_un,(KA_T addr, int len));
+#endif	/* TCPV>=450 */
+
+
 #if	PTXV>=400
 /*
  * ent_fa() - enter fattach addresses in NAME column addition
@@ -85,16 +93,18 @@ ent_fa(a1, a2, d)
 	KA_T *a2;			/* second fattach address */
 	char *d;			/* direction ("->" or "<-") */
 {
-	char buf[128], *cp;
+	char buf[128], *cp, tbuf[32];
 	MALLOC_S len;
 
 	if (Lf->nma)
 	    return;
 	if (!a1)
-	    (void) sprintf(buf, "(FA:%s%s)", d, print_kptr(*a2, (char *)NULL));
+	    (void) snpf(buf, sizeof(buf), "(FA:%s%s)", d,
+		print_kptr(*a2, (char *)NULL, 0));
 	else
-	    (void) sprintf(buf, " (FA:%s%s%s)", print_kptr(*a1, (char *)NULL),
-		d, print_kptr(*a2, (char *)NULL));
+	    (void) snpf(buf, sizeof(buf), " (FA:%s%s%s)",
+		print_kptr(*a1, tbuf, sizeof(tbuf)), d,
+		print_kptr(*a2, (char *)NULL, 0));
 	len = strlen(buf) + 1;
 	if ((cp = (char *)malloc(len)) == NULL) {
 	    (void) fprintf(stderr,
@@ -102,7 +112,7 @@ ent_fa(a1, a2, d)
 		Pn, Lp->pid, Lf->fd);
 	    Exit(1);
 	}
-	(void) strcpy(cp, buf);
+	(void) snpf(cp, len, "%s", buf);
 	Lf->nma = cp;
 }
 #endif	/* PTXV>=400 */
@@ -113,8 +123,9 @@ ent_fa(a1, a2, d)
  */
 
 static struct l_dev *
-finddev(dev, stream)
+finddev(dev, rdev, stream)
 	dev_t *dev;			/* device */
+	dev_t *rdev;			/* raw device */
 	int stream;			/* stream if 1 */
 {
 	struct clone *c;
@@ -129,14 +140,14 @@ finddev_again:
 
 #endif	/* defined(HASDCACHE) */
 
-	if ((dp = lkupdev(dev, 0, 0)))
+	if ((dp = lkupdev(dev, rdev, 0, 0)))
 	    return(dp);
 /*
  * Search for clone.
  */
 	if (stream && Clone) {
 	    for (c = Clone; c; c = c->next) {
-		if (major(*dev) == minor(Devtp[c->dx].rdev)) {
+		if (major(*rdev) == minor(Devtp[c->dx].rdev)) {
 
 #if	defined(HASDCACHE)
 		    if (DCunsafe && !Devtp[c->dx].v && !vfy_dev(&Devtp[c->dx]))
@@ -238,7 +249,6 @@ process_node(va)
 {
 	char *cp, tbuf[32];
 	static KA_T cvops = (KA_T)0;
-	dev_t dev = 0;
 	int dl, fp, j, k, l, lp, pt, px;
 	struct l_dev *dp;
 	struct fifonode f;
@@ -268,11 +278,11 @@ process_node(va)
 	struct snode s;
 	KA_T sa;
 	struct stdata sd;
-	int sin = 0;
-	int sns = 0;
+	unsigned char sin = 0;
+	unsigned char sns = 0;
+	unsigned char srn = 0;
+	unsigned char srv = 0;
 	KA_T sp;
-	int srn = 0;
-	int srv = 0;
 	char *ty;
 	enum vtype type;
 	struct vnode v, rv, uv;
@@ -386,10 +396,10 @@ read_vnode:
 	 * Read the namenode and enter the fattach addresses.
 	 */
 	    if (!v.v_data || kread((KA_T)v.v_data, (char *)&n, sizeof(n))) {
-		(void) sprintf(Namech,
+		(void) snpf(Namech, Namechl,
 		    "vnode at %s: can't read namenode (%s)",
-		    print_kptr(va, tbuf),
-		    print_kptr((KA_T)v.v_data, (char *)NULL));
+		    print_kptr(va, tbuf, sizeof(tbuf)),
+		    print_kptr((KA_T)v.v_data, (char *)NULL, 0));
 		enter_nm(Namech);
 		return;
 	    }
@@ -413,9 +423,17 @@ read_vnode:
 	/*
 	 * The file system type is unknown.
 	 */
-	    (void) sprintf(Namech,
+
+#if	PTXV>=450
+	    if ((v.v_flag & VSVTX) && !strcmp(Lf->fd, "txt")) {
+		Lf->sf = 0;
+		return;
+	    }
+#endif	/* PTXV>=450 */
+
+	    (void) snpf(Namech, Namechl,
 		"unknown file system type, v_op: %s",
-		print_kptr((KA_T)v.v_op, (char *)NULL));
+		print_kptr((KA_T)v.v_op, (char *)NULL, 0));
 	    enter_nm(Namech);
 	    return;
 	}
@@ -443,9 +461,9 @@ read_vnode:
 	    lvfs = (struct l_vfs *)NULL;
 	else {
 	    if (!(lvfs = readvfs(&v))) {
-		(void) sprintf(Namech, "bad vfs for %s at %s",
-		    print_kptr(va, tbuf),
-		    print_kptr((KA_T)v.v_vfsp, (char *)NULL));
+		(void) snpf(Namech, Namechl, "bad vfs for %s at %s",
+		    print_kptr(va, tbuf, sizeof(tbuf)),
+		    print_kptr((KA_T)v.v_vfsp, (char *)NULL, 0));
 		enter_nm(Namech);
 		return;
 	    }
@@ -459,11 +477,11 @@ read_vnode:
 	case N_CDFS:
 	    if (Selinet)
 		return;
-	    if (!v.v_data || read_cdnode(&v, lvfs, &dev)) {
-		(void) sprintf(Namech,
+	    if (!v.v_data || read_cdnode(&v, lvfs)) {
+		(void) snpf(Namech, Namechl,
 		    "vnode at %s: can't read cd_inode (%s)",
-		    print_kptr(va, tbuf),
-		    print_kptr((KA_T)v.v_data, (char *)NULL));
+		    print_kptr(va, tbuf, sizeof(tbuf)),
+		    print_kptr((KA_T)v.v_data, (char *)NULL, 0));
 		enter_nm(Namech);
 		return;
 	    }
@@ -474,11 +492,11 @@ read_vnode:
 	case N_VXFS:
 	    if (Selinet)
 		return;
-	    if (!v.v_data || read_vxinode(&v, lvfs, &dev)) {
-		(void) sprintf(Namech,
+	    if (!v.v_data || read_vxinode(&v, lvfs)) {
+		(void) snpf(Namech, Namechl,
 		    "vnode at %s: can't read vx_inode (%s)",
-		    print_kptr(va, tbuf),
-		    print_kptr((KA_T)v.v_data, (char *)NULL));
+		    print_kptr(va, tbuf, sizeof(tbuf)),
+		    print_kptr((KA_T)v.v_data, (char *)NULL, 0));
 		enter_nm(Namech);
 		return;
 	    }
@@ -499,10 +517,10 @@ read_vnode:
 	    if (Selinet)
 		return;
 	    if (!v.v_data || kread((KA_T)v.v_data, (char *)&f, sizeof(f))) {
-		(void) sprintf(Namech,
+		(void) snpf(Namech, Namechl,
 		    "vnode at %s: can't read fifonode (%s)",
-		    print_kptr(va, tbuf),
-		    print_kptr((KA_T)v.v_data, (char *)NULL));
+		    print_kptr(va, tbuf, sizeof(tbuf)),
+		    print_kptr((KA_T)v.v_data, (char *)NULL, 0));
 		enter_nm(Namech);
 		return;
 	    }
@@ -521,7 +539,7 @@ read_vnode:
 
 #if	defined(HASVXFS)
 		if (vvops && vvops == (KA_T)rv.v_op
-		&&  read_vxinode(&rv, lvfs, &dev) == 0) {
+		&&  read_vxinode(&rv, lvfs) == 0) {
 		    svx = 1;
 		    break;
 		}
@@ -540,10 +558,10 @@ read_vnode:
 	    if (Selinet)
 		return;
 	    if (!v.v_data || readrnode((KA_T)v.v_data, &r)) {
-		(void) sprintf(Namech,
+		(void) snpf(Namech, Namechl,
 		    "vnode at %s: can't read rnode (%s)",
-		    print_kptr(va, tbuf),
-		    print_kptr((KA_T)v.v_data, (char *)NULL));
+		    print_kptr(va, tbuf, sizeof(tbuf)),
+		    print_kptr((KA_T)v.v_data, (char *)NULL, 0));
 		enter_nm(Namech);
 		return;
 	    }
@@ -556,10 +574,10 @@ read_vnode:
 	    if (Selinet)
 		return;
 	    if (!v.v_data || kread((KA_T)v.v_data, (char *)&n, sizeof(n))) {
-		(void) sprintf(Namech,
+		(void) snpf(Namech, Namechl,
 		    "vnode at %s: can't read namenode (%s)",
-		    print_kptr(va, tbuf),
-		    print_kptr((KA_T)v.v_data, (char *)NULL));
+		    print_kptr(va, tbuf, sizeof(tbuf)),
+		    print_kptr((KA_T)v.v_data, (char *)NULL, 0));
 		enter_nm(Namech);
 		return;
 	    }
@@ -618,8 +636,8 @@ read_vnode:
 		     * Enter path from sockaddr_un.
 		     */
 			un.sun_path[sizeof(un.sun_path) - 1] - '\0';
-			if ((l = strlen(un.sun_path)) >= (MAXPATHLEN - 2))
-			    l = MAXPATHLEN - 2;
+			if ((l = strlen(un.sun_path)) >= (int)(Namechl - 2))
+			    l = (int)(Namechl - 2);
 			(void) strncpy(Namech, un.sun_path, l);
 			Namech[l] = '\0';
 # endif	/* TCPV>=450 */
@@ -683,9 +701,10 @@ read_vnode:
 	 * Read the snode.
 	 */
 	    if (kread(sa, (char *)&s, sizeof(s))) {
-		(void) sprintf(Namech,
+		(void) snpf(Namech, Namechl,
 		    "vnode at %s: can't read snode (%s)",
-		    print_kptr(va, tbuf), print_kptr(sa, (char *)NULL));
+		    print_kptr(va, tbuf, sizeof(tbuf)),
+		    print_kptr(sa, (char *)NULL, 0));
 		enter_nm(Namech);
 		return;
 	   }
@@ -696,10 +715,10 @@ read_vnode:
 	    * Read the "real" vnode, if there is one, then read its inode.
 	    */
 		if (readvnode((KA_T)s.s_realvp, &rv)) {
-		    (void) sprintf(Namech,
+		    (void) snpf(Namech, Namechl,
 			"snode at %s; can't read real vnode at %s",
-			print_kptr(sa, tbuf),
-			print_kptr((KA_T)s.s_realvp, (char *)NULL));
+			print_kptr(sa, tbuf, sizeof(tbuf)),
+			print_kptr((KA_T)s.s_realvp, (char *)NULL, 0));
 		    return;
 		}
 		srv = 1;
@@ -729,12 +748,12 @@ read_vnode:
 	 *    /dev/* name if it exists for s.s_dev;
 	 *    "STR:" otherwise.
 	 */
-	    (void) strcpy(Namech, "STR:");
+	    (void) snpf(Namech, Namechl, "STR:");
 	    Lf->is_stream = 1;
 	    k = strlen(Namech);
 	    cp = NULL;
-	    if ((dp = finddev(&s.s_dev, 1)) != NULL) {
-		(void) strcpy(&Namech[k], dp->name);
+	    if ((dp = finddev(&DevDev, &s.s_dev, 1)) != NULL) {
+		(void) snpf(&Namech[k], Namechl - k, "%s", dp->name);
 		k += strlen(dp->name);
 		if ((cp = strrchr(dp->name, '/')) != NULL)
 		    cp++;
@@ -842,7 +861,8 @@ read_vnode:
 				p = (KA_T)q.q_ptr;
 #endif	/* TCPV>=420 */
 
-				(void) strcpy(Lf->iproto, IPtab[px].nm);
+				(void) snpf(Lf->iproto, sizeof(Lf->iproto),
+				    "%s", IPtab[px].nm);
 			    }
 			}
 			continue;
@@ -850,14 +870,14 @@ read_vnode:
 		    if (q.q_ptr && strcasecmp(tbuf, "sockmod") == 0 && !sop)
 			sop = (KA_T)q.q_ptr;
 		    if (k) {
-			if ((k + 2) > (MAXPATHLEN - 1))
+			if ((k + 2) > (int)(Namechl - 1))
 			    break;
-			(void) strcpy(&Namech[k], "->");
+			(void) snpf(&Namech[k], Namechl - k, "->");
 			k += 2;
 		    }
-		    if ((k + l) > (MAXPATHLEN - 1))
+		    if ((k + l) > (int)(Namechl - 1))
 			break;
-		    (void) strcpy(&Namech[k], tbuf);
+		    (void) snpf(&Namech[k], Namechl - k, "%s", tbuf);
 		    k += l;
 		}
 	    }
@@ -873,7 +893,7 @@ read_vnode:
 		    break;
 		if (Fnet)
 		    Lf->sf |= SELNET;
-		if ((k + 1) > (MAXPATHLEN - 1))
+		if ((k + 1) > (int)(Namechl - 1))
 		    break;
 		Namech[k++] = ' ';
 		Namech[k] = '\0';
@@ -915,10 +935,10 @@ read_vnode:
 	    if (v.v_type != VBLK) {
 		if (!v.v_data
 		||  kread((KA_T)v.v_data, (char *)&i, sizeof(i))) {
-		    (void) sprintf(Namech,
+		    (void) snpf(Namech, Namechl,
 			"vnode at %s: can't read inode (%s)",
-			print_kptr(va, tbuf),
-			print_kptr((KA_T)v.v_data, (char *)NULL));
+			print_kptr(va, tbuf, sizeof(tbuf)),
+			print_kptr((KA_T)v.v_data, (char *)NULL, 0));
 		    enter_nm(Namech);
 		    return;
 		}
@@ -950,8 +970,8 @@ read_vnode:
 
 #if	defined(HAS_NFS)
 	case N_NFS:
-	    if (lvfs) {
-		dev = lvfs->dev;
+	    if (lvfs && lvfs->fsname) {
+		Lf->dev = lvfs->dev;
 		Lf->dev_def = 1;
 	    }
 	    break;
@@ -959,36 +979,36 @@ read_vnode:
 
 #if	PTXV>=400
 	case N_NM:
-	    dev = n.nm_vattr.va_fsid;
+	    Lf->dev = n.nm_vattr.va_fsid;
 	    Lf->dev_def = 1;
 	    break;
 #endif	/* PTXV>=400 */
 
 	case N_FIFO:
-	    if (srv && sin && lvfs) {
-		dev = lvfs->dev;
+	    if (srv && sin && lvfs && lvfs->fsname) {
+		Lf->dev = lvfs->dev;
 		Lf->dev_def = 1;
 	    }
 
 #if	defined(HAS_NFS)
-	    else if (srv && srn && lvfs) {
-		dev = lvfs->dev;
+	    else if (srv && srn && lvfs && lvfs->fsname) {
+		Lf->dev = lvfs->dev;
 		Lf->dev_def = 1;
 	    }
 #endif	/* defined(HAS_NFS) */
 
 #if	defined(HASVXFS)
 	    else if (srv && svx)
-		Lf->dev_def = 1;	/* dev is set in read_vxnode() */
+		break;			/* Lf->dev_def set in read_vxinode() */
 #endif	/* defined(HASVXFS) */
 
 	    else
-		enter_dev_ch(print_kptr((KA_T)va, (char *)NULL));
+		enter_dev_ch(print_kptr((KA_T)va, (char *)NULL, 0));
 	    break;
 
 #if	defined(HASVXFS)
 	case N_VXFS:
-	    /* dev is set in read_vxnode() */
+	    /* dev is set in read_vxinode() */
 	    break;
 #endif	/* defined(HASVXFS) */
 
@@ -997,33 +1017,37 @@ read_vnode:
 	    if (v.v_rdev == NODEV)
 		enter_dev_ch("NODEV");
 	    else {
-		dev = v.v_rdev;
-		Lf->dev_def = 1;
+		Lf->dev = DevDev;
+		Lf->rdev = v.v_rdev;
+		Lf->dev_def = Lf->rdev_def = 1;
 	    }
 	    break;
 #endif	/* TCPV>=420 */
 
 	case N_SPEC:
+	    if (lvfs && lvfs->fsname) { 
+		Lf->dev = lvfs->dev;
+		Lf->dev_def = 1;
+	    }
 	    if (sop)
-		dev = so.lux_dev.addr.tu_addr.dev;
+		Lf->rdev = so.lux_dev.addr.tu_addr.dev;
 	    else
-		dev = sns ? s.s_dev : v.v_rdev;
-	    Lf->dev_def = 1;
+		Lf->rdev = sns ? s.s_dev : v.v_rdev;
+	    Lf->rdev_def = 1;
 	    break;
 	default:
-	    if (type == VCHR || type == VBLK) {
-		dev = v.v_rdev;
+	    if (srv) {
+		Lf->dev = rv.v_rdev;
 		Lf->dev_def = 1;
-	    } else {
-		if (srv) {
-		    dev = rv.v_rdev;
-		    Lf->dev_def = 1;
-		} else if (i.i_devvp
-		       &&  !kread((KA_T)i.i_devvp, (char *)&uv, sizeof(uv)))
-		{
-		    dev = uv.v_rdev;
-		    Lf->dev_def = 1;
-		}
+	    } else if (i.i_devvp
+		   &&  !kread((KA_T)i.i_devvp, (char *)&uv, sizeof(uv)))
+	    {
+		Lf->dev = uv.v_rdev;
+		Lf->dev_def = 1;
+	    }
+	    if ((type == VCHR) || (type == VBLK)) {
+		Lf->rdev = v.v_rdev;
+		Lf->rdev_def = 1;
 	    }
 	}
 /*
@@ -1039,7 +1063,7 @@ read_vnode:
 
 #if	defined(HASVXFS)
 	case N_VXFS:
-	    /* Lf->inode and Lf->inp_ty are set in read_vxnode() */
+	    /* Lf->inode and Lf->inp_ty are set in read_vxinode() */
 	    break;
 #endif	/* defined(HASVXFS) */
 
@@ -1058,7 +1082,7 @@ read_vnode:
 
 #if	defined(HASVXFS)
 	    else if (svx)
-		Lf->inp_ty = 1;		/* inode set in read_vxnode() */
+		Lf->inp_ty = 1;		/* inode set in read_vxinode() */
 #endif	/* defined(HASVXFS) */
 
 	     break;
@@ -1088,7 +1112,7 @@ read_vnode:
 	case N_SOCK:
 	    /* The sock_vnodeops type vnode has no inode number. */
 	    if (!Lf->iproto[0]) {
-		(void) strcpy(Lf->iproto, "SOCK");
+		(void) snpf(Lf->iproto, sizeof(Lf->iproto), "SOCK");
 		Lf->inp_ty = 2;
 	    }
 	    break;
@@ -1117,7 +1141,7 @@ read_vnode:
 
 #if	defined(HASVXFS)
 	     case N_VXFS:
-		/* Lf->sz and Lf->sz_def are set in read_vxnode() */
+		/* Lf->sz and Lf->sz_def are set in read_vxinode() */
 		break;
 #endif	/* defined(HASVXFS) */
 
@@ -1125,13 +1149,13 @@ read_vnode:
 
 #if	defined(HASVXFS)
 		if (srv && svx)
-		    Lf->sz_def = 0;	/* unset what read_vxnode() set */
+		    Lf->sz_def = 0;	/* unset what read_vxinode() set */
 #endif	/* defined(HASVXFS) */
 
 
 #if	PTXV<400
 		if (Fsize || (Lf->access != 'r' && Lf->access != 'w')) {
-		    (void) sprintf(Namech, "rd=%#x; wr=%#x",
+		    (void) snpf(Namech, Namechl, "rd=%#x; wr=%#x",
 			f.fn_rptr, f.fn_wptr);
 		    Lf->sz = (SZOFFTYPE)f.fn_size;
 		    Lf->sz_def = 1;
@@ -1139,9 +1163,9 @@ read_vnode:
 		}
 		Lf->off = (unsigned long)
 			  (Lf->access == 'r') ? f.fn_rptr : f.fn_wptr;
-		(void) sprintf(Namech, "%s=%#x",
-			       (Lf->access == 'r') ? "wr" : "rd",
-			       (Lf->access == 'r') ? f.fn_wptr : f.fn_rptr);
+		(void) snpf(Namech, Namechl, "%s=%#x",
+		    (Lf->access == 'r') ? "wr" : "rd",
+		    (Lf->access == 'r') ? f.fn_wptr : f.fn_rptr);
 #endif	/* PTXV<400 */
 
 		Lf->off_def = 1;
@@ -1204,7 +1228,7 @@ read_vnode:
 
 #if	defined(HASVXFS)
 	    case N_VXFS:
-		/* Lf->nlink and Lf->nlink_def are set in read_vxnode() */
+		/* Lf->nlink and Lf->nlink_def are set in read_vxinode() */
 	    break;
 #endif	/* defined(HASVXFS) */
 
@@ -1280,16 +1304,13 @@ read_vnode:
 	case VREG:
 	case VDIR:
 	    ty = (type == VREG) ? "VREG" : "VDIR";
-	    Lf->dev = dev;
 	    break;
 	case VBLK:
 	    ty = "VBLK";
-	    Lf->dev = dev;
 	    Ntype = N_BLK;
 		break;
 	case VCHR:
 	    ty = "VCHR";
-	    Lf->dev = dev;
 	    if (Ntype != N_SPEC || !sop)
 		Ntype = N_CHR;
 	    break;
@@ -1298,25 +1319,23 @@ read_vnode:
 	    break;
 	case VSOCK:
 	    ty = usst ? "unix" : "SOCK";
-	    Lf->dev = dev;
 	    break;
 	case VBAD:
 	    ty = "VBAD";
 	    break;
 	case VFIFO:
-	    Lf->dev = dev;
 	    ty = "FIFO";
 	    break;
 	default:
 	    if (type > 9999)
-		(void) sprintf(Lf->type, "*%03d", type % 1000);
+		(void) snpf(Lf->type, sizeof(Lf->type), "*%03d", type % 1000);
 	    else
-		(void) sprintf(Lf->type, "%4d", type);
-	    (void) strcpy(Namech, "unknown type");
+		(void) snpf(Lf->type, sizeof(Lf->type), "%4d", type);
+	    (void) snpf(Namech, Namechl, "unknown type");
 	    ty = NULL;
 	}
 	if (ty)
-	    (void) strcpy(Lf->type, ty);
+	    (void) snpf(Lf->type, sizeof(Lf->type), "%s", ty);
 	Lf->ntype = Ntype;
 
 #if	defined(HASBLKDEV)
@@ -1324,7 +1343,7 @@ read_vnode:
  * If this is a VBLK file and it's missing an inode number, try to
  * supply one.
  */
-	if (Lf->inp_ty == 0 && type == VBLK && Lf->dev_def)
+	if ((Lf->inp_ty == 0) && (type == VBLK))
 	    find_bl_ino();
 #endif	/* defined(HASBLKDEV) */
 
@@ -1332,7 +1351,7 @@ read_vnode:
  * If this is a VCHR file and it's missing an inode number, try to
  * supply one.
  */
-	if (Lf->inp_ty == 0 && type == VCHR && Lf->dev_def)
+	if ((Lf->inp_ty == 0) && (type == VCHR))
 	    find_ch_ino();
 
 #if	PTXV>=400
@@ -1342,8 +1361,8 @@ read_vnode:
  * address there.
  */
 	if (Ntype == N_FIFO && f.fn_mate && Namech[0] == '\0' && !sin && !srn)
-	    (void) sprintf(Namech, "->%s",
-		print_kptr((KA_T)f.fn_mate, (char *)NULL));
+	    (void) snpf(Namech, Namechl, "->%s",
+		print_kptr((KA_T)f.fn_mate, (char *)NULL, 0));
 #endif	/* PTXV>=400 */
 
 /*
@@ -1353,7 +1372,7 @@ read_vnode:
 	if (Ntype == N_SPEC && sop) {
 	    if (Funix)
 		Lf->sf |= SELUNX;
-	    (void) strcpy(Lf->type, "unix");
+	    (void) snpf(Lf->type, sizeof(Lf->type), "unix");
 	    if ((Namech[0] == '\0' || !strncmp(Namech, "STR:", strlen("STR:")))
 	    &&  so.laddr.buf)
 	    {
@@ -1371,7 +1390,7 @@ read_vnode:
 		&& !kread((KA_T)so.laddr.buf, (char *)&ua, sizeof(ua)))
 		{
 		    ua.sun_path[sizeof(ua.sun_path) - 1] = '\0';
-		    (void) strcpy(Namech, ua.sun_path);
+		    (void) snpf(Namech, Namechl, "%s", ua.sun_path);
 		    if (is_file_named(Namech, 0))
 			Lf->sf |= SELNM;
 		}
@@ -1382,7 +1401,8 @@ read_vnode:
 /*
  * Test for specified file.
  */
-	if (Sfile && is_file_named(NULL, (type == VCHR) ? 1 : 0))
+	if (Sfile && is_file_named((char *)NULL,
+				   ((type == VCHR) || (type ==VBLK)) ? 1 : 0))
 	    Lf->sf |= SELNM;
 /*
  * Enter name characters.
@@ -1429,11 +1449,7 @@ read_vl_un(addr, len)
  * Enter path from variable length sockaddr_un.
  */
 	if (!kread(addr, (char *)vu, len)) {
-	    nml = len - offsetof(struct sockaddr_un, sun_path);
-	    if (nml > (MAXPATHLEN - 2))
-		nml = MAXPATHLEN - 2;
-	    (void) strncpy(Namech, vu->sun_path, nml);
-	    Namech[nml - 1] = '\0';
+	    (void) snpf(Namech, Namechl, "%s", vu->sun_path);
 	    return(1);
 	}
 	return(0);
