@@ -32,7 +32,7 @@
 #ifndef lint
 static char copyright[] =
 "@(#) Copyright 1995 Purdue Research Foundation.\nAll rights reserved.\n";
-static char *rcsid = "$Id: dproc.c,v 1.11 99/06/22 08:24:28 abe Exp $";
+static char *rcsid = "$Id: dproc.c,v 1.14 2000/11/03 18:14:15 abe Exp $";
 #endif
 
 #include "lsof.h"
@@ -60,6 +60,9 @@ _PROTOTYPE(static void get_kernel_access,(void));
 #if	PTXV>=400
 _PROTOTYPE(static void process_text,(KA_T xv, KA_T as));
 _PROTOTYPE(static void store_va,(KA_T va));
+# if	PTXV>=450
+_PROTOTYPE(static KA_T mhp2vp,(KA_T mhp));
+# endif	/* PTXV>=450 */
 #endif	/* PTXV>=400 */
 
 
@@ -93,6 +96,11 @@ gather_proc_info()
 #else	/* PTXV>=400 */
 	struct ucred pc;
 	struct proc_data pd;
+# if	PTXV>=450
+	KA_T cv = (KA_T)NULL;
+	proc_resrc_t pr;
+	int prs = 0;
+# endif	/* PTXV>=450 */
 #endif	/* PTXV<400 */
 
 #if	PTXV<400
@@ -122,14 +130,6 @@ gather_proc_info()
 		Exit(1);
 	    }
 	}
-
-#if     defined(HASNCACHE)
-/*
- * Read kernel name cache.
- */
-        ncache_load();
-#endif  /* defined(HASNCACHE) */
-
 /*
  * Examine proc structures and their associated information.
  */
@@ -141,8 +141,11 @@ gather_proc_info()
 	    pa = (KA_T)(Kp + (long)(px * sizeof(struct proc)));
 	    if (kread(pa, (char *)&ps, sizeof(ps)))
 		continue;
+
+#if	PTXV<450
 	    if (p->p_stat == 0 || p->p_stat == SZOMB)
 		continue;
+#endif	/* PTXV<450 */
 
 #if	PTXV<400
 	/*
@@ -223,11 +226,32 @@ gather_proc_info()
 #endif	/* PTXV<400 */
 
 	    {
+
+#if	PTXV>=450
+		cv = va;
+#endif	/* PTXV>=450 */
+
 		alloc_lfile(CWD, -1);
 		process_node(va);
 		if (Lf->sf)
 		    link_lfile();
 	    }
+
+#if	PTXV>=450
+	    else if ((va = (KA_T)p->p_resrcl.l_flink)) {
+		if (!kread(va, (char *)&pr, sizeof(pr))) {
+		    prs = 1;
+		    if ((va = (KA_T)pr.pr_cdir)) {
+			cv = va;
+			alloc_lfile(CWD, -1);
+			process_node(va);
+			if (Lf->sf)
+			    link_lfile();
+		    }
+		}
+	    }
+#endif	/* PTXV>=450 */
+
 	/*
 	 * Save root directory information.
 	 */
@@ -235,7 +259,11 @@ gather_proc_info()
 #if	PTXV<400
 	    if ((va = (KA_T)u->u_rdir))
 #else	/* PTXV>=400 */
+# if	PTXV<450
 	    if ((va = (KA_T)p->p_rdir))
+# else	/* PTXV>=450 */
+	    if ((va = (KA_T)p->p_rdir) && (va != cv))
+# endif	/* PTXV<450 */
 #endif	/* PTXV<400 */
 
 	    {
@@ -245,12 +273,32 @@ gather_proc_info()
 		    link_lfile();
 	    }
 
+#if	PTXV>=450
+	    else if (prs || (va = (KA_T)p->p_resrcl.l_flink)) {
+		if (prs || !kread(va, (char *)&pr, sizeof(pr))) {
+		    if ((va = (KA_T)pr.pr_rdir) && (va != cv)) {
+			alloc_lfile(RTD, -1);
+			process_node(va);
+			if (Lf->sf)
+			    link_lfile();
+		    }
+		}
+	    }
+#endif	/* PTXV>=450 */
+
+
 #if	PTXV>=400
 	/*
 	 * Process PTX 4 and above executable and library text accesses.
 	 */
+
+# if	PTXV<450
 	    if (p->p_execvp || p->p_as)
 		process_text((KA_T)p->p_execvp, (KA_T)p->p_as);
+# else	/* PTXV>=450 */
+	    if (pd.pd_execvp || p->p_as)
+		process_text((KA_T)pd.pd_execvp, (KA_T)p->p_as);
+# endif	/* PTXV<450 */
 #endif	/* PTXV>=400 */
 
 	/*
@@ -432,7 +480,7 @@ get_kernel_access()
 	if (get_Nl_value("nofile", Drive_Nl, &v) < 0 || !v
 	||  kread((KA_T)v, (char *)&Nf, sizeof(Nf)) || Nf < 3) {
 	    (void) fprintf(stderr, "%s: bad open file count (%d) from %s\n",
-		Pn, Nf, print_kptr(v, (char *)NULL));
+		Pn, Nf, print_kptr(v, (char *)NULL, 0));
 	    Exit(1);
 	}
 /*
@@ -512,6 +560,24 @@ kread(addr, buf, len)
 	br = read(Kd, buf, len);
 	return((br == len) ? 0 : 1);
 }
+  
+  
+#if	PTXV>=450
+/*
+ * mhp2vp() - connvert mhandle pointer to vnode pointer
+ */
+
+static KA_T
+mhp2vp(mhp)
+	KA_T mhp;				/* mhandle pointer */
+{
+	struct mfile mh;
+
+	if (!mhp || kread(mhp, (char *)&mh, sizeof(mh)))
+	    return((KA_T)NULL);
+	return((KA_T)mh.mf_vp);
+}
+#endif	/* PTXV>=450 */
 
 
 #if	PTXV>=400
@@ -529,6 +595,8 @@ process_text(xv, as)
 	int i, j;
 	MALLOC_S len;
 	static int nd = 0;
+	KA_T va;
+
 /*
  * Save the address of the executable vnode.
  */
@@ -545,11 +613,16 @@ process_text(xv, as)
 	 * Save the vnodes of the short, static list.
 	 */
 	    if (Asopsm && Asopsm == (KA_T)a.as_ops) {
-		for (i = j = 0; i < AS_SVSM_MMAP_MAX; i++) {
-		    if (a.as_mvp.mvp_small[i]) {
-			store_va((KA_T)a.as_mvp.mvp_small[i]);
-			j++;
-		    }
+		for (i = 0; i < AS_SVSM_MMAP_MAX; i++) {
+
+#if	PTXV<450
+		    va = (KA_T)a.as_mvp.mvp_small[i];
+#else	PTXV>=450
+		    va = mhp2vp((KA_T)a.as_mhandle.handle_small[i]);
+#endif	/* PTXV<450 */
+
+		    if (va)
+			store_va(va);
 		} 
 	    }
 	/*
@@ -557,34 +630,44 @@ process_text(xv, as)
 	 * the addresses of the vnodes it contains.
 	 */
 	    if (Asoplg && Asoplg == (KA_T)a.as_ops) {
-		if ((i = a.as_mvp.mvp_dynamic_size) > 0 && a.as_mvp.mvp_large) {
+
+#if	PTXV<450
+		if (((i = a.as_mvp.mvp_dynamic_size) > 0)
+		&&  (va = (KA_T)a.as_mvp.mvp_large))
+#else	PTXV>=450
+		if (((i = a.as_mhandle.handle_dynamic_size) > 0)
+		&&  (va = (KA_T)a.as_mhandle.handle_large))
+#endif	/* PTXV<450 */
+
+		{
 		    len = (MALLOC_S)(i * sizeof(KA_T));
 		    if (!da) {
-			if (!(da = (KA_T *)malloc(len))) {
-
-no_vm_vp_space:
-
-			    (void) fprintf(stderr,
-				"%s: no space for dynamic vnodes: PID %d\n",
-				Pn, Lp->pid);
-			    Exit(1);
-			}
+			da = (KA_T *)malloc(len);
 			nd = i;
 		    } else if (i >= nd) {
-			if (!(da = (KA_T *)realloc(da, len)))
-			    goto no_vm_vp_space;
+			da = (KA_T *)realloc((MALLOC_P *)da, len);
 			nd = i;
 		    }
-		    if (!kread((KA_T)a.as_mvp.mvp_large, (char *)da,
-			(READLEN_T)len))
-		    {
+		    if (!da) {
+			(void) fprintf(stderr,
+			    "%s: no space for dynamic vnodes: PID %d\n",
+			    Pn, Lp->pid);
+			Exit(1);
+		    }
+		    if (!kread(va, (char *)da, (READLEN_T)len)) {
 			for (j = 0; j < i; j++) {
-			    if (da[j])
-				store_va(da[j]);
+
+#if	PTXV<450
+			    if ((va = da[j]))
+#else	/* PTXV>=450 */
+			    if ((va = mhp2vp(da[j])))
+#endif	/* PTXV<450 */
+
+				store_va(va);
 			}
 		    }
 		}
-	   }
+	    }
 	}
 /*
  * Process the unique vnode addresses accumulated from the executable,
@@ -843,10 +926,10 @@ LNC_isroot(np, cp, ty)
 	    }
 	    if (!mtp)
 		return(0);
-	    (void) strcpy(buf, Lf->fsdir);
+	    (void) snpf(buf, sizeof(buf), "%s", Lf->fsdir);
 	    if (buf[len - 1] != '/')
 		buf[len++] = '/';
-	    (void) strcpy(&buf[len], cp);
+	    (void) snpf(&buf[len], sizeof(buf) - len, "%s", cp);
 	    if (statsafely(buf, &sb) != 0
 	    ||  (unsigned long)sb.st_ino != Lf->inode)
 		return(0);
@@ -1069,7 +1152,7 @@ ncache_lookup(buf, blen, fp)
 	    return(NULL);
 	cp = buf + blen - nl - 1;
 	rlen = blen - nl - 1;
-	(void) strcpy(cp, lc->nm);
+	(void) snpf(cp, nl + 1, "%s", lc->nm);
 /*
  * Look up the name cache entries that are parents of the node address.
  * Quit when:
@@ -1104,14 +1187,25 @@ ncache_lookup(buf, blen, fp)
 static void
 VN_load()
 {
-	struct ncache *cp;
-	int i, kcl, kcn, len;
+	int i, kcl, kcn, len, nce;
 	KA_T ka;
 	static struct ncache *kc = (struct ncache *)NULL;
 	static KA_T kcp = (KA_T)0;
 	static int kcla = 0;
+	struct ncache *kctp;
 	static KA_T kla = (KA_T)0;
 	struct lnch lc;
+	static int nca = 1;
+
+#if	PTXV>=450
+	char *cp;
+	dnlc_t dnlc;
+	static KA_T dnlcma = (KA_T)0;
+	static KA_T kcap = (KA_T)0;
+	static KA_T kcapa = (KA_T)0;
+	int nb;
+#endif	/* PTXV>=450 */
+
 /*
  * Do first-time setup, as required.
  */
@@ -1123,18 +1217,34 @@ VN_load()
 	 */
 	    if (get_Nl_value(X_NCSIZE, Drive_Nl, &kla) < 0 || !kla)
 		return;
+
+# if	PTXV<450
 	    if (get_Nl_value(X_NCACHE, Drive_Nl, &kcp) < 0 || !kcp)
 		return;
+# else	/* PTXV>=450 */
+	    if (get_Nl_value(X_DNLCA, Drive_Nl, &kcapa) < 0 || !kcapa)
+		return;
+	    if (get_Nl_value(X_DNLCM, Drive_Nl, &dnlcma) < 0 || !dnlcma)
+		return;
+	    if (kread(dnlcma, (char *)&nca, sizeof(nca)) || (nca < 0))
+		return;
+	    nca++;
+# endif	/* PTXV<450 */
+
 	}
 /*
- * Get kernel's vnode name cache length.
+ * Get kernel's vnode name cache length.  For PTX 4.5 and above, that's the
+ * number of entries in each DNLC array.
  */
-	if (kread(kla, (char *)&kcn, sizeof(kcn)) || kcn < 1)
+	if (kread(kla, (char *)&kcn, sizeof(kcn)) || kcn < 0)
 	    return;
+	kcn++;
 /*
- * Allocate space for a copy of the kernel's vnode name cache.
+ * Allocate space for copies of the kernel's vnode name cache(s).  For PTX
+ * below 4.5, there's but one cache.  PTX 4.5 and above may have multiple
+ * caches.
  */
-	kcl = kcn * sizeof(struct ncache);
+	kcl = kcn * nca * sizeof(struct ncache);
 	if (!kc) {
 	    kc = (struct ncache *)malloc(kcl);
 	    kcla = kcl;
@@ -1149,29 +1259,59 @@ VN_load()
 	    Exit(1);
 	}
 /*
- * Read the kernel's vnode name cache.
+ * Read the kernel's vnode name cache(s).  For PTX below 4.5 there is but one
+ * cache.  PTX 4.5 and above may have multiple caches.  There we must read
+ * the cache addresses from the kernel's DNLC array table.
  */
+
+# if	PTXV<450
 	if (!kc || !kcl || !kcp || kread(kcp, (char *)&ka, sizeof(ka)))
 	    return;
 	if (!ka || kread(ka, (char *)kc, kcl))
 	    return;
+	nce = kcn;
+# else	/* PTXV>=450 */
+	if (!kc || !kcl || !kcapa
+	||  kread(kcapa,(char *)&kcap, sizeof(kcap))
+	||  !kcap)
+	    return;
+	for (i = nce = 0, cp = (char *)kc,
+		 nb = (int)(kcn * sizeof(struct ncache));
+	     i < nca;
+	     i++, kcap = (KA_T)((char *)kcap + sizeof(dnlc_t)))
+	{
+	    if (kread((KA_T)kcap, (char *)&dnlc, sizeof(dnlc)))
+		return;
+	    if (!(ka = (KA_T)dnlc.dnlc_cache))
+		continue;
+	    if (kread(ka, cp, nb))
+		continue;
+	    cp += nb;
+	    nce += kcn;
+	}
+	if (!nce)
+	    return;
+# endif	/* PTXV<450 */
+
 /*
  * Scan the kernel's vnode cache and create local name cache entries.
  */
 	lc.type = 0;
-	for (cp = kc, i = 0; i < kcn; cp++, i++) {
-	    if (!cp->NCACHE_VP || !cp->NCACHE_NAME[0] || cp->NCACHE_NAMLEN < 1)
+	for (kctp = kc, i = 0; i < nce; kctp++, i++) {
+	    if (!kctp->NCACHE_VP || !kctp->NCACHE_NAME[0]
+	    ||  kctp->NCACHE_NAMLEN < 1)
 		continue;
-	    if (cp->NCACHE_NAMLEN < 3 && cp->NCACHE_NAME[0] == '.') {
-		if (cp->NCACHE_NAMLEN == 1
-		||  (cp->NCACHE_NAMLEN == 2 && cp->NCACHE_NAME[1] == '.'))
+	    if ((kctp->NCACHE_NAMLEN < 3) && (kctp->NCACHE_NAME[0] == '.')) {
+		if ((kctp->NCACHE_NAMLEN == 1)
+		||  ((kctp->NCACHE_NAMLEN == 2)
+		&&   (kctp->NCACHE_NAME[1] == '.')))
 		    continue;
 	    }
-	    len = (cp->NCACHE_NAMLEN > NC_NAMLEN) ? NC_NAMLEN
-						  : cp->NCACHE_NAMLEN;
-	    lc.np = (KA_T)cp->NCACHE_VP;
-	    lc.dp = (KA_T)cp->NCACHE_DP;
-	    LNC_enter(&lc, cp->NCACHE_NAME, len);
+	    len = (kctp->NCACHE_NAMLEN > NC_NAMLEN) ? NC_NAMLEN
+						    : kctp->NCACHE_NAMLEN;
+	    lc.np = (KA_T)kctp->NCACHE_VP;
+	    lc.dp = (KA_T)kctp->NCACHE_DP;
+	    LNC_enter(&lc, kctp->NCACHE_NAME, len);
 	}
 /*
  * If not repeating, free space allocated for the copy of the kernel's
