@@ -33,7 +33,7 @@
 
 
 /*
- * $Id: dproto.h,v 1.4 99/01/25 07:12:22 abe Exp $
+ * $Id: dproto.h,v 1.5 2000/12/04 14:33:45 abe Exp $
  */
 
 
@@ -41,6 +41,7 @@ _PROTOTYPE(extern void completevfs,(struct l_vfs *vfs, dev_t *dev));
 _PROTOTYPE(extern void findclones,(void));
 _PROTOTYPE(extern int is_file_named,(char *p, int cd));
 _PROTOTYPE(extern void process_socket,(char *pn, int fam, int ipid, KA_T pa, void *lsp));
+_PROTOTYPE(extern int read_cdnode,(struct vnode *v, struct l_vfs *vfs));
 _PROTOTYPE(extern int readinode,(KA_T ia, struct inode *i));
 _PROTOTYPE(extern struct l_vfs *readvfs,(struct vnode *va));
 
@@ -53,7 +54,7 @@ _PROTOTYPE(extern void process_unix_socket,(struct socket *s, struct unpcb *u));
 #endif	/* PTXV>=421 */
 
 #if	defined(HASVXFS)
-_PROTOTYPE(extern int read_vxinode,(struct vnode *v, struct l_vfs *vfs, dev_t *dev));
+_PROTOTYPE(extern int read_vxinode,(struct vnode *v, struct l_vfs *vfs));
 # if	defined(HASNCACHE) && defined(HASVXFSDNLC)
 _PROTOTYPE(extern int read_vxvnode,(KA_T va, struct vnode *v));
 # endif	/* defined(HASNCACHE) && defined(HASVXFSDNLC) */
