@@ -32,7 +32,7 @@
 #ifndef lint
 static char copyright[] =
 "@(#) Copyright 1995 Purdue Research Foundation.\nAll rights reserved.\n";
-static char *rcsid = "$Id: dsock.c,v 1.10 99/01/25 07:12:40 abe Exp $";
+static char *rcsid = "$Id: dsock.c,v 1.11 2000/08/01 14:21:59 abe Exp $";
 #endif
 
 
@@ -143,7 +143,7 @@ print_tcptpi(nl)
 		    cp = "TS_WACK_DREQ11";
 		    break;
 		default:
-		    (void) sprintf(buf, "UNKNOWN_TPI_STATE_%d", s);
+		    (void) snpf(buf, sizeof(buf), "UNKNOWN_TPI_STATE_%d", s);
 		    cp = buf;
 		}
 #else	/* TCPV>=450 */
@@ -275,16 +275,16 @@ process_socket(pn, fam, ipid, pa, lsp)
 /*
  * Enter protocol name and address.
  */
-	(void) strcpy(Lf->iproto, pn);
+	(void) snpf(Lf->iproto, sizeof(Lf->iproto), "%s", pn);
 	Lf->inp_ty = 2;
 	if (pa)
-	    enter_dev_ch(print_kptr(pa, (char *)NULL));
+	    enter_dev_ch(print_kptr(pa, (char *)NULL, 0));
 /*
  * Process protocols.
  */
 	switch (fam) {
 	case AF_INET:
-	    (void) strcpy(Lf->type, "inet");
+	    (void) snpf(Lf->type, sizeof(Lf->type), "inet");
 	    if (Fnet)
 		Lf->sf |= SELNET;
 	/*
@@ -292,13 +292,14 @@ process_socket(pn, fam, ipid, pa, lsp)
 	 */
 	    if (pa) {
 		if (kread(pa, (char *)&pcb, sizeof(pcb))) {
-		    (void) sprintf(Namech, "%s: can't read PCB from %s",
-			pn, print_kptr(pa, (char *)NULL));
+		    (void) snpf(Namech, Namechl, "%s: can't read PCB from %s",
+			pn, print_kptr(pa, (char *)NULL, 0));
 		    enter_nm(Namech);
 		    return;
 		}
 	    } else {
-		(void) sprintf(Namech, "%s: no protocol control block", pn);
+		(void) snpf(Namech, Namechl, "%s: no protocol control block",
+		    pn);
 		enter_nm(Namech);
 		return;
 	    }
@@ -432,13 +433,13 @@ process_socket(pn, fam, ipid, pa, lsp)
 		}
 		break;
 	    default:
-		(void) sprintf(Namech,
+		(void) snpf(Namech, Namechl,
 		    "%s: unknown IP ID (%d) for AF_INET family", pn, ipid);
 	    }
 	    break;
 	default:
-	    (void) strcpy(Lf->type, "sock");
-	    (void) sprintf(Namech, "%s: unknown family (%d) for IP ID %d",
+	    (void) snpf(Lf->type, sizeof(Lf->type), "sock");
+	    (void) snpf(Namech, Namechl, "%s: unknown family (%d) for IP ID %d",
 		pn, fam, ipid );
 	}
 	if (Namech[0])
@@ -461,8 +462,8 @@ process_unix_socket(s, u)
 
 	if (Funix)
 	    Lf->sf |= SELUNX;
-	(void) strcpy(Lf->type, "unix");
-	enter_dev_ch(print_kptr((KA_T)s->so_pcb, (char *)NULL));
+	(void) snpf(Lf->type, sizeof(Lf->type), "unix");
+	enter_dev_ch(print_kptr((KA_T)s->so_pcb, (char *)NULL, 0));
 	if (u->unp_ino) {
 	    Lf->inode = (unsigned long)u->unp_ino;
 	    Lf->inp_ty = 1;
@@ -479,6 +480,7 @@ process_unix_socket(s, u)
 	||  kread((KA_T)cu.unp_socket, (char *)&cs, sizeof(cs))
 	||  !cs.so_pcb)
 	    return;
-	(void) sprintf(Namech,"->%s",print_kptr((KA_T)cs.so_pcb,(char *)NULL));
+	(void) snpf(Namech, Namechl, "->%s",
+	    print_kptr((KA_T)cs.so_pcb, (char *)NULL, 0));
 }
 #endif	/* TCPV>=420 */
