@@ -32,7 +32,7 @@
 #ifndef lint
 static char copyright[] =
 "@(#) Copyright 1995 Purdue Research Foundation.\nAll rights reserved.\n";
-static char *rcsid = "$Id: dstore.c,v 1.6 99/06/22 08:24:41 abe Exp $";
+static char *rcsid = "$Id: dstore.c,v 1.7 2000/06/25 17:34:16 abe Exp $";
 #endif
 
 
@@ -56,6 +56,8 @@ struct drive_Nl Drive_Nl[] = {
 	{ "asopsm",	"asops_small"	},
 	{ "cvops",	"cdfs_vnodeops"	},
 	{ "clmaj",	"clone_c_major"	},
+	{ X_DNLCA,	"dnlc_array"	},
+	{ X_DNLCM,	"dnlc_mask"	},
 	{ "fvops",	"fifo_vnodeops"	},
         { X_NCACHE,	"ncache"	},
         { X_NCSIZE,	"ncsize"	},
