@@ -1,5 +1,5 @@
 /*
- * ddev.c - SunOS (Solaris 1.x and 2.x) device support functions for lsof
+ * ddev.c - Solaris device support functions for lsof
  */
 
 
@@ -32,13 +32,20 @@
 #ifndef lint
 static char copyright[] =
 "@(#) Copyright 1994 Purdue Research Foundation.\nAll rights reserved.\n";
-static char *rcsid = "$Id: ddev.c,v 1.13 99/10/22 08:23:34 abe Exp $";
+static char *rcsid = "$Id: ddev.c,v 1.16 2001/03/19 12:19:54 abe Exp $";
 #endif
 
 
 #include "lsof.h"
 
 
+/*
+ * Local definitions
+ */
+
+#define	LIKE_NODE_TTL	"like device special "
+
+
 /*
  * Local static values
  */
@@ -46,6 +53,10 @@ static char *rcsid = "$Id: ddev.c,v 1.13 99/10/22 08:23:34 abe Exp $";
 static int Devx = 0;			/* current Devtp[] index */
 
 
+/*
+ * Local function prototypes
+ */
+
 _PROTOTYPE(static void make_devtp,(struct stat *s, char *p));
 _PROTOTYPE(static int rmdupdev,(struct l_dev ***dp, int n, int ty));
 
@@ -97,20 +108,18 @@ make_devtp(s, p)
  */
 
 int
-printchdevname(rdev, f)
-	dev_t *rdev;			/* device */
+printchdevname(dev, rdev, f)
+	dev_t *dev;			/* device */
+	dev_t *rdev;			/* raw device */
 	int f;				/* 1 = print trailing '\n' */
 {
 	struct clone *c;
 	struct l_dev *dp;
-
-#if	defined(solaris)
 	struct pseudo *p;
-#endif
 
 	readdev(0);
 /*
- * Search device table for match.
+ * Search device table for a full match.
  */
 
 #if	defined(HASDCACHE)
@@ -119,14 +128,36 @@ printchdevname_again:
 
 #endif	/* defined(HASDCACHE) */
 
-	if ((dp = lkupdev(rdev, 1, 0))) {
+	if ((dp = lkupdev(dev, rdev, 1, 0))) {
 	    safestrprt(dp->name, stdout, f);
 	    return(1);
 	}
+/*
+ * Search device table for a match without inode number and dev.
+ */
+	if ((dp = lkupdev(&DevDev, rdev, 0, 0))) {
+
+	/*
+	 * A raw device match was found.  Record it as a name column addition.
+	 */
+	    char *cp;
+	    int len;
+
+	    len = (int)(1 + strlen(LIKE_NODE_TTL) + strlen(dp->name) + 1);
+	    if (!(cp = (char *)malloc((MALLOC_S)(len + 1)))) {
+		(void) fprintf(stderr, "%s: no nma space for: (%s%s)\n",
+		    Pn, LIKE_NODE_TTL, dp->name);
+		Exit(1);
+	    }
+	    (void) snpf(cp, len + 1, "(%s%s)", LIKE_NODE_TTL, dp->name);
+	    (void) add_nma(cp, len);
+	    (void) free((MALLOC_P *)cp);
+	    return(0);
+	}
 /*
  * Search for clone parent.
  */
-	if (Lf->is_stream && Clone) {
+	if (Lf->is_stream && Clone && (*dev == DevDev)) {
 	    for (c = Clone; c; c = c->next) {
 		if (major(*rdev) == minor(c->cd.rdev)) {
 
@@ -140,21 +171,21 @@ printchdevname_again:
 		}	
 	    }
 	}
-
-#if	defined(solaris)
 /*
  * Search for pseudo device match on major device only.
  */
-	for (p = Pseudo; p; p = p->next) {
-	    if (major(*rdev) == major(p->pd.rdev)) {
+	if (*dev == DevDev) {
+	    for (p = Pseudo; p; p = p->next) {
+		if (major(*rdev) == major(p->pd.rdev)) {
 
 # if	defined(HASDCACHE)
-		if (DCunsafe && !p->pd.v && vfy_dev(&p->pd))
-		    goto printchdevname_again;
+		    if (DCunsafe && !p->pd.v && vfy_dev(&p->pd))
+			goto printchdevname_again;
 # endif	/* defined(HASDCACHE) */
 
-		safestrprt(p->pd.name, stdout, f);
-		return(1);
+		    safestrprt(p->pd.name, stdout, f);
+		    return(1);
+		}
 	    }
 	}
 
@@ -168,13 +199,11 @@ printchdevname_again:
 	    goto printchdevname_again;
 	}
 #endif	/* defined(HASDCACHE) */
-#endif	/* solaris */
 
 	return(0);
 }
 
 
-#if	defined(solaris)
 /*
  * read_clone() - read Solaris clone device information
  */
@@ -343,7 +372,6 @@ read_clone()
 	if (path)
 	    (void) free((FREE_P *)path);
 }
-#endif	/* solaris */
 
 
 /*
@@ -376,10 +404,6 @@ readdev(skip)
 	MALLOC_S pl;
 	struct stat sb;
 
-#if	!defined(solaris)
-	struct clone *c;
-#endif	/* !defined(solaris) */
-
 	if (Sdev)
 	    return;
 
@@ -397,8 +421,6 @@ readdev(skip)
 #endif	/* defined(HASDCACHE) */
 
 	Devx = 0;
-
-#if	defined(solaris)
 	if (!(ppath = mkstrcat(DVCH_DEVPATH, -1, "/", 1, "pseudo", -1,
 		      (MALLOC_S *)NULL)))
 	{
@@ -407,8 +429,6 @@ readdev(skip)
 	    Exit(1);
 	}
 	read_clone();
-#endif	/* defined(solaris) */
-
 	Dstk = (char **)NULL;
 	Dstkn = Dstkx = 0;
 	(void) stkdir(DVCH_DEVPATH);
@@ -433,7 +453,7 @@ readdev(skip)
 	 * Create a directory name buffer with a trailing slash.
 	 */
 	    if (path) {
-		(void) free((FREE_P *)NULL);
+		(void) free((FREE_P *)path);
 		path = (char *)NULL;
 	    }
 	    if (!(path = mkstrcat(Dstk[Dstkx], -1, "/", 1, (char *)NULL, -1,
@@ -494,15 +514,12 @@ readdev(skip)
 	     */
 		if ((sb.st_mode & S_IFMT) == S_IFDIR) {
 
-#if	defined(solaris)
 		/*
 		 * Skip Solaris /DVCH_DEV_PATH/pseudo sub-directory;
 		 * it has been examined in read_clone().
 		 */
 		    if (strcmp(fp, ppath) == 0)
 			continue;
-#endif	/* solaris */
-
 		    (void) stkdir(fp);
 			continue;
 		}
@@ -512,44 +529,6 @@ readdev(skip)
 		 * Make Devtp[] entry.
 		 */
 		    make_devtp(&sb, fp);
-
-#if	!defined(solaris)
-		/*
-		 * Save information on non-Solaris clone devices.
-		 */
-		    if (major(sb.st_rdev) == CLONEMAJ) {
-
-		    /*
-		     * Allocate space for a clone entry.
-		     */
-			if (!(c = (struct clone *)malloc(sizeof(struct clone))))
-			{
-			    (void) fprintf(stderr,
-				"%s: no space for clone device: ", Pn);
-			    safestrprt(fp, stderr, 1);
-			    Exit(1);
-			}
-		    /*
-		     * Allocate space for the path name.
-		     */
-			if (!(c->cd.name = mkstrcpy(fp, (MALLOC_S *)NULL))) {
-			    (void) fprintf(stderr,
-				"%s: no space for clone name: ", Pn);
-			    safestrprt(fp, stderr, 1);
-			    Exit(1);
-			}
-		    /*
-		     * Fill in the clone entry and link it to the rest.
-		     */
-			c->cd.inode = sb.st_ino;
-			c->n = 0;
-			c->cd.rdev = sb.st_rdev;
-			c->cd.v = 0;
-			c->next = Clone;
-		  	Clone = c;
-		    }
-#endif	/* !solaris */
-
 		}
 
 #if	defined(HASBLKDEV)
@@ -687,8 +666,6 @@ clr_sect()
 	    }
 	    Clone = (struct clone *)NULL;
 	}
-
-# if	defined(solaris)
 	if (Pseudo) {
 	    struct pseudo *p, *p1;
 
@@ -700,8 +677,6 @@ clr_sect()
 	    }
 	    Pseudo = (struct pseudo *)NULL;
 	}
-# endif	/* defined(solaris) */
-
 }
 
 
@@ -837,7 +812,7 @@ bad_clone_sect:
 	 */
 	    for (c = Clone, n = 0; c; c = c->next, n++)
 		;
-	    (void) sprintf(buf, "clone section: %d\n", n);
+	    (void) snpf(buf, sizeof(buf), "clone section: %d\n", n);
 	    if (wr2DCfd(buf, &DCcksum))
 		return(1);
 	/*
@@ -849,8 +824,8 @@ bad_clone_sect:
 	 * not use the new format lines.
 	 */
 	    for (c = Clone; c; c = c->next) {
-		(void) sprintf(buf, " %lx %d %ld %s\n", (long)c->cd.rdev, c->n,
-		    (long)c->cd.inode, c->cd.name);
+		(void) snpf(buf, sizeof(buf), " %lx %d %ld %s\n",
+		    (long)c->cd.rdev, c->n, (long)c->cd.inode, c->cd.name);
 		if (wr2DCfd(buf, &DCcksum))
 		    return(1);
 	    }
@@ -862,6 +837,7 @@ bad_clone_sect:
 	(void) fprintf(stderr, "%s: internal rw_clone_sect error: %d\n",
 	    Pn, m);
 	Exit(1);
+	return(1);		/* to make code analyzers happy */
 }
 
 
@@ -883,7 +859,6 @@ rereaddev()
 }
 
 
-# if	defined(solaris)
 /*
  * rw_pseudo_sect() - read/write the device cache pseudo section
  */
@@ -995,7 +970,7 @@ bad_pseudo_sect:
 		    Exit(1);
 		}
 		*(cp + len - 1) = '\0';
-		(void) strcpy(p->pd.name, cp);
+		(void) snpf(p->pd.name, len, "%s", cp);
 		p->pd.v = 0;
 		p->next = Pseudo;
 		Pseudo = p;
@@ -1008,7 +983,7 @@ bad_pseudo_sect:
 	 */
 	    for (p = Pseudo, n = 0; p; p = p->next, n++)
 		;
-	    (void) sprintf(buf, "pseudo section: %d\n", n);
+	    (void) snpf(buf, sizeof(buf), "pseudo section: %d\n", n);
 	    if (wr2DCfd(buf, &DCcksum))
 		return(1);
 	/*
@@ -1020,7 +995,7 @@ bad_pseudo_sect:
 	 * not use the new format lines.
 	 */
 	    for (p = Pseudo; p; p = p->next) {
-		(void) sprintf(buf, " %lx %ld %s\n", (long)p->pd.rdev,
+		(void) snpf(buf, sizeof(buf), " %lx %ld %s\n", (long)p->pd.rdev,
 		    (long)p->pd.inode, p->pd.name);
 		if (wr2DCfd(buf, &DCcksum))
 		    return(1);
@@ -1034,7 +1009,6 @@ bad_pseudo_sect:
 	    Pn, m);
 	return(1);
 }
-# endif	/* defined_solaris */
 
 
 /*
@@ -1082,10 +1056,7 @@ rmdupdev(dp, n, ty)
 	struct clone *c, *cp;
 	struct l_dev **d;
 	int i, j, k;
-
-#if	defined(solaris)
 	struct pseudo *p, *pp;
-#endif	/* defined(solaris) */
 
 	for (i = j = 0, d = *dp; i < n ;) {
 	    for (k = i + 1; k < n; k++) {
@@ -1114,8 +1085,6 @@ rmdupdev(dp, n, ty)
 		    (void) free((FREE_P *)c);
 		    break;
 		}
-
-#if	defined(solaris)
 	    /*
 	     * See if we're deleting a duplicate pseudo device.  If so,
 	     * delete its pseudo table entry.
@@ -1137,8 +1106,6 @@ rmdupdev(dp, n, ty)
 		    (void) free((FREE_P *)p);
 		    break;
 		}
-#endif	/* defined(solaris) */
-
 	    }
 	    if (i != j)
 		d[j] = d[i];
