@@ -1,5 +1,5 @@
 /*
- * dfile.c - SunOS (Solaris 1.x and 2.x) file processing functions for lsof
+ * dfile.c - Solaris file processing functions for lsof
  */
 
 
@@ -32,7 +32,7 @@
 #ifndef lint
 static char copyright[] =
 "@(#) Copyright 1994 Purdue Research Foundation.\nAll rights reserved.\n";
-static char *rcsid = "$Id: dfile.c,v 1.11 99/10/22 08:23:39 abe Exp $";
+static char *rcsid = "$Id: dfile.c,v 1.13 2000/12/04 14:35:13 abe Exp $";
 #endif
 
 
@@ -59,6 +59,9 @@ static int HbyCdCt = 0;			/* HbyCd entry count */
 static struct hsfile *HbyFdi =		/* hash by file buckets */
 	(struct hsfile *)NULL;
 static int HbyFdiCt = 0;		/* HbyFdi entry count */
+static struct hsfile *HbyFrd =		/* hash by file raw device buckets */
+	(struct hsfile *)NULL;
+static int HbyFrdCt = 0;		/* HbyFrd entry count */
 static struct hsfile *HbyFsd =		/* hash by file system buckets */
 	(struct hsfile *)NULL;
 static int HbyFsdCt = 0;		/* HbyFsd entry count */
@@ -82,9 +85,16 @@ static int HbyNmCt = 0;			/* HbyNm entry count */
 					 * (m must be a power of 2) */
 #define	SFNMHASH	4096		/* Sfile hash by name bucket count
 					   (power of 2!) */
+#define	SFRDHASH	1024		/* Sfile hash by raw device number
+					 * bucket count (power of 2!) */
+#define SFHASHRDEVI(maj, min, rmaj, rmin, ino, mod) ((int)(((int)((((int)(maj+1))*((int)((min+1))))+((int)(rmaj+1)*(int)(rmin+1))+ino)*31415)&(mod-1)))
+					/* hash for Sfile by major device,
+					 * minor device, major raw device,
+					 * minor raw device, and inode, modulo
+					 * mod (mod must be a power of 2) */
 
 
-#if	defined(solaris) && solaris<20500
+#if	solaris<20500
 /*
  * get_max_fd() - get maximum file descriptor plus one
  */
@@ -98,7 +108,7 @@ get_max_fd()
 	    return(-1);
 	return(r.rlim_cur);
 }
-#endif	/* defined(solaris) && solaris<20500 */
+#endif	/* solaris<20500 */
 
 
 /*
@@ -119,20 +129,13 @@ hashSfile()
 	if (!Sfile || hs)
 	    return;
 /*
- * Preset the clone major device for Solaris of SunOS.
+ * Preset the clone major device for Solaris.
  */
-
-#if	defined(solaris)
 	if (HaveCloneMaj) {
 	    cmaj = CloneMaj;
 	    hvc = 1;
 	} else
 	    hvc = 0;
-#else	/* !defined(SOLARIS) */
-	cmaj = CLONEMAJ;
-	hvc = 1;
-#endif	/* defined(solaris) */
-
 /*
  * Allocate hash buckets by clone device, (device,inode), file system device,
  * and file name.
@@ -155,6 +158,14 @@ hashSfile()
 		Pn, SFDIHASH);
 	    Exit(1);
 	}
+	if (!(HbyFrd = (struct hsfile *)calloc((MALLOC_S)SFRDHASH,
+					       sizeof(struct hsfile))))
+	{
+	    (void) fprintf(stderr,
+		"%s: can't allocate space for %d rdev hash buckets\n",
+		Pn, SFRDHASH);
+	    Exit(1);
+	}
 	if (!(HbyFsd = (struct hsfile *)calloc((MALLOC_S)SFFSHASH,
 					       sizeof(struct hsfile))))
 	{
@@ -177,7 +188,7 @@ hashSfile()
  * bucket chains.
  */
 	for (s = Sfile; s; s = s->next) {
-	    for (i = 0; i < 3; i++) {
+	    for (i = 0; i < 4; i++) {
 		if (i == 0) {
 		    if (!s->aname)
 			continue;
@@ -197,10 +208,23 @@ hashSfile()
 						  SFFSHASH)];
 			HbyFsdCt++;
 		    }
-		} else {
-		    if (!hvc || (major(s->dev) != cmaj))
+		} else if (i == 2) {
+		    if (s->type
+		    &&  ((s->mode == S_IFCHR) || (s->mode == S_IFBLK)))
+		    {
+			sh = &HbyFrd[SFHASHRDEVI(major(s->dev),
+						 minor(s->dev),
+						 major(s->rdev),
+						 minor(s->rdev),
+						 s->i,
+						 SFRDHASH)];
+			HbyFrdCt++;
+		    } else
 			continue;
-		    sh = &HbyCd[SFHASHDEVINO(0, minor(s->dev), 0, SFCDHASH)];
+		} else {
+		    if (!hvc || (major(s->rdev) != cmaj))
+			continue;
+		    sh = &HbyCd[SFHASHDEVINO(0, minor(s->rdev), 0, SFCDHASH)];
 		    HbyCdCt++;
 		}
 		if (!sh->s) {
@@ -238,9 +262,11 @@ is_file_named(p, nt, vt, ps)
 	int ps;				/* print status: 0 = don't copy name
 					 * to Namech */
 {
+	char *ep;
 	int f = 0;
 	struct sfile *s;
 	struct hsfile *sh;
+	size_t sz;
 /*
  * Check for a path name match, as requested.
  */
@@ -253,14 +279,16 @@ is_file_named(p, nt, vt, ps)
 	    }
 	}
 /*
- * Check for a Sun clone file.
+ * Check for a Solaris clone file.
  */
-	if (!f && HbyCdCt && nt == N_STREAM && Lf->dev_def) {
-	    for (sh = &HbyCd[SFHASHDEVINO(0, major(Lf->dev), 0, SFCDHASH)];
+	if (!f && HbyCdCt && nt == N_STREAM && Lf->dev_def && Lf->rdev_def
+	&&  (Lf->dev == DevDev))
+	{
+	    for (sh = &HbyCd[SFHASHDEVINO(0, major(Lf->rdev), 0, SFCDHASH)];
 		 sh;
 		 sh = sh->next)
 	    {
-		if ((s = sh->s) && (major(Lf->dev) == minor(s->dev))) {
+		if ((s = sh->s) && (major(Lf->rdev) == minor(s->rdev))) {
 		    f = 1;
 		    break;
 		}
@@ -287,7 +315,7 @@ is_file_named(p, nt, vt, ps)
 	    }
 	}
 /*
- * Check for a file system.
+ * Check for a file system match.
  */
 	if (!f && HbyFsdCt && Lf->dev_def) {
 	    for (sh = &HbyFsd[SFHASHDEVINO(major(Lf->dev), minor(Lf->dev), 0,
@@ -303,13 +331,36 @@ is_file_named(p, nt, vt, ps)
 		}
 	    }
 	}
+/*
+ * Check for a character or block device match.
+ */
+	if (!f && HbyFrdCt
+	&&  ((vt = VCHR) || (vt = VBLK))
+	&&  Lf->dev_def && (Lf->dev == DevDev)
+	&&  Lf->rdev_def
+	&& (Lf->inp_ty == 1 || Lf->inp_ty == 3))
+	{
+	    for (sh = &HbyFrd[SFHASHRDEVI(major(Lf->dev), minor(Lf->dev),
+					  major(Lf->rdev), minor(Lf->rdev),
+					  Lf->inode, SFRDHASH)];
+		 sh;
+		 sh = sh->next)
+	    {
+		if ((s = sh->s) && (s->dev == Lf->dev)
+		&&  (s->rdev == Lf->rdev) && (s->i == Lf->inode))
+		{
+		    f = 1;
+		    break;
+		}
+	    }
+	}
 /*
  * Convert the name if a match occurred.
  */
 	if (f) {
 	    if (f == 2) {
 		if (ps)
-		    (void) strcpy(Namech, p);
+		    (void) snpf(Namech, Namechl, "%s", p);
 	    } else {
 		if (ps && s->type) {
 
@@ -317,9 +368,11 @@ is_file_named(p, nt, vt, ps)
 		 * If the search argument isn't a file system, propagate it
 		 * to Namech[]; otherwise, let printname() compose the name.
 		 */
-		    (void) strcpy(Namech, s->name);
-		    if (s->devnm)
-			(void) sprintf(endnm(), " (%s)", s->devnm);
+		    (void) snpf(Namech, Namechl, "%s", s->name);
+		    if (s->devnm) {
+			ep = endnm(&sz);
+			(void) snpf(ep, sz, " (%s)", s->devnm);
+		    }
 		}
 	    }
 	    s->f = 1;
@@ -329,14 +382,14 @@ is_file_named(p, nt, vt, ps)
 }
 
 
-#if	defined(solaris)
 /*
  * print_dev() - print device
  */
 
 char *
-print_dev(lf)
+print_dev(lf, dev)
 	struct lfile *lf;		/* file whose device is to be printed */
+	dev_t *dev;			/* device to be printed */
 {
 	static char buf[128];
 /*
@@ -346,7 +399,7 @@ print_dev(lf)
  * We would like to use the L_MAXMAJ definition from <sys/sysmacros.h> all
  * the time, but it's not always correct in all versions of Solaris.
  */
-	(void) sprintf(buf, "%d,%d", (int)((lf->dev >> L_BITSMINOR) &
+	(void) snpf(buf, sizeof(buf), "%d,%d", (int)((*dev >> L_BITSMINOR) &
 
 #if	solaris>=20501
 	    L_MAXMAJ
@@ -354,29 +407,9 @@ print_dev(lf)
 	    0x3fff
 #endif	/* solaris>=20501 */
 
-	    ), (int)(lf->dev & L_MAXMIN));
+	    ), (int)(*dev & L_MAXMIN));
 	return(buf);
 }
-#else	/* !defined(solaris) */
-
-
-/*
- * print_ino() - print inode
- */
-
-char *
-print_ino(lf)
-	struct lfile *lf;		/* file whose device is to be printed */
-{
-	static char buf[128];
-
-	if ((long)lf->inode < 0L)
-	    (void) sprintf(buf, " %ld", (long)lf->inode);
-	else
-	    (void) sprintf(buf, " %lu", (unsigned long)lf->inode);
-	return(buf);
-}
-#endif	/* defined(slaris) */
 
 
 /*
@@ -395,8 +428,8 @@ process_file(fp)
 #endif	/* defined(FILEPTR) */
 
 	if (kread(fp, (char *)&f, sizeof(f))) {
-	    (void) sprintf(Namech, "can't read file struct from %s",
-		print_kptr(fp, (char *)NULL));
+	    (void) snpf(Namech, Namechl, "can't read file struct from %s",
+		print_kptr(fp, (char *)NULL, 0));
 	    enter_nm(Namech);
 	    return;
 	}
@@ -431,48 +464,16 @@ process_file(fp)
 		Lf->fsv |= FSV_FG;
 	    }
 	    if (Fsv & FSV_NI) {
-
-# if	defined(solaris)
 		Lf->fna = (KA_T)f.f_vnode;
-# else	/* !defined(solaris) */
-		Lf->fna = (KA_T)f.f_data;
-# endif	/* defined(solaris) */
-
 		Lf->fsv |= FSV_NI;
 	    }
 #endif	/* defined(HASFSTRUCT) */
 
-
-#if	defined(solaris)
 	/*
 	 * Solaris file structures contain a vnode pointer.  Process it.
 	 */
 	    process_node((KA_T)f.f_vnode);
 	    return;
-#else	/* !solaris */
-
-	/*
-	 * Process non-Solaris file structure by its type.
-	 */
-	    switch (f.f_type) {
-
-	    case DTYPE_VNODE:
-		if (!Selinet)
-		    process_node((KA_T)f.f_data);
-		return;
-	    case DTYPE_SOCKET:
-		process_socket((KA_T)f.f_data);
-		return;
-	    default:
-		if (f.f_type != 0 || f.f_ops) {
-		    (void) sprintf(Namech, "%s file struct, ty=%#x, op=%#x",
-			print_kptr(fp, (char *)NULL), f.f_type, f.f_ops);
-		    enter_nm(Namech);
-		    return;
-		}
-	    }
-#endif	/* solaris */
-
 	}
 	enter_nm("no more information"); }
 
