@@ -1,5 +1,5 @@
 /*
- * dmnt.c - SunOS (Solaris 1.x and 2.x) mount support functions for lsof
+ * dmnt.c - Solaris mount support functions for lsof
  */
 
 
@@ -32,7 +32,7 @@
 #ifndef lint
 static char copyright[] =
 "@(#) Copyright 1994 Purdue Research Foundation.\nAll rights reserved.\n";
-static char *rcsid = "$Id: dmnt.c,v 1.10 99/11/28 06:44:37 abe Exp $";
+static char *rcsid = "$Id: dmnt.c,v 1.11 2000/12/04 14:35:13 abe Exp $";
 #endif
 
 
@@ -105,13 +105,8 @@ readmnt()
 	struct mounts *mtp;
 	char *opt, *opte;
 	struct stat sb;
-
-#if	defined(solaris)
 	struct mnttab me;
 	struct mnttab *mp;
-#else	/* !defined(solaris) */
-	struct mntent *mp;
-#endif	/* defined(solaris) */
 
 #if	defined(HASPROCFS)
 	int procfs = 0;
@@ -122,35 +117,18 @@ readmnt()
 /*
  * Open access to the mount table and read mount table entries.
  */
-
-#if	defined(solaris)
 	if (!(mfp = fopen(MNTTAB, "r"))) {
             (void) fprintf(stderr, "%s: can't access %s\n", Pn, MNTTAB);
             return(0);
         }
-	for (mp = &me; getmntent(mfp, mp) == 0;)
-#else	/* !defined(solaris) */
-        if (!(mfp = setmntent(MOUNTED, "r"))) {
-            (void) fprintf(stderr, "%s: can't access %s\n", Pn, MOUNTED);
-            return(0);
-        }
-        while ((mp = getmntent(mfp)))
-#endif	/* defined(solaris) */
-
-	{
+	for (mp = &me; getmntent(mfp, mp) == 0;) {
 
 	/*
 	 * Skip loop-back mounts, since they are aliases for legitimate file
 	 * systems and there is no way to determine that a vnode refers to a
 	 * loop-back alias.
 	 */
-
-#if	defined(solaris)
 	    if (strcmp(mp->mnt_fstype, MNTTYPE_LO) == 0)
-#else	/* !defined(solaris) */
-	    if (strcmp(mp->mnt_type, MNTTYPE_LO) == 0)
-#endif	/* defined(solaris) */
-
 		continue;
 	/*
 	 * Save pointers to the directory and file system names for later use.
@@ -158,27 +136,14 @@ readmnt()
 	 * Check the file system name.  If it doesn't begin with a `/'
 	 * but contains a `:' not followed by a '/', ignore this entry.
 	 */
-
-#if	defined(solaris)
 	    dir = mp->mnt_mountp;
 	    fs = mp->mnt_special;
-#else	/* !defined(solaris) */
-	    dir = mp->mnt_dir;
-	    fs = mp->mnt_fsname;
-#endif	/* defined(solaris) */
-
 	    if (*fs != '/' && (cp = strchr(fs, ':')) && *(cp+1) != '/')
 		continue;
 	/*
 	 * Check for a "ignore" type (SunOS) or "ignore" option (Solaris).
 	 */
-
-#if	defined(solaris)
 	    if (hasmntopt(mp, MNTOPT_IGNORE))
-#else	/* !defined(solaris) */
-	    if (strcmp(mp->mnt_type, MNTTYPE_IGNORE) == 0)
-#endif	/* defined(solaris) */
-
 		ignore = 1;
 	    else
 		ignore = 0;
@@ -216,25 +181,14 @@ no_space_for_mount:
 	 * or "ignore" option (Solaris) and try to use the mount entry's
 	 * device number instead.
 	 */
-
-#if	defined(solaris)
 	    opt = hasmntopt(mp, MNTOPT_DEV);
 	    devl = strlen(MNTOPT_DEV);
-#else	/* !defined(solaris) */
-	    opt = hasmntopt(mp, MNTINFO_DEV);
-	    devl = strlen(MNTINFO_DEV);
-#endif	/* defined(solaris) */
-
 	    if (ignore) {
 		if (!opt
 		||  !(opte = getmntdev(opt, devl, &sb,
 
 #if	defined(HASFSTYPE)
-# if	defined(solaris)
 			     mp->mnt_fstype
-# else	/* !defined(solaris) */
-			     (char *)NULL
-# endif	/* defined(solaris) */
 #else	/* !defined(HASFSTYPE) */
 			     (char *)NULL
 #endif	/* defined(HASFSTYPE) */
@@ -246,15 +200,8 @@ no_space_for_mount:
 		if (!Fwarn) {
 			(void) fprintf(stderr,
 			    "%s: WARNING: can't stat() ", Pn);
-
-#if	defined(solaris)
 			safestrprt(mp->mnt_fstype, stderr, 0);
 			(void) fprintf(stderr, " file system ");
-#else	/* !defined(solaris) */
-			safestrprt(mp->mnt_type, stderr, 0);
-			(void) fprintf(stderr, " file system ");
-#endif	/* defined(solaris) */
-
 			safestrprt(dir, stderr, 1);
 			(void) fprintf(stderr,
 			    "      Output information may be incomplete.\n");
@@ -263,11 +210,7 @@ no_space_for_mount:
 		    if ((opte = getmntdev(opt, devl, &sb,
 
 #if	defined(HASFSTYPE)
-# if	defined(solaris)
 					   mp->mnt_fstype
-# else	/* !defined(solaris) */
-					   (char *)NULL
-# endif	/* defined(solaris) */
 #else	/* !defined(HASFSTYPE) */
 					   (char *)NULL
 #endif	/* defined(HASFSTYPE) */
@@ -277,15 +220,7 @@ no_space_for_mount:
 			if (!Fwarn) {
 			    (void) fprintf(stderr,
 				"      assuming \"%.*s\" from %s\n",
-				(opte - opt), opt,
-
-#if	defined(solaris)
-				MNTTAB
-#else	/* !defined(solaris) */
-				MOUNTED
-#endif	/* defined(solaris) */
-
-			    );
+				(opte - opt), opt, MNTTAB);
 			}
 		    } else
 			opt = (char *)NULL;
@@ -312,10 +247,10 @@ no_space_for_mount:
 	    mtp->inode = sb.st_ino;
 	    mtp->mode = sb.st_mode;
 
-#if	defined(solaris) && solaris>=80000
+#if	solaris>=80000
 	    mtp->nlink = sb.st_nlink;
 	    mtp->size = sb.st_size;
-#endif	/* defined(solaris) && solaris>=80000 */
+#endif	/* solaris>=80000 */
 
 #if	defined(HASPROCFS)
 
@@ -361,16 +296,10 @@ no_space_for_mount:
 		AFSdev = mtp->dev;
 		AFSdevStat = 1;
 	    }
-#endif	/* defined(HAS_AFS) && defined(solaris) && solaris>=20600 */
+#endif	/* defined(HAS_AFS) && solaris>=20600 */
 
         }
-
-#if	defined(solaris)
 	(void) fclose(mfp);
-#else	/* !defined(solaris) */
-        (void) endmntent(mfp);
-#endif	/* defined(solaris) */
-
 /*
  * Clean up and return local mount info table address.
  */
@@ -396,11 +325,6 @@ readvfs(ka, la, lv)
 	struct vfs *v, tv;
 	struct l_vfs *vp;
 
-#if	!defined(solaris)
-	struct mount m;
-	dev_t td;
-# endif	/* !defined(solaris) */
-
 	if (!ka && !la)
 	    return((struct l_vfs *)NULL);
 	for (vp = Lvfs; vp; vp = vp->next) {
@@ -436,39 +360,18 @@ readvfs(ka, la, lv)
 /*
  * Fake the device number for an AFS device.
  */
-# if	defined(solaris)
 	if (v->vfs_fstype == AFSfstype) {
 	    if (!AFSdevStat)
 		(void) readmnt();
 	    v->vfs_dev = AFSdevStat ? AFSdev : 0;
 	}
-# endif	/* defined(solaris) */
 #endif	/* defined(HAS_AFS) */
 
 /*
  * Complete mount information.
  */
 
-#if	defined(solaris)
 	(void) completevfs(vp, (dev_t *)&v->vfs_dev);
-#else	/* !defined(solaris) */
-	if (Ntype == N_HSFS || Ntype == N_PCFS) {
-	    td = (dev_t)(v->vfs_fsid.val[0] & 0xffff);
-	    (void) completevfs(vp, &td);
-	} else if (v->vfs_data) {
-	    if (!kread((KA_T)v->vfs_data, (char *)&m, sizeof(m)))
-		(void) completevfs(vp, &m.m_dev);
-	}
-# if	defined(HAS_AFS)
-	else if (!lv->v_data && AFSVfsp && (KA_T)lv->v_vfsp == AFSVfsp) {
-	    if (!AFSdevStat)
-		(void) readmnt();
-	    td = AFSdevStat ? AFSdev : 0;
-	    (void) completevfs(vp, &td);
-	}
-# endif	/* defined(HAS_AFS) */
-#endif	/* defined(solaris) */
-
 	vp->next = Lvfs;
 	vp->addr = ka;
 	Lvfs = vp;
