@@ -1,5 +1,5 @@
 /*
- * dproc.c - SunOS (Solaris 1.x and 2.x)  process access functions for lsof
+ * dproc.c - Solaris lsof functions for accessing process infomation
  */
 
 
@@ -32,48 +32,105 @@
 #ifndef lint
 static char copyright[] =
 "@(#) Copyright 1994 Purdue Research Foundation.\nAll rights reserved.\n";
-static char *rcsid = "$Id: dproc.c,v 1.16 99/10/22 08:23:48 abe Exp $";
+static char *rcsid = "$Id: dproc.c,v 1.23 2001/07/05 12:26:57 abe Exp $";
 #endif
 
 #include "lsof.h"
 
-#if	defined(solaris) && solaris<20500
+#if	solaris<20500
 #include "kernelbase.h"
-#endif	/* defined(solaris) && solaris<20500 */
+#endif	/* solaris<20500 */
 
 
 /*
  * Local definitions
  */
 
-#if	defined(solaris) && solaris>=20500
-KA_T Kb = (KA_T)NULL;			/* KERNELBASE for Solaris 2.5 */
-#endif	/* defined(solaris) && solaris>=20500 */
+#if	solaris>=20501
+#define	KVMHASHBN	8192		/* KVM hash bucket count -- MUST BE
+					 * A POWER OF 2!!! */
+#define	HASHKVM(va)	((int)((va * 31415) >> 3) & (KVMHASHBN-1))
+					/* virtual address hash function */
+
+# if	solaris<70000
+#define	KAERR	(u_longlong_t)-1	/* kvm_physaddr() error return */
+#define	KBUFT	char			/* kernel read buffer type */
+#define	KPHYS	u_longlong_t		/* kernel physical address type */
+#define	KVIRT	u_int			/* kernel virtual address type */
+# else	/* solaris>=70000 */
+#define	KAERR	(uint64_t)-1		/* kvm_physaddr() error return */
+#define	KBUFT	void			/* kernel read buffer type */
+#define	KPHYS	uint64_t		/* kernel physical address type */
+#define	KVIRT	uintptr_t		/* kernel virtual address type */
+# endif	/* solaris<70000 */
+#endif	/* solaris>=20501 */
+
+
+/*
+ * Local structures
+ */
+
+#if	solaris>=20501
+typedef struct kvmhash {
+	KVIRT vpa;			/* virtual page address */
+	KPHYS pa;			/* physical address */
+	struct kvmhash *nxt;		/* next virtual address */
+} kvmhash_t;
+#endif	/* solaris>=20501 */
+
+
+/*
+ * Local variables
+ */
+
+#if	solaris>=20501
+static struct as *Kas = (struct as *)NULL;
+					/* pointer to kernel's address space
+					 * map in kernel virtual memory */
+static kvmhash_t **KVMhb = (kvmhash_t **)NULL;
+					/* KVM hash buckets */
+static int PageSz = 0;			/* page size */
+static int PSMask = 0;			/* page size mask */
+static int PSShft = 0;			/* page size shift */
+
+# if	solaris<70000
+static struct as Kam;			/* kernel's address space map */
+static int Kmd = -1;			/* memory device file descriptor */
+# endif	/* solaris<70000 */
+#endif	/* solaris>=20501 */
+
+#if	solaris>=20500
+static KA_T Kb = (KA_T)NULL;		/* KERNELBASE for Solaris 2.5 */
+#endif	/* solaris>=20500 */
 
 static int Knp = PROCDFLT;		/* numer of proc structures known
 					 * to the kernel */
-static int Np;				/* number of P[], Pgrp[], and Pid[]
+static int Np;				/* number of P[], Pgid[], and Pid[]
 					 * entries  */
 static struct proc *P = NULL;		/* local proc structure table */
 static KA_T Sgvops;			/* [_]segvn_ops address */
-
-#if	defined(solaris)
+static int *Pgid = NULL;		/* process group IDs for P[] entries */
 static int *Pid = NULL;			/* PIDs for P[] entries */
-static int *Pgrp = NULL;		/* process group IDs for P[] entries */
 static gid_t Savedgid;			/* saved (effective) GID */
 static int Switchgid = 0;		/* must switch GIDs for kvm_open() */
-#endif	/* defined(solaris) */
 
 
-_PROTOTYPE(static void close_kvm,(void));
+/*
+ * Local function prototypes
+ */
+
 _PROTOTYPE(static void get_kernel_access,(void));
-_PROTOTYPE(static void open_kvm,(void));
 _PROTOTYPE(static void process_text,(KA_T pa));
 _PROTOTYPE(static void read_proc,(void));
-
-#if	defined(solaris)
 _PROTOTYPE(static void readfsinfo,(void));
-#endif	/* defined(solaris) */
+
+#if	solaris>=20501
+_PROTOTYPE(static void readkam,(KA_T addr));
+#endif	/* solaris>=20501 */
+
+#if	solaris>=20501 && solaris<70000
+_PROTOTYPE(extern u_longlong_t kvm_physaddr,(kvm_t *, struct as *, u_int));
+#endif	/* solaris>=20501 && solaris<70000 */
 
 
 
@@ -81,16 +138,26 @@ _PROTOTYPE(static void readfsinfo,(void));
  * close_kvm() - close kernel virtual memory access
  */
 
-static void
+void
 close_kvm()
 {
 	if (!Kd)
 	    return;
-	if (kvm_close(Kd) != 0) {
-	    (void) fprintf(stderr, "%s: kvm_close failed\n", Pn);
-	    Exit(1);
+	if (Kd) {
+	    if (kvm_close(Kd) != 0) {
+		(void) fprintf(stderr, "%s: kvm_close failed\n", Pn);
+		Exit(1);
+	    }
+	    Kd = (kvm_t *)NULL;
 	}
-	Kd = (kvm_t *)NULL;
+
+#if	solaris>=20501 && solaris<70000
+	if (Kmd >= 0) {
+	    (void) close(Kmd);
+	    Kmd = -1;
+	}
+#endif	/* solaris>=20501 && solaris<70000 */
+
 }
 
 
@@ -102,40 +169,32 @@ void
 gather_proc_info()
 {
 	MALLOC_S bc;
-	int ft = 1;
+	static int ft = 1;
 	int i, j;
 	struct proc *p;
-	int pgrp, pid, px;
+	int pgid, pid, px;
 	long pofv;
 	short pss, sf;
 	struct user *u;
 	uid_t uid;
 
-#if	defined(solaris)
-# if	solaris>=20400
+#if	solaris>=20400
 	int k;
 
-#  if	!defined(NFPCHUNK)
+# if	!defined(NFPCHUNK)
 #define	uf_ofile	uf_file
 #define	uf_pofile	uf_flag
 #define	u_flist		u_finfo.fi_list
 #define	u_nofiles	u_finfo.fi_nfiles
 #define	NFPREAD		64
-#  else	/* defined(NFPCHUNK) */
+# else	/* defined(NFPCHUNK) */
 #define	NFPREAD		NFPCHUNK
-#  endif	/* !defined(NFPCHUNK) */
+# endif	/* !defined(NFPCHUNK) */
 	uf_entry_t uf[NFPREAD];
-# endif	/* solaris>=20400 */
-# if	solaris>=20500
+#endif	/* solaris>=20400 */
+#if	solaris>=20500
 	struct cred pc;
-# endif	/* solaris>=20500 */
-#else	/* !solaris */
-	struct ucwd cd;
-	int dl, nf;
-	struct file **uf;
-	static int xnf = 0;
-	struct file **xuf = (struct file **)NULL;
-#endif	/* solaris */
+#endif	/* solaris>=20500 */
 
 #if	defined(HASFSTRUCT)
 	static int npofb = 0;
@@ -151,20 +210,29 @@ gather_proc_info()
 		Sgvops = (KA_T)NULL;
 	    ft = 0;
 	}
+
+#if	solaris>=20501
+	else
+
+	/*
+	 * If not the first time, re-read the kernel's address space map.
+	 */
+	    readkam((KA_T)NULL);
+#endif	/* solaris>=20501 */
+
 /*
- * Read kernel name cache and process table.
+ * Read the process table.
+ *
+ * The Solaris process table is a linked list whose head pointer is acquired
+ * by open_kvm()'s call to kvm_open().  For Solaris close_kvm() is called so
+ * the process table head pointer can be re-acquired by a call to open_kvm()
+ * immediately before the actual reading of the process table.
+ *
+ * Since the SunOS 4.1.x process table is sequential, there's no worry about
+ * a stale or incorrect list head pointer.  Thus no close_kvm() is needed.
+ * A kvm_setproc() call is sufficient.
  */
-
-#if	defined(HASNCACHE)
-# if	defined(solaris)
-	open_kvm();
-# endif	/* defined(solaris) */
-	ncache_load();
-# if	defined(solaris)
 	close_kvm();
-# endif	/* defined(solaris) */
-#endif	/* defined(HASNCACHE) */
-
 	read_proc();
 /*
  * Loop through processes.
@@ -175,84 +243,40 @@ gather_proc_info()
 	 * Get the process ID.
 	 */
 
-#if	defined(solaris)
-	    if (Fpgrp)
-		pgrp = Pgrp[px];
+	    if (Fpgid)
+		pgid = Pgid[px];
 	    else
-		pgrp = 0;
+		pgid = 0;
 	    pid = Pid[px];
-# if solaris<20500
+
+#if solaris<20500
 	    uid = p->p_uid;
-# else	/* solaris >=20500 */
+#else	/* solaris >=20500 */
 	/*
 	 * Read credentials for Solaris 2.5 and above process.
 	 */
 	    if (kread((KA_T)p->p_cred, (char *)&pc, sizeof(pc)))
 		continue;
 	    uid = pc.cr_uid;
-# endif	/* solaris<20500 */
-#else	/* !defined(solaris) */
-	    pgrp = (int)p->p_pgrp;
-	    pid = p->p_pid;
-	    uid = p->p_suid ? p->p_suid : p->p_uid;
-#endif	/* defined(solaris) */
+#endif	/* solaris<20500 */
 
 	/*
 	 * See if the process is excluded.
 	 */
-	    if  (is_proc_excl(pid, pgrp, (UID_ARG)uid, &pss, &sf))
+	    if  (is_proc_excl(pid, pgid, (UID_ARG)uid, &pss, &sf))
 		continue;
 	/*
 	 * Get the user area associated with the process.
 	 */
-
-#if	defined(solaris)
 	    u = &p->p_user;
-#else	/* !defined(solaris) */
-	    if (!(u = kvm_getu(Kd, p)))
-		continue;
-#endif	/* defined(solaris) */
-
 	/*
 	 * Allocate a local process structure and start filling it.
 	 */
 	    if (is_cmd_excl(u->u_comm, &pss, &sf))
 		continue;
-	    alloc_lproc(pid, pgrp, (int)p->p_ppid, (UID_ARG)uid, u->u_comm,
+	    alloc_lproc(pid, pgid, (int)p->p_ppid, (UID_ARG)uid, u->u_comm,
 		(int)pss, (int)sf);
 	    Plf = (struct lfile *)NULL;
-
-#if	!defined(solaris)
-	/*
-	 * Read the SunOS current working directory and root directory
-	 * paths.
-	 */
-	    *Cwd = *Rtd = '\0';
-	    if (u->u_cwd && !kread((KA_T)u->u_cwd, (char *)&cd, sizeof(cd))) {
-		if (cd.cw_dir) {
-		    dl = (int)((char *)u->u_cwd + cd.cw_len - cd.cw_dir);
-		    if (dl >= MAXPATHLEN)
-			dl = MAXPATHLEN - 1;
-		    if (dl < 2 || kread((KA_T)cd.cw_dir, Cwd, dl))
-			*Cwd = '\0';
-		    else
-			Cwd[dl] = '\0';
-		}
-		if (cd.cw_root) {
-		    if (cd.cw_dir >= cd.cw_root)
-			dl = (int)(cd.cw_dir - cd.cw_root);
-		    else
-			dl = (int)((char *)u->u_cwd + cd.cw_len - cd.cw_root);
-			if (dl >= MAXPATHLEN)
-			    dl = MAXPATHLEN - 1;
-			if (dl < 2 || kread((KA_T)cd.cw_root, Rtd, dl))
-			    *Rtd = '\0';
-			else
-			    Rtd[dl] = '\0';
-		}
-	    }
-#endif	/* ! solaris */
-
 	/*
 	 * Save current working directory information.
 	 */
@@ -294,10 +318,7 @@ gather_proc_info()
 	    }
 	/*
 	 * Save information on file descriptors.
-	 */
-
-#if	defined(solaris)
-	/*
+	 *
 	 * Under Solaris the file pointers are stored in dynamically-linked
 	 * ufchunk structures, each containing NFPREAD file pointers.  The
 	 * first ufchunk structure is in the user area.
@@ -305,7 +326,8 @@ gather_proc_info()
 	 * Under Solaris 2.4 the file pointers are in a dynamically allocated,
 	 * contiguous memory block.
 	 */
-# if	solaris<20400
+
+#if	solaris<20400
 	    for (i = 0, j = 0; i < u->u_nofiles; i++) {
 		if (++j > NFPCHUNK) {
 		    if (!u->u_flist.uf_next)
@@ -316,7 +338,7 @@ gather_proc_info()
 		    j = 1;
 		}
 		if (!u->u_flist.uf_ofile[j-1])
-# else	/* solaris>=20400 */
+#else	/* solaris>=20400 */
 	    for (i = 0, j = NFPREAD; i < u->u_nofiles; i++) {
 		if (++j > NFPREAD) {
 		    k = u->u_nofiles - i;
@@ -331,84 +353,18 @@ gather_proc_info()
 		    j = 1;
 		}
 		if (!uf[j-1].uf_ofile)
-# endif	/* solaris<20400 */
-#else	/* !solaris */
-
-	/*
-	 * Under SunOS 4.1.[123], the file pointers may be located in the
-	 * user structure or in a separately allocated area.  Determine
-	 * which is the case and read the pointers.
-	 */
-	    if ((KA_T)u->u_ofile
-		== ((KA_T)&u->u_ofile_arr[0] - (KA_T)&u
-		+ (KA_T)p->p_uarea))
-	    {
-		nf = NOFILE_IN_U;
-		pofp = &u->u_pofile_arr[0];
-		uf = &u->u_ofile_arr[0];
-	    } else {
-		nf = u->u_lastfile + 1;
-		bc = (MALLOC_S)(nf * sizeof(struct file *));
-		if (bc >xnf) {
-		    if (!xuf)
-			xuf = (struct file **)malloc(bc);
-		    else
-			xuf = (struct file **)realloc((MALLOC_P *)xuf, bc);
-		    if (!xuf) {
-			(void) fprintf(stderr,
-			    "%s: PID %d: no file * space\n", Pn, Lp->pid);
-			Exit(1);
-		    }
-		    xnf = bc;
-		}
-		if (kread((KA_T)u->u_ofile, (char *)xuf, (int)bc))
-		    nf = 0;
-		uf = xuf;
-
-# if	defined(HASFSTRUCT)
-		if (Fsv & FSV_FG) {
-		    bc = (MALLOC_S)nf;
-		    if (bc > npofb) {
-			if (!pofb)
-			    pofb = (char *)malloc(bc);
-			else
-			    pofb = (char *)realloc((MALLOC_P *)pofb, bc);
-			if (!pofb) {
-			    (void) fprintf(stderr,
-				"%s: PID %d: no pofile space\n", Pn, Lp->pid);
-			    Exit(1);
-			}
-			npofb = bc;
-		    }
-		    if (!u->u_pofile
-		    ||  kread((KA_T)u->u_pofile, pofb, bc))
-			zeromem(pofb, bc);
-		    pofp = pofb;
-		} else
-		    pofp = (char *)NULL;
-# endif	/* defined(HASFSTRUCT) */
-
-	    }
-	    *Cwd = *Rtd = '\0';
-	    for (i = 0; i < nf; i++) {
-		if (!uf[i])
-#endif	/* solaris */
+#endif	/* solaris<20400 */
 
 		    continue;
 		alloc_lfile((char *)NULL, i);
 
-#if	defined(solaris)
-# if	solaris<20400
+#if	solaris<20400
 		pofv = (long)u->u_flist.uf_pofile[j-1];
 		process_file((KA_T)u->u_flist.uf_ofile[j-1]);
-# else	/* solaris>=20400 */
+#else	/* solaris>=20400 */
 		pofv = uf[j-1].uf_pofile;
 		process_file((KA_T)uf[j-1].uf_ofile);
-# endif	/* solaris <20400 */
-#else	/* !solaris */
-		pofv = pofp ? pofp[i] : 0l;
-		process_file((KA_T)uf[i]);
-#endif	/* solaris */
+#endif	/* solaris <20400 */
 
 		if (Lf->sf) {
 
@@ -437,7 +393,7 @@ static void
 get_kernel_access()
 {
 	char er[64];
-	int l;
+	int i;
 	KA_T v;
 
 #if	defined(HAS_AFS)
@@ -447,17 +403,9 @@ get_kernel_access()
 /*
  * Check the Solaris or SunOS version number; check the SunOS architecture.
  */
-	(void) ckkv(
+	(void) ckkv("Solaris", LSOF_VSTR, (char *)NULL, (char *)NULL);
 
-#if	defined(solaris)
-		    "Solaris", LSOF_VSTR, (char *)NULL, (char *)NULL
-#else	/* !defined(solaris) */
-		    "SunOS", LSOF_VSTR, (char *)NULL, LSOF_ARCH
-#endif	/* defined(solaris) */
-
-	);
-
-#if	defined(solaris) && solaris>=70000
+#if	solaris>=70000
 /*
  * Compare the Solaris 7 lsof compilation bit size with the kernel bit size.
  * Quit on a mismatch.
@@ -496,7 +444,7 @@ get_kernel_access()
 		Exit(1);
 	    }
 	}
-#endif	/* defined(solaris) && solaris>=70000 */
+#endif	/* solaris>=70000 */
 
 /*
  * Get kernel symbols.
@@ -518,13 +466,7 @@ get_kernel_access()
 		    Pn, Nll);
 		Exit(1);
 	    }
-
-# if	defined(solaris)
 	    (void) memcpy((void *)nl, (void *)Nl, (size_t)Nll);
-# else	/* !defined(solaris) */
-	    (void) memcpy((char *)nl, (char *)Nl, (int)Nll);
-# endif	/* defined(solaris) */
-
 	}
 #endif	/* defined(HAS_AFS) */
 
@@ -552,8 +494,6 @@ get_kernel_access()
 #endif	/* defined(HAS_AFS) */
 
 #if	defined(WILLDROPGID)
-
-# if	defined(solaris)
 /*
  * If Solaris kernel memory is coming from KMEM, and the process is willing
  * to surrender GID permission, set up for GID switching after the first
@@ -564,8 +504,6 @@ get_kernel_access()
 	    if (Setgid)
 		Switchgid = 1;
 	}
-# endif	/* defined(solaris) */
-
 /*
  * If kernel memory isn't coming from KMEM, drop setgid permission
  * before attempting to open the (Memory) file.
@@ -592,8 +530,7 @@ get_kernel_access()
 	||  Knp < 1)
 	    Knp = PROCDFLT;
 
-#if	defined(solaris)
-# if	solaris>=20500
+#if	solaris>=20500
 /*
  * Get the kernel's KERNELBASE value for Solaris 2.5 and above.
  */
@@ -602,10 +539,10 @@ get_kernel_access()
 	||  kread((KA_T)v, (char *)&Kb, sizeof(Kb))) {
 	    (void) fprintf(stderr,
 		"%s: can't read kernel base address from %s\n",
-		Pn, print_kptr(v, (char *)NULL));
+		Pn, print_kptr(v, (char *)NULL, 0));
 	    Exit(1);
 	}
-# endif	/* solaris >=20500 */
+#endif	/* solaris>=20500 */
 
 /*
  * Get the Solaris clone major device number, if possible.
@@ -614,18 +551,20 @@ get_kernel_access()
 	if (get_Nl_value("clmaj", Drive_Nl, &v) >= 0 && v
 	&&  kread((KA_T)v, (char *)&CloneMaj, sizeof(CloneMaj)) == 0)
 	    HaveCloneMaj = 1;
+
+#if	solaris>=20501
 /*
- * The Solaris process table is a linked list whose head pointer is acquired
- * by open_kvm()'s call to kvm_open().  We call close_kvm() here so that the
- * process table head pointer can be re-acquired by a call to open_kvm()
- * immediately before the actual reading of the process table.
- *
- * Since the SunOS 4.1.x process table is sequential, we need not worry
- * about a stale or incorrect list head pointer.  Thus we don't have to call
- * close_kvm().  Instead, we call kvm_setproc() each time we read it.
+ * Get the kernel's virtual to physical map structure for Solaris 2.5.1 and
+ * above.
  */
-	close_kvm();
-#endif	/* defined(solaris) */
+	if (get_Nl_value("kasp", Drive_Nl, &v) >= 0 && v) {
+	    PageSz = getpagesize();
+	    PSMask = PageSz - 1;
+	    for (i = 1, PSShft = 0; i < PageSz; i <<= 1, PSShft++)
+		;
+	    (void) readkam(v);
+	}
+#endif	/* solaris>=20501 */
 
 }
 
@@ -638,8 +577,6 @@ void
 initialize()
 {
 	get_kernel_access();
-
-#if	defined(solaris)
 /*
  * Read Solaris file system information and construct the clone table.
  *
@@ -647,26 +584,11 @@ initialize()
  */
 	readfsinfo();
 
-# if	defined(HASDCACHE)
+#if	defined(HASDCACHE)
 	readdev(0);
-# else	/* !defined(HASDCACHE) */
+#else	/* !defined(HASDCACHE) */
 	read_clone();
-# endif	/*defined(HASDCACHE) */
-#else	/* !defined(solaris) */
-/*
- * Create current working and root directory buffers.
- */
-	if ((Cwd = (char *)malloc(MAXPATHLEN)) == NULL) {
-	    (void) fprintf(stderr, "%s: no space for cwd buffer\n", Pn);
-	    Exit(1);
-	}
-	if ((Rtd = (char *)malloc(MAXPATHLEN)) == NULL) {
-	    (void) fprintf(stderr, "%s: no space for root directory buffer\n",
-		Pn);
-	    Exit(1);
-	}
-	*Cwd = *Rtd = '\0';
-#endif	/* solaris */
+#endif	/*defined(HASDCACHE) */
 
 }
 
@@ -681,7 +603,7 @@ kread(addr, buf, len)
 	char *buf;			/* buffer to receive data */
 	READLEN_T len;			/* length to read */
 {
-	int br;
+	register int br;
 /*
  * Because lsof reads kernel data and follows pointers found there at a
  * rate considerably slower than the kernel, lsof sometimes acquires
@@ -691,17 +613,105 @@ kread(addr, buf, len)
  * kernel's _kernelbase variable for Solaris 2.5 and above).
  */
 
-#if	defined(solaris) && solaris>=20500
+#if	solaris>=20500
 #define	KVMREAD	kvm_kread
 	if (addr < Kb)
-#else	/* !defined(solaris) || solaris<20500 */
-#define	KVMREAD	kvm_read
+#else	/* solaris<20500 */
+#define	KVMREAD kvm_read
 	if (addr < (KA_T)KERNELBASE)
-#endif	/* defined(solaris) && solaris>=20500 */
+#endif	/* solaris>=20500 */
 
-		return(1);
+	    return(1);
+
+#if	solaris>=20501
+
+/*
+ * Make sure the virtual address represents real physical memory by testing
+ * it with kvm_physaddr().
+ *
+ * For Solaris below 7 read the kernel data with llseek() and read().  For
+ * Solaris 7 and above use kvm_pread().
+ */
+	if (Kas) {
+
+# if	solaris>20501
+	    register int b2r;
+	    register char *bp;
+# endif	/* solaris>20501 */
+
+	    register int h, ip, tb;
+	    register kvmhash_t *kp;
+	    KPHYS pa;
+	    register KVIRT va, vpa;
+
+# if	solaris<20600
+	    for (tb = 0, va = (KVIRT)addr;
+		 tb < len;
+		 tb += br, va += (KVIRT)br)
+# else	/* solaris>=20600 */
+	    for (bp = buf, tb = 0, va = (KVIRT)addr;
+		 tb < len;
+		 bp += br, tb += br, va += (KVIRT)br)
+# endif	/* solaris<20600 */
+
+	    {
+		vpa = (va & (KVIRT)~PSMask) >> PSShft;
+		ip = (int)(va & (KVIRT)PSMask);
+		h = HASHKVM(vpa);
+		for (kp = KVMhb[h]; kp; kp = kp->nxt) {
+		    if (kp->vpa == vpa) {
+			pa = kp->pa;
+			break;
+		    }
+		}
+		if (!kp) {
+		    if ((pa = kvm_physaddr(Kd, Kas, va)) == KAERR)
+			return(1);
+		    if (!(kp = (kvmhash_t *)malloc(sizeof(kvmhash_t)))) {
+			(void) fprintf(stderr, "%s: no kvmhash_t space\n", Pn);
+			Exit(1);
+		    }
+		    kp->nxt = KVMhb[h];
+		    pa = kp->pa = (pa & ~(KPHYS)PSMask);
+		    kp->vpa = vpa;
+		    KVMhb[h] = kp;
+		}
+
+# if	solaris<20600
+		br = (int)(len - tb);
+		if ((ip + br) > PageSz)
+		    br = PageSz - ip;
+# else	/* solaris>=20600 */
+		b2r = (int)(len - tb);
+		if ((ip + b2r) > PageSz)
+		    b2r = PageSz - ip;
+		pa |= (KPHYS)ip;
+
+#  if	solaris<70000
+		if (llseek(Kmd, (offset_t)pa, SEEK_SET) == (offset_t)-1)
+		    return(1);
+		if ((br = (int)read(Kmd, (void *)bp, (size_t)b2r)) <= 0)
+		    return(1);
+#  else	/* solaris>=70000 */
+		if ((br = kvm_pread(Kd, pa, (void *)bp, (size_t)b2r)) <= 0)
+		    return(1);
+#  endif	/* solaris<70000 */
+# endif	/* solaris<20600 */
+
+	    }
+
+# if	solaris>=20600
+	    return(0);
+# endif	/* solaris>=20600 */
+
+	}
+#endif	/* solaris>=20501 */
+
+/*
+ * Use kvm_read for Solaris < 2.5; use kvm_kread() Solaris >= 2.5.
+ */
 	br = KVMREAD(Kd, (u_long)addr, buf, len);
-	return((br == len) ? 0 : 1);
+	return(((READLEN_T)br == len) ? 0 : 1);
 }
 
 
@@ -709,27 +719,19 @@ kread(addr, buf, len)
  * open_kvm() - open kernel virtual memory access
  */
 
-static void
+void
 open_kvm()
 {
 	if (Kd)
 	    return;
 
-#if	defined(solaris) && defined(WILLDROPGID)
+#if	defined(WILLDROPGID)
 /*
  * If this Solaris process began with setgid permission and its been
  * surrendered, regain it.
  */
-	if (Switchgid == 2 && !Setgid) {
-	    if (setgid(Savedgid) != 0) {
-		(void) fprintf(stderr,
-		    "%s: can't set effective GID to %d: %s\n",
-		    Pn, (int)Savedgid, strerror(errno));
-		Exit(1);
-	    }
-	    Setgid = 1;
-	}
-#endif	/* defined(solaris) && defined(WILLDROPGID) */
+	(void) restoregid();
+#endif	/* defined(WILLDROPGID) */
 
 	if (!(Kd = kvm_open(Nmlst, Memory, NULL, O_RDONLY, NULL))) {
 	    (void) fprintf(stderr, "%s: kvm_open (namelist=%s, core=%s): %s\n",
@@ -740,21 +742,26 @@ open_kvm()
 	    Exit(1);
 	}
 
+#if	solaris>=20501 && solaris<70000
+	if ((Kmd = open((Memory ? Memory : KMEM), O_RDONLY)) < 0) {
+	    (void) fprintf(stderr, "%s: open(\"/dev/mem\"): %s\n", Pn, 
+		strerror(errno));
+	    Exit(1);
+	}
+#endif	/* solaris>=20501 && solaris<70000 */
+
 #if	defined(WILLDROPGID)
 /*
  * If this process has setgid permission, and is willing to surrender it,
  * do so.
  */
 	(void) dropgid();
-
-# if	defined(solaris)
 /*
  * If this Solaris process must switch GIDs, enable switching after the
  * first call to this function.
  */
 	if (Switchgid == 1)
 	    Switchgid = 2;
-# endif	/* defined(solaris) */
 #endif	/* define(WILLDROPGID) */
 
 }
@@ -764,11 +771,151 @@ open_kvm()
  * process_text() - process text access information
  */
 
-#if	defined(solaris) && solaris>=20400
+#if	solaris>=90000
+#include <sys/avl.h>
+
+/*
+ * Avl trees are implemented as follows: types in AVL trees contain an
+ * avl_node_t.  These avl_nodes connect to other avl nodes embedded in
+ * objects of the same type.  The avl_tree contains knowledge about the
+ * size of the structure and the offset of the AVL node in the object
+ * so we can convert between AVL nodes and (in this case) struct seg. 
+ *
+ * This code was provided by Casper Dik <Casper.Dik@holland.sun.com>.
+ */
+
+#define READ_AVL_NODE(n,o,s) \
+	if (kread((KA_T)AVL_NODE2DATA(n, o), (char*) s, sizeof(*s))) \
+		return -1
+
+static int
+get_first_seg(avl_tree_t *av, struct seg *s)
+{
+	avl_node_t *node = av->avl_root;
+	size_t off = av->avl_offset;
+	int count = 0;
+
+	while (node != NULL && ++count < MAXSEGS * 2) {
+	    READ_AVL_NODE(node, off, s);
+	    node = s->s_tree.avl_child[0];
+	    if (node == NULL)
+		return 0;
+	}
+	return -1;
+}
+
+static int
+get_next_seg(avl_tree_t *av, struct seg *s)
+{
+	avl_node_t *node = &s->s_tree;
+	size_t off = av->avl_offset;
+	int count = 0;
+
+	if (node->avl_child[1]) {
+	    /*
+	     * Has right child, go all the way to the leftmost child of
+	     * the right child.
+	     */
+	    READ_AVL_NODE(node->avl_child[1], off, s);
+	    while (node->avl_child[0] != NULL && ++count < 2 * MAXSEGS)
+		 READ_AVL_NODE(node->avl_child[0],off,s);
+	    if (count < 2 * MAXSEGS)
+		return 0;
+	} else {
+	    /*
+	     * No right child, go up until we find a node we're not a right
+	     * child of.
+	     */
+	    for (;count < 2 * MAXSEGS; count++) {
+		int index = AVL_XCHILD(node);
+		avl_node_t *parent = AVL_XPARENT(node);
+
+		if (parent == NULL)
+		    return -1;
+
+		READ_AVL_NODE(parent, off, s);
+
+		if (index == 0)
+		    return 0;
+	    }
+	}
+	return -1;
+}
+
+static void
+process_text(pa)
+	KA_T pa;			/* address space description pointer */
+{
+	struct as as;
+	int i, j, k, l;
+	struct seg s;
+	struct segvn_data vn;
+	avl_tree_t *avtp;
+	KA_T v[MAXSEGS];
+/*
+ * Get address space description.
+ */
+	if (kread((KA_T)pa, (char *)&as, sizeof(as))) {
+	    alloc_lfile(" txt", -1);
+	    (void) snpf(Namech, Namechl, "can't read text segment list (%s)",
+		print_kptr(pa, (char *)NULL, 0));
+	    enter_nm(Namech);
+	    if (Lf->sf)
+		link_lfile();
+	    return;
+	}
+/*
+ * Loop through the segments.  The loop should stop when the segment
+ * pointer returns to its starting point, but just in case, it's stopped
+ * when MAXSEGS have been recorded or 2*MAXSEGS have been examined.
+ */
+	avtp = &as.a_segtree;
+
+	for (i = j = k = 0; i < MAXSEGS && j < 2*MAXSEGS; j++) {
+	    if (j == 0 ? get_first_seg(avtp, &s) : get_next_seg(avtp, &s))
+		break;
+		
+	    if ((KA_T)s.s_ops == Sgvops && s.s_data) {
+		if (kread((KA_T)s.s_data, (char *)&vn, sizeof(vn)))
+		    break;
+		if (vn.vp) {
+			
+		/*
+		 * This is a virtual node segment.
+		 *
+		 * If its vnode pointer has not been seen already,
+		 * print its information.
+		 */
+		    for (l = 0; l < k; l++) {
+			if (v[l] == (KA_T)vn.vp)
+			    break;
+		    }
+		    if (l >= k) {
+			alloc_lfile(" txt", -1);
+
+# if	defined(FILEPTR)
+			FILEPTR = (struct file *)NULL;
+# endif	/* defined(FILEPTR) */
+
+			process_node((KA_T)vn.vp);
+			if (Lf->sf) {
+			    link_lfile();
+			    i++;
+			}
+			v[k++] = (KA_T)vn.vp;
+		    }
+		}
+	    }
+	}
+}
+
+#else	/* solaris<90000 */
+
+# if	solaris>=20400
 #define S_NEXT s_next.list
-#else	/* !defined(solaris) || solaris<20400 */
+# else	/* solaris<20400 */
 #define S_NEXT s_next
-#endif	/* defined(solaris) && solaris>=20400 */
+# endif	/* solaris>=20400 */
 
 static void
 process_text(pa)
@@ -784,8 +931,8 @@ process_text(pa)
  */
 	if (kread((KA_T)pa, (char *)&as, sizeof(as))) {
 	    alloc_lfile(" txt", -1);
-	    (void) sprintf(Namech, "can't read text segment list (%s)",
-		print_kptr(pa, (char *)NULL));
+	    (void) snpf(Namech, Namechl, "can't read text segment list (%s)",
+		print_kptr(pa, (char *)NULL, 0));
 	    enter_nm(Namech);
 	    if (Lf->sf)
 		link_lfile();
@@ -819,9 +966,9 @@ process_text(pa)
 		    if (l >= k) {
 			alloc_lfile(" txt", -1);
 
-#if	defined(FILEPTR)
+# if	defined(FILEPTR)
 			FILEPTR = (struct file *)NULL;
-#endif	/* defined(FILEPTR) */
+# endif	/* defined(FILEPTR) */
 
 			process_node((KA_T)vn.vp);
 			if (Lf->sf) {
@@ -838,18 +985,18 @@ process_text(pa)
 	 * limit on the loop.)
 	 */
 
-#if	!defined(solaris) || solaris<20400
+# if	solaris<20400
 	    if (s.s_next == as.a_segs)
-#else	/* solaris>=20400 */
+# else	/* solaris>=20400 */
 	    if (s.s_next.list == as.a_segs.list)
-#endif	/* !defined(solaris) && solaris<20400 */
+# endif	/* solaris<20400 */
 
 		break;
 	}
 }
+#endif  /* solaris>=90000 */
 
 
-#if	defined(solaris)
 /*
  * readfsinfo() - read file system information
  */
@@ -858,7 +1005,7 @@ static void
 readfsinfo()
 {
 	char buf[FSTYPSZ+1];
-	int i;
+	int i, len;
 
 	if ((Fsinfomax = sysfs(GETNFSTYP)) == -1) {
 	    (void) fprintf(stderr, "%s: sysfs(GETNFSTYP) error: %s\n",
@@ -883,12 +1030,13 @@ readfsinfo()
 		continue;
 	    }
 	    buf[FSTYPSZ] = '\0';
-	    if (!(Fsinfo[i-1] = (char *)malloc((MALLOC_S)(strlen(buf) + 1)))) {
+	    len = strlen(buf) + 1;
+	    if (!(Fsinfo[i-1] = (char *)malloc((MALLOC_S)len))) {
 		(void) fprintf(stderr,
 		    "%s: no space for file system entry %s\n", Pn, buf);
 		Exit(1);
 	    }
-	    (void) strcpy(Fsinfo[i-1], buf);
+	    (void) snpf(Fsinfo[i-1], len, "%s", buf);
 
 # if	defined(HAS_AFS)
 	    if (strcasecmp(buf, "afs") == 0)
@@ -897,7 +1045,58 @@ readfsinfo()
 
 	}
 }
-#endif	/* solaris */
+
+
+#if	solaris>=20501
+/*
+ * readkam() - read kernel's address map structure
+ */
+
+static void
+readkam(addr)
+	KA_T addr;			/* kernel virtual address */
+{
+	register int i;
+	register kvmhash_t *kp, *kpp;
+	static KA_T kas = (KA_T)NULL;
+	size_t sz;
+
+	if (addr)
+	    kas = addr;
+	Kas = (struct as *)NULL;
+
+#if	solaris<70000
+	if (kas && !kread(kas, (char *)&Kam, sizeof(Kam)))
+	    Kas = (KA_T)&Kam;
+#else	/* solaris>=70000 */
+	Kas = (struct as *)kas;
+#endif	/* solaris<70000 */
+
+	if (Kas) {
+	    if (!KVMhb) {
+		if (!(KVMhb = (kvmhash_t **)calloc(KVMHASHBN,
+						   sizeof(kvmhash_t *))))
+		{
+		     (void) fprintf(stderr,
+			"%s: no space (%d) for KVM hash buckets\n",
+			Pn, (KVMHASHBN * sizeof(kvmhash_t *)));
+		    Exit(1);
+		}
+	    } else if (!addr) {
+		for (i = 0; i < KVMHASHBN; i++) {
+		    if ((kp = KVMhb[i])) {
+			while (kp) {
+			    kpp = kp->nxt;
+			    (void) free((void *)kp);
+			    kp = kpp;
+			}
+			KVMhb[i] = (kvmhash_t *)NULL;
+		    }
+		}
+	    }
+	}
+}
+#endif	/* solaris>=20501 */
 
 
 /*
@@ -913,11 +1112,7 @@ read_proc()
 	static int sz = 0;
 	int try;
 	struct proc *p;
-
-#if	defined(solaris)
 	struct pid pg, pids;
-#endif	/* defined(solaris) */
-
 /*
  * Try PROCTRYLM times to read a valid proc table.
  */
@@ -933,15 +1128,13 @@ read_proc()
 		    (void) fprintf(stderr, "%s: no proc table space\n", Pn);
 		    Exit(1);
 		}
-
-#if	defined(solaris)
 	/*
-	 * Pre-allocate space for Solaris PGRP and PID numbers.
+	 * Pre-allocate space for Solaris PGID and PID numbers.
 	 */
 		len = (MALLOC_S)(sz * sizeof(int));
-		if (Fpgrp) {
-		    if (!(Pgrp = (int *)malloc(len))) {
-			(void) fprintf(stderr, "%s: no PGRP table space\n", Pn);
+		if (Fpgid) {
+		    if (!(Pgid = (int *)malloc(len))) {
+			(void) fprintf(stderr, "%s: no PGID table space\n", Pn);
 			Exit(1);
 		    }
 		}
@@ -949,8 +1142,6 @@ read_proc()
 		    (void) fprintf(stderr, "%s: no PID table space\n", Pn);
 		    Exit(1);
 		}
-#endif	/* defined(solaris) */
-
 	    }
 	/*
 	 * Prepare for a proc table scan.
@@ -969,19 +1160,18 @@ read_proc()
 		if (p->p_stat == 0 || p->p_stat == SZOMB)
 		    continue;
 
-#if	defined(solaris)
-# if	solaris >=20500
+#if	solaris >=20500
 		/*
 		 * Check Solaris 2.5 and above p_cred pointer.
 		 */
 	    	    if (!p->p_cred)
 			continue;
-# endif	/* solaris >=20500 */
+#endif	/* solaris >=20500 */
 
 		/*
-		 * Read Solaris PGRP and PID numbers.
+		 * Read Solaris PGID and PID numbers.
 		 */
-		if (Fpgrp) {
+		if (Fpgid) {
 		    if (!p->p_pgidp
 		    ||  kread((KA_T)p->p_pgidp, (char *)&pg, sizeof(pg)))
 			continue;
@@ -989,8 +1179,6 @@ read_proc()
 		if (!p->p_pidp
 		||  kread((KA_T)p->p_pidp, (char *)&pids, sizeof(pids)))
 		    continue;
-#endif	/* defined(solaris) */
-
 		if (Np >= sz) {
 
 		/*
@@ -1003,16 +1191,14 @@ read_proc()
 			    "%s: no more (%d) proc space\n", Pn, sz);
 			Exit(1);
 		    }
-
-#if	defined(solaris)
 		/*
-		 * Expand the Solaris PGRP and PID tables.
+		 * Expand the Solaris PGID and PID tables.
 		 */
 		    len = (MALLOC_S)(sz * sizeof(int));
-		    if (Fpgrp) {
-			if (!(Pgrp = (int *)realloc((MALLOC_P *)Pgrp, len))) {
+		    if (Fpgid) {
+			if (!(Pgid = (int *)realloc((MALLOC_P *)Pgid, len))) {
 			    (void) fprintf(stderr,
-				"%s: no more (%d) PGRP space\n", Pn, sz);
+				"%s: no more (%d) PGID space\n", Pn, sz);
 			    Exit(1);
 			}
 		    }
@@ -1021,20 +1207,14 @@ read_proc()
 			    "%s: no more (%d) PID space\n", Pn, sz);
 			Exit(1);
 		    }
-#endif	/* defined(solaris) */
-
 		}
-
-#if	defined(solaris)
 	    /*
-	     * Save the Solaris PGRP and PID numbers in
+	     * Save the Solaris PGID and PID numbers in
 	     * local tables.
 	     */
-		if (Fpgrp)
-		    Pgrp[Np] = (int)pg.pid_id;
+		if (Fpgid)
+		    Pgid[Np] = (int)pg.pid_id;
 		Pid[Np] = (int)pids.pid_id;
-#endif	/* defined(solaris) */
-
 	    /*
 	     * Save the proc structure in a local table.
 	     */
@@ -1045,11 +1225,7 @@ read_proc()
 	 */
 	    if (Np >= PROCMIN)
 		break;
-
-#if	defined(solaris)
 	    close_kvm();
-#endif	/* defined(solaris) */
-
 	}
 /*
  * Quit if no proc structures were stored in the local table.
@@ -1070,17 +1246,15 @@ read_proc()
 		    Pn, Np);
 		Exit(1);
 	    }
-
-#if	defined(solaris)
 	/*
-	 * Reduce the Solaris PGRP and PID tables to their minimum if
+	 * Reduce the Solaris PGID and PID tables to their minimum if
 	 * not in repeat mode.
 	 */
 	    len = (MALLOC_S)(Np * sizeof(int));
-	    if (Fpgrp) {
-		if (!(Pgrp = (int *)realloc((MALLOC_P *)Pgrp, len))) {
+	    if (Fpgid) {
+		if (!(Pgid = (int *)realloc((MALLOC_P *)Pgid, len))) {
 		    (void) fprintf(stderr,
-			"%s: can't reduce PGRP table to %d\n", Pn, Np);
+			"%s: can't reduce PGID table to %d\n", Pn, Np);
 		    Exit(1);
 		}
 	    }
@@ -1089,7 +1263,26 @@ read_proc()
 		    "%s: can't reduce PID table to %d\n", Pn, Np);
 		Exit(1);
 	    }
-#endif	/* defined(solaris) */
-
 	}
 }
+
+
+#if	defined(WILLDROPGID)
+/*
+ * restoregid() -- restore setgid permission, as required
+ */
+
+void
+restoregid()
+{
+	if (Switchgid == 2 && !Setgid) {
+	    if (setgid(Savedgid) != 0) {
+		(void) fprintf(stderr,
+		    "%s: can't set effective GID to %d: %s\n",
+		    Pn, (int)Savedgid, strerror(errno));
+		Exit(1);
+	    }
+	    Setgid = 1;
+	}
+}
+#endif	/* defined(WILLDROPGID) */
