@@ -1,5 +1,5 @@
 /*
- * dproto.h - SunOS (Solaris 1.x and 2.x)  function prototypes for lsof
+ * dproto.h - Solaris function prototypes for lsof
  *
  * The _PROTOTYPE macro is defined in the common proto.h.
  */
@@ -33,13 +33,14 @@
 
 
 /*
- * $Id: dproto.h,v 1.8 99/05/04 09:04:06 abe Exp $
+ * $Id: dproto.h,v 1.13 2001/07/05 12:26:21 abe Exp $
  */
 
 
 _PROTOTYPE(extern void completevfs,(struct l_vfs *vfs, dev_t *dev));
 _PROTOTYPE(extern int is_file_named,(char *p, int nt, enum vtype vt, int ps));
 _PROTOTYPE(extern struct l_vfs *readvfs,(KA_T ka, struct vfs *la, struct vnode *lv));
+_PROTOTYPE(extern int vop2ty,(struct vnode *vp));
 
 #if	defined(HAS_AFS)
 _PROTOTYPE(extern struct vnode *alloc_vcache,(void));
@@ -51,22 +52,23 @@ _PROTOTYPE(extern int readafsnode,(KA_T va, struct vnode *v, struct afsnode *an)
 #if	defined(HASDCACHE)
 _PROTOTYPE(extern int rw_clone_sect,(int m));
 _PROTOTYPE(extern void clr_sect,(void));
-# if	defined(solaris)
 _PROTOTYPE(extern int rw_pseudo_sect,(int m));
-# endif	/* defined_solaris */
 #endif	/* defined(HASDCACHE) */
 
 #if	defined(HASVXFS)
-_PROTOTYPE(extern int read_vxnode,(KA_T va, struct vnode *v, struct l_vfs *vfs,dev_t *dev, unsigned long *ino, long *nl, unsigned long *sz));
+_PROTOTYPE(extern int read_vxnode,(KA_T va, struct vnode *v, struct l_vfs *vfs,
+				   struct l_ino *li, KA_T *vnops));
 #endif	/* defined(HASVXFS) */
 
-#if	defined(solaris)
+_PROTOTYPE(extern void close_kvm,(void));
+_PROTOTYPE(extern void open_kvm,(void));
 _PROTOTYPE(extern void process_socket,(KA_T sa, char *ty));
 _PROTOTYPE(extern void read_clone,(void));
-#else	/* !defined(solaris) */
-_PROTOTYPE(extern void process_socket,(KA_T sa));
-#endif	/* defined(solaris) */
 
-#if	defined(solaris) && solaris<20500
+#if	solaris<20500
 _PROTOTYPE(extern int get_max_fd,(void));
-#endif	/* defined(solaris) && solaris<20500 */
+#endif	/* solaris<20500 */
+
+#if	defined(WILLDROPGID)
+_PROTOTYPE(extern void restoregid,(void));
+#endif	/* defined(WILLDROPGID) */
