@@ -32,7 +32,7 @@
 #ifndef lint
 static char copyright[] =
 "@(#) Copyright 1996 Purdue Research Foundation.\nAll rights reserved.\n";
-static char *rcsid = "$Id: dnode.c,v 1.14 99/06/22 09:52:49 abe Exp $";
+static char *rcsid = "$Id: dnode.c,v 1.21 2001/04/19 12:10:37 abe Exp $";
 #endif
 
 
@@ -45,6 +45,10 @@ static char *rcsid = "$Id: dnode.c,v 1.14 99/06/22 09:52:49 abe Exp $";
 #undef	IEXEC
 #endif	/* UNIXWAREV>=70000 */
 
+#if	defined(HAS_UW_CFS)
+#include <fs/nsc_cfs/cnode.h>
+#endif	/* defined(HAS_UW_CFS) */
+
 #include <sys/fs/ufs_inode.h>
 
 #if	defined(HASXNAMNODE)
@@ -55,7 +59,7 @@ static char *rcsid = "$Id: dnode.c,v 1.14 99/06/22 09:52:49 abe Exp $";
 _PROTOTYPE(static void ent_fa,(KA_T *a1, KA_T *a2, char *d));
 _PROTOTYPE(static int get_vty,(struct vnode *v, KA_T va, struct vfs *kv, int *fx));
 _PROTOTYPE(static int examine_stream,(KA_T vs, struct queue *q, char *mn, char *sn, KA_T *sqp));
-_PROTOTYPE(static struct l_dev * findstrdev,(dev_t *dev));
+_PROTOTYPE(static struct l_dev * findstrdev,(dev_t *dev, dev_t *rdev));
 _PROTOTYPE(static char isvlocked,(struct vnode *va));
 _PROTOTYPE(static int readlino,(int fx, struct vnode *v, struct l_ino *i));
 
@@ -94,10 +98,12 @@ ent_fa(a1, a2, d)
 	if (Lf->nma)
 	    return;
 	if (!a1)
-	    (void) sprintf(buf, "(FA:%s%s)", d, print_kptr(*a2, (char *)NULL));
+	    (void) snpf(buf, sizeof(buf), "(FA:%s%s)", d,
+		print_kptr(*a2, (char *)NULL, 0));
 	else
-	    (void) sprintf(buf, " (FA:%s%s%s)", print_kptr(*a1, tbuf), d,
-		print_kptr(*a2, (char *)NULL));
+	    (void) snpf(buf, sizeof(buf), " (FA:%s%s%s)",
+		print_kptr(*a1, tbuf, sizeof(tbuf)), d,
+		print_kptr(*a2, (char *)NULL, 0));
 	len = strlen(buf) + 1;
 	if ((cp = (char *)malloc(len)) == NULL) {
 	    (void) fprintf(stderr,
@@ -105,7 +111,7 @@ ent_fa(a1, a2, d)
 		Pn, Lp->pid, Lf->fd);
 	    Exit(1);
 	}
-	(void) strcpy(cp, buf);
+	(void) snpf(cp, len, "%s", buf);
 	Lf->nma = cp;
 }
 
@@ -132,8 +138,8 @@ examine_stream(vs, q, mn, sn, sqp)
  * Read stream's head.
  */
 	if (!vs || readstdata(vs, &sd)) {
-	    (void) sprintf(Namech, "can't read stream head from %s",
-		print_kptr(vs, (char *)NULL));
+	    (void) snpf(Namech, Namechl, "can't read stream head from %s",
+		print_kptr(vs, (char *)NULL, 0));
 	    enter_nm(Namech);
 	    return(1);
 	}
@@ -150,8 +156,8 @@ examine_stream(vs, q, mn, sn, sqp)
 	 * Read stream queue entry.
 	 */
 	    if (kread(qp, (char *)q, sizeof(struct queue))) {
-		(void) sprintf(Namech, "can't read stream queue from %s",
-		    print_kptr(qp, (char *)NULL));
+		(void) snpf(Namech, Namechl, "can't read stream queue from %s",
+		    print_kptr(qp, (char *)NULL, 0));
 		enter_nm(Namech);
 		return(1);
 	    }
@@ -159,9 +165,9 @@ examine_stream(vs, q, mn, sn, sqp)
 	 * Read queue's information structure.
 	 */
 	    if (!q->q_qinfo || readstqinit((KA_T)q->q_qinfo, &qi)) {
-		(void) sprintf(Namech, "can't read qinit for %s from %s",
-		    print_kptr(qp, tbuf),
-		    print_kptr((KA_T)q->q_qinfo, (char *)NULL));
+		(void) snpf(Namech, Namechl, "can't read qinit for %s from %s",
+		    print_kptr(qp, tbuf, sizeof(tbuf)),
+		    print_kptr((KA_T)q->q_qinfo, (char *)NULL, 0));
 		enter_nm(Namech);
 		return(1);
 	    }
@@ -169,9 +175,10 @@ examine_stream(vs, q, mn, sn, sqp)
 	 * Read module information structure.
 	 */
 	    if (!qi.qi_minfo || readstmin((KA_T)qi.qi_minfo, &mi)) {
-		(void) sprintf(Namech, "can't read module info for %s from %s",
-		    print_kptr((KA_T)q->q_qinfo, tbuf),
-		    print_kptr((KA_T)qi.qi_minfo, (char *)NULL));
+		(void) snpf(Namech, Namechl,
+		    "can't read module info for %s from %s",
+		    print_kptr((KA_T)q->q_qinfo, tbuf, sizeof(tbuf)),
+		    print_kptr((KA_T)qi.qi_minfo, (char *)NULL, 0));
 		enter_nm(Namech);
 		return(1);
 	    }
@@ -179,9 +186,10 @@ examine_stream(vs, q, mn, sn, sqp)
 	 * Read module name.
 	 */
 	    if (!mi.mi_idname || kread((KA_T)mi.mi_idname, mn, STRNML-1)) {
-		(void) sprintf(Namech, "can't read module name for %s from %s",
-		    print_kptr((KA_T)qi.qi_minfo, tbuf),
-		    print_kptr((KA_T)mi.mi_idname, (char *)NULL));
+		(void) snpf(Namech, Namechl,
+		    "can't read module name for %s from %s",
+		    print_kptr((KA_T)qi.qi_minfo, tbuf, sizeof(tbuf)),
+		    print_kptr((KA_T)mi.mi_idname, (char *)NULL, 0));
 		enter_nm(Namech);
 		return(1);
 	    }
@@ -203,8 +211,9 @@ examine_stream(vs, q, mn, sn, sqp)
  */
 
 static struct l_dev *
-findstrdev(dev)
+findstrdev(dev, rdev)
 	dev_t *dev;			/* device */
+	dev_t *rdev;			/* raw device */
 {
 	struct clone *c;
 	struct l_dev *dp;
@@ -218,14 +227,14 @@ findstrdev_again:
 
 #endif	/* HASDCACHE */
 
-	if ((dp = lkupdev(dev, 0, 0)))
+	if ((dp = lkupdev(dev, rdev, 0, 0)))
 	    return(dp);
 /*
  * Search for clone.
  */
 	if (Clone) {
 	    for (c = Clone; c; c = c->next) {
-		if (major(*dev) == minor(Devtp[c->dx].rdev)) {
+		if (major(*rdev) == minor(Devtp[c->dx].rdev)) {
 
 #if	HASDCACHE
 		    if (DCunsafe && !Devtp[c->dx].v && !vfy_dev(&Devtp[c->dx]))
@@ -243,7 +252,11 @@ findstrdev_again:
 /*
  * get_vty() - get vnode type
  *
- * return: vnode type as an N_* symbol value (-1 if unrecognized)
+ * return: vnode type as an N_* symbol value
+ *	   N_REGLR if no special file system type applies
+ *	   -1 if the vnode type is VUNNAMED
+ *	   -2 if the vfs structure has an illegal type index
+ *	   -3 if the vfs structure can't be read
  */
 
 static int
@@ -257,9 +270,13 @@ get_vty(v, va, kv, fx)
 	int nty = N_REGLR;
 	char tbuf[32];
 
+	if (v->v_type == VUNNAMED) {
+	    *fx = 0;
+	    return(-1);
+	}
 	if (!v->v_vfsp) {
 	    *fx = 0;
-	    if (v->v_type == VFIFO && v->v_stream)
+	    if ((v->v_type == VFIFO) || v->v_stream)
 		return(N_STREAM);
 	    return(N_REGLR);
 	}
@@ -276,6 +293,9 @@ get_vty(v, va, kv, fx)
 		    nty = N_NFS;
 		else if (!strcmp(Fsinfo[fxt-1], "namefs"))
 		    nty = N_NM;
+		else if (!strcmp(Fsinfo[fxt-1], "nsc_cfs"))
+		    nty = N_CFS;
+
 
 #if	defined(HASPROCFS)
 		else if (!strcmp(Fsinfo[fxt-1], "proc"))
@@ -283,17 +303,18 @@ get_vty(v, va, kv, fx)
 #endif	/* defined(HASPROCFS) */
 
 	    } else {
-		(void) sprintf(Namech, "vnode@%s: bad file system index (%d)",
-		    print_kptr(va, (char *)NULL), fxt);
+		(void) snpf(Namech, Namechl,
+		    "vnode@%s: bad file system index (%d)",
+		    print_kptr(va, (char *)NULL, 0), fxt);
 		enter_nm(Namech);
-		return(-1);
+		return(-2);
 	    }
 	} else {
-	    (void) sprintf(Namech, "vnode@%s: bad vfs pointer (%s)",
-		print_kptr(va, tbuf),
-		print_kptr((KA_T)v->v_vfsp, (char *)NULL));
+	    (void) snpf(Namech, Namechl, "vnode@%s: bad vfs pointer (%s)",
+		print_kptr(va, tbuf, sizeof(tbuf)),
+		print_kptr((KA_T)v->v_vfsp, (char *)NULL, 0));
 	    enter_nm(Namech);
-	    return(-1);
+	    return(-3);
 	}
 	if (nty == N_REGLR) {
 	    if (v->v_type == VFIFO)
@@ -357,29 +378,33 @@ void
 process_node(na)
 	KA_T na;			/* vnode kernel space address */
 {
-	char *cp;
-	dev_t dev;
-	int ds = 0;
+	char *cp, *ep;
+	dev_t dev, rdev;
+	unsigned char devs = 0;
+	unsigned char ni = 0;
+	unsigned char rdevs = 0;
 	struct l_dev *dp;
 	struct fifonode f;
 	int fx, rfx;
 	struct l_ino i;
+	int is = 1;
 	int j, k;
+	KA_T ka;
 	struct vfs kv, rkv;
 	struct module_info mi;
 	char mn[STRNML];
 	struct mnode mno;
-	int ni = 0;
+	MALLOC_S msz;
 	struct namenode nn;
 	int px;
 	struct queue q;
 	struct rnode r;
-	struct snode rs;
 	struct vnode rv, v;
 	struct snode s;
 	unsigned char sd = 1;
 	struct so_so so;
 	KA_T sqp = (KA_T)NULL;
+	size_t sz;
 	char tbuf[32], *ty;
 	enum vtype type;
 	struct sockaddr_un ua;
@@ -422,8 +447,11 @@ process_node(na)
 /*
  * Determine the vnode type.
  */
-	if ((Ntype = get_vty(&v, na, &kv, &fx)) < 0)
+	if ((Ntype = get_vty(&v, na, &kv, &fx)) < 0) {
+	    if (Ntype == -1)
+		Lf->sf = 0;
 	    return;
+	}
 /*
  * Determine the lock state.
  */
@@ -437,26 +465,32 @@ get_lock_state:
 	switch (Ntype) {
 	case N_FIFO:
 	    if (!v.v_data || readfifonode((KA_T)v.v_data, &f)) {
-		(void) sprintf(Namech,
+		(void) snpf(Namech, Namechl,
 		    "vnode@%s: can't read fifonode (%s)",
-		    print_kptr(na, tbuf),
-		    print_kptr((KA_T)v.v_data, (char *)NULL));
+		    print_kptr(na, tbuf, sizeof(tbuf)),
+		    print_kptr((KA_T)v.v_data, (char *)NULL, 0));
 		enter_nm(Namech);
 		return;
 	    }
 	    if (f.fn_realvp) {
 		if (readvnode((KA_T)f.fn_realvp, &rv)) {
-		    (void) sprintf(Namech,
+		    (void) snpf(Namech, Namechl,
 			"fifonode@%s: can't read real vnode (%s)",
-			print_kptr((KA_T)v.v_data, tbuf),
-			print_kptr((KA_T)f.fn_realvp, (char *)NULL));
+			print_kptr((KA_T)v.v_data, tbuf, sizeof(tbuf)),
+			print_kptr((KA_T)f.fn_realvp, (char *)NULL, 0));
 		    enter_nm(Namech);
 		    return;
 		}
-		if (!rv.v_data || readlino(fx, &rv, &i)) {
-		    (void) sprintf(Namech, "fifonode@%s: can't read inode (%s)",
-			print_kptr((KA_T)v.v_data, tbuf),
-			print_kptr((KA_T)rv.v_data, (char *)NULL));
+
+#if	defined(HASNCACHE)
+		Lf->na = (KA_T)f.fn_realvp;
+#endif	/* defined(HASNCACHE) */
+
+		if (!rv.v_data || (is = readlino(fx, &rv, &i))) {
+		    (void) snpf(Namech, Namechl,
+			"fifonode@%s: can't read inode (%s)",
+			print_kptr((KA_T)v.v_data, tbuf, sizeof(tbuf)),
+			print_kptr((KA_T)rv.v_data, (char *)NULL, 0));
 		    enter_nm(Namech);
 		    return;
 		}
@@ -465,18 +499,18 @@ get_lock_state:
 	    break;
 	case N_NFS:
 	    if (!v.v_data || readrnode((KA_T)v.v_data, &r)) {
-		(void) sprintf(Namech, "vnode@%s: can't read rnode (%s)",
-		    print_kptr(na, tbuf),
-		    print_kptr((KA_T)v.v_data, (char *)NULL));
+		(void) snpf(Namech, Namechl, "vnode@%s: can't read rnode (%s)",
+		    print_kptr(na, tbuf, sizeof(tbuf)),
+		    print_kptr((KA_T)v.v_data, (char *)NULL, 0));
 		enter_nm(Namech);
 		return;
 	    }
 	    break;
 	case N_NM:
 	    if (!v.v_data || kread((KA_T)v.v_data, (char *)&nn, sizeof(nn))) {
-		(void) sprintf(Namech, "vnode@%s: no namenode (%s)",
-		    print_kptr(na, tbuf),
-		    print_kptr((KA_T)v.v_data, (char *)NULL));
+		(void) snpf(Namech, Namechl, "vnode@%s: no namenode (%s)",
+		    print_kptr(na, tbuf, sizeof(tbuf)),
+		    print_kptr((KA_T)v.v_data, (char *)NULL, 0));
 		enter_nm(Namech);
 		return;
 	    }
@@ -491,14 +525,17 @@ get_lock_state:
 	 */
 	    (void) ent_fa(&na, (KA_T *)&nn.nm_mountpt, "->");
 	    if (kread((KA_T)nn.nm_mountpt, (char *)&rv, sizeof(rv))) {
-		(void) sprintf(Namech,
+		(void) snpf(Namech, Namechl,
 		    "vnode@%s: can't read namenode's mounted vnode (%s)",
-		    print_kptr(na, tbuf),
-		    print_kptr((KA_T)nn.nm_mountpt, (char *)NULL));
+		    print_kptr(na, tbuf, sizeof(tbuf)),
+		    print_kptr((KA_T)nn.nm_mountpt, (char *)NULL, 0));
 		return;
 	    }
-	    if ((Ntype = get_vty(&rv, (KA_T)nn.nm_mountpt, &rkv, &rfx)) < 0)
+	    if ((Ntype = get_vty(&rv, (KA_T)nn.nm_mountpt, &rkv, &rfx)) < 0) {
+		if (Ntype == -1)
+		    Lf->sf = 0;
 		return;
+	    }
 	/*
 	 * Unless the mounted-over/to node is another "namefs" node, promote
 	 * it to the vnode of interest.
@@ -514,9 +551,9 @@ get_lock_state:
 	case N_PROC:
 	    ni = 1;
 	    if (!v.v_data || kread((KA_T)v.v_data, (char *)&pr, sizeof(pr))) {
-		(void) sprintf(Namech, "vnode@%s: can't read prnode (%s)",
-		    print_kptr(na, tbuf),
-		    print_kptr((KA_T)v.v_data, (char *)NULL));
+		(void) snpf(Namech, Namechl, "vnode@%s: can't read prnode (%s)",
+		    print_kptr(na, tbuf, sizeof(tbuf)),
+		    print_kptr((KA_T)v.v_data, (char *)NULL, 0));
 		enter_nm(Namech);
 		return;
 	    }
@@ -529,46 +566,49 @@ get_lock_state:
 		pid = (long)prc.prc_pid;
 		switch(pr.pr_type) {
 		case PR_PIDDIR:
-		    (void) sprintf(Namech, "/%s/%ld", HASPROCFS, pid);
+		    (void) snpf(Namech, Namechl, "/%s/%ld", HASPROCFS, pid);
 		    break;
 		case PR_AS:
-		    (void) sprintf(Namech, "/%s/%ld/as", HASPROCFS, pid);
+		    (void) snpf(Namech, Namechl, "/%s/%ld/as", HASPROCFS, pid);
 		    break;
 		case PR_CTL:
-		    (void) sprintf(Namech, "/%s/%ld/ctl", HASPROCFS, pid);
+		    (void) snpf(Namech, Namechl, "/%s/%ld/ctl", HASPROCFS, pid);
 		    break;
 		case PR_STATUS:
-		    (void) sprintf(Namech, "/%s/%ld/status", HASPROCFS, pid);
+		    (void) snpf(Namech, Namechl, "/%s/%ld/status", HASPROCFS,
+			pid);
 		    break;
 		case PR_MAP:
-		    (void) sprintf(Namech, "/%s/%ld/map", HASPROCFS, pid);
+		    (void) snpf(Namech, Namechl, "/%s/%ld/map", HASPROCFS, pid);
 		    break;
 		case PR_CRED:
-		    (void) sprintf(Namech, "/%s/%ld/cred", HASPROCFS, pid);
+		    (void) snpf(Namech, Namechl, "/%s/%ld/cred", HASPROCFS, pid);
 		    break;
 		case PR_SIGACT:
-		    (void) sprintf(Namech, "/%s/%ld/sigact", HASPROCFS, pid);
+		    (void) snpf(Namech, Namechl, "/%s/%ld/sigact", HASPROCFS,
+			pid);
 		    break;
 		case PR_OBJECTDIR:
-		    (void) sprintf(Namech, "/%s/%ld/object", HASPROCFS, pid);
+		    (void) snpf(Namech, Namechl, "/%s/%ld/object", HASPROCFS,
+			pid);
 		    break;
 		case PR_LWPDIR:
-		    (void) sprintf(Namech, "/%s/%ld/lwp", HASPROCFS, pid);
+		    (void) snpf(Namech, Namechl, "/%s/%ld/lwp", HASPROCFS, pid);
 		    break;
 		case PR_LWPIDDIR:
-		    (void) sprintf(Namech, "/%s/%ld/lwp/%d",
+		    (void) snpf(Namech, Namechl, "/%s/%ld/lwp/%d",
 			HASPROCFS, pid, prc.prc_lwpid);
 		    break;
 		case PR_LWPCTL:
-		    (void) sprintf(Namech, "/%s/%ld/lwp/%d/lwpctl",
+		    (void) snpf(Namech, Namechl, "/%s/%ld/lwp/%d/lwpctl",
 			HASPROCFS, pid, prc.prc_lwpid);
 		    break;
 		case PR_LWPSTATUS:
-		    (void) sprintf(Namech, "/%s/%ld/lwp/%d/lwpstatus",
+		    (void) snpf(Namech, Namechl, "/%s/%ld/lwp/%d/lwpstatus",
 			HASPROCFS, pid, prc.prc_lwpid);
 		    break;
 		case PR_LWPSINFO:
-		    (void) sprintf(Namech, "/%s/%ld/lwp/%d/lwpsinfo",
+		    (void) snpf(Namech, Namechl, "/%s/%ld/lwp/%d/lwpsinfo",
 			HASPROCFS, pid, prc.prc_lwpid);
 		    break;
 		}
@@ -586,22 +626,23 @@ get_lock_state:
 		break;
 	    }
 	    if (kread((KA_T)pr.pr_proc, (char *)&p, sizeof(p))) {
-		(void) sprintf(Namech, "prnode@%s: can't read proc (%s)",
-		    print_kptr((KA_T)v.v_data, tbuf),
-		    print_kptr((KA_T)pr.pr_proc, (char *)NULL));
+		(void) snpf(Namech, Namechl, "prnode@%s: can't read proc (%s)",
+		    print_kptr((KA_T)v.v_data, tbuf, sizeof(tbuf)),
+		    print_kptr((KA_T)pr.pr_proc, (char *)NULL, 0));
 		enter_nm(Namech);
 		return;
 	    }
 	    if (!p.p_pidp || kread((KA_T)p.p_pidp, (char *)&pids, sizeof(pids)))
 	    {
-		(void) sprintf(Namech, "proc struct at %s: can't read pid (%s)",
-		    print_kptr((KA_T)pr.pr_proc, tbuf),
-		    print_kptr((KA_T)p.p_pidp, (char *)NULL));
+		(void) snpf(Namech, Namechl,
+		    "proc struct at %s: can't read pid (%s)",
+		    print_kptr((KA_T)pr.pr_proc, tbuf, sizeof(tbuf)),
+		    print_kptr((KA_T)p.p_pidp, (char *)NULL, 0));
 		enter_nm(Namech);
 		return;
 	    }
 	    pid = (long)pids.pid_id;
-	    (void) sprintf(Namech, "/%s/%ld", HASPROCFS, pid);
+	    (void) snpf(Namech, Namechl, "/%s/%ld", HASPROCFS, pid);
 	    i.number = pid + PR_INOBIAS;
 	    if (!p.p_as ||  kread((KA_T)p.p_as, (char *)&as, sizeof(as)))
 		sd = 0;
@@ -614,6 +655,18 @@ get_lock_state:
 	case N_STREAM:
 	    if (v.v_stream) {
 		Lf->is_stream = ni = 1;
+
+#if	UNIXWAREV>=70101
+		if (process_unix_sockstr(&v, na)) {
+
+		/*
+		 * The stream is a UNIX socket stream.  No more need be done;
+		 * process_unix_stream() has done it all.
+		 */
+		    return;
+		}
+#endif	/* UNIXWAREV>=70101 */
+
 	    /*
 	     * Get the queue pointer and module name at the end of the stream.
 	     * The module name indentifies socket streams.
@@ -633,19 +686,20 @@ get_lock_state:
 		if (sqp && kread(sqp, (char *)&so, sizeof(so)) == 0)
 		    break;
 		sqp = (KA_T)NULL;
-		(void) strcpy(Namech, "STR");
+		(void) snpf(Namech, Namechl, "STR");
 		j = strlen(Namech);
 		if (v.v_type == VCHR) {
 	    /*
 	     * If this is a VCHR stream, look up the device name and record it.
 	     */
-		    if ((dp = findstrdev((dev_t *)&v.v_rdev))) {
+		    if ((dp = findstrdev(&DevDev, (dev_t *)&v.v_rdev))) {
 			Lf->inode = (unsigned long)dp->inode;
 			Lf->inp_ty = 1;
 			Namech[j++] = ':';
 			k = strlen(dp->name);
-			if ((j + k) <= (MAXPATHLEN - 1)) {
-			    (void) strcpy(&Namech[j], dp->name);
+			if ((j + k) <= (Namechl - 1)) {
+			    (void) snpf(&Namech[j], Namechl - j, "%s",
+				dp->name);
 			    j += k;
 			    if ((cp = strrchr(Namech, '/'))
 			    &&  *(cp + 1) == '\0')
@@ -660,16 +714,16 @@ get_lock_state:
 	     * Follow the "STR" and possibly the device name with "->" and
 	     * the module name.
 	     */
-		if ((j + 2) <= (MAXPATHLEN - 1)) {
-		    (void) strcpy(&Namech[j], "->");
+		if ((j + 2) <= (Namechl - 1)) {
+		    (void) snpf(&Namech[j], Namechl - j, "->");
 		    j += 2;
 		}
 		if (mn[0]) {
-		    if ((j + strlen(mn)) <= (MAXPATHLEN - 1))
-			(void) strcpy(&Namech[j], mn);
+		    if ((j + strlen(mn)) <= (Namechl - 1))
+			(void) snpf(&Namech[j], Namechl - j, mn);
 		} else {
-		    if ((j + strlen("none")) <= (MAXPATHLEN - 1))
-			(void) strcpy(&Namech[j], "none");
+		    if ((j + strlen("none")) <= (Namechl - 1))
+			(void) snpf(&Namech[j], Namechl - j, "none");
 	        }
 	    }
 	    break;
@@ -682,78 +736,89 @@ get_lock_state:
 	 */
 	    if (v.v_type == VCHR) {
 		if (!v.v_data || readsnode((KA_T)v.v_data, &s)) {
-		    (void) sprintf(Namech,
+		    (void) snpf(Namech, Namechl,
 			"vnode@%s: can't read snode (%s)",
-			print_kptr(na, tbuf),
-			print_kptr((KA_T)v.v_data, (char *)NULL));
+			print_kptr(na, tbuf, sizeof(tbuf)),
+			print_kptr((KA_T)v.v_data, (char *)NULL, 0));
 		    enter_nm(Namech);
 		    return;
 		}
 		if (s.s_realvp) {
 		    if (readvnode((KA_T)s.s_realvp, &rv)) {
-			(void) sprintf(Namech,
+			(void) snpf(Namech, Namechl,
 			    "snode@%s: can't read real vnode (%s)",
-			    print_kptr((KA_T)v.v_data, tbuf),
-			    print_kptr((KA_T)s.s_realvp, (char *)NULL));
+			    print_kptr((KA_T)v.v_data, tbuf, sizeof(tbuf)),
+			    print_kptr((KA_T)s.s_realvp, (char *)NULL, 0));
 			enter_nm(Namech);
 			return;
 		    }
-		    if (!rv.v_data || readlino(fx, &rv, &i)) {
-			(void) sprintf(Namech,
+		    if (!rv.v_data || (is = readlino(fx, &rv, &i))) {
+			(void) snpf(Namech, Namechl,
 			    "snode@%s: unknown inode@%s; fx=",
-			     print_kptr((KA_T)v.v_data, tbuf),
-			     print_kptr((KA_T)rv.v_data, (char *)NULL));
-			    if (fx < 1 || fx > Fsinfomax)
-				(void) sprintf(endnm(), "%d", fx);
-			    else
-				(void) strcpy(endnm(), Fsinfo[fx - 1]);
+			    print_kptr((KA_T)v.v_data, tbuf, sizeof(tbuf)),
+			    print_kptr((KA_T)rv.v_data, (char *)NULL, 0));
+			ep = endnm(&sz);
+			if (fx < 1 || fx > Fsinfomax)
+			    (void) snpf(ep, sz, "%d", fx);
+			else
+			    (void) snpf(ep, sz, "%s", Fsinfo[fx - 1]);
 			enter_nm(Namech);
 			return;
-		     }
+		    }
 		}
 	    /*
 	     * If there's no real vnode, look for a common vnode and a
 	     * common snode.
 	     */
-		else if (s.s_commonvp) {
-		    if (readvnode((KA_T)s.s_commonvp, &rv)) {
-			(void) sprintf(Namech,
-			    "snode@%s: can't read real vnode (%s)",
-			    print_kptr((KA_T)v.v_data, tbuf),
-			    print_kptr((KA_T)s.s_commonvp, (char *)NULL));
+		else if ((ka = (KA_T)s.s_commonvp)) {
+		    if (readvnode(ka, &rv)) {
+			(void) snpf(Namech, Namechl,
+			    "snode@%s: can't read common vnode (%s)",
+			    print_kptr((KA_T)v.v_data, tbuf, sizeof(tbuf)),
+			    print_kptr((KA_T)ka, (char *)NULL, 0));
 			enter_nm(Namech);
 			return;
 		    }
-		    if (!rv.v_data || readsnode((KA_T)rv.v_data, &rs)) {
-			(void) sprintf(Namech, "vnode@%s: can't read snode(%s)",
-			    print_kptr((KA_T)s.s_commonvp, tbuf),
-			    print_kptr((KA_T)rv.v_data, (char *)NULL));
-			enter_nm(Namech);
-			return;
+		    if (get_vty(&rv, ka, &rkv, &rfx) < 0)
+		        Lf->is_com = ni = 1;
+		    else {
+			if ((is = readlino(rfx, &rv, &i))) {
+			    (void) snpf(Namech, Namechl,
+				"vnode@%s: unknown successor@%s; fx=",
+				print_kptr((KA_T)ka, tbuf, sizeof(tbuf)),
+				print_kptr((KA_T)v.v_data, (char *)NULL, 0));
+			    ep = endnm(&sz);
+			    if (rfx < 1 || rfx > Fsinfomax)
+				(void) snpf(ep, sz, "%d", fx);
+			    else
+				(void) snpf(ep, sz, "%s", Fsinfo[fx - 1]);
+			    enter_nm(Namech);
+			    return;
+			}
 		    }
-		    Lf->is_com = ni = 1;
 		} else
 		    ni = 1;
 		break;
 	    }
 	    if (v.v_data == NULL) {
-		(void) sprintf(Namech, "vnode@%s: no further information",
-		    print_kptr(na, (char *)NULL));
+		(void) snpf(Namech, Namechl, "vnode@%s: no further information",
+		    print_kptr(na, (char *)NULL, 0));
 		enter_nm(Namech);
 		return;
 	    }
 	/*
 	 * Read inode information.
 	 */
-	    if (readlino(fx, &v, &i)) {
-		(void) sprintf(Namech,
+	    if ((is = readlino(fx, &v, &i))) {
+		(void) snpf(Namech, Namechl,
 		    "vnode@%s: unknown successor@%s; fx=",
-		    print_kptr(na, tbuf),
-		    print_kptr((KA_T)v.v_data, (char *)NULL));
-		    if (fx < 1 || fx > Fsinfomax)
-			(void) sprintf(endnm(), "%d", fx);
-		    else
-			(void) strcpy(endnm(), Fsinfo[fx - 1]);
+		    print_kptr(na, tbuf, sizeof(tbuf)),
+		    print_kptr((KA_T)v.v_data, (char *)NULL, 0));
+		ep = endnm(&sz);
+		if (fx < 1 || fx > Fsinfomax)
+		    (void) snpf(ep, sz, "%d", fx);
+		else
+		    (void) snpf(ep, sz, "%s", Fsinfo[fx - 1]);
 		enter_nm(Namech);
 		return;
 	    }
@@ -764,42 +829,55 @@ get_lock_state:
 	switch (Ntype) {
 	case N_NFS:
 	    dev = r.r_attr.va_fsid;
-	    ds = 1;
+	    devs = 1;
 	    break;
 
 #if	defined(HASPROCFS)
 	case N_PROC:
 	    dev = kv.vfs_dev;
-	    ds = 1;
+	    devs = 1;
 	    break;
 #endif	/* defined(HASPROCFS) */
 
 	case N_STREAM:
 	    if (sqp) {
 		if (so.lux_dev.size >= 8) {
-		    dev = so.lux_dev.addr.tu_addr.dev;
-		    ds = 1;
+		    dev = DevDev;
+		    rdev = so.lux_dev.addr.tu_addr.dev;
+		    devs = rdevs = 1;
 		} else
-		    enter_dev_ch(print_kptr(sqp, (char *)NULL));
+		    enter_dev_ch(print_kptr(sqp, (char *)NULL, 0));
 		break;
 	    }
 	    if (v.v_type == VFIFO) {
 		KA_T ta;
 
 		if ((ta = (KA_T)(v.v_data ? v.v_data : v.v_stream)))
-		    enter_dev_ch(print_kptr(ta, (char *)NULL));
+		    enter_dev_ch(print_kptr(ta, (char *)NULL, 0));
 		break;
 	    }
 	    /* fall through */
 	default:
-	    dev = (v.v_type == VCHR || ni) ? v.v_rdev : i.dev;
-	    ds = 1;
+	    if (!ni) {
+		dev = i.dev;
+		devs = (Ntype == N_CFS) ? i.dev_def : 1;
+	    } else if ((Ntype == N_STREAM)
+		   &&  ((v.v_type == VCHR) || (v.v_type == VBLK)))
+	    {
+		dev = DevDev;
+		devs = 1;
+	    }
+	    if ((v.v_type == VCHR) || (v.v_type == VBLK)) {
+		rdev = v.v_rdev;
+		rdevs = (Ntype == N_CFS) ? i.rdev_def : 1;
+	    }
 	}
 	type = v.v_type;
 /*
  * Obtain the inode number.
  */
 	switch (Ntype) {
+
 	case N_NFS:
 	    Lf->inode = (unsigned long)r.r_attr.va_nodeid;
 	    Lf->inp_ty = 1;
@@ -814,21 +892,24 @@ get_lock_state:
 
 	case N_FIFO:
 	    if (!f.fn_realvp) {
-		enter_dev_ch(print_kptr((KA_T)v.v_data, (char *)NULL));
+		enter_dev_ch(print_kptr((KA_T)v.v_data, (char *)NULL, 0));
 		Lf->inode = (unsigned long)f.fn_ino;
 		Lf->inp_ty = 1;
 		if (f.fn_flag & ISPIPE)
-		    (void) strcpy(Namech, "PIPE");
-		if (f.fn_mate)
-		    (void) sprintf(endnm(), "->%s",
-			print_kptr((KA_T)f.fn_mate, (char *)NULL));
+		    (void) snpf(Namech, Namechl, "PIPE");
+		if (f.fn_mate) {
+		    ep = endnm(&sz);
+		    (void) snpf(ep, sz, "->%s",
+			print_kptr((KA_T)f.fn_mate, (char *)NULL, 0));
+		}
 		break;
 	    }
 	    /* fall through */
+	case N_CFS:
 	case N_REGLR:
 	    if (!ni) {
 		Lf->inode = (unsigned long)i.number;
-		Lf->inp_ty = 1;
+		Lf->inp_ty = (Ntype == N_CFS) ? i.number_def : 1;
 	    }
 	    break;
 	case N_STREAM:
@@ -844,6 +925,7 @@ get_lock_state:
 	    Lf->off_def = 1;
 	else {
 	    switch (Ntype) {
+
 	    case N_FIFO:
 	    case N_STREAM:
 		if (!Fsize)
@@ -861,11 +943,12 @@ get_lock_state:
 		break;
 #endif	/* defined(HASPROCFS) */
 
+	    case N_CFS:
 	    case N_REGLR:
 		if (type == VREG || type == VDIR) {
 		    if (!ni) {
 			Lf->sz = (SZOFFTYPE)i.size;
-			Lf->sz_def = sd;
+			Lf->sz_def = (Ntype == N_CFS) ? i.size_def : sd;
 		    }
 		} else if (type == VCHR && !Fsize)
 		    Lf->off_def = 1;
@@ -890,6 +973,7 @@ get_lock_state:
 	    case N_PROC:
 #endif	/* defined(HASPROCFS) */
 
+	    case N_CFS:
 	    case N_REGLR:
 		if (!ni) {
 		    Lf->nlink = (long)i.nlink;
@@ -910,27 +994,31 @@ get_lock_state:
  */
 	Lf->lmi_srch = 1;
 /*
+ * Save the device numbers and their states.
+ *
  * Format the vnode type, and possibly the device name.
  */
+	if (type != VFIFO) {
+	    Lf->dev = dev;
+	    Lf->dev_def = devs;
+	    Lf->rdev = rdev;
+	    Lf->rdev_def = rdevs;
+	}
 	switch (type) {
 	case VNON:
 	    ty ="VNON";
 	    break;
 	case VREG:
+	    ty = "VREG";
+	    break;
 	case VDIR:
-	    ty = (type == VREG) ? "VREG" : "VDIR";
-	    Lf->dev_def = ds;
-	    Lf->dev = dev;
+	    ty = "VDIR";
 	    break;
 	case VBLK:
 	    ty = "VBLK";
-	    Lf->dev = dev;
-	    Lf->dev_def = ds;
 	    Ntype = N_BLK;
 	    break;
 	case VCHR:
-	    Lf->dev = dev;
-	    Lf->dev_def = ds;
 	    ty = "VCHR";
 	    if (Lf->is_stream == 0)
 		Ntype = N_CHR;
@@ -951,7 +1039,9 @@ get_lock_state:
 	case VFIFO:
 	    if (!Lf->dev_ch || Lf->dev_ch[0] == '\0') {
 		Lf->dev = dev;
-		Lf->dev_def = ds;
+		Lf->dev_def = devs;
+		Lf->rdev = rdev;
+		Lf->rdev_def = rdevs;
 	    }
 	    ty = "FIFO";
 	    break;
@@ -960,26 +1050,26 @@ get_lock_state:
 	    break;
 	default:
 	    if (type > 9999)
-		(void) sprintf(Lf->type, "*%03d", type % 1000);
+		(void) snpf(Lf->type, sizeof(Lf->type), "*%03d", type % 1000);
 	    else
-		(void) sprintf(Lf->type, "%4d", type);
-	    (void) strcpy(Namech, "unknown type");
+		(void) snpf(Lf->type, sizeof(Lf->type), "%4d", type);
+	    (void) snpf(Namech, Namechl, "unknown type");
 	    ty = NULL;
 	}
 	if (ty)
-	    (void) strcpy(Lf->type, ty);
+	    (void) snpf(Lf->type, sizeof(Lf->type), "%s", ty);
 	Lf->ntype = Ntype;
 /*
  * If this is a VBLK file and it's missing an inode number, try to
  * supply one.
  */
-	if (Lf->inp_ty == 0 && Lf->ntype == N_BLK && Lf->dev_def)
+	if ((Lf->inp_ty == 0) && (Lf->ntype == N_BLK))
 	    find_bl_ino();
 /*
  * If this is a VCHR file and it's missing an inode number, try to
  * supply one.
  */
-	if (Lf->inp_ty == 0 && type == VCHR && Lf->dev_def)
+	if ((Lf->inp_ty == 0) && (type == VCHR))
 	    find_ch_ino();
 /*
  * If this is a stream with a "sockmod" module whose q_ptr leads to an
@@ -989,12 +1079,12 @@ get_lock_state:
 	if (Ntype == N_STREAM && sqp) {
 	    if (Funix)
 		Lf->sf |= SELUNX;
-	    (void) strcpy(Lf->type, "unix");
+	    (void) snpf(Lf->type, sizeof(Lf->type), "unix");
 	    if (!Namech[0]
 	    &&  so.laddr.buf && so.laddr.len == sizeof(ua)
 	    &&  !kread((KA_T)so.laddr.buf, (char *)&ua, sizeof(ua))) {
 		ua.sun_path[sizeof(ua.sun_path) - 1] = '\0';
-		(void) strcpy(Namech, ua.sun_path);
+		(void) snpf(Namech, Namechl, "%s", ua.sun_path);
 		if (Sfile && is_file_named(Namech, 0))
 		    Lf->sf = SELNM;
 		if (so.lux_dev.size >= 8) {
@@ -1002,9 +1092,11 @@ get_lock_state:
 		    Lf->inp_ty = 1;
 		}
 	    }
-	    if (so.so_conn)
-		(void) sprintf(endnm(), "->%s",
-		    print_kptr((KA_T)so.so_conn, (char *)NULL));
+	    if (so.so_conn) {
+		ep = endnm(&sz);
+		(void) snpf(ep, sz, "->%s",
+		    print_kptr((KA_T)so.so_conn, (char *)NULL, 0));
+	    }
 	    Lf->is_stream = 0;
 	}
 /*
@@ -1029,8 +1121,8 @@ get_lock_state:
 			pfi->f = 1;
 			Lf->sf |= SELNM;
 			if (!Namech[0] && pfi->nm) {
-			    (void) strncpy(Namech, pfi->nm, MAXPATHLEN - 1);
-			    Namech[MAXPATHLEN-1] = '\0';
+			    (void) strncpy(Namech, pfi->nm, Namechl - 1);
+			    Namech[Namechl-1] = '\0';
 			}
 			break;
 		    }
@@ -1040,12 +1132,28 @@ get_lock_state:
 #endif	/* defined(HASPROCFS) */
 
 	{
-	    if (Sfile && is_file_named(NULL, (type == VCHR) ? 1 : 0))
+	    if (Sfile && is_file_named((char *)NULL,
+				       ((type == VCHR) || (type == VBLK)) ? 1
+									  : 0))
 		Lf->sf |= SELNM;
 	}
 /*
- * Enter name characters.
+ * Enter name characters.  If there's an l_ino structure with a file name
+ * pointer, and no name column addition exists, make what the l_ino file
+ * name pointer addresses a name column addition.
  */
+	if (!Lf->nma && !is && i.nm) {
+	    if ((msz = (MALLOC_S)strlen(i.nm))) {
+		if (!(cp = (char *)malloc(msz + 1))) {
+		    (void) fprintf(stderr,
+			"%s: can't allocate %d bytes for l_ino name addition\n",
+			msz, Pn);
+		    Exit(1);
+		}
+		(void) snpf(cp, msz + 1, "%s", i.nm);
+		Lf->nma = cp;
+	    }
+	}
 	if (Namech[0])
 	    enter_nm(Namech);
 }
@@ -1061,6 +1169,11 @@ readlino(fx, v, i)
 	struct vnode *v;		/* vnode pointing to inode */
 	struct l_ino *i;		/* local inode */
 {
+
+#if	defined(HAS_UW_CFS)
+	cnode_t cn;
+#endif	/* defined(HAS_UW_CFS) */
+
 	struct vnode fa;
 	struct mnode mn;
 	struct inode sn;
@@ -1081,10 +1194,44 @@ readlino(fx, v, i)
 	    i->rdev = v->v_rdev;
 	    i->nlink = (long)sn.i_nlink;
 	    i->nlink_def = 1;
+	    i->nm = (char *)NULL;
 	    i->number = sn.i_number;
 	    i->size = sn.i_size;
 	    return(0);
-	} else if (!strcmp(Fsinfo[fx-1], "s5"))
+	}
+
+#if	defined(HAS_UW_CFS)
+	else if (!strcmp(Fsinfo[fx-1], "nsc_cfs")) {
+	    if (kread((KA_T)v->v_data, (char *)&cn, sizeof(cn)))
+		return(1);
+	    if (cn.c_attr.va_mask & AT_FSID) {
+		i->dev = cn.c_attr.va_fsid;
+		i->dev_def = 1;
+	    } else
+		i->dev_def = 0;
+	    i->rdev = v->v_rdev;
+	    i->rdev_def = 1;
+	    if (cn.c_attr.va_mask & AT_NLINK) {
+		i->nlink = cn.c_attr.va_nlink;
+		i->nlink_def = 1;
+	    } else
+		i->nlink_def = 0;
+	    i->nm = (char *)NULL;
+	    if (cn.c_attr.va_mask & AT_NODEID) {
+		i->number = cn.c_attr.va_nodeid;
+		i->number_def = 1;
+	    } else
+		i->number_def = 0;
+	    if (cn.c_attr.va_mask & AT_SIZE) {
+		i->size = cn.c_attr.va_size;
+		i->size_def = 1;
+	    } else
+		i->size_def = 0;
+	    return(0);
+	}
+#endif	/* defined(HAS_UW_CFS) */
+
+	else if (!strcmp(Fsinfo[fx-1], "s5"))
 	    return(reads5lino(v, i));
  	else if (!strcmp(Fsinfo[fx-1], "vxfs"))
 	    return(readvxfslino(v, i));
@@ -1100,6 +1247,7 @@ readlino(fx, v, i)
 	    i->dev = xn.x_dev;
 	    i->nlink = (long)xn.x_count;
 	    i->nlink_def = 1;
+	    i->nm = (char *)NULL;
 	    i->rdev = xn.x_fsid;
 	    i->size = xn.x_size;
 	    return(0);
@@ -1112,10 +1260,19 @@ readlino(fx, v, i)
 	    i->dev = mn.mno_fsid;
 	    i->nlink = (long)mn.mno_nlink;
 	    i->nlink_def = 1;
+	    i->nm = (char *)NULL;
 	    i->number = mn.mno_nodeid;
 	    i->rdev = mn.mno_rdev;
 	    i->size = mn.mno_size;
 	    return(0);
 	}
+
+#if	UNIXWAREV>=70000
+	else if (!strcmp(Fsinfo[fx-1], "cdfs"))
+	    return readcdfslino(v, i);
+	else if (!strcmp(Fsinfo[fx-1], "dosfs"))
+	    return readdosfslino(v, i);
+#endif	/* UNIXWAREV>=70000 */
+
 	return(1);
 }
