@@ -37,7 +37,7 @@
 # if	!defined(lint)
 static char copyright[] =
 "@(#) Copyright 1997 Purdue Research Foundation.\nAll rights reserved.\n";
-static char *rcsid = "$Id: dvch.c,v 1.9 99/10/23 14:39:54 abe Exp $";
+static char *rcsid = "$Id: dvch.c,v 1.12 2001/02/19 12:12:33 abe Exp $";
 # endif	/* !defined(lint) */
 
 #include "../lsof.h"
@@ -563,7 +563,7 @@ dcpath(rw, npw)
 	     */
 
 	    case 'U':
-		(void) sprintf(hn, "%d", (int)Myuid);
+		(void) snpf(hn, sizeof(hn), "%d", (int)Myuid);
 		if ((i + (l = strlen(hn))) >= (int)sizeof(buf))
 		    ierr = 2;
 		else {
@@ -695,7 +695,7 @@ close_exit:
 		return(1);
 	    }
 	    if ((int)(s->st_mode & 07777) != ((DCpathX == 2) ? 0644 : 0600)) {
-		(void) sprintf(buf, "doesn't have %04o modes",
+		(void) snpf(buf, sizeof(buf), "doesn't have %04o modes",
 	    	    (DCpathX == 2) ? 0644 : 0600);
 		w = buf;
 	    } else if ((s->st_mode & S_IFMT) != S_IFREG)
@@ -774,7 +774,7 @@ close_exit:
 int
 read_dcache()
 {
-	char buf[MAXPATHLEN*2], cbuf[32], *cp;
+	char buf[MAXPATHLEN*2], cbuf[64], *cp;
 	int i, len, ov;
 	struct stat sb, devsb;
 /*
@@ -860,8 +860,19 @@ read_close:
 	cp = "s";
 # endif	/* defined(DCACHE_PSEUDO) */
 
-	(void) sprintf(cbuf, "%d section%s\n", i, cp);
-	if (strcmp(buf, cbuf) != 0) {
+	(void) snpf(cbuf, sizeof(cbuf), "%d section%s", i, cp);
+	len = strlen(cbuf);
+	(void) snpf(&cbuf[len], sizeof(cbuf) - len, ", dev=%lx\n",
+		    (long)DevDev);
+	if (!strncmp(buf, cbuf, len) && (buf[len] == '\n')) {
+	    if (!Fwarn) {
+		(void) fprintf(stderr, "%s: WARNING: no /dev device in %s: ",
+		    Pn, DCpath[DCpathX]);
+		safestrprt(buf, stderr, 1);
+	    }
+	    goto read_close;
+	}
+	if (strcmp(buf, cbuf)) {
 	    if (!Fwarn) {
 		(void) fprintf(stderr,
 		    "%s: WARNING: bad section count line in %s: ",
@@ -1065,7 +1076,7 @@ read_dhdr:
 /*
  * Read and check the CRC section; it must be the last thing in the file.
  */
-	(void) sprintf(cbuf, "CRC section: %x\n", DCcksum);
+	(void) snpf(cbuf, sizeof(cbuf), "CRC section: %x\n", DCcksum);
 	if (!fgets(buf, sizeof(buf), DCfs) || strcmp(buf, cbuf) != 0) {
 	    if (!Fwarn) {
 		(void) fprintf(stderr,
@@ -1204,7 +1215,7 @@ bad_clone_index:
 	 */
 	    for (c = Clone, n = 0; c; c = c->next, n++)
 		;
-	    (void) sprintf(buf, "clone section: %d\n", n);
+	    (void) snpf(buf, sizeof(buf), "clone section: %d\n", n);
 	    if (wr2DCfd(buf, &DCcksum))
 		return(1);
 	/*
@@ -1226,7 +1237,7 @@ bad_clone_index:
 		    DCfd = -1;
 		    return(1);
 		}
-		(void) sprintf(buf, "%d %s\n", j, dp->name);
+		(void) snpf(buf, sizeof(buf), "%d %s\n", j, dp->name);
 		if (wr2DCfd(buf, &DCcksum))
 		    return(1);
 	    }
@@ -1278,7 +1289,8 @@ write_dcache()
 	cp = "s";
 # endif	/* defined(DCACHE_PSEUDO) */
 
-	(void) sprintf(buf, "%d section%s\n", i, cp);
+	(void) snpf(buf, sizeof(buf), "%d section%s, dev=%lx\n", i, cp,
+	    (long)DevDev);
 	(void) crcbld();
 	DCcksum = 0;
 	if (wr2DCfd(buf, &DCcksum))
@@ -1286,13 +1298,13 @@ write_dcache()
 /*
  * Write the device section from the contents of Sdev[] and Devtp[].
  */
-	(void) sprintf(buf, "device section: %d\n", Ndev);
+	(void) snpf(buf, sizeof(buf), "device section: %d\n", Ndev);
 	if (wr2DCfd(buf, &DCcksum))
 	    return;
 	for (i = 0; i < Ndev; i++) {
 	    dp = Sdev[i];
-	    (void) sprintf(buf, "%lx %ld %s\n", (long)dp->rdev, (long)dp->inode,
-		dp->name);
+	    (void) snpf(buf, sizeof(buf), "%lx %ld %s\n", (long)dp->rdev,
+			(long)dp->inode, dp->name);
 	    if (wr2DCfd(buf, &DCcksum))
 		return;
 	}
@@ -1301,13 +1313,13 @@ write_dcache()
 /*
  * Write the block device section from the contents of BSdev[] and BDevtp[].
  */
-	(void) sprintf(buf, "block device section: %d\n", BNdev);
+	(void) snpf(buf, sizeof(buf), "block device section: %d\n", BNdev);
 	if (wr2DCfd(buf, &DCcksum))
 	    return;
 	if (BNdev) {
 	    for (i = 0; i < BNdev; i++) {
 		dp = BSdev[i];
-		(void) sprintf(buf, "%x %ld %s\n", (int)dp->rdev,
+		(void) snpf(buf, sizeof(buf), "%x %ld %s\n", (int)dp->rdev,
 		    (long)dp->inode, dp->name);
 		if (wr2DCfd(buf, &DCcksum))
 		    return;
@@ -1334,7 +1346,7 @@ write_dcache()
 /*
  * Write the CRC section and close the file.
  */
-	(void) sprintf(buf, "CRC section: %x\n", DCcksum);
+	(void) snpf(buf, sizeof(buf), "CRC section: %x\n", DCcksum);
 	if (wr2DCfd(buf, (unsigned *)NULL))
 		return;
 	if (close(DCfd) != 0) {
