@@ -37,7 +37,7 @@
 # if	!defined(lint)
 static char copyright[] =
 "@(#) Copyright 1997 Purdue Research Foundation.\nAll rights reserved.\n";
-static char *rcsid = "$Id: pdvn.c,v 1.4 99/06/22 09:55:51 abe Exp $";
+static char *rcsid = "$Id: pdvn.c,v 1.5 2000/12/04 14:05:26 abe Exp $";
 # endif	/* !defined(lint) */
 
 #include "../lsof.h"
@@ -57,19 +57,27 @@ static char d1[] = "d"; static char *d2 = d1;
  */
 
 
+/*
+ * Local definitions
+ */
+
+#define	LIKE_NODE_TTL	"like device special "
+
+
 #if	defined(HASBLKDEV)
 /*
  * printbdevname() - print block device name
  */
 
 int
-printbdevname(rdev, f)
-	dev_t *rdev;			/* device */
+printbdevname(dev, rdev, f)
+	dev_t *dev;			/* device */
+	dev_t *rdev;			/* raw device */
 	int f;				/* 1 = follow with '\n' */
 {
 	struct l_dev *dp;
 
-	if ((dp = lkupbdev(rdev, 1, 1))) {
+	if ((dp = lkupbdev(dev, rdev, 1, 1))) {
 	    safestrprt(dp->name, stdout, f);
 	    return(1);
 	}
@@ -84,8 +92,9 @@ printbdevname(rdev, f)
  */
 
 int
-printchdevname(rdev, f)
-	dev_t *rdev;			/* device */
+printchdevname(dev, rdev, f)
+	dev_t *dev;			/* device */
+	dev_t *rdev;			/* raw device */
 	int f;				/* 1 = print trailing '\n' */
 {
 
@@ -106,7 +115,7 @@ printchdevname_again:
 /*
  * Search for clone.
  */
-	if (Lf->is_stream && Clone) {
+	if (Lf->is_stream && Clone && (*dev == DevDev)) {
 	    r = 0;	/* Don't let lkupdev() rebuild the device cache,
 			 * because when it has been rebuilt we want to
 			 * search again for clones. */
@@ -127,12 +136,34 @@ printchdevname_again:
 #endif	/* defined(HAS_STD_CLONE) */
 
 /*
- * Search device table for match.
+ * Search device table for a full match.
  */
-	if ((dp = lkupdev(rdev, 1, r))) {
+	if ((dp = lkupdev(dev, rdev, 1, r))) {
 	    safestrprt(dp->name, stdout, f);
 	    return(1);
 	}
+/*
+ * Search device table for a match without inode number and dev.
+ */
+	if ((dp = lkupdev(&DevDev, rdev, 0, r))) {
+
+	/*
+	 * A raw device match was found.  Record it as a name column addition.
+	 */
+	    char *cp;
+	    int len;
+
+	    len = (int)(1 + strlen(LIKE_NODE_TTL) + strlen(dp->name) + 1);
+	    if (!(cp = (char *)malloc((MALLOC_S)(len + 1)))) {
+		(void) fprintf(stderr, "%s: no nma space for: (%s%s)\n",
+		    Pn, LIKE_NODE_TTL, dp->name);
+		Exit(1);
+	    }
+	    (void) snpf(cp, len + 1, "(%s%s)", LIKE_NODE_TTL, dp->name);
+	    (void) add_nma(cp, len);
+	    (void) free((MALLOC_P *)cp);
+	    return(0);
+	}
 
 #if	defined(HASDCACHE)
 /*
