@@ -31,7 +31,7 @@
 
 
 /*
- * $Id: lsof.h,v 1.35 10/.0/.1 .0:.5:.2 abe Exp $
+ * $Id: lsof.h,v 1.41 2001/06/19 13:23:39 abe Exp $
  */
 
 
@@ -48,6 +48,11 @@
 
 #include <ctype.h>
 #include <errno.h>
+
+# if	defined(HASSETLOCALE)
+#include <locale.h>
+# endif	/* defined(HASSETLOCALE) */
+
 #include <netdb.h>
 #include <pwd.h>
 #include <stdio.h>
@@ -168,6 +173,7 @@ struct l_dev {
 
 #include <sys/types.h>			/* just in case -- because utmp.h
 					 * may need it */
+#include "./regex.h"
 
 # if	defined(EMPTY)
 #undef	EMPTY
@@ -196,7 +202,7 @@ extern int optind;
 #define	FSV_CT		0x2		/* file struct count status */
 #define	FSV_FG		0x4		/* file struct flags */
 #define	FSV_NI		0x8		/* file struct node ID status */
-#define	IDINCR		10		/* PID/PGRP table malloc() increment */
+#define	IDINCR		10		/* PID/PGID table malloc() increment */
 
 # if	!defined(INADDR_LOOPBACK)
 #define	INADDR_LOOPBACK	(u_long)0x7f000001
@@ -213,7 +219,7 @@ static struct utmp dummy_utmp;		/* to get login name length */
 #define	LOGINML		sizeof(dummy_utmp.ut_name)
 					/* login name length */
 #define	LPROCINCR	128		/* Lproc[] allocation increment */
-#define	LSOF_URL	"ftp://vic.cc.purdue.edu/pub/tools/unix/lsof"
+#define	LSOF_URL	"ftp://vic.cc.purdue.edu/pub/tools/unix/lsof/"
 #define	MIN_AF_ADDR	sizeof(struct in_addr)
 					/* minimum AF_* address length */
 
@@ -233,34 +239,36 @@ static struct utmp dummy_utmp;		/* to get login name length */
 
 #define	N_REGLR		0		/* regular file system node */
 #define	N_AFS		1		/* AFS node */
-#define	N_AUSX		2		/* Auspex LFS node */
-#define	N_AUTO		3		/* automount node */
-#define	N_BLK		4		/* block device node */
-#define	N_CACHE		5		/* cached file system node */
-#define	N_CDFS		6		/* CD-ROM node */
-#define	N_CHR		7		/* character device node */
-#define	N_COM		8		/* streams common device node */
-#define	N_DOOR		9		/* DOOR node */
-#define	N_FIFO		10		/* FIFO node */
-#define	N_HSFS		11		/* High Sierra node */
-#define	N_KERN		12		/* BSD /kern node */
-#define	N_LOFS		13		/* loopback node */
-#define	N_MNT		14		/* mount file system device node */
-#define	N_MPC		15		/* multiplexed device node */
-#define	N_MVFS		16		/* multi-volume file system node (?) */
-#define	N_NFS		17		/* NFS node */
-#define	N_NM		18		/* named file system node */
-#define	N_OBJF		19		/* objfs file system node */
-#define	N_PCFS		20		/* PC file system node */
-#define	N_PIPE		21		/* pipe device node */
-#define	N_PROC		22		/* /proc node */
-#define	N_SOCK		23		/* sock_vnodeops node */
-#define	N_SPEC		24		/* spec_vnodeops node */
-#define	N_STREAM	25		/* stream node */
-#define	N_TMP		26		/* tmpfs node */
-#define	N_UFS		27		/* Unix file system node */
-#define	N_VXFS		28		/* Veritas file system node */
-#define	N_XFS		29		/* XFS node */
+#define	N_AFPFS		2		/* Apple Darwin AppleShare */
+#define	N_AUSX		3		/* Auspex LFS node */
+#define	N_AUTO		4		/* automount node */
+#define	N_BLK		5		/* block device node */
+#define	N_CACHE		6		/* cached file system node */
+#define	N_CDFS		7		/* CD-ROM node */
+#define	N_CFS		8		/* CFS node */
+#define	N_CHR		9		/* character device node */
+#define	N_COM		10		/* streams common device node */
+#define	N_DOOR		11		/* DOOR node */
+#define	N_FIFO		12		/* FIFO node */
+#define	N_HSFS		13		/* High Sierra node */
+#define	N_KERN		15		/* BSD /kern node */
+#define	N_LOFS		15		/* loopback node */
+#define	N_MNT		16		/* mount file system device node */
+#define	N_MPC		17		/* multiplexed device node */
+#define	N_MVFS		18		/* multi-volume file system node (?) */
+#define	N_NFS		19		/* NFS node */
+#define	N_NM		20		/* named file system node */
+#define	N_OBJF		21		/* objfs file system node */
+#define	N_PCFS		22		/* PC file system node */
+#define	N_PIPE		23		/* pipe device node */
+#define	N_PROC		24		/* /proc node */
+#define	N_SOCK		25		/* sock_vnodeops node */
+#define	N_SPEC		26		/* spec_vnodeops node */
+#define	N_STREAM	27		/* stream node */
+#define	N_TMP		28		/* tmpfs node */
+#define	N_UFS		29		/* Unix file system node */
+#define	N_VXFS		30		/* Veritas file system node */
+#define	N_XFS		31		/* XFS node */
 
 # if	!defined(OFFDECDIG)
 #define	OFFDECDIG	8		/* maximum number of digits in the
@@ -324,8 +332,8 @@ extern int NmColW;
 #define NODETTL		"NODE"
 extern int NodeColW;
 #define OFFTTL		"OFFSET"
-#define	PGRPTTL		"PGRP"
-extern int PgrpColW;
+#define	PGIDTTL		"PGID"
+extern int PgidColW;
 #define	PIDTTL		"PID"
 extern int PidColW;
 #define	PPIDTTL		"PPID"
@@ -363,7 +371,7 @@ extern char *NiTtl;
 #define	SELNFS		0x010		/* select NFS files (-N) */
 #define	SELNLINK	0x020		/* select based on link count */
 #define	SELNM		0x040		/* select by name */
-#define	SELPGRP		0x080		/* select process group IDs (-g) */
+#define	SELPGID		0x080		/* select process group IDs (-g) */
 #define	SELPID		0x100		/* select PIDs (-p) */
 #define	SELUID		0x200		/* select UIDs (-u) */
 #define	SELUNX		0x400		/* select Unix socket (-U) */
@@ -410,6 +418,14 @@ struct int_lst {
 	int f;				/* find state */
 };
 
+typedef struct lsof_rx {		/* regular expression table entry */
+	char *exp;			/* original regular expression */
+	regex_t cx;			/* compiled expression */
+	int mc;				/* match count */ 
+} lsof_rx_t;
+extern lsof_rx_t *CmdRx;
+extern int NCmdRxU;
+
 # if	defined(HASFSTRUCT)
 struct pff_tab {			/* print file flags table structure */
 	long val;			/* flag value */
@@ -453,6 +469,7 @@ extern int DCunsafe;
 # endif	/* defined(HASDCACHE) */
 
 extern int DChelp;
+extern dev_t DevDev;
 extern struct l_dev *Devtp;
 extern char **Dstk;
 extern int Dstkn;
@@ -467,6 +484,7 @@ extern int Fhost;
 
 # if	defined(HASNCACHE)
 extern int Fncache;
+extern int NcacheReload;
 # endif	/* defined(HASNCACHE) */
 
 extern int Fnet;
@@ -476,7 +494,7 @@ extern int Foffset;
 extern int Fovhd;
 extern int Fport;
 extern int FportMap;
-extern int Fpgrp;
+extern int Fpgid;
 extern int Fppid;
 extern int Fsize;
 extern int Fsv;
@@ -512,7 +530,7 @@ extern struct fieldsel FieldSel[];
 
 extern int Hdr;
 
-enum IDType {PGRP, PID};
+enum IDType {PGID, PID};
 
 struct lfile {
 	char access;
@@ -536,6 +554,7 @@ struct lfile {
 					 * 1 = printname() search required */
 	unsigned char nlink_def;	/* link count definition status */
 	unsigned char off_def;		/* offset definition status */
+	unsigned char rdev_def;		/* rdev definition status */
 	unsigned char sz_def;		/* size definition status */
 
 # if	defined(HASFSTRUCT)
@@ -551,6 +570,7 @@ struct lfile {
 	SZOFFTYPE off;
 	SZOFFTYPE sz;
 	dev_t dev;
+	dev_t rdev;
 	unsigned long inode;
 	long nlink;			/* link count */
 	char *dev_ch;
@@ -576,35 +596,36 @@ struct lfile {
 	} li[2];			/* li[0]: local
 					 * li[1]: foreign */
 	struct ltstate {		/* local TCP/TPI state */
-		int type;		/* state type: -1 == none
-					 *		0 == TCP
-					 *		1 == TPI or socket (SS_*) */
-		union {
-			int i;		/* integer state */
-			unsigned int u;	/* unsigned integer state */
-		} state;
+	    int type;			/* state type:
+					 *   -1 == none
+					 *    0 == TCP
+					 *    1 == TPI or socket (SS_*) */
+	    union {
+		int i;			/* integer state */
+		unsigned int ui;	/* unsigned integer state */
+	    } state;
 
 # if	defined(HASTCPTPIQ)
-		unsigned long rq;	/* receive queue length */
-		unsigned long sq;	/* send queue length */
-		unsigned char rqs;	/* rq status: 0 = none */
-		unsigned char sqs;	/* sq status: 0 = none */
+	    unsigned long rq;		/* receive queue length */
+	    unsigned long sq;		/* send queue length */
+	    unsigned char rqs;		/* rq status: 0 = none */
+	    unsigned char sqs;		/* sq status: 0 = none */
 # endif	/* defined(HASTCPTPIQ) */
 
 # if	defined(HASTCPTPIW)
-		unsigned char rws;	/* rw status: 0 = none */
-		unsigned char wws;	/* ww status: 0 = none */
-		unsigned long rw;	/* read window size */
-		unsigned long ww;	/* write window size */
+	    unsigned char rws;		/* rw status: 0 = none */
+	    unsigned char wws;		/* ww status: 0 = none */
+	    unsigned long rw;		/* read window size */
+	    unsigned long ww;		/* write window size */
 # endif	/* defined(HASTCPTPIW) */
 
 	} lts;
 	char *nm;
 	char *nma;			/* NAME column addition */
 
-# if	defined(HASNCACHE)
+# if	defined(HASNCACHE) && HASNCACHE<2
 	KA_T na;			/* file structure's node address */
-# endif	/* defined(HASNCACHE) */
+# endif	/* defined(HASNCACHE) && HASNCACHE<2 */
 
 # if	defined(HASNCACHE) && defined(HASNCAPID)
 	unsigned long id;		/* capability ID */
@@ -634,7 +655,7 @@ struct lproc {
 				 	 *	  1 = wholly selected
 				 	 *	  2 = partially selected */
 	int pid;			/* process ID */
-	int pgrp;			/* process group ID */
+	int pgid;			/* process group ID */
 	int ppid;			/* parent process ID */
 	uid_t uid;			/* user ID */
 	struct lfile *file;		/* open files of process */
@@ -647,13 +668,14 @@ extern char *Memory;
 extern struct mounts *Mtprocfs;
 # endif
 
-extern int Mxpgrp;
+extern int Mxpgid;
 extern int Mxpid;
 extern int Mxuid;
 extern gid_t Mygid;
 extern int Mypid;
 extern uid_t Myuid;
 extern char *Namech;
+extern size_t Namechl;
 extern int Ndev;
 
 # if	defined(HASNLIST)
@@ -666,7 +688,7 @@ extern int Nll;
 extern long Nlink;
 extern int Nlproc;
 extern char *Nmlst;
-extern int Npgrp;
+extern int Npgid;
 extern int Npid;
 extern int Npuns;
 extern int Ntype;
@@ -722,7 +744,7 @@ extern int Setgid;
 extern int Selinet;
 extern int Setuidroot;
 extern struct sfile *Sfile;
-extern struct int_lst *Spgrp;
+extern struct int_lst *Spgid;
 extern struct int_lst *Spid;
 extern struct seluid *Suid;
 extern char *SzOffFmt_0t;
