@@ -34,7 +34,7 @@
 #ifndef lint
 static char copyright[] =
 "@(#) Copyright 1994 Purdue Research Foundation.\nAll rights reserved.\n";
-static char *rcsid = "$Id: main.c,v 1.28 99/11/28 06:33:52 abe Exp $";
+static char *rcsid = "$Id: main.c,v 1.35 2001/06/19 13:23:51 abe Exp $";
 #endif
 
 
@@ -76,16 +76,27 @@ main(argc, argv)
 	struct nwad *np, *npn;
 	char options[128];
 	int rc = 0;
+	struct stat sb;
 	struct sfile *sfp;
 	struct lproc **slp = (struct lproc **)NULL;
 	int sp = 0;
 	struct str_lst *str;
 	int version = 0;
-
+/*
+ * Save program name.
+ */
 	if ((Pn = strrchr(argv[0], '/')))
 	    Pn++;
 	else
 	    Pn = argv[0];
+
+#if	defined(HASSETLOCALE)
+/*
+ * Set locale to environment's definition.
+ */
+	(void) setlocale(LC_CTYPE, "");
+#endif	/* defined(HASSETLOCALE) */
+
 /*
  * Common initialization.
  */
@@ -98,10 +109,11 @@ main(argc, argv)
 	    (void) fprintf(stderr, "%s: no space for name buffer\n", Pn);
 	    Exit(1);
 	}
+	Namechl = (size_t)(MAXPATHLEN + 1);
 /*
  * Create option mask.
  */
-	(void) sprintf(options,
+	(void) snpf(options, sizeof(options),
 	    "?a%sbc:D:d:%sf:F:g:hi:%slL:%sMnNo:Op:Pr:%ssS:tT:u:UvVw%s",
 
 #if	defined(HAS_AFS) && defined(HASAOPT)
@@ -172,8 +184,13 @@ main(argc, argv)
 		Fblock = 1;
 		break;
 	    case 'c':
-		if (enter_str_lst("c", GOv, &Cmdl))
-		    err = 1;
+		if (GOv && (*GOv == '/')) {
+		    if (enter_cmd_rx(GOv))
+			err = 1;
+		} else {
+		    if (enter_str_lst("c", GOv, &Cmdl))
+			err = 1;
+		}
 		break;
 
 #if	defined(HASNCACHE)
@@ -348,10 +365,10 @@ main(argc, argv)
 		    if (*GOv == '-' || *GOv == '+') {
 			GOx1 = GObk[0];
 			GOx2 = GObk[1];
-		    } else if (enter_id(PGRP, GOv))
+		    } else if (enter_id(PGID, GOv))
 			err = 1;
 		}
-		Fpgrp = 1;
+		Fpgid = 1;
 		break;
 	    case 'h':
 	    case '?':
@@ -632,7 +649,7 @@ main(argc, argv)
 /*
  * Compute the selection flags.
  */
-	if (Cmdl)
+	if (Cmdl || CmdRx)
 	    Selflags |= SELCMD;
 	if (Fdl)
 	    Selflags |= SELFD;
@@ -642,8 +659,8 @@ main(argc, argv)
 	    Selflags |= SELNFS;
 	if (Funix)
 	    Selflags |= SELUNX;
-	if (Npgrp)
-	    Selflags |= SELPGRP;
+	if (Npgid)
+	    Selflags |= SELPGID;
 	if (Npid)
 	    Selflags |= SELPID;
 	if (Nuid && Nuidincl)
@@ -665,6 +682,15 @@ main(argc, argv)
 		Selinet = 1;
 	    Selall = 0;
 	}
+/*
+ * Get the device for /dev.
+ */
+	if (stat("/dev", &sb)) {
+	    (void) fprintf(stderr, "%s: can't stat(/dev): %s\n", Pn,
+		strerror(errno));
+	    Exit(1);
+	}
+	DevDev = sb.st_dev;
 /*
  * Process the file arguments.
  */
@@ -687,6 +713,7 @@ main(argc, argv)
 	(void) dropgid();
 #endif	/* defined(WILLDROPGID) */
 
+
 #if	defined(HASDCACHE)
 /*
  * If there is a device cache, prepare the device table.
@@ -698,13 +725,13 @@ main(argc, argv)
 /*
  * Define the size and offset print formats.
  */
-	(void) sprintf(options, "0t%%%su", SZOFFPSPEC);
+	(void) snpf(options, sizeof(options), "0t%%%su", SZOFFPSPEC);
 	SzOffFmt_0t = sv_fmt_str(options);
-	(void) sprintf(options, "%%%su", SZOFFPSPEC);
+	(void) snpf(options, sizeof(options), "%%%su", SZOFFPSPEC);
 	SzOffFmt_d = sv_fmt_str(options);
-	(void) sprintf(options, "%%*%su", SZOFFPSPEC);
+	(void) snpf(options, sizeof(options), "%%*%su", SZOFFPSPEC);
 	SzOffFmt_dv = sv_fmt_str(options);
-	(void) sprintf(options, "%%#%sx", SZOFFPSPEC);
+	(void) snpf(options, sizeof(options), "%%#%sx", SZOFFPSPEC);
 	SzOffFmt_x = sv_fmt_str(options);
 /*
  * Gather and report process information every RptTm seconds.
@@ -742,6 +769,13 @@ main(argc, argv)
 	    }
 	    if ((n = Nlproc)) {
 
+#if	defined(HASNCACHE)
+	    /*
+	     * If using the kernel name cache, force its reloading.
+	     */
+		NcacheReload = 1;
+#endif	/* defined(HASNCACHE) */
+
 	    /*
 	     * Print the selected processes and count them.
 	     *
@@ -805,6 +839,19 @@ main(argc, argv)
 		safestrprt(str->str, stdout, 1);
 	    }
 	}
+	for (i = 0; i < NCmdRxU; i++) {
+	
+	/*
+	 * Check command regular expressions.
+	 */
+	    if (CmdRx[i].mc)
+		continue;
+	    rv = 1;
+	    if (Fverbose) {
+		(void) printf("%s: no command found for regex: ", Pn);
+		safestrprt(CmdRx[i].exp, stdout, 1);
+	    }
+	}
 	for (sfp = Sfile; sfp; sfp = sfp->next) {
 
 	/*
@@ -918,17 +965,17 @@ main(argc, argv)
 		(void) printf("%s: process ID not located: %d\n",
 		    Pn, Spid[i].i);
 	}
-	for (i = 0; i < Npgrp; i++) {
+	for (i = 0; i < Npgid; i++) {
 
 	/*
 	 * Check process group ID specifications.
 	 */
-	    if (Spgrp[i].f)
+	    if (Spgid[i].f)
 		continue;
 	    rv = 1;
 	    if (Fverbose)
 		(void) printf("%s: process group ID not located: %d\n",
-		    Pn, Spgrp[i].i);
+		    Pn, Spgid[i].i);
 	}
 	for (i = 0; i < Nuid; i++) {
 
@@ -953,6 +1000,7 @@ main(argc, argv)
 	if (!rv && ErrStat)
 	    rv = 1;
 	Exit(rv);
+	return(rv);		/* to make code analyzers happy */
 }
 
 
@@ -1101,6 +1149,6 @@ sv_fmt_str(f)
 		"%s: can't allocate %d bytes for format: %s\n", Pn, (int)l, f);
 	    Exit(1);
 	}
-	(void) strcpy(cp, f);
+	(void) snpf(cp, l, "%s", f);
 	return(cp);
 }
