@@ -32,13 +32,22 @@
 #ifndef lint
 static char copyright[] =
 "@(#) Copyright 1994 Purdue Research Foundation.\nAll rights reserved.\n";
-static char *rcsid = "$Id: misc.c,v 1.17 99/07/06 13:05:13 abe Exp $";
+static char *rcsid = "$Id: misc.c,v 1.19 2001/01/02 12:45:48 abe Exp $";
 #endif
 
 
 #include "lsof.h"
 
 
+/*
+ * Local definitions
+ */
+
+#if	!defined(MAXSYMLINKS)
+#define	MAXSYMLINKS	32
+#endif	/* !defined(MAXSYMLINKS) */
+
+
 /*
  * Local function prototypes
  */
@@ -658,7 +667,7 @@ mkstrcpy(src, rlp)
 	ns = (char *)malloc(len + 1);
 	if (ns) {
 	    if (src)
-		(void) strcpy(ns, src);
+		(void) snpf(ns, len + 1, "%s", src);
 	    else
 		*ns = '\0';
 	}
@@ -779,6 +788,7 @@ Readlink(arg)
 	char *argp1, *argp2;
 	int i, len, llen, slen;
 	char lbuf[MAXPATHLEN];
+	static char *op = (char *)NULL;
 	static int ss = 0;
 	char *s1;
 	static char **stk = (char **)NULL;
@@ -793,8 +803,14 @@ Readlink(arg)
 		safestrprt(arg, stderr, 0);
 		(void) fprintf(stderr, "): -b was specified.\n");
 	    }
+	    op = (char *)NULL;
 	    return(arg);
 	}
+/*
+ * Save the original path.
+ */
+	if (!op)
+	    op = arg;
 /*
  * Evaluate each component of the argument for a symbolic link.
  */
@@ -807,8 +823,9 @@ path_too_long:
 		if (!Fwarn) {
 		    (void) fprintf(stderr,
 			"%s: readlink() path too long: ", Pn);
-		    safestrprt(arg, stderr, 1);
+		    safestrprt(op ? op : arg, stderr, 1);
 		}
+		op = (char *)NULL;
 		return((char *)NULL);
 	    }
 	    (void) strncpy(tbuf, arg, len);
@@ -890,6 +907,7 @@ path_too_long:
 		stk[i] = (char *)NULL;
 	    }
 	    sx = 0;
+	    op = (char *)NULL;
 	    return(arg);
 	}
 /*
@@ -904,6 +922,28 @@ no_readlink_space:
 	    safestrprt(abuf, stderr, 1);
 	    Exit(1);
 	}
+	if (sx >= MAXSYMLINKS) {
+
+	/*
+	 * If there are too many symbolic links, report an error, clear
+	 * the stack, and return no path.
+	 */
+	    if (!Fwarn) {
+		(void) fprintf(stderr,
+		    "%s: too many (> %d) symbolic links in readlink() path: ",
+			Pn, MAXSYMLINKS);
+		safestrprt(op ? op : arg, stderr, 1);
+	    }
+	    for (i = 0; i < sx; i++) {
+		(void) free((FREE_P *)stk[i]);
+		stk[i] = (char *)NULL;
+	    }
+	    (void) free((FREE_P *)stk);
+	    stk = (char **)NULL;
+	    ss = sx = 0;
+	    op = (char *)NULL;
+	    return((char *)NULL);
+	}
 	if (++sx > ss) {
 	    if (!stk)
 		stk = (char **)malloc(sizeof(char *) * sx);
@@ -930,8 +970,8 @@ readstdata(addr, buf)
 {
 	if (!addr
 	||  kread(addr, (char *)buf, sizeof(struct stdata))) {
-	    (void) sprintf(Namech, "no stream data in %s",
-		print_kptr(addr, (char *)NULL));
+	    (void) snpf(Namech, Namechl, "no stream data in %s",
+		print_kptr(addr, (char *)NULL, 0));
 	    return(1);
 	}
 	return(0);
@@ -950,13 +990,13 @@ readsthead(addr, buf)
 	KA_T qp;
 
 	if (!addr) {
-	    (void) strcpy(Namech, "no stream queue head");
+	    (void) snpf(Namech, Namechl, "no stream queue head");
 	    return(1);
 	}
 	for (qp = addr; qp; qp = (KA_T)buf->q_next) {
 	    if (kread(qp, (char *)buf, sizeof(struct queue))) {
-		(void) sprintf(Namech, "bad stream queue link at %s",
-		    print_kptr(qp, (char *)NULL));
+		(void) snpf(Namech, Namechl, "bad stream queue link at %s",
+		    print_kptr(qp, (char *)NULL, 0));
 		return(1);
 	    }
 	}
@@ -975,8 +1015,8 @@ readstidnm(addr, buf, len)
 	READLEN_T len;			/* buffer length */
 {
 	if (!addr || kread(addr, buf, len)) {
-	    (void) sprintf(Namech, "can't read module ID name from %s",
-		print_kptr(addr, (char *)NULL));
+	    (void) snpf(Namech, Namechl, "can't read module ID name from %s",
+		print_kptr(addr, (char *)NULL, 0));
 	    return(1);
 	}
 	return(0);
@@ -993,8 +1033,8 @@ readstmin(addr, buf)
 	struct module_info *buf;	/* receiving buffer address */
 {
 	if (!addr || kread(addr, (char *)buf, sizeof(struct module_info))) {
-	    (void) sprintf(Namech, "can't read module info from %s",
-		print_kptr(addr, (char *)NULL));
+	    (void) snpf(Namech, Namechl, "can't read module info from %s",
+		print_kptr(addr, (char *)NULL, 0));
 	    return(1);
 	}
 	return(0);
@@ -1011,8 +1051,8 @@ readstqinit(addr, buf)
 	struct qinit *buf;		/* receiving buffer address */
 {
 	if (!addr || kread(addr, (char *)buf, sizeof(struct qinit))) {
-	    (void) sprintf(Namech, "can't read queue info from %s",
-		print_kptr(addr, (char *)NULL));
+	    (void) snpf(Namech, Namechl, "can't read queue info from %s",
+		print_kptr(addr, (char *)NULL, 0));
 	    return(1);
 	}
 	return(0);
@@ -1057,7 +1097,7 @@ safepup(c, cl)
 		rp = "\\t";
 		break;
 	    default:
-		(void) sprintf(up, "^%c", c + 0x40);
+		(void) snpf(up, sizeof(up), "^%c", c + 0x40);
 		rp = up;
 	    }
 	    len = 2;
@@ -1065,7 +1105,7 @@ safepup(c, cl)
 	    rp = "^?";
 	    len = 2;
 	} else {
-	    (void) sprintf(up, "\\x%02x", (int)(c & 0xff));
+	    (void) snpf(up, sizeof(up), "\\x%02x", (int)(c & 0xff));
 	    rp = up;
 	    len = 4;
 	}
