@@ -32,13 +32,50 @@
 #ifndef lint
 static char copyright[] =
 "@(#) Copyright 1994 Purdue Research Foundation.\nAll rights reserved.\n";
-static char *rcsid = "$Id: proc.c,v 1.24 99/08/03 08:20:45 abe Exp $";
+static char *rcsid = "$Id: proc.c,v 1.28 2001/02/13 02:09:44 abe Exp $";
 #endif
 
 
 #include "lsof.h"
 
 
+/*
+ * add_nma() - add to NAME column addition
+ */
+
+void
+add_nma(cp, len)
+	char *cp;			/* string to add */
+	int len;			/* string length */
+{
+	int nl;
+
+	if (!cp || !len)
+	    return;
+	if (Lf->nma) {
+	    nl = (int)strlen(Lf->nma);
+	    Lf->nma = (char *)realloc((MALLOC_P *)Lf->nma,
+				      (MALLOC_S)(len + nl + 2));
+	} else {
+	    nl = 0;
+	    Lf->nma = (char *)malloc((MALLOC_S)(len + 1));
+	}
+	if (!Lf->nma) {
+	    (void) fprintf(stderr, "%s: no name addition space: PID %ld, FD %s",
+		Pn, (long)Lp->pid, Lf->fd);
+	    Exit(1);
+	}
+	if (nl) {
+	    Lf->nma[nl] = ' ';
+	    (void) strncpy(&Lf->nma[nl + 1], cp, len);
+	    Lf->nma[nl + 1 + len] = '\0';
+	} else {
+	    (void) strncpy(Lf->nma, cp, len);
+	    Lf->nma[len] = '\0';
+	}
+}
+
+
 #if	defined(HASFSTRUCT)
 _PROTOTYPE(static char *alloc_fflbuf,(char **bp, int *al, int lr));
 
@@ -110,6 +147,7 @@ alloc_lfile(nm, num)
 	Lf->access = Lf->lock = ' ';
 	Lf->dev_def = Lf->inp_ty = Lf->is_com = Lf->is_nfs = Lf->is_stream
 		    = Lf->lmi_srch = Lf->nlink_def = Lf->off_def = Lf->sz_def
+		    = Lf->rdev_def
 		    = (unsigned char)0;
 	Lf->li[0].af = Lf->li[1].af = 0;
 	Lf->lts.type = -1;
@@ -142,17 +180,17 @@ alloc_lfile(nm, num)
 	    Lf->fd[FDLEN - 1] = '\0';
 	} else if (num >= 0) {
 	    if (num < 10000)
-		(void) sprintf(Lf->fd, "%4d", num);
+		(void) snpf(Lf->fd, sizeof(Lf->fd), "%4d", num);
 	    else
-		(void) sprintf(Lf->fd, "*%03d", num % 1000);
+		(void) snpf(Lf->fd, sizeof(Lf->fd), "*%03d", num % 1000);
 	} else
 	    Lf->fd[0] = '\0';
 	Lf->dev_ch = Lf->fsdir = Lf->fsdev = Lf->nm = Lf->nma = (char *)NULL;
 	Lf->ch = -1;
 
-#if	defined(HASNCACHE)
+#if	defined(HASNCACHE) && HASNCACHE<2
 	Lf->na = (KA_T)NULL;
-#endif	/* defined(HASNCACHE) */
+#endif	/* defined(HASNCACHE) && HASNCACHE<2 */
 
 	Lf->next = (struct lfile *)NULL;
 	Lf->ntype = Ntype = N_REGLR;
@@ -201,9 +239,9 @@ alloc_lfile(nm, num)
  */
 
 void
-alloc_lproc(pid, pgrp, ppid, uid, cmd, pss, sf)
+alloc_lproc(pid, pgid, ppid, uid, cmd, pss, sf)
 	int pid;			/* Process ID */
-	int pgrp;			/* process group ID */
+	int pgid;			/* process group ID */
 	int ppid;			/* parent process ID */
 	UID_ARG uid;			/* User ID */
 	char *cmd;			/* command */
@@ -235,7 +273,7 @@ alloc_lproc(pid, pgrp, ppid, uid, cmd, pss, sf)
 	}
 	Lp = &Lproc[Nlproc++];
 	Lp->pid = pid;
-	Lp->pgrp = pgrp;
+	Lp->pgid = pgid;
 	Lp->ppid = ppid;
 	Lp->file = (struct lfile *)NULL;
 	Lp->sf = (short)sf;
@@ -421,6 +459,8 @@ is_cmd_excl(cmd, pss, sf)
 	short *pss;			/* process state */
 	short *sf;			/* process select flags */
 {
+	char buf[1024];
+	int i;
 	struct str_lst *sp;
 /*
  * The command is not excluded if no command selection was requested,
@@ -438,7 +478,19 @@ is_cmd_excl(cmd, pss, sf)
 	    }
 	}
 /*
- * The command name doesn't match any -c <command> specification.
+ * The command name doesn't match any -c <command> specification.  See if it
+ * matches a -c /RE/[bix] specification.
+ */
+	for (i = 0; i < NCmdRxU; i++) {
+	    if (!regexec(&CmdRx[i].cx, cmd, 0, NULL, 0)) {
+		CmdRx[i].mc = 1;
+		*pss |= PS_PRI;
+		*sf |= SELCMD;
+		return(0);
+	    }
+	}
+/*
+ * The command name matches no -c specification.
  *
  * It's excluded if the only selection condition is command name,
  * or if command name selection is part of an ANDed set.
@@ -472,9 +524,9 @@ is_file_sel(lf)
  */
 
 int
-is_proc_excl(pid, pgrp, uid, pss, sf)
+is_proc_excl(pid, pgid, uid, pss, sf)
 	int pid;			/* Process ID */
-	int pgrp;			/* process group ID */
+	int pgid;			/* process group ID */
 	UID_ARG uid;			/* User ID */
 	short *pss;			/* process select state for lproc */
 	short *sf;			/* select flags for lproc */
@@ -518,18 +570,18 @@ is_proc_excl(pid, pgrp, uid, pss, sf)
  * If the listing of processes has been specified by process group ID, see
  * if this one is specified.
  */
-	if (Npgrp && (Selflags & SELPGRP)) {
-	    for (i = 0; i < Npgrp; i++) {
-		if (Spgrp[i].i == pgrp) {
-		    Spgrp[i].f = 1;
+	if (Npgid && (Selflags & SELPGID)) {
+	    for (i = 0; i < Npgid; i++) {
+		if (Spgid[i].i == pgid) {
+		    Spgid[i].f = 1;
 		    *pss = PS_PRI;
-		    *sf = SELPGRP;
-		    if (Selflags == SELPGRP)
+		    *sf = SELPGID;
+		    if (Selflags == SELPGID)
 			return(0);
 		    break;
 		}
 	    }
-	    if (Selflags == SELPGRP && ! *sf)
+	    if (Selflags == SELPGID && ! *sf)
 		return(1);
 	}
 /*
@@ -580,7 +632,7 @@ is_proc_excl(pid, pgrp, uid, pss, sf)
  *	Otherwise, it's not excluded by the tests of this function.
  */
 	if ( ! *sf)
-	    return((Fand && (Selflags & (SELPGRP|SELPID|SELUID))) ? 1 : 0);
+	    return((Fand && (Selflags & (SELPGID|SELPID|SELUID))) ? 1 : 0);
 /*
  * When the process group ID, PID, or UID is selected and the process group
  * ID, PID, or UID list option has been specified:
@@ -592,9 +644,9 @@ is_proc_excl(pid, pgrp, uid, pss, sf)
  *	If list option ANDing has not been specified, reply that the
  *	process is not excluded by the tests of this function.
  */
-	if (Selflags & (SELPGRP|SELPID|SELUID)) {
+	if (Selflags & (SELPGID|SELPID|SELUID)) {
 	    if (Fand)
-		return(((Selflags & (SELPGRP|SELPID|SELUID)) != *sf) ? 1 : 0);
+		return(((Selflags & (SELPGID|SELPID|SELUID)) != *sf) ? 1 : 0);
 	    return(0);
 	}
 /*
@@ -643,8 +695,8 @@ print_fflags(ffg, pof)
 	long pof;		/* process open files flags value */
 {
 	int al, ct, fx;
-	static char *bp = (char *)NULL;
 	static int bl = 0;
+	static char *bp = (char *)NULL;
 	char *sep;
 	int sepl;
 	struct pff_tab *tp;
@@ -676,7 +728,7 @@ print_fflags(ffg, pof)
 		    break;
 		al = (int)strlen(tp->nm) + sepl;
 		bp = alloc_fflbuf(&bp, &bl, al + ct);
-		(void) sprintf((bp + ct), "%s%s", sep, tp->nm);
+		(void) snpf(bp + ct, al + 1, "%s%s", sep, tp->nm);
 		sep = ",";
 		sepl = 1;
 		wf &= ~(tp->val);
@@ -687,10 +739,10 @@ print_fflags(ffg, pof)
 	 * in hex.
 	 */
 	    if (wf || FsvFlagX) {
-		(void) sprintf(xbuf, "0x%lx", wf);
+		(void) snpf(xbuf, sizeof(xbuf), "0x%lx", wf);
 		al = (int)strlen(xbuf) + sepl;
 		bp = alloc_fflbuf(&bp, &bl, al + ct);
-		(void) sprintf((bp + ct), "%s%s", sep, xbuf);
+		(void) snpf(bp + ct, al + 1, "%s%s", sep, xbuf);
 		ct += al;
 	    }
 	}
@@ -754,8 +806,8 @@ print_proc()
 		return(rv);
 	    rv = 1;
 	    (void) printf("%c%d%c", LSOF_FID_PID, Lp->pid, Terminator);
-	    if (FieldSel[LSOF_FIX_PGRP].st && Fpgrp)
-		(void) printf("%c%d%c", LSOF_FID_PGRP, Lp->pgrp, Terminator);
+	    if (FieldSel[LSOF_FIX_PGID].st && Fpgid)
+		(void) printf("%c%d%c", LSOF_FID_PGID, Lp->pgid, Terminator);
 
 #if	defined(HASPPID)
 	    if (FieldSel[LSOF_FIX_PPID].st && Fppid)
@@ -826,7 +878,7 @@ print_proc()
 	    if (FieldSel[LSOF_FIX_FA].st && (Fsv & FSV_FA)
 	    &&  (Lf->fsv & FSV_FA)) {
 		(void) printf("%c%s%c", LSOF_FID_FA,
-		    print_kptr(Lf->fsa, (char *)NULL), Terminator);
+		    print_kptr(Lf->fsa, (char *)NULL, 0), Terminator);
 		lc++;
 	    }
 	    if (FieldSel[LSOF_FIX_CT].st && (Fsv & FSV_CT)
@@ -843,7 +895,7 @@ print_proc()
 	    if (FieldSel[LSOF_FIX_NI].st && (Fsv & FSV_NI)
 	    &&  (Lf->fsv & FSV_NI)) {
 		(void) printf("%c%s%c", LSOF_FID_NI,
-		    print_kptr(Lf->fna, (char *)NULL), Terminator);
+		    print_kptr(Lf->fna, (char *)NULL, 0), Terminator);
 		lc++;
 	    }
 #endif	/* defined(HASFSTRUCT) */
