@@ -31,7 +31,7 @@
 
 
 /*
- * $Id: proto.h,v 1.19 99/11/28 06:33:28 abe Exp $
+ * $Id: proto.h,v 1.24 2001/02/13 02:10:06 abe Exp $
  */
 
 
@@ -63,8 +63,9 @@
 #endif	/* gcc && gcc>=2.7 */
 
 
+_PROTOTYPE(extern void add_nma,(char *cp, int len));
 _PROTOTYPE(extern void alloc_lfile,(char *nm, int num));
-_PROTOTYPE(extern void alloc_lproc,(int pid, int pgrp, int ppid, UID_ARG uid, char *cmd, int pss, int sf));
+_PROTOTYPE(extern void alloc_lproc,(int pid, int pgid, int ppid, UID_ARG uid, char *cmd, int pss, int sf));
 _PROTOTYPE(extern void childx,(void));
 _PROTOTYPE(extern int ck_file_arg,(int i, int ac, char *av[], int fv, int rs, struct stat *sbp));
 _PROTOTYPE(extern void ckkv,(char *d, char *er, char *ev, char *ea));
@@ -76,7 +77,8 @@ _PROTOTYPE(extern int comppid,(COMP_P *a1, COMP_P *a2));
 _PROTOTYPE(extern void dropgid,(void));
 #endif	/* defined(WILLDROPGID) */
 
-_PROTOTYPE(extern char *endnm,(void));
+_PROTOTYPE(extern char *endnm,(size_t *sz));
+_PROTOTYPE(extern int enter_cmd_rx,(char *x));
 _PROTOTYPE(extern void enter_dev_ch,(char *m));
 _PROTOTYPE(extern int enter_dir,(char *d, int descend));
 _PROTOTYPE(extern int enter_fd,(char *f));
@@ -84,9 +86,9 @@ _PROTOTYPE(extern int enter_network_address,(char *na));
 _PROTOTYPE(extern int enter_id,(enum IDType ty, char *p));
 _PROTOTYPE(extern void enter_nm,(char *m));
 _PROTOTYPE(extern int enter_str_lst,(char *opt, char *s, struct str_lst **lp));
-_PROTOTYPE(extern int enter_uid,(char *u));
-_PROTOTYPE(extern int examine_lproc,(void));
+_PROTOTYPE(extern int enter_uid,(char *us));
 _PROTOTYPE(extern void ent_inaddr,(unsigned char *la, int lp, unsigned char *fa, int fp, int af));
+_PROTOTYPE(extern int examine_lproc,(void));
 _PROTOTYPE(extern void Exit,(int xv)) exiting;
 _PROTOTYPE(extern void find_ch_ino,(void));
 _PROTOTYPE(extern void free_lproc,(struct lproc *lp));
@@ -98,25 +100,24 @@ _PROTOTYPE(extern void initialize,(void));
 _PROTOTYPE(extern int is_cmd_excl,(char *cmd, short *pss, short *sf));
 _PROTOTYPE(extern int is_file_sel,(struct lfile *lf));
 _PROTOTYPE(extern int is_nw_addr,(unsigned char *ia, int p, int af));
-_PROTOTYPE(extern int is_proc_excl,(int pid, int pgrp, UID_ARG uid, short *pss, short *sf));
+_PROTOTYPE(extern int is_proc_excl,(int pid, int pgid, UID_ARG uid, short *pss, short *sf));
 _PROTOTYPE(extern int is_readable,(char *path, int msg));
 _PROTOTYPE(extern int kread,(KA_T addr, char *buf, READLEN_T len));
 _PROTOTYPE(extern void link_lfile,(void));
-_PROTOTYPE(extern struct l_dev *lkupdev,(dev_t *dev, int i, int r));
+_PROTOTYPE(extern struct l_dev *lkupdev,(dev_t *dev,dev_t *rdev,int i,int r));
 _PROTOTYPE(extern int main,(int argc, char *argv[]));
 _PROTOTYPE(extern int lstatsafely,(char *path, struct stat *buf));
 _PROTOTYPE(extern char *mkstrcpy,(char *src, MALLOC_S *rlp));
 _PROTOTYPE(extern char *mkstrcat,(char *s1, int l1, char *s2, int l2, char *s3, int l3, MALLOC_S *clp));
-_PROTOTYPE(extern int printchdevname,(dev_t *rdev, int f));
+_PROTOTYPE(extern int printchdevname,(dev_t *dev, dev_t *rdev, int f));
 _PROTOTYPE(extern void print_file,(void));
 _PROTOTYPE(extern void print_init,(void));
-_PROTOTYPE(extern void printiproto,(int p));
 _PROTOTYPE(extern void printname,(int nl));
-_PROTOTYPE(extern char *print_kptr,(KA_T kp, char *buf));
+_PROTOTYPE(extern char *print_kptr,(KA_T kp, char *buf, size_t bufl));
 _PROTOTYPE(extern int print_proc,(void));
 _PROTOTYPE(extern void printrawaddr,(struct sockaddr *sa));
 _PROTOTYPE(extern void print_tcptpi,(int nl));
-_PROTOTYPE(extern char *printuid,(UID_ARG u, int *ty));
+_PROTOTYPE(extern char *printuid,(UID_ARG uid, int *ty));
 _PROTOTYPE(extern void printunkaf,(int fam, int ty));
 _PROTOTYPE(extern char *printsockty,(int ty));
 _PROTOTYPE(extern void process_file,(KA_T fp));
@@ -136,8 +137,8 @@ _PROTOTYPE(extern char *x2dev,(char *s, dev_t *d));
 
 #if	defined(HASBLKDEV)
 _PROTOTYPE(extern void find_bl_ino,(void));
-_PROTOTYPE(extern struct l_dev *lkupbdev,(dev_t *dev, int i, int r));
-_PROTOTYPE(extern int printbdevname,(dev_t *rdev, int f));
+_PROTOTYPE(extern struct l_dev *lkupbdev,(dev_t *dev,dev_t *rdev,int i,int r));
+_PROTOTYPE(extern int printbdevname,(dev_t *dev, dev_t *rdev, int f));
 #endif	/* defined(HASBLKDEV) */
 
 #if	defined(HASCDRNODE)
@@ -191,7 +192,7 @@ _PROTOTYPE(extern int readpipenode,(KA_T pa, struct pipenode *p));
 #endif	/* defined(HASPIPENODE) */
 
 #if	defined(HASPRINTDEV)
-_PROTOTYPE(extern char *HASPRINTDEV,(struct lfile *lf));
+_PROTOTYPE(extern char *HASPRINTDEV,(struct lfile *lf, dev_t *dev));
 #endif	/* defined(HASPRINTDEV) */
 
 #if	defined(HASPRINTINO)
@@ -202,10 +203,6 @@ _PROTOTYPE(extern char *HASPRINTINO,(struct lfile *lf));
 _PROTOTYPE(extern void HASPRINTNM,(struct lfile *lf));
 #endif	/* defined(HASPRINTNM) */
 
-#if	defined(HASPRIVNMCACHE)
-_PROTOTYPE(extern int HASPRIVNMCACHE,(struct lfile *lf));
-#endif	/* defined(HASPRIVNMCACHE) */
-
 #if	defined(HASPRINTOFF)
 _PROTOTYPE(extern char *HASPRINTOFF,(struct lfile *lf, int ty));
 #endif	/* defined(HASPRINTOFF) */
@@ -214,10 +211,22 @@ _PROTOTYPE(extern char *HASPRINTOFF,(struct lfile *lf, int ty));
 _PROTOTYPE(extern char *HASPRINTSZ,(struct lfile *lf));
 #endif	/* defined(HASPRINTSZ) */
 
+#if	defined(HASPRIVNMCACHE)
+_PROTOTYPE(extern int HASPRIVNMCACHE,(struct lfile *lf));
+#endif	/* defined(HASPRIVNMCACHE) */
+
+#if	!defined(HASPRIVPRIPP)
+_PROTOTYPE(extern void printiproto,(int p));
+#endif	/* !defined(HASPRIVPRIPP) */
+
 #if	defined(HASRNODE)
 _PROTOTYPE(extern int readrnode,(KA_T ra, struct rnode *r));
 #endif	/* defined(HASRNODE) */
 
+#if	defined(HASSPECDEVD)
+_PROTOTYPE(extern void HASSPECDEVD,(char *p, struct stat *s));
+#endif	/* defined(HASSPECDEVD) */
+
 #if	defined(HASSNODE)
 _PROTOTYPE(extern int readsnode,(KA_T sa, struct snode *s));
 #endif	/* defined(HASSNODE) */
@@ -237,3 +246,7 @@ _PROTOTYPE(extern int readtnode,(KA_T ta, struct tmpnode *t));
 #if	defined(HASVNODE)
 _PROTOTYPE(extern int readvnode,(KA_T va, struct vnode *v));
 #endif	/* defined(HASVNODE) */
+
+#if	defined(USE_LIB_SNPF)
+_PROTOTYPE(extern int snpf,(char *str, int len, char *fmt, ...));
+#endif	/* defined(USE_LIB_SNPF) */
