@@ -32,7 +32,7 @@
 #ifndef lint
 static char copyright[] =
 "@(#) Copyright 1994 Purdue Research Foundation.\nAll rights reserved.\n";
-static char *rcsid = "$Id: store.c,v 1.22 10/.0/.1 .0:.5:.4 abe Exp $";
+static char *rcsid = "$Id: store.c,v 1.26 2001/02/13 02:10:43 abe Exp $";
 #endif
 
 
@@ -62,6 +62,8 @@ struct clone *Clone = (struct clone *)NULL;
 struct str_lst *Cmdl = (struct str_lst *)NULL;
 				/* command names selected with -c */
 int CmdColW;			/* COMMAND column width */
+lsof_rx_t *CmdRx = (lsof_rx_t *)NULL;
+				/* command regular expression table */
 
 #if	defined(HASDCACHE)
 unsigned DCcksum;		/* device cache file checksum */
@@ -95,6 +97,7 @@ int DCunsafe = 0;		/* device cache file is potentially unsafe,
 int DChelp = 0;			/* -D? status */
 
 int DevColW;			/* DEVICE column width */
+dev_t DevDev;			/* devce number of /dev or its equivalent */
 struct l_dev *Devtp = (struct l_dev *)NULL;
 				/* device table pointer */
 
@@ -118,6 +121,7 @@ int Ffilesys = 0;		/* -f option status:
 
 #if	defined(HASNCACHE)
 int Fncache = 1;		/* -C option status */
+int NcacheReload = 1;		/* 1 == call ncache_load() */
 #endif	/* defined(HASNCACHE) */
 
 int Ffield = 0;			/* -f and -F status */
@@ -138,7 +142,7 @@ int FportMap = 1;		/* +|-M option status */
 int FportMap = 0;		/* +|-M option status */
 #endif	/* defined(HASPMAPENABLED) */
 
-int Fpgrp = 0;			/* -g option status */
+int Fpgid = 0;			/* -g option status */
 int Fppid = 0;			/* -R option status */
 int Fsize = 0;			/* -s option status */
 int FcColW;			/* FCT column width */
@@ -185,7 +189,7 @@ struct fieldsel FieldSel[] = {
     { LSOF_FID_NI,     0,  LSOF_FNM_NI,     &Fsv,     FSV_NI	 }, /* 14 */
     { LSOF_FID_OFFSET, 0,  LSOF_FNM_OFFSET, NULL,     0		 }, /* 15 */
     { LSOF_FID_PID,    1,  LSOF_FNM_PID,    NULL,     0		 }, /* 16 */
-    { LSOF_FID_PGRP,   0,  LSOF_FNM_PGRP,   &Fpgrp,   1		 }, /* 17 */
+    { LSOF_FID_PGID,   0,  LSOF_FNM_PGID,   &Fpgid,   1		 }, /* 17 */
     { LSOF_FID_PROTO,  0,  LSOF_FNM_PROTO,  NULL,     0		 }, /* 18 */
     { LSOF_FID_PPID,   0,  LSOF_FNM_PPID,   &Fppid,   1		 }, /* 19 */
     { LSOF_FID_SIZE,   0,  LSOF_FNM_SIZE,   NULL,     0		 }, /* 20 */
@@ -248,13 +252,15 @@ struct mounts *Mtprocfs = (struct mounts *)NULL;
 				/* /proc mount entry */
 #endif	/* defined(HASPROCFS) */
 
-int Mxpgrp = 0;			/* maximum process groip ID table entries */
+int Mxpgid = 0;			/* maximum process group ID table entries */
 int Mxpid = 0;			/* maximum PID table entries */
 int Mxuid = 0;			/* maximum UID table entries */
 gid_t Mygid;			/* real GID of this lsof process */
 int Mypid;			/* lsof's process ID */
 uid_t Myuid;			/* real UID of this lsof process */
 char *Namech = (char *)NULL;	/* name characters for printing */
+size_t Namechl = (size_t)0;	/* sizeof(Namech) */
+int NCmdRxU = 0;		/* number of CmdRx[] entries */
 int Ndev = 0;			/* number of entries in Devtp[] */
 
 #if	defined(HASNLIST)
@@ -270,7 +276,7 @@ int NlColW;			/* NLINK column width */
 int NmColW;			/* NAME column width */
 char *Nmlst = (char *)NULL;	/* namelist file path */
 int NodeColW;			/* NODE column width */
-int Npgrp = 0;			/* -g option count */
+int Npgid = 0;			/* -g option count */
 int Npid = 0;			/* -p option count */
 int Npuns;			/* number of unselected PIDs (starts at Npid) */
 int Ntype;			/* node type (see N_* symbols) */
@@ -281,7 +287,7 @@ struct nwad *Nwad = (struct nwad *)NULL;
 				/* list of network addresses */
 int OffDecDig = OFFDECDIG;	/* offset decimal form (0t...) digit limit */
 int OffColW;			/* OFFSET column width */
-int PgrpColW;			/* PGRP column width */
+int PgidColW;			/* PGID column width */
 int PidColW;			/* PID column width */
 struct lfile *Plf = (struct lfile *)NULL;
 				/* previous local file structure */
@@ -310,7 +316,7 @@ int Selinet = 0;		/* select only Internet socket files */
 int Setuidroot = 0;		/* setuid-root state */
 struct sfile *Sfile = (struct sfile *)NULL;
 				/* chain of files to search for */
-struct int_lst *Spgrp = (struct int_lst *)NULL;
+struct int_lst *Spgid = (struct int_lst *)NULL;
 				/* process group IDs to search for */
 struct int_lst *Spid = (struct int_lst *)NULL;
 				/* Process IDs to search for */
